package edu.uoc.pfc.business.objects;

import java.io.Serializable;

import edu.uoc.pfc.business.transfers.TOUser;

/**
 * <p>Clase que implementa la relaci&oacute;n que existe entre los Usuarios de la 
 * aplicaci&oacute;n y las diferentes Aulas que se van creando dentro de la misma.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOClassroomUser implements Serializable 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de la clase.</p> */
	private static final long serialVersionUID = 1L;
	/** <p>Identificador de un profesor.</p> */
	public static final String ROLE_TEACHER = "T";
	/** <p>Identificador de un alumno.</p> */
	public static final String ROLE_STUDENT = "S";
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la relaci&oacute;n entre Usuario y Aula.</p> */
	private int id;
	/** <p>El Usuario.</p> */
	private BOUser user;
	/** <p>El Aula.</p> */
	private BOClassroom classroom;
	/** <p>Role del usuario: T = Profesor y S = Estudiante.</p> */
	private String role;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>BOClassroomUser</code> e inicializa sus atributos
	 * a los valores por defecto.</p>
	 */
	public BOClassroomUser()
	{
		// Llamamos al construicto de la superclase.
		super();
	}
	
	/**
	 * <p>Crea un objeto <code>BOClassroomUser</code> a partir de la informaci&oacute;n 
	 * que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param user <code>BOUser</code> el Usuario.
	 * @param classroom <code>BOClassroom</code> el Aula.
	 * @param role <code>String</code> rol del Usuario dentro del Aula (Alumno o Profesor).
	 */
	public BOClassroomUser(BOUser user, BOClassroom classroom, String role) {
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.user = user;
		this.classroom = classroom;
		this.role = role;
	}

	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOUser</code> con la informaci&oacute;n del usuario
	 * que est&aacute; asignado al Aula y que representa el objeto sobre el que se 
	 * invoca este m&eacute;todo.</p>
	 * 
	 * @return <code>TOUser</code> con la informaci&oacute;n del usuario.
	 */
	public TOUser getTOUser()
	{
		// Construimos un objeto TOUser y lo retornamos.
		return (new TOUser(this.user.getId(), this.user.getLogin()));
	}
	
	/**
	 * <p>Retorna un objeto <code>TOUser</code> con toda la informaci&oacute;n
	 * del usuario que est&aacute; asignado al Aula y que representa el objeto sobre 
	 * el que se invoca este m&eacute;todo.</p>.
	 * 
	 * @return <code>TOUser</code> con la informaci&oacute;n del usuario.
	 */
	public TOUser getTOUserPersonalData()
	{
		// Construimos un objeto TOUser.
		TOUser toUser = new TOUser(this.user.getId(), this.user.getName(), 
				this.user.getSurname());
		// Guardamos la direccin de correo electrnico.
		toUser.setEmail(this.user.getEmail());
		// Guardamos la informacin del ltimo acceso.
		toUser.setLastLogin(this.user.getLastLogin());
		// Retornamos el objeto "toUser"
		return toUser;
	}
	
	// -------- GETTERS & SETTERS.
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public BOUser getUser() 
	{
		return user;
	}
	public void setUser(BOUser user) 
	{
		this.user = user;
	}
	public BOClassroom getClassroom() 
	{
		return classroom;
	}
	public void setClassroom(BOClassroom classroom) 
	{
		this.classroom = classroom;
	}
	public String getRole() 
	{
		return role;
	}
	public void setRole(String role) 
	{
		this.role = role;
	}

}
