package edu.uoc.pfc.business.objects;

import java.io.Serializable;

import edu.uoc.pfc.business.transfers.TOConfig;

/**
 * <p>Clase que recoger&aacute; la informaaci&oacute;n de configuraci&oacute;n de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOConfig implements Serializable
{
	
	// -------- CONSTANTES
	/** <p>N&uacute;mero de serie por defecto para los objetos de la clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la configuraci&oacute;n.</p> */
	protected int id;
	/** <p>N&uacute;mero de intentos de acceso a una cuenta de usuario.</p> */
	protected int loginAttempts;
	/** <p>Indica si se trata de la configuraci&oacute;n activa (en uso).</p> */
	protected boolean active;
	/** <p>Indica cual es el curso activo en la aplicaci&oacute;n.</p>*/
	protected int course;
	/** <p>Indica si est&aacute; permitido o no el env&iacute;o de correos electr&oacute;nicos.</p> */
	protected boolean sendEmail;
	/** <p>Servidor SMTP para el env&iacute;o de correos electr&oacute;nicos.</p> */
	protected String smtpServer;
	/** <p>Puerto del servidor SMTP.</p> */
	protected int smtpPort;
	/** <p>Indica si el servidor SMTP precisa de autentificaci&oacute;n o no.</p> */
	protected boolean smtpAuthentication;
	/** <p>Indica si el servidor SMTP utiliza TLS (Transport Layer Security).</p> */
	protected boolean smtpTLS;
	/** <p>Nombre de usuario dentro del servidor SMTP.</p> */
	protected String smtpUser;
	/** <p>Contrasea del usuario dentro del servidor SMTP.</p> */
	protected String smtpPassword;
	/** <p>Direcci&oacute;n desde la que se env&iacute;n las peticiones de recordar <i>login</i>.</p> */
	protected String forgotLoginFrom;
	/** <p>Asunto de las peticiones de recordar <i>login</i>.</p> */
	protected String forgotLoginSubject;
	/** <p>Texto de los correos asociados a las peticiones de recordar <i>login</i>.</p> */
	protected String forgotLoginText;
	/** <p>Direcci&oacute;n desde la que se env&iacute;n las peticiones de recordar <i>contrase&ntilde;a</i>.</p> */
	protected String forgotPasswordFrom;
	/** <p>Asunto de las peticiones de recordar <i>contrase&ntilde;a</i>.</p> */
	protected String forgotPasswordSubject;
	/** <p>Texto de los correos asociados a las peticiones de recordar <i>contrase&ntilde;a</i>.</p> */
	protected String forgotPasswordText;
	/** <p>Direcci&oacute;n de correo electr&oacute;nico de los administradores de la aplicaci&oacute;n.</p> */
	protected String adminEmail;
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOConfig</code> cuya informaci&oacute;n ser&aacute; equivalente al 
	 * objeto sobre el que se invoca al m&eacute;todo.</p>
	 * 
	 * @return <code>TOConfig</code> con la informaci&oacute;n equivalente.
	 */
	public TOConfig getTOConfig()
	{
		// Retornamos un objeto equivalente.
		return (new TOConfig(this.getId(), this.getLoginAttempts(), 
				this.isActive(), this.getCourse(), this.isSendEmail(), 
				this.getSmtpServer(), this.getSmtpPort(), 
				this.isSmtpAuthentication(), this.isSmtpTLS(), 
				this.getSmtpUser(), this.getSmtpPassword(), 
				this.getForgotLoginFrom(), this.getForgotLoginSubject(), 
				this.getForgotLoginText(), this.getForgotPasswordFrom(), 
				this.getForgotLoginSubject(), this.getForgotPasswordText(), 
				this.getAdminEmail()));
	}
	
	/**
	 * <p>Establece los atributos de la configuraci&oacute; a los valores que se le pasan como
	 * par&aacute;metros dejando el resto (los correspondientes a los env&iacute;os de emails desde
	 * un servidor SMTP) vac&iacute;os.</p>
	 * 
	 * @param loginAttemtp <code>int</code> intentos de acceso a la aplicaci&oacute;n.
	 * @param course <code>int</code> curso actual.
	 * @param adminEmail <code>String</code> direcci&oacute;n de correo electr&oacute;nico del
	 * administrador de la aplicaci&oacute;n.
	 */
	public void setNoSMTPEmail(int loginAttemtp, int course, String adminEmail)
	{
		// Establecemos los atributos a los valores de los parmetros.
		this.loginAttempts = loginAttemtp;
		this.course = course;
		this.adminEmail = adminEmail;
		// Establecemos el resto de atributos.
		this.sendEmail = false;
		this.smtpServer = "";
		this.smtpPort = 0;
		this.smtpAuthentication = false;
		this.smtpTLS = false;
		this.smtpUser = "";
		this.smtpPassword = "";
		this.forgotPasswordFrom = "";
		this.forgotPasswordSubject = "";
		this.forgotPasswordText = "";
		this.forgotLoginFrom = "";
		this.forgotLoginSubject = "";
		this.forgotLoginText = "";
	}
	
	/**
	 * <p>Establece los atributos de la configuraci&oacute;n a los valores que se le pasan como
	 * parmetros (indicando de esta manera que &iacute; que se permite el env&iacute;o de 
	 * emails desde un servidor SMTP).</p>
	 * 
	 * @param loginAttempts code>int</code> n&uacute;meros de intentos de acceso a la cuenta.
	 * @param activeCourse <code>int</code> curso actual (activo).
	 * @param adminEmail <code>String</code> email del administrador.
	 * @param smtpServer <code>String</code> servidor SMTP.
	 * @param smtpPort <code>int</code> puerto del servidor SMTP.
	 * @param smtpAuthentication <code>boolean</code> indica si el servidor SMTP requiero autentificaci&oacute;n
	 * @param smtpTLS <code>boolean</code> indica si el servidor SMTP admite el protocolo TLS.
	 * @param smtpUser <code>String</code> usuario servidor SMTP
	 * @param smtpPassword <code>String</code> password servidor SMTP.
	 * @param fpFrom <code>String</code> email de quien env&iacute;a el correo de recuperaci&oacute; de contrase&ntilde;a.
	 * @param fpSubject <code>String</code> asunto del correo de recuperaci&oacute; de contrase&ntilde;a.
	 * @param fpText <code>String</code> texto del correo de recuperaci&oacute; de contrase&ntilde;a.
	 * @param flFrom <code>String</code> email de quien env&iacute;a el correo de recuperaci&oacute; de login.
	 * @param flSubject <code>String</code> asunto del correo de recuperaci&oacute; de login.
	 * @param flText <code>String</code> texto del correo de recuperaci&oacute; de login.
	 */
	public void setSMTPEmail(int loginAttempts, int activeCourse, String adminEmail,
			String smtpServer, int smtpPort, boolean smtpAuthentication, boolean smtpTLS, 
			String smtpUser, String smtpPassword, String fpFrom, String fpSubject, String fpText,
			String flFrom, String flSubject, String flText)
	{	
		// Indicamos que s se permite el envo de emails.
		this.sendEmail = true;
		// Establecemos los atributos de los parmetros.
		this.loginAttempts = loginAttempts;
		this.course = activeCourse;
		this.adminEmail = adminEmail;
		this.smtpServer = smtpServer;
		this.smtpPort = smtpPort;
		this.smtpAuthentication = smtpAuthentication;
		this.smtpTLS = smtpTLS;
		this.smtpUser = smtpUser;
		this.smtpPassword = smtpPassword;
		this.forgotPasswordFrom = fpFrom;
		this.forgotPasswordSubject = fpSubject;
		this.forgotPasswordText = fpText;
		this.forgotLoginFrom = flFrom;
		this.forgotLoginSubject = flSubject;
		this.forgotLoginText = flText;
	}
	
	// -------- GETTERS & SETTERS
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public int getLoginAttempts()
	{
		return loginAttempts;
	}
	public void setLoginAttempts(int loginAttempts)
	{
		this.loginAttempts = loginAttempts;
	}
	public boolean isActive()
	{
		return active;
	}
	public void setActive(boolean active)
	{
		this.active = active;
	}
	public int getCourse()
	{
		return course;
	}
	public void setCourse(int course)
	{
		this.course = course;
	}
	public boolean isSendEmail()
	{
		return sendEmail;
	}
	public void setSendEmail(boolean sendEmail)
	{
		this.sendEmail = sendEmail;
	}
	public String getSmtpServer()
	{
		return smtpServer;
	}
	public void setSmtpServer(String smtpServer)
	{
		this.smtpServer = smtpServer;
	}
	public int getSmtpPort()
	{
		return smtpPort;
	}
	public void setSmtpPort(int smtpPort)
	{
		this.smtpPort = smtpPort;
	}
	public boolean isSmtpAuthentication()
	{
		return smtpAuthentication;
	}
	public void setSmtpAuthentication(boolean smtpAuthentication)
	{
		this.smtpAuthentication = smtpAuthentication;
	}
	public boolean isSmtpTLS()
	{
		return smtpTLS;
	}
	public void setSmtpTLS(boolean smtpTLS)
	{
		this.smtpTLS = smtpTLS;
	}
	public String getSmtpUser()
	{
		return smtpUser;
	}
	public void setSmtpUser(String smtpUser)
	{
		this.smtpUser = smtpUser;
	}
	public String getSmtpPassword()
	{
		return smtpPassword;
	}
	public void setSmtpPassword(String smtpPassword)
	{
		this.smtpPassword = smtpPassword;
	}
	public String getForgotPasswordFrom()
	{
		return forgotPasswordFrom;
	}
	public void setForgotPasswordFrom(String forgotPasswordFrom)
	{
		this.forgotPasswordFrom = forgotPasswordFrom;
	}
	public String getForgotPasswordSubject()
	{
		return forgotPasswordSubject;
	}
	public void setForgotPasswordSubject(String forgotPasswordSubject)
	{
		this.forgotPasswordSubject = forgotPasswordSubject;
	}
	public String getForgotPasswordText()
	{
		return forgotPasswordText;
	}
	public void setForgotPasswordText(String forgotPasswordText)
	{
		this.forgotPasswordText = forgotPasswordText;
	}
	public String getForgotLoginFrom()
	{
		return forgotLoginFrom;
	}
	public void setForgotLoginFrom(String forgotLoginFrom)
	{
		this.forgotLoginFrom = forgotLoginFrom;
	}
	public String getForgotLoginSubject()
	{
		return forgotLoginSubject;
	}
	public void setForgotLoginSubject(String forgotLoginSubject)
	{
		this.forgotLoginSubject = forgotLoginSubject;
	}
	public String getForgotLoginText()
	{
		return forgotLoginText;
	}
	public void setForgotLoginText(String forgotLoginText)
	{
		this.forgotLoginText = forgotLoginText;
	}
	public String getAdminEmail()
	{
		return adminEmail;
	}
	public void setAdminEmail(String adminEmail)
	{
		this.adminEmail = adminEmail;
	}
	
}
