package edu.uoc.pfc.business.objects;

import java.io.Serializable;

import edu.uoc.pfc.business.transfers.TOQuestion;

/**
 * <p>Clase que modela a las diferentes preguntas que un Profesor puede definir dentro
 * de su Banco de Preguntas.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOQuestion implements Serializable
{

	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	/** <p>Tipo de Pregunta: Verdadero o Falso.</p> */
	public static final String TYPE_TRUE_OR_FALSE = "TF";
	/** <p>Tipo de Pregunta: Respuesta M&uacute;ltiple.</p> */
	public static final String TYPE_MULTIPLE_CHOICES = "MC";
	/** <p>Tipo de Pregunta: Respuesta corta.</p> */
	public static final String TYPE_SHORT_ANSWER = "SA";
	/** <p>Tipo de Pregunta: Num&eacute;rica.</p> */
	public static final String TYPE_NUMERIC = "NU";
	/** <p>Tipo de Pregunta: Emparejar.</p> */
	public static final String TYPE_MATCH = "MA";
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la Pregunta.</p> */
	protected int id;
	/** <p>Categor&iacute;a a la que pertenece.</p> */
	protected BOQuestionCategory category;
	/** <p>T&iacute;tulo de la Pregunta.</p> */
	protected String title;
	/** <p>Enunciado de la Pregunta.</p> */
	protected String statement;
	/** <p>Tipo de Pregunta.</p> */
	protected String type;
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOQuestion</code> con la misma informaci&oacute;n que el
	 * objeto <code>BOQuestion</code> sobre el cual se ha llamado a este m&eacute;todo.</p>
	 * 
	 * @return <code>TOQuestion</code> con la misma informaci&oacute;n que este objeto (this).
	 */
	public TOQuestion getTOQuestion()
	{
		// Creamos un objeto TOQuestion a partir de la inforamcin de "this".
		TOQuestion toQuestion = new TOQuestion(this.id, this.title, this.statement, this.type);
		// Le asignamos el identificador de la Categora de Preguntas siempre que Categoty no sea nulo.
		if (this.category != null)
		{
			// Asignamos el identificador de la categora.
			toQuestion.setCategoryId(this.category.getId());
		}
		return toQuestion;
	}
	
	// -------- GETTERS & SETTERS.
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public BOQuestionCategory getCategory()
	{
		return category;
	}
	public void setCategory(BOQuestionCategory category)
	{
		this.category = category;
	}
	public String getTitle()
	{
		return title;
	}
	public void setTitle(String title)
	{
		this.title = title;
	}
	public String getStatement()
	{
		return statement;
	}
	public void setStatement(String statement)
	{
		this.statement = statement;
	}
	public String getType()
	{
		return type;
	}
	public void setType(String type)
	{
		this.type = type;
	}

}
