package edu.uoc.pfc.business.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import edu.uoc.pfc.business.transfers.TOQuestion;
import edu.uoc.pfc.business.transfers.TOQuestionCategory;

/**
 * <p>Clase que modela a las diferentes Categor&iacute;as de Preguntas que un Profesor puede
 * definir dentro de su Banco de Preguntas.</p> 
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOQuestionCategory implements Serializable
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la Categor&iacute;a de Preguntas dentro del sistema.</p> */
	private int id;
	/** <p>Usuario (Profesor) al que pertenece la Categor&iacute;a de Preguntas.</p> */
	private BOUser user;
	/** <p>Nombre de la Categor&iacute;a de Preguntas.</p> */
	private String name;
	/** <p>Descripci&oacute;n de la Categor&iacute;a de Preguntas.</p> */
	private String description;
	/** <p>Conjunto de Preguntas que pertencen a esta Categor&iacugte;a.</p> */
	private Set<BOQuestion> questions;
	
	// --------
	/**
	 * <p>Constructor por defecto. Construye un objeto <code>BOQuestionCategory</code> e inicializa
	 * sus atributos a los valores por defecto.</p>
	 */
	public BOQuestionCategory()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOQuestionCategory</code> a partir de la informaci&oacute;n 
	 * que se pasa como par&aacute;metro.</p>
	 * 
	 * @param teacher <code>BOUser</code> el Profesor due&ntilde;o de la Categor&iacute;a.
	 * @param name <code>String</code> nombre de la Categor&iacute;a de Preguntas.
	 * @param description <code>String</code> descripci&oacute;n de la Categor&iacute;a.
	 */
	public BOQuestionCategory(BOUser teacher, String name, String description)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Establecemos los valores de los atributos.
		this.user = teacher;
		this.name = name;
		this.description = description;
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOQuestionCategory</code> con la informaci&oacute;n equivalente
	 * al objeto sobre el cual se invoca al m&eacute;todo. Esta informaci&oacute;n incluye el 
	 * identificador, nombre y el n&uacute;mero de preguntas dentro del categor&iacute;a.</p>
	 * 
	 * @return <code>TOQuestionCategory</code> con la informaci&oacute;n equivalente.
	 */
	public TOQuestionCategory getTOQuestionCategory()
	{
		// Definimos una variable local para el total de preguntas de la categora.
		int total_questions = 0;
		// Vemos si "questions" es distinto de nulo.
		if (this.questions != null)
		{
			// Hay preguntas dentro de la categora.
			total_questions = this.questions.size();
		}	
		// Creamos un objeto TOQuestionCategory y lo retornamos.
		return (new TOQuestionCategory(this.id, this.name, total_questions));
	}
	
	/**
	 *<p>Retorna un objeto <code>TOQuestionCategory</code> con la informaci&oacute;n completa 
	 * equivalente al objeto sobre el cual se invoca al m&eacute;todo. </p>
	 * 
	 * @return <code>TOQuestionCategory</code> con la informaci&oacute;n equivalente.
	 */
	public TOQuestionCategory getTOQuestionCategoryComplete()
	{
		// Creamos una lista de objetos TOQuestion para almacenar las preguntas.
		List<TOQuestion> list_questions = new ArrayList<TOQuestion>();
		// Recorremos el conjunto de Preguntas de la Categora.
		for (BOQuestion question : this.questions)
		{
			// Aadimos a la lista el objeto TOQuestion equivalente.
			list_questions.add(question.getTOQuestion());
		}
		// Ordenamos la lista de preguntas.
		Collections.sort(list_questions);
		// Creamos un objeto TOQuestionCategory y lo retornamos.
		return (new TOQuestionCategory(this.id, this.name, this.description, list_questions));
	}
	
	// --------- GETTERS & SETTERS.
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public BOUser getUser()
	{
		return user;
	}
	public void setUser(BOUser user)
	{
		this.user = user;
	}
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	public Set<BOQuestion> getQuestions()
	{
		return questions;
	}
	public void setQuestions(Set<BOQuestion> questions)
	{
		this.questions = questions;
	}	
	
}
