package edu.uoc.pfc.business.objects;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import edu.uoc.pfc.business.transfers.TOQuestionMatch;
import edu.uoc.pfc.business.transfers.TOQuestionMatchAnswer;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que modela las preguntas de tipo "Emparejar".</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOQuestionMatch extends BOQuestion {
	
	// -------- CONSTANTES.	
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>N&uacute;mero de parejas a emparejar.</p> */
	private int options;
	/** <p>Conjunto de respuestas.</p> */
	private Set<BOQuestionMatchAnswer> answers;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>BOQuestionMatch</code> e inicializa sus atributos
	 * a los valores por defecto.</p>
	 */
	public BOQuestionMatch()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOQuestionMatch</code> a partir de la informaci&oacute;n
	 * contenida en el objeto <code>TOQuestionMatch</code> que recibe como parmetro.</p>
	 * 
	 * @param to <code>TOQuestionMatch</code> con la informaci&oacute;n a partir de la cual
	 * construir el objeto.
	 */
	public BOQuestionMatch(TOQuestionMatch to)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Comprobamos que el parmetro no es nulo.
		if (to != null)
		{
			// Establecemos los valores de los atributos.
			this.title = WebUtils.stringToHTML(to.getTitle());
			this.statement = WebUtils.stringToHTML(to.getStatement());
			this.type = BOQuestion.TYPE_MATCH;
			this.options = to.getOptions();
		}
	}
	
	// -------- GETTERS & SETTERS.
	/**
	 * <p>Retorna un objeto <code>TOQuestionMatch</code> cuya informaci&oacute;n es 
	 * equivalente al objeto "this" sobre el que se invoca el m&eacute;todo.</p>
	 * 
	 * @return <code>TOQuestionMatch</code> con la informaci&oacute;n de la pregunta.
	 */
	public TOQuestionMatch getTOQuestionMatch()
	{
		// Creamos una lista de objeto TOQuestionMatchAnswer
		List<TOQuestionMatchAnswer> list_answers = new ArrayList<TOQuestionMatchAnswer>();
		// Recorremos el conjunto de opciones de respuesta.
		for (BOQuestionMatchAnswer answer : this.answers)
		{
			// Obtenemos el objeto TOQuestionMatchAnswer y lo aadimos a la lista.
			list_answers.add(answer.getTOQuestionMatchAnswer());
		}
		// Ordenamos la lista de opciones de respuesta.
		Collections.sort(list_answers);
		// Creamos un objeto TOQuestionMatch y lo retornamos.
		return (new TOQuestionMatch(this.id, this.title, this.statement, 
				list_answers, this.category.getId()));		
	}
	
	// -------- GETTERS & SETTERS.
	public int getOptions() 
	{
		return options;
	}
	public void setOptions(int options) 
	{
		this.options = options;
	}
	public Set<BOQuestionMatchAnswer> getAnswers() 
	{
		return answers;
	}
	public void setAnswers(Set<BOQuestionMatchAnswer> answers) 
	{
		this.answers = answers;
	}
	
}
