package edu.uoc.pfc.business.objects;

import java.io.Serializable;

import edu.uoc.pfc.business.transfers.TOQuestionMatchAnswer;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que modela cada una de las respuestas de una pregunta de tipo 
 * "Emparajar" (cada instancia representar&aacute; una parejera de opcines
 * correcta).</p> 
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOQuestionMatchAnswer implements Serializable 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la Respuesta.</p> */
	private int id;
	/** <p>Uno de los dos miembros del par de la respuesta.</p> */
	private String text1;
	/** <p>El otro miembro de la pareja.</p> */
	private String text2;
	/** <p>La Pregunta a la que pertenece la respuesta.</p> */
	private BOQuestionMatch question;
	/** <p>Posici&oacute;n en la que ser&aacute; mostrada la respuesta.</p> */
	private int position;
	/** <p>Texto de refuerzo.</p> */
	private String explain;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>BOQuestionMatchAnswer</code> e inicializa sus atributos
	 * a los valores por defecto.</p>
	 */
	public BOQuestionMatchAnswer()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOQuestionMatchAnswer</code> a partir de la informaci&oacute;n
	 * contenida en los objetos que se le pasan como par&aacute;metros.</p>
	 * 
	 * @param boMA <code>BOQuestionMatch</code> con la informaci&oacute;n de la pregunta a la que
	 * pertenece la respuesta.
	 * @param answer <code>TOQuestionMatchAnswer</code> con la infomraci&oacute;n de la respuesta.
	 */
	public BOQuestionMatchAnswer(BOQuestionMatch boMA, TOQuestionMatchAnswer answer)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Comprobamos que ambos parmetros no sean nulos.
		if ((boMA != null) && (answer != null))
		{
			// Establecemos los valores de los atributos.
			this.text1 = WebUtils.stringToHTML(answer.getText1());
			this.text2 = WebUtils.stringToHTML(answer.getText2());
			this.question = boMA;
			this.position = answer.getPosition();
			this.explain = WebUtils.stringToHTML(answer.getExplain());
		}
	}
	
	// -------- GETTERS & SETTERS.
	/**
	 * <p>Retorna un objeto <code>TOQuestionMatchAnswer</code> con la misma informaci&oacute;n
	 * que el objeto "this" sobre el cual se invoca al m&eacute;todo.</p>
	 * 
	 * @return <code>TOQuestionMatchAnswer</code> con la misma informaci&oacute;n que el objeto
	 * sobre el que se ha llamado al m&eacute;todo.
	 */
	public TOQuestionMatchAnswer getTOQuestionMatchAnswer()
	{
		// Construimos un objeto TOQuestionMatchAnswer y lo retornamos.
		return (new TOQuestionMatchAnswer(this.id, this.text1, this.text2, 
				this.position, this.explain));
	}
	
	// -------- GETTERS & SETTERS
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public String getText1() 
	{
		return text1;
	}
	public void setText1(String text1) 
	{
		this.text1 = text1;
	}
	public String getText2() 
	{
		return text2;
	}
	public void setText2(String text2) 
	{
		this.text2 = text2;
	}
	public BOQuestionMatch getQuestion() 
	{
		return question;
	}
	public void setQuestion(BOQuestionMatch question) 
	{
		this.question = question;
	}
	public int getPosition() 
	{
		return position;
	}
	public void setPosition(int position) 
	{
		this.position = position;
	}
	public String getExplain() 
	{
		return explain;
	}
	public void setExplain(String explain) 
	{
		this.explain = explain;
	}
	
}
