package edu.uoc.pfc.business.objects;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import edu.uoc.pfc.business.transfers.TOQuestionMultipleChoices;
import edu.uoc.pfc.business.transfers.TOQuestionMultipleChoicesAnswer;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que modela a las pregunta de tipo "Opci&oacute;n M&uacute;ltiple" en el sistema.</p> 
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOQuestionMultipleChoices extends BOQuestion 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>N&uacute;mero de respuestas correctas.</p> */
	private int correct;
	/** <p>Conjunto de respuestas asociadas a la pregunta.</p> */
	private Set<BOQuestionMultipleChoicesAnswer> answers;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Crea un objeto <code>BOQuestionMultipleChoices</code> e inicializa los
	 * atributos a sus valores por defecto.</p>
	 */
	public BOQuestionMultipleChoices()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Crea un objeto <code>BOQuestionMultipleChoices</code> a partir de la informaci&oacute;n
	 * contenida en el objeto <code>TOQuestionMultipleChoices</code>.</p>
	 * 
	 * @param to <code>TOQuestionMultipleChoices</code> con la informaci&aocute;n a partir de 
	 * la cual se ha de crear el objeto.
	 */
	public BOQuestionMultipleChoices(TOQuestionMultipleChoices to)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.title = WebUtils.stringToHTML(to.getTitle());
		this.statement = WebUtils.stringToHTML(to.getStatement());
		this.type = BOQuestion.TYPE_MULTIPLE_CHOICES;
		// Comprobamos que el parmetro no sea nulo.
		if ((to != null) && (to.getAnswers() != null))
		{
			// Asignamos los valores a los atributos.
			this.correct = to.getAnswers().size();
		}
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOQuestionMultipleChoices</code> que contendr&aacute; la misma
	 * informaci&oacute;n que el objeto "this" sobre el que se invoca al m&eacute;todo.</p>
	 * 
	 * @return <code>TOQuestionMultipleChoices</code> con la misma informaci&oacute;n que el 
	 * objeto "this" sobre el que se ha llamado al m&eacute;todo.
	 */
	public TOQuestionMultipleChoices getTOQuestionMultipleChoices()
	{
		
		// Creamos una lista con las opciones de respuesta asociadas a la pregunta.
		List<TOQuestionMultipleChoicesAnswer> list_answers = new ArrayList<TOQuestionMultipleChoicesAnswer>();
		// Recorremos todas las respuesta.
		for (BOQuestionMultipleChoicesAnswer boAnswer : this.answers)
		{
			// Obtenemos el objeto TOQuestionMultipleChoicesAnswer equivalente a boAnswer y lo 
			// aadimos a la lista de respuestas.
			list_answers.add(boAnswer.getTOQuestionMultipleChoicesAnswer());
		}
		// Ordenamos la lista de respuetas.
		Collections.sort(list_answers);
		// Creamos un objeto TOQuestionMultipleChoices y lo retornamos.
		return (new TOQuestionMultipleChoices(this.id, this.title, this.statement, 
				list_answers, this.getCategory().getId()));
	}
		
	// -------- GETTERS & SETTERS.
	public int getCorrect() 
	{
		return correct;
	}
	public void setCorrect(int correct) 
	{
		this.correct = correct;
	}
	public Set<BOQuestionMultipleChoicesAnswer> getAnswers() 
	{
		return answers;
	}
	public void setAnswers(Set<BOQuestionMultipleChoicesAnswer> answers) 
	{
		this.answers = answers;
	}
	
}