package edu.uoc.pfc.business.objects;

import java.io.Serializable;

import edu.uoc.pfc.business.transfers.TOQuestionMultipleChoicesAnswer;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que modela cada una de las opciones de respuesta en una pregunta de 
 * tipo "Opci&oacute;n M&uacute;ltiple".</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOQuestionMultipleChoicesAnswer implements Serializable 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de la clase. </p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Indentificador de la opci&oacute;n.</p> */
	private int id;
	/** <p>Pregunta a la que pertenece.</p> */
	private BOQuestionMultipleChoices question;
	/** <p>Posici&oacute;n que ocupa dentro de las opciones respuesta.</p> */
	private int order;
	/** <p>Texto de la respuesta.</p> */
	private String answer;
	/** <p>Indica si es correcta o no.</p> */
	private boolean correct;
	/** <p>Texto de refuerzo asociado a la opci&oacute;n.</p> */
	private String text;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Crea un objeto <code>BOQuestionMultipleChoicesAnswer</code> e inicializa
	 * sus atributos a los valores por defecto.</p>
	 */
	public BOQuestionMultipleChoicesAnswer()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Crea un objeto <code>BOQuestionMultipleChoicesAnswer</code> a partir de la 
	 * informaci&oacute;n que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param question <code>BOQuestionMultipleChoices</code> pregunta a la cual pertenece la opci&oacute;n.
	 * @param to <code>TOQuestionMultipleChoicesAnswer</code> con la inforamaci&oacute;n
	 * a partir de la cual crear la respuesta.
	 */
	public BOQuestionMultipleChoicesAnswer(BOQuestionMultipleChoices question, 
			TOQuestionMultipleChoicesAnswer to)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Comprobamos que ambos parmetros no sean nulos.
		if ((question != null) && (to != null))
		{
			// Asignamos los valores de los atributos.
			this.question = question;
			this.order = to.getOrder();
			this.answer = WebUtils.stringToHTML(to.getAnswer());
			this.correct = to.isCorrect();
			this.text = WebUtils.stringToHTML(to.getText());
		}
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOQuestionMultipleChoicesAnswer</code> cuya informaci&oacute;n ser&aacute;
	 * la misma que la del objeto "this" sobre el que se invoca al m&eacute;todo.</p>
	 * 
	 * @return <code>TOQuestionMultipleChoicesAnswer</code> con la misma informaci&oacute;n que el objeto
	 * "this" sobre el que se ha invocado al m&eacute;todo.
	 */
	public TOQuestionMultipleChoicesAnswer getTOQuestionMultipleChoicesAnswer()
	{
		// Construimos un objeto TOQuestionMultipleChoicesAnswer a partir de "this" y lo retornamos.
		return (new TOQuestionMultipleChoicesAnswer(this.id, this.order, this.answer, 
				this.correct, this.text));
	}
	
	// -------- GETTERS & SETTERS.
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public BOQuestionMultipleChoices getQuestion() 
	{
		return question;
	}
	public void setQuestion(BOQuestionMultipleChoices question) 
	{
		this.question = question;
	}
	public int getOrder() 
	{
		return order;
	}
	public void setOrder(int order) 
	{
		this.order = order;
	}
	public String getAnswer() 
	{
		return answer;
	}
	public void setAnswer(String answer) 
	{
		this.answer = answer;
	}
	public boolean isCorrect() 
	{
		return correct;
	}
	public void setCorrect(boolean correct) 
	{
		this.correct = correct;
	}
	public String getText() 
	{
		return text;
	}
	public void setText(String text) 
	{
		this.text = text;
	}

}
