package edu.uoc.pfc.business.objects;

import edu.uoc.pfc.business.transfers.TOQuestionNumeric;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que modela a las Preguntas de tipo Num&eacute;rico.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOQuestionNumeric extends BOQuestion
{

	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Valor correcto de la respuesta.</p> */
	private double value;
	/** <p>Margen de error permitido en la respuesta para ser considerada correcta.</p> */
	private double error; 
	/** <p>Texto de refuerzo.</p> */
	private String text;
	
	// ------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>BOQuestionNumeric</code> e inicializa sus atributos a
	 * los valores por defecto.</p>
	 */
	public BOQuestionNumeric()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOQuestionNumeric</code> e inicializa sus atributos a
	 * los valores recogidos en los atributos del objeto <code>TOQuestionNumeric</code> que 
	 * se le pasa como par&aacute;metro.</p>
	 * 
	 * @param to <code>TOQuestionNumeric</code> con la informaci&oacute;n a partir de la cual
	 * se construir&aacute; el objeto.
	 */
	public BOQuestionNumeric(TOQuestionNumeric to)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Comprobamos que "to" no sea nulo.
		if (to != null)
		{
			// Asignamos los valores de los atributos.
			this.title = WebUtils.stringToHTML(to.getTitle());
			this.statement = WebUtils.stringToHTML(to.getStatement());
			this.type = BOQuestion.TYPE_NUMERIC;
			this.value = to.getValue();
			this.error = to.getError();
			this.text = WebUtils.stringToHTML(to.getText());
		}
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOQuestionNumeric</code> que contiene informaci&oacute;n equivalente
	 * al objeto "this" sobre el que se llama a este m&eacute;todo.</p>
	 * 
	 * @return <code>TOQuestionNumeric</code> que representa al objeto "this" sobre el que se llama 
	 * a este m&eacute;todo.
	 */
	public TOQuestionNumeric getTOQuestionNumeric()
	{
		// Construimos un objeto TOQuestionNumeric y lo retornamos.
		return (new TOQuestionNumeric(this.id, this.title, this.statement, this.value, 
				this.error, this.text, this.getCategory().getId()));
	}
	
	// -------- GETTERS & SETTERS.
	public double getValue()
	{
		return value;
	}
	public void setValue(double value)
	{
		this.value = value;
	}
	public double getError()
	{
		return error;
	}
	public void setError(double error)
	{
		this.error = error;
	}
	public String getText()
	{
		return text;
	}
	public void setText(String text)
	{
		this.text = text;
	}	

}
