package edu.uoc.pfc.business.objects;

import edu.uoc.pfc.business.transfers.TOQuestionShortAnswer;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que modela a las preguntas de tipo "Respuesta Corta" dentro del sistema.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOQuestionShortAnswer extends BOQuestion 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Indica si la respuesta es sensible a may&uacute;sculas y min%uacute;sculas.</p> */
	private boolean caseSensitive;
	/** <p>Valor de la respuesta correcta.</p> */
	private String answer;
	/** <p>Texto de refuerzo.</p> */
	private String text;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>BOQuestionShortAnswer</code> e inicializa sus atributos
	 * a los valores por defecto.</p>
	 */
	public BOQuestionShortAnswer()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOQuestionShortAnswer</code> e inicializa sus atributos a
	 * partir de los valores contenidos en el objeto <code>TOQuestionShortAnswer</code> que se
	 * recibe como par&aacute;metro.</p>
	 * 
	 * @param to <code>TOQuestionShortAnswer</code> con los valores de la pregunta.
	 */
	public BOQuestionShortAnswer(TOQuestionShortAnswer to)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Comprobamos que el parmetro no sea nulo.
		if (to != null)
		{
			// Asignamos los valores de los atributos.
			this.title = WebUtils.stringToHTML(to.getTitle());
			this.statement = WebUtils.stringToHTML(to.getStatement());
			this.type = BOQuestion.TYPE_SHORT_ANSWER;
			this.answer =WebUtils.stringToHTML(to.getAnswer());
			this.caseSensitive = to.isCaseSensitive();
			this.text = WebUtils.stringToHTML(to.getText());
		}
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOQuestionShortAnswer</code> el cual contendr&aacute;
	 * la misma informaci&oacute;n que el objeto "this" sobre el que se invoca este m&eacute;todo.</p>
	 * 
	 * @return <code>TOQuestionShortAnswer</code> con la informaci&oacute;n equivalente.
	 */
	public TOQuestionShortAnswer getTOQuestionShortAnswer()
	{
		// Retornamos un objeto TOQuestionShortAnswer con la informacin equivalente.
		return (new TOQuestionShortAnswer(this.id, this.title, this.statement, 
				this.answer, this.caseSensitive, this.text, this.category.getId()));
	}
	
	// -------- GETTERS & SETTERs.
	public boolean isCaseSensitive() 
	{
		return caseSensitive;
	}
	public void setCaseSensitive(boolean caseSensitive) 
	{
		this.caseSensitive = caseSensitive;
	}
	public String getAnswer() 
	{
		return answer;
	}
	public void setAnswer(String answer) 
	{
		this.answer = answer;
	}
	public String getText() 
	{
		return text;
	}
	public void setText(String text) 
	{
		this.text = text;
	}

}
