package edu.uoc.pfc.business.objects;

import edu.uoc.pfc.business.transfers.TOQuestionTrueOrFalse;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Modela a las preguntas de tipo "Verdadero/Falso".</p>
 *  
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOQuestionTrueOrFalse extends BOQuestion 
{

	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Opci&aocute;n correcta.</p> */
	private boolean value;
	/** <p>Texto de refuerzo de la opci&oacute;n verdadera.</p> */
	private String txtTrue;
	/** <p>Texto de refuerzo de la opci&oacute;n falsa.</p> */
	private String txtFalse;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>BOQuestionTrueOrFalse</code> e inicializa los atributos
	 * a sus valores por defecto.</p>
	 */
	public BOQuestionTrueOrFalse() 
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOQuestionTrueOrFalse</code> a partir de la informaci&oacute;n
	 * del objeto <code>TOQuestionTrueOrFalse</code> que se recibe como par&aacute;metro.</p>
	 * 
	 * @param question <code>TOQuestionTrueOrFalse</code> con la informaci&oacute;n a partir de la
	 * cual se ha de construir el <code>BOQuestionTrueOrFalse</code>.
	 */
	public BOQuestionTrueOrFalse(TOQuestionTrueOrFalse question)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Comprobamos si el parmetro es nulo.
		if (question != null)
		{
			// Inicializamos los atributos.
			this.id = question.getId();
			this.type = BOQuestion.TYPE_TRUE_OR_FALSE;
			this.title = WebUtils.stringToHTML(question.getTitle());
			this.statement = WebUtils.stringToHTML(question.getStatement());
			this.value = question.isValue();
			this.txtTrue = WebUtils.stringToHTML(question.getTxtTrue());
			this.txtFalse = WebUtils.stringToHTML(question.getTxtFalse());
		}
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que retorna un objeto <code>TOQuestionTrueOrFalse</code> con la misma
	 * informaci&oacute;n que el objeto "this" sobre el que se invoca a este m&eacute;todo.</p>
	 * 
	 * @return <code>TOQuestionTrueOrFalse</code> con la informaci&aocute;n equivalente a "this".
	 */
	public TOQuestionTrueOrFalse getTOQuestionTrueOrFalse()
	{
		// Construimos un objeto TOQuestionTrueOrFalse equivalente y lo retornamos.
		return (new TOQuestionTrueOrFalse(this.id, this.title, 
				this.statement, this.value, this.txtTrue, this.txtFalse,
				this.category.getId()));
	}
	
	// -------- GETTERS & SETTERS
	public boolean isValue() 
	{
		return value;
	}
	public void setValue(boolean value) 
	{
		this.value = value;
	}
	public String getTxtTrue() 
	{
		return txtTrue;
	}
	public void setTxtTrue(String txtTrue) 
	{
		this.txtTrue = txtTrue;
	}
	public String getTxtFalse() 
	{
		return txtFalse;
	}
	public void setTxtFalse(String txtFalse) 
	{
		this.txtFalse = txtFalse;
	}
	
}
