package edu.uoc.pfc.business.objects;

import java.io.Serializable;

import edu.uoc.pfc.business.transfers.TOSubjectCurriculum;

/**
 * <p>Clase que modela la relaci&oacute;n que existe entre una asignatura y un plan de estudios. 
 * Una misma asignatura puede ser impartida en varios planes de estudios diferentes y en cada
 * uno de ellos puede tener un car&aacute;cter diferentes (optativo, obligatorio, etc.) adem&aacute;s
 * de un n&uacute;mero de cr&eacute;ditos y descripci&oacute;n distinta.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOSubjectCurriculum implements Serializable, Comparable<BOSubjectCurriculum>
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la relaci&oacute;n dentro del sistema.</p> */
	protected int id;
	/** <p>El plan de estudios.</p> */
	protected BOCurriculum curriculum;
	/** <p>C&oacute;digo de la asignatura.</p> */
	protected String code;
	/** <p>N&uacute;mero de cr&eacute;ditos.</p> */
	protected double credits;
	/** <p>Tipo de asignatura.</p> */
	protected String type;
	/** <p>Descripci&oacute;n de la asignatura.</p> */
	protected String description;
	/** <p>La asignatura.</p> */
	protected BOSubject subject;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>BOSubjectCurriculum</code> he inicializa sus atributos a los
	 * valores por defecto.</p>
	 */
	public BOSubjectCurriculum()
	{
		// Constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOSubjectCurriculum</code> he inicializa sus atributos a los 
	 * valores que se le pasan como par&aacute;metros.</p>
	 * 
	 * @param subject <code>BOSubject</code> la Asignatura.
	 * @param curriculum <code>BOCurriculum</code> el Plan de Estudios.
	 * @param code <code>String</code> c&oacute;digo de la Asignatura.
	 * @param credits <code>double</code> n&uacute;mero de cr&eacute;ditos de la Asignatura.
	 * @param type <code>String</code> tipo de Asignatura.
	 */
	public BOSubjectCurriculum(BOSubject subject, BOCurriculum curriculum, String code, double credits, String type)
	{
		// Constructor de la superclase.
		super();
		// Asignamso los valores a los atributos.
		this.subject = subject;
		this.curriculum = curriculum;
		this.code = code;
		this.credits = credits;
		this.type = type;
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Obtiene el objeto <code>TOSubjectCurriculum</code> que contiene la misma
	 * informaci&oacute;n que el objeto <code>BOSubjectCurriculum</code> sobre el cual
	 * se invoca a este m&eeacute;todo.</p>
	 * 
	 * @return <code>TOSubjectCurriculum</code> con la misma informaci&oacute;n que 
	 * el objeto sobre el que se ha llamado a este m&eeacute;todo.
	 */
	public TOSubjectCurriculum getTOSubjectCurriculum()
	{
		return null;
	}
	
	@Override
	public int compareTo(BOSubjectCurriculum o) 
	{
		// Comprobamos que tanto este objeto como el que se recibe como
		// parmetro tienen una asignatura asignada.
		if ((this.curriculum != null) && (o.getCurriculum() != null))
		{
			// Comprobamos que los nombres de los planes de estudio. 
			return this.curriculum.name.compareTo(o.getCurriculum().getName());
		}
		else if (o.getCurriculum() == null)
		{
			// En este caso quiere decir que es el objeto que se recibe 
			// como parmetro es nulo. Retornamos que este (this) es mayor.
			return 1;
		}
		else
		{
			// En este caso quiere decir que "this.curriculum" es nulo por 
			// lo que retornamos que es mayor el parmetro.
			return -1;
		}
	}




	// -------- GETTERS & SETTERS.
	public BOSubject getSubject()
	{
		return subject;
	}
	public void setSubject(BOSubject subject)
	{
		this.subject = subject;
	}
	public BOCurriculum getCurriculum()
	{
		return curriculum;
	}
	public void setCurriculum(BOCurriculum curriculum)
	{
		this.curriculum = curriculum;
	}
	public String getCode()
	{
		return code;
	}
	public void setCode(String code)
	{
		this.code = code;
	}
	public double getCredits()
	{
		return credits;
	}
	public void setCredits(double credits)
	{
		this.credits = credits;
	}
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public String getType()
	{
		return type;
	}
	public void setType(String type)
	{
		this.type = type;
	}
	
}
