package edu.uoc.pfc.business.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import edu.uoc.pfc.business.transfers.TOTest;
import edu.uoc.pfc.business.transfers.TOTestQuestion;

/**
 * <p>Clase que ser&aacute; utilizada para modelar los diferentes test (Cuestionarios) que 
 * un Profesor elabora dentro del sistema y que los Alumnos contestar&aacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOTest implements Serializable 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de la clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador del Cuestionario dentro del sistema.</p> */
	private int id;
	/** <p><code>BOUser</code> que representa al Profesor al que pertence el Cuestionario.</p> */
	private BOUser teacher;
	/** <p>T&iacute;tulo del Cuestionario.</p> */
	private String title;
	/** <p>Texto de introducci&oacute;n.</p>*/
	private String introduction;
	/** <p>Indica si se han de barajar las preguntas.</p> */
	private boolean shuffleQuestions;
	/** <p>Indica si se han de barajar las respuestas.</p> */
	private boolean shuffleAnswers;
	/** <p>Indica si se mostrar&aacute; el resultado.</p> */
	private boolean showResult;
	/** <p>Indica si se mostrar&aacute; la revisi&oacute;n.</p> */
	private boolean showReview;
	/** <p>Conjunto de Preguntas que forman el Test.</p> */
	private Set<BOTestQuestion> questions;
	/** <p>Indica si el Cuestionario est&aacute; publicado en un Aula o no.</p> */
	private boolean published;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>BOTest</code> e inicializa sus atributos a los valores por defecto.</p>	
	 */
	public BOTest()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOTest</code> e inicializa sus atributos a los valores que se le pasan
	 * como par&aacute;metros.</p>
	 * 
	 * @param teacher <code>BOUser</code> el Profesor que crea el Cuestionario.
	 * @param title <code>String</code> t&iacute;tulo del Cuestionario.
	 * @param instroduction <code>String</code> texto de introducci&oacute;n.
	 * @param shuffleQuestions <code>boolean</code> indica si hay que barajar las preguntas.
	 * @param shuffleAnswers <code>boolean</code> indica si hay que barajar las respuestas.
	 * @param showResult <code>boolean</code> indica si se mostrar&aacute; el resultado.
	 * @param showReview <code>boolean</code> indica si se mostrar&aacute; la revisi&oacute;n.
	 */
	public BOTest(BOUser teacher, String title, String instroduction, boolean shuffleQuestions,
			boolean shuffleAnswers, boolean showResult, boolean showReview)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.teacher = teacher;
		this.title = title;
		this.introduction = instroduction;
		this.shuffleQuestions = shuffleQuestions;
		this.shuffleAnswers = shuffleAnswers;
		this.showResult = showResult;
		this.showReview = showReview;
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto TOTest con la informaci&oacute;n asociada al
	 * Cuestionario representado por el objeto "this" sobre el cual se invoca.</p>
	 * 
	 * @return <code>TOTest</code> con la misma informaci&oacute;n que 
	 * el objeto "this".
	 */
	public TOTest getTOTest()
	{
		// Declaramos una variable local que contendr el nmero de asignaciones de preguntas.
		int questions = 0;
		// Comprobamos si "this.questions" es distinto de nulo.
		if (this.questions != null)
		{
			// El nmero de preguntas es el tamao (elementos) de "this.questions".
			questions = this.questions.size();
		}
		// Creamos un nuevo objeto TOTest y lo retornamos.
		return (new TOTest(this.id, this.title, questions, this.published));
	}
	
	/**
	 * <p>Retorna un objeto <code>TOTest</code> con toda la informaci&oacute;n correspondiente
	 * al objeto <code>BOTest</code> sobre el cual se invoca a este m&eacute;todo.</p>
	 * 
	 * @return <code>TOTest</code> con la misma informaci&oacute;n que el objeto sobre el que
	 * se ha invocado a este m&eacute;todo.
	 */
	public TOTest getTOTestComplete()
	{
		// Creamos una lista de objeto TOTestQuestion para almacenar las preguntas.
		List<TOTestQuestion> questions = new ArrayList<TOTestQuestion>();
		// Recorremos el conjunto de preguntas asociadas al test.
		for (BOTestQuestion test_question : this.questions)
		{
			// Aadimos la pregunta a la lista de preguntas.
			questions.add(test_question.getTOTestQuestion());
		}
		// Contruimos un objeto TOTest y lo retornamos.
		return (new TOTest(this.id, this.title, this.introduction, this.shuffleQuestions, 
				this.shuffleAnswers, this.showResult, this.showReview, questions, this.published));
	}
	
	// -------- GETTERS & SETTERS.
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public BOUser getTeacher() 
	{
		return teacher;
	}
	public void setTeacher(BOUser teacher) 
	{
		this.teacher = teacher;
	}
	public String getTitle() 
	{
		return title;
	}
	public void setTitle(String title) 
	{
		this.title = title;
	}
	public String getIntroduction() 
	{
		return introduction;
	}
	public void setIntroduction(String introduction) 
	{
		this.introduction = introduction;
	}
	public boolean isShuffleQuestions() 
	{
		return shuffleQuestions;
	}
	public void setShuffleQuestions(boolean shuffleQuestions) 
	{
		this.shuffleQuestions = shuffleQuestions;
	}
	public boolean isShuffleAnswers() 
	{
		return shuffleAnswers;
	}
	public void setShuffleAnswers(boolean shuffleAnswers) 
	{
		this.shuffleAnswers = shuffleAnswers;
	}
	public boolean isShowResult() 
	{
		return showResult;
	}
	public void setShowResult(boolean showResult) 
	{
		this.showResult = showResult;
	}
	public boolean isShowReview() 
	{
		return showReview;
	}
	public void setShowReview(boolean showReview) 
	{
		this.showReview = showReview;
	}
	public Set<BOTestQuestion> getQuestions() 
	{
		return questions;
	}
	public void setQuestions(Set<BOTestQuestion> questions) 
	{
		this.questions = questions;
	}
	public boolean isPublished() 
	{
		return published;
	}
	public void setPublished(boolean published) 
	{
		this.published = published;
	}
	
}
