package edu.uoc.pfc.business.objects;

import java.io.Serializable;
import java.util.Date;
import java.util.Set;

import edu.uoc.pfc.business.transfers.TOTest;

/**
 * <p>Clase que modela la relaci&oacute;n existente entre los Cuestionarios y las Aulas
 * definidas en el sistema, es decir, recoge la asignaci&oacute;n de los Cuestionarios 
 * a las Aulas.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOTestClassroom implements Serializable 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de la clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la asignaci&oacute;n en el sistema.</p> */
	private int id;
	/** <p>Cuestionario.</p> */
	private BOTest test;
	/** <p>Aula.</p> */
	private BOClassroom classroom;
	/** <p>Fecha de publicaci&oacute;n del Cuestionario en el Aula.</p> */
	private Date openDate;
	/** <p>Fecha de cierre del Cuestionario en el Aula.</p> */
	private Date closeDate;
	/** <p>Conjunto de Respuestas del Cuestionario recibidas en el Aula.</p> */
	private Set<BOTestAnswer> answers;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Crea un objeto <code>BOTestClassroom</code> e inicializa todos sus atributos a los valores
	 * por defecto.</p>
	 */
	public BOTestClassroom()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Crea un objeto <code>BOTestClassroom</code> e inicializa sus atributos a los valores que
	 * se le pasan como par&aacute;metro.</p>
	 * 
	 * @param test <code>BOTest</code> con la informaci&oacute;n del Cuestionario.
	 * @param classroom <code>BOClassroom</code> con la informacin del Aula.
	 * @param openDate <code>Date</code> fecha de publicaci&oacute;n del Cuestionarios en el Aula.
	 * @param closeDate <code>Date</code> fecha de cierre del Cuestionario en el Aula.
	 */
	public BOTestClassroom(BOTest test, BOClassroom classroom, Date openDate, Date closeDate)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.test = test;
		this.classroom = classroom;
		this.openDate = openDate;
		this.closeDate = closeDate;
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOTest</code> con la informaci&oacute;n equivalente al objeto
	 * "this" sobre el cual se invoca a este m&eacute;todo.</p>
	 * 
	 * @return <code>TOTest</code> con la informaci&oacute;n equivalente al objeto "this".
	 */
	public TOTest getTOTest()
	{
		// Construimos un objeto TOTest a partir de la informacin de "this".
		TOTest toTest = new TOTest(this.test.getId(), this.test.getTitle(), this.openDate, 
				this.closeDate, this.test.getQuestions().size()); 
		// Asignamos el resto de informacin a "toTest".
		toTest.setShowReview(this.test.isShowReview());
		toTest.setShowResult(this.test.isShowResult());
		toTest.setShuffleAnswers(this.test.isShuffleAnswers());
		toTest.setShuffleQuestions(this.test.isShuffleQuestions());
		// Retornamos el objeto toTest;
		return toTest;
	}
	
	// -------- GETTERS & SETTERS
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public BOTest getTest() 
	{
		return test;
	}
	public void setTest(BOTest test) 
	{
		this.test = test;
	}
	public BOClassroom getClassroom() 
	{
		return classroom;
	}
	public void setClassroom(BOClassroom classroom) 
	{
		this.classroom = classroom;
	}
	public Date getOpenDate() 
	{
		return openDate;
	}
	public void setOpenDate(Date openDate) 
	{
		this.openDate = openDate;
	}
	public Date getCloseDate() 
	{
		return closeDate;
	}
	public void setCloseDate(Date closeDate) 
	{
		this.closeDate = closeDate;
	}
	public Set<BOTestAnswer> getAnswers() 
	{
		return answers;
	}
	public void setAnswers(Set<BOTestAnswer> answers) 
	{
		this.answers = answers;
	}
	
}
