package edu.uoc.pfc.business.objects;

import java.io.Serializable;

import edu.uoc.pfc.business.transfers.TOTestQuestion;

/**
 * <p>Clase que modela la relaci&oacute;n que existe entre las Preguntas del Banco 
 * de Preguntas de un Profesor y los Cuestionarios que dicho Profesor elabora.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public class BOTestQuestion implements Serializable 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la relaci&oacute;n dentro del sistema.</p> */
	private int id;
	/** <p>El Cuestionario al que pertenece la relaci&oacute;n.</p> */
	private BOTest test;
	/** <p>La Pregunta que est&aacute; relacionada con el Cuestionario.</p> */
	private BOQuestion question;
	/** <p>El valor de la Pregunta dentro del Test.</p> */
	private double value;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>BOTestQuestion</code> e inicializa sus atributos a los valores
	 * por defecto.</p>
	 */
	public BOTestQuestion()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOTestQuestion</code> e inicializa sus atributos a los valores 
	 * que se reciben como par&aacute;metros.</p>
	 * 
	 * @param test <code>BOTest</code> el Cuestionario.
	 * @param question <code>BOQuestion</code> la Pregunta.
	 * @param value <code>double</code> el valor de la Pregunta dentro del Cuestionario.
	 */
	public BOTestQuestion(BOTest test, BOQuestion question, double value)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.test = test;
		this.question = question;
		this.value = value;
	}
	
	// --------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>TOTestQuestion</code> con la misma informaci&oacute;n que el 
	 * objeto <code>BOTestQuestion</code> sobre el cual se invoca al m&eacute;todo.</p>
	 * 
	 * @return <code>TOTestQuestion</code> con la misma informaci&oacute;n que el objeto "this"
	 * sobre el cual se ha invocado al m&eacute;todo.
	 */
	public TOTestQuestion getTOTestQuestion()
	{
		// Creamos un objeto TOTestQuestion a partir de la informacin de "this" y lo retornamos.
		return (new TOTestQuestion(this.id, this.value, this.question.getTOQuestion()));
	}
	
	// -------- GETTERS & SETTERS.
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public BOTest getTest() 
	{
		return test;
	}
	public void setTest(BOTest test) 
	{
		this.test = test;
	}
	public BOQuestion getQuestion()
	{
		return question;
	}
	public void setQuestion(BOQuestion question) 
	{
		this.question = question;
	}
	public double getValue() 
	{
		return value;
	}
	public void setValue(double value) 
	{
		this.value = value;
	}

}
