package edu.uoc.pfc.business.objects;

import java.io.Serializable;
import java.util.Date;

import org.apache.commons.beanutils.BeanUtils;

import edu.uoc.pfc.business.transfers.TOUser;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que modela los usuarios de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class BOUser implements Serializable 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador del usuario. </p> */
	protected int id;
	/** <p>Nombre de usuario (login).</p> */
	protected String login;
	/** <p>Contrase&ntilde;a de aacceso (password).</p> */
	protected String password;
	/** <p>Indica si la cuenta del usuario est&aacute; activa o no.</p> */
	protected boolean active;
	/** <p>Fecha de registro de la cuenta en el sistema.</p> */
	protected Date created;
	/** <p>Fecha y Hora en la que se ha accedido por &uacute;tima vez a la cuenta.</p> */
	protected Date lastLogin;
	/** <p>Indica si el usuario es un <i>administrador</i>.</p> */
	protected boolean admin;
	/** <p>Indica si el usuario es un <i>profesor</i>.</p> */
	protected boolean teacher;
	/** <p>Indica si el usuario es un <i>alumno</i>.</p> */
	protected boolean student;
	/** <p>Nombre real del usuario.</p> */
	protected String name;
	/** <p>Apellidos del usuario.</p> */
	protected String surname;	
	/** <p>Direcci&oacute;n de correo electr&oacugte;nico.</p> */
	protected String email;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>BOUser</code> e inicializa todos los atributos
	 * a sus valores por defecto.</p>
	 */
	public BOUser()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>BOUser</code> a partir de los valores almacenados
	 * en los atributos del objeto <code>TOUser</code> que recibe como par&aacute;metro.</p>
	 * @param to <code>TOUser</code> con los datos a asignar a los atributos.
	 */
	public BOUser(TOUser to)
	{
		// Constructor de la superclase.
		super();
		// Comprobamos que el parmetro que se ha recibido no es nulo.
		if (to != null)
		{
			try 
			{
				// Pasamos los valores de los atributos de uno a uno.
				BeanUtils.copyProperties(this, to);
			} 
			catch (Exception ex) 
			{ 
				// Capturamos la excepcin pero no hacemos nada porque
				// estamos seguros de que todas las propiedades de TOUser
				// y BOUser son las mismas y del mismo tipo.
			}
		}
	}
	
	/**
	 * <p>Construye un objeto <code>BOUser</code> a partir de la informaci&oacute;n que se le
	 * pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del usuario.
	 */
	public BOUser(int id)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los atributos.
		this.id = id;
	}
	
	
	// -------- MTODOS PBLICOS
	/**
	 * <p>Vuelca la infomaci&oacute;n del objeto <code>BOUser</code> sobre el cual
	 * se llama al m&eacute;todo a un objeto <code>TOUser</code>.</p>
	 * 
	 * @return <code>TOUser</code> con la informaci&oacute;n del objeto <code>BOUser</code>.
	 */
	public TOUser getTOUser()
	{
		try
		{
			// El objeto que retornaremos.
			TOUser user = new TOUser();
			// Pasamos los valores de los atributos de uno a otro.
			BeanUtils.copyProperties(user, this);
			// Transformamos el nombre y los apellidos.
			user.setName(WebUtils.HTMLtoString(this.name));
			user.setSurname(WebUtils.HTMLtoString(this.surname));
			// Retornamos el objeto.
			return user;
		}
		catch (Exception ex)
		{
			// Capturamos la excepcin y retornamos null indicando que
			// no se han podido capturar los datos.
			return null;
		}
	}
	
	@Override
	public boolean equals(Object obj)
	{
		// Vemos si el parmetro es null o no es de tipo BOUser
		if ((obj == null) || (!(obj instanceof BOUser)))
		{
			// En este caso consideramos que no son iguales.
			return false;
		}
		// Cast del parmetros.
		BOUser objParam = (BOUser) obj;
		// Retornamos el resultado de comparar los identificadores de ambos objetos.
		return (this.id == objParam.getId());
	}

	// -------- GETTERS & SETTERS
	public String getLogin()
	{
		return login;
	}
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public void setLogin(String login)
	{
		this.login = login;
	}
	public String getPassword()
	{
		return password;
	}
	public void setPassword(String password)
	{
		this.password = password;
	}
	public Date getLastLogin()
	{
		return lastLogin;
	}
	public void setLastLogin(Date lastLogin)
	{
		this.lastLogin = lastLogin;
	}
	public boolean isActive()
	{
		return active;
	}
	public void setActive(boolean active)
	{
		this.active = active;
	}
	public boolean isAdmin()
	{
		return admin;
	}
	public void setAdmin(boolean admin)
	{
		this.admin = admin;
	}
	public boolean isTeacher()
	{
		return teacher;
	}
	public void setTeacher(boolean teacher)
	{
		this.teacher = teacher;
	}
	public boolean isStudent()
	{
		return student;
	}
	public void setStudent(boolean student)
	{
		this.student = student;
	}
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	public String getSurname()
	{
		return surname;
	}
	public void setSurname(String surname)
	{
		this.surname = surname;
	}
	public String getEmail()
	{
		return email;
	}
	public void setEmail(String email)
	{
		this.email = email;
	}
	public Date getCreated()
	{
		return created;
	}
	public void setCreated(Date created)
	{
		this.created = created;
	}
	
}
