package edu.uoc.pfc.business.transfers;

import java.io.Serializable;
import java.util.List;

public class TOClassroom implements Serializable, Comparable<TOClassroom> 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// --------- ATRIBUTOS.
	/** <p>Identificador del Aula.</p> */
	private int id;
	/** <p>Identificador del Plan de Estudios.</p> */
	private int curriculumId;
	/** <p>Nombre del Plan de Estudios.</p> */
	private String curriculumName;
	/** <p>Identificador de la Asignatura.</p> */
	private int subjectId;
	/** <p>Nombre de la Asignatura.</p> */
	private String subjectName;
	/** <p>Letra asignada al Aula.</p>. */
	private String letter;
	/** <p>A&ntilde;o del Aula.</p> */
	private int course;
	
	/** <p>Profesor del Aula.</p> */
	private TOUser teacher;
	/** <p>Alumnos del Aula.</p> */
	private List<TOUser> students;
	/** <p>Cuestionarios asignados al Aula.</p> */
	private List<TOTest> test;
	
	// -------- CONSTRUCTORES
	/**
	 * <p>Construye un objeto <code>TOClassroom</code> a partir de la informaci&oacute;n
	 * que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Aula.
	 * @param curriculumId <code>int</code> identificador del Plan de Estudios del Aula.
	 * @param curriculumName <code>String</code> nombre del Plan de Estudios.
	 * @param subjectId <code>int</code> identificador de la Asignatura del Aula.
	 * @param subjectName <code>String</code> nombre de la Asignatura.
	 * @param letter <code>String</code> letra del Aula.
	 * @param year <code>course</code> a&nitlde;o del Aula.
	 */
	public TOClassroom(int id, int curriculumId, String curriculumName,
			int subjectId, String subjectName, String letter, int course) 
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.id = id;
		this.curriculumId = curriculumId;
		this.curriculumName = curriculumName;
		this.subjectId = subjectId;
		this.subjectName = subjectName;
		this.letter = letter;
		this.course = course;
	}
	
	/**
	 * <p>Construye un objeto <code>TOClassroom</code> a partir de la informaci&oacute;n
	 * que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Aula.
	 * @param curriculumId <code>int</code> identificador del Plan de Estudios del Aula.
	 * @param curriculumName <code>String</code> nombre del Plan de Estudios.
	 * @param subjectId <code>int</code> identificador de la Asignatura del Aula.
	 * @param subjectName <code>String</code> nombre de la Asignatura.
	 * @param letter <code>String</code> letra del Aula.
	 * @param year <code>course</code> a&nitlde;o del Aula.
	 * @param teacher <code>TOTeacher</code> profesor asignado al Aula.
	 * @param students <code>List&lt;TOUser&gt;</code> alumnos asignados al Aula.
	 */
	public TOClassroom(int id, int curriculumId, String curriculumName,
			int subjectId, String subjectName, String letter, int course,
			TOUser teacher, List<TOUser> students) {
		this(id, curriculumId, curriculumName, subjectId, subjectName, letter, course);
		this.teacher = teacher;
		this.students = students;
	}

	// -------- MTODOS PBLICOS.
	@Override
	public int compareTo(TOClassroom o) 
	{
		// Comprobamos si el parmetro es nulo.
		if (o == null)
		{
			// Como el parmetro es null retornamos que "this" es mayor.
			return 1;
		}
		// Vemos si los nombres de los Planes de Estudio son los mismos.
		if (this.curriculumName.equals(o.getCurriculumName()))
		{
			// Comprobamos si los nombres de las asignaturas son los mismos.
			if (this.subjectName.equals(o.getSubjectName()))
			{
				// Vemos si el ao es el mismo.
				if (this.course == o.getCourse())
				{
					// El ao es el mismo por lo que simplemente retornamos
					// el resultado de comparar las letras de las aulas.
					return (this.letter.compareTo(o.getLetter()));
				}
				else
				{
					// El ao no es el mismo as que retornamos el resultado
					// de realizar su resta para ver cul es mayor.
					return (this.course - o.getCourse());
				}
			}
			else
			{
				// Los nombres de las Asignaturas no son los mismos por lo que
				// retornamos el resultado de su comparacin.
				return this.subjectName.compareTo(o.getSubjectName());
			}
		}
		else
		{
			// Los nombres de los Planes de Estudio son diferentes
			// as que retornamos el resultado de su comparacin.
			return this.curriculumName.compareTo(o.getCurriculumName());
		}
	}
		
	// -------- GETTERS & SETTERS
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public int getCurriculumId() 
	{
		return curriculumId;
	}
	public void setCurriculumId(int curriculumId) 
	{
		this.curriculumId = curriculumId;
	}
	public String getCurriculumName() 
	{
		return curriculumName;
	}
	public void setCurriculumName(String curriculumName) 
	{
		this.curriculumName = curriculumName;
	}
	public int getSubjectId() 
	{
		return subjectId;
	}
	public void setSubjectId(int subjectId) 
	{
		this.subjectId = subjectId;
	}
	public String getSubjectName() 
	{
		return subjectName;
	}
	public void setSubjectName(String subjectName) 
	{
		this.subjectName = subjectName;
	}
	public String getLetter()
	{
		return letter;
	}
	public void setLetter(String letter)
	{
		this.letter = letter;
	}
	public int getCourse() 
	{
		return course;
	}
	public void setCourse(int course) 
	{
		this.course = course;
	}
	public TOUser getTeacher() 
	{
		return teacher;
	}
	public void setTeacher(TOUser teacher) 
	{
		this.teacher = teacher;
	}
	public List<TOUser> getStudents() 
	{
		return students;
	}
	public void setStudents(List<TOUser> students) 
	{
		this.students = students;
	}
	public List<TOTest> getTest() 
	{
		return test;
	}
	public void setTest(List<TOTest> test) 
	{
		this.test = test;
	}
	
}
