package edu.uoc.pfc.business.transfers;

import java.io.Serializable;

/**
 * <p>Clase que ser&aacute; utilizada para realizar el intercambio de la informaci&oacute;n de configuraci&oacute;n
 * de la aplicaci&oacute;n entre las distintas capas de la arquitectura.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOConfig implements Serializable
{
	
	// -------- CONSTANTES
	/** <p>N&uacute;mero de serie por defecto para los objetos de la clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la configuraci&oacute;n.</p> */
	private int id;
	/** <p>N&uacute;mero de intentos de acceso a una cuenta de usuario.</p> */
	private int loginAttempts;
	/** <p>Indica si se trata de la configuraci&oacute;n activa (en uso).</p> */
	private boolean active;
	/** <p>Recoge el curso que est&aacute; activo dentro de la aplicaci&oacute;n.</p> */
	private int course;
	/** <p>Indica si est&aacute; permitido o no el env&iacute;o de correos electr&oacute;nicos.</p> */
	private boolean sendEmail;
	/** <p>Servidor SMTP para el env&iacute;o de correos electr&oacute;nicos.</p> */
	private String smtpServer;
	/** <p>Puerto del servidor SMTP.</p> */
	private int smtpPort;
	/** <p>Indica si el servidor SMTP precisa de autentificaci&oacute;n o no.</p> */
	private boolean smtpAuthentication;
	/** <p>Indica si el servidor SMTP utiliza TLS (Transport Layer Security).</p> */
	private boolean smtpTLS;
	/** <p>Nombre de usuario dentro del servidor SMTP.</p> */
	private String smtpUser;
	/** <p>Contrasea del usuario dentro del servidor SMTP.</p> */
	private String smtpPassword;
	/** <p>Direcci&oacute;n desde la que se env&iacute;n las peticiones de recordar <i>login</i>.</p> */
	private String forgotLoginFrom;
	/** <p>Asunto de las peticiones de recordar <i>login</i>.</p> */
	private String forgotLoginSubject;
	/** <p>Texto de los correos asociados a las peticiones de recordar <i>login</i>.</p> */
	private String forgotLoginText;
	/** <p>Direcci&oacute;n desde la que se env&iacute;n las peticiones de recordar <i>contrase&ntilde;a</i>.</p> */
	private String forgotPasswordFrom;
	/** <p>Asunto de las peticiones de recordar <i>contrase&ntilde;a</i>.</p> */
	private String forgotPasswordSubject;
	/** <p>Texto de los correos asociados a las peticiones de recordar <i>contrase&ntilde;a</i>.</p> */
	private String forgotPasswordText;
	/** <p>Direcci&oacute;n de correo electr&oacute;nico de los administradores de la aplicaci&oacute;n.</p> */
	private String adminEmail;
	
	// -------- CONSTRUCTORES
	/**
	 * <p>Construye un objeto <code>TOConfig</code> e inicializa sus atributos a los
	 * valores por defecto.</p>
	 */
	public TOConfig()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>TOConfig</code> e inicializa sus atributos a los valores
	 * que se pasan como par&aacute;metro.</p> 
	 * 	
	 * @param id <code>int</code> identificador de la configuraci&oacute;n.
	 * @param loginAttempts <code>int</code> intentos de acceso a una cuenta de Usuario.
	 * @param active <code>boolean</code> indica si la configuraci&oacute;n est&aacute; activa o no.
	 * @param course <code>int</code> curso activo.
	 * @param sendEmail <code>boolean</code> indica si se enviar&aacute;n notificaciones v&iacute;a
	 * correo electr&oacute;nico.
	 * @param smtpServer <code>String</code> servidor de correo SMTP.
	 * @param smtpPort <code>int</code> puerto SMTP.
	 * @param smtpAuthentication <code>boolean</code> indica si el servidor SMTP requiere 
	 * autentificaci&oacute;n o no.
	 * @param smtpTLS <code>boolean</code> indica si el servidor SMTP admite el protocolo TLS.
	 * @param smtpUser <code>String</code> usuario SMTP.
	 * @param smtpPassword <code>String</code> contrase&ntilde;a SMTP.
	 * @param forgotLoginFrom <code>String</code> campo "de" en el correo "recordar login".
	 * @param forgotLoginSubject <code>String</code> campo "asunto" en el corro "recordar login".
	 * @param forgotLoginText <code>String</code> texto del correo "recordar login".
	 * @param forgotPasswordFrom <code>String</code> campo "de" en el correo "recordar contrase&ntilde;a".
	 * @param forgotPasswordSubject <code>String</code> campo "asunto" en el corro "recordar contrase&ntilde;a".
	 * @param forgotPasswordText <code>String</code> campo "de" en el correo "recordar contrase&ntilde;a".
	 * @param adminEmail <code>String</code> email de los administradores de al aplicaci&oacute;n.
	 */
	public TOConfig(int id, int loginAttempts, boolean active, int course,
			boolean sendEmail, String smtpServer, int smtpPort,
			boolean smtpAuthentication, boolean smtpTLS, String smtpUser,
			String smtpPassword, String forgotLoginFrom,
			String forgotLoginSubject, String forgotLoginText,
			String forgotPasswordFrom, String forgotPasswordSubject,
			String forgotPasswordText, String adminEmail) 
	{
		// Constructor de la superclase.
		super();
		// Asignamos los valores a los atributos.
		this.id = id;
		this.loginAttempts = loginAttempts;
		this.active = active;
		this.course = course;
		this.sendEmail = sendEmail;
		this.smtpServer = smtpServer;
		this.smtpPort = smtpPort;
		this.smtpAuthentication = smtpAuthentication;
		this.smtpTLS = smtpTLS;
		this.smtpUser = smtpUser;
		this.smtpPassword = smtpPassword;
		this.forgotLoginFrom = forgotLoginFrom;
		this.forgotLoginSubject = forgotLoginSubject;
		this.forgotLoginText = forgotLoginText;
		this.forgotPasswordFrom = forgotPasswordFrom;
		this.forgotPasswordSubject = forgotPasswordSubject;
		this.forgotPasswordText = forgotPasswordText;
		this.adminEmail = adminEmail;
	}



	// -------- GETTERS & SETTERS
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public int getLoginAttempts()
	{
		return loginAttempts;
	}
	public void setLoginAttempts(int loginAttempts)
	{
		this.loginAttempts = loginAttempts;
	}
	public boolean isActive()
	{
		return active;
	}
	public void setActive(boolean active)
	{
		this.active = active;
	}
	public int getCourse()
	{
		return course;
	}
	public void setCourse(int course)
	{
		this.course = course;
	}
	public boolean isSendEmail()
	{
		return sendEmail;
	}
	public void setSendEmail(boolean sendEmail)
	{
		this.sendEmail = sendEmail;
	}
	public String getSmtpServer()
	{
		return smtpServer;
	}
	public void setSmtpServer(String smtpServer)
	{
		this.smtpServer = smtpServer;
	}
	public int getSmtpPort()
	{
		return smtpPort;
	}
	public void setSmtpPort(int smtpPort)
	{
		this.smtpPort = smtpPort;
	}
	public boolean isSmtpAuthentication()
	{
		return smtpAuthentication;
	}
	public void setSmtpAuthentication(boolean smtpAuthentication)
	{
		this.smtpAuthentication = smtpAuthentication;
	}
	public boolean isSmtpTLS()
	{
		return smtpTLS;
	}
	public void setSmtpTLS(boolean smtpTLS)
	{
		this.smtpTLS = smtpTLS;
	}
	public String getSmtpUser()
	{
		return smtpUser;
	}
	public void setSmtpUser(String smtpUser)
	{
		this.smtpUser = smtpUser;
	}
	public String getSmtpPassword()
	{
		return smtpPassword;
	}
	public void setSmtpPassword(String smtpPassword)
	{
		this.smtpPassword = smtpPassword;
	}
	public String getForgotLoginFrom()
	{
		return forgotLoginFrom;
	}
	public void setForgotLoginFrom(String forgotLoginFrom)
	{
		this.forgotLoginFrom = forgotLoginFrom;
	}
	public String getForgotLoginSubject()
	{
		return forgotLoginSubject;
	}
	public void setForgotLoginSubject(String forgotLoginSubject)
	{
		this.forgotLoginSubject = forgotLoginSubject;
	}
	public String getForgotLoginText()
	{
		return forgotLoginText;
	}
	public void setForgotLoginText(String forgotLoginText)
	{
		this.forgotLoginText = forgotLoginText;
	}
	public String getForgotPasswordFrom()
	{
		return forgotPasswordFrom;
	}
	public void setForgotPasswordFrom(String forgotPasswordFrom)
	{
		this.forgotPasswordFrom = forgotPasswordFrom;
	}
	public String getForgotPasswordSubject()
	{
		return forgotPasswordSubject;
	}
	public void setForgotPasswordSubject(String forgotPasswordSubject)
	{
		this.forgotPasswordSubject = forgotPasswordSubject;
	}
	public String getForgotPasswordText()
	{
		return forgotPasswordText;
	}
	public void setForgotPasswordText(String forgotPasswordText)
	{
		this.forgotPasswordText = forgotPasswordText;
	}
	public String getAdminEmail()
	{
		return adminEmail;
	}
	public void setAdminEmail(String adminEmail)
	{
		this.adminEmail = adminEmail;
	}
		
}
