package edu.uoc.pfc.business.transfers;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que se encargar de recoger la informacin de los diferentes planes
 * de estudio con los que se trabaja en la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOCurriculum implements Serializable, Comparable<TOCurriculum>
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador del plan de estudios.</p> */
	private int id;
	/** <p>Nombre del plan de estudios.</p> */
	private String name;
	/** <p>Tipo de Plan de Estudios.</p> */
	private String type;
	/** <p>Descripci&oacute;n del plan de estudios.</p> */
	private String description;
	/** <p>Fecha de creaci&oacute;n del plan de estudios.</p> */
	private Date created;
	/** <p>Asignaturas que contiene el plan de estudios.</p> */
	private List<TOCurriculumSubject> subjects;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Crea un objeto <code>TOCurriculum</code> a partir de la informaci&oacute;n que se
	 * le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador en el sistema del Plan de Estudios.
	 * @param name <code>String</code> nombre del Plan de Estudios.
	 * @param type <code>String</code> tipo del Plan de Estudios.
	 * @param created <code>Date</code> fecha de creaci&oacute;n.
	 * @param description <code>String</code> descripci&oacute;n del Plan de Estudios.
	 */
	public TOCurriculum(int id, String name, String type, Date created, String description)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores a los atributos.
		this.id = id;
		this.name = name;
		this.type = type;
		this.created = created;
		this.description = description;
	}
	
	
	// --------- MTODOS PBLICOS.
	@Override
	public int compareTo(TOCurriculum o)
	{
		// En primer lugar comprobamos que el parmetro no sea nulo.
		if (o == null)
		{
			// Si el parmetro es nulo entonces consideraremos que el objeto sobre
			// el cual se invoca este mtodo (this) es mayor. Retornamos 1.
			return 1;
		}
		// Si se llega hasta aqu querr decir que el parmetro no es nulo. Simplemente
		// retornaremos el resultado de comprobar (compareTo()) de los nombres de los
		// planes de estudio de cada uno de los objetos.
		return (this.name.toLowerCase().compareTo(o.getName().toLowerCase()));
	}
	
	/**
	 * <p>Retorna el valor del atributo <i>description</i> formateado de tal manera que
	 * pueda ser mostrado correctamente en un navegador web.</p>
	 * 
	 * @return <code>String</code> con el valor del atributo <i>description</i> formateado.
	 */
	public String getDescriptionHTML()
	{
		// Simplemente retornamos el texto formateado.
		return WebUtils.stringToHMTLParagraph(this.description);
	}
	
	/**
	 * <p>Retorna el valor del atributo <i>description</i> formateado de tal manera que
	 * pueda ser mostrado sin problemas dentro de una textarea de un formulario.</p>
	 * 
	 * @return <code>String</code> con el valor del atributo <i>description</i> formateado.
	 */
	public String getDescriptionForm()
	{
		// Simplemente retornamos el texto formateado.
		return WebUtils.HTMLtoString(this.description);
	}
	
	// -------- GETTERS & SETTERS.
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public String getName() 
	{
		return name;
	}
	public void setName(String name) 
	{
		this.name = name;
	}
	public String getDescription() 
	{
		return description;
	}
	public void setDescription(String description) 
	{
		this.description = description;
	}
	public Date getCreated()
	{
		return created;
	}
	public void setCreated(Date created)
	{
		this.created = created;
	}
	public List<TOCurriculumSubject> getSubjects() 
	{
		return subjects;
	}
	public void setSubjects(List<TOCurriculumSubject> subjects) 
	{
		this.subjects = subjects;
	}
	public String getType() 
	{
		return type;
	}
	public void setType(String type) 
	{
		this.type = type;
	}
	
}
