package edu.uoc.pfc.business.transfers;

import java.io.Serializable;

/**
 * <p>Clase que sirve para recoger la relaci&oacute;n que existe entre un Plan de Estudios
 * y las Asignaturas que dentro de &eacute;l se imparten.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOCurriculumSubject implements Serializable, Comparable<TOCurriculumSubject>
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	/** <p>Identificador de la Asignatura dentro del sistema.</p> */
	private int id;
	/** <p>Nombre de la Asignatura.</p> */
	private String name;
	/** <p>C&oacute;digo de la Asignatura dentro del Plan de Estudios.</p> */
	private String code;
	/** <p>N&uacute;mero de cr&eacute;ditos de la Asignatura en el Plan de Estudios.</p> */
	private double credits;
	/** <p>Tipo de Asignatura dentro del Plan de Estudios.</p> */
	private String type;
	
	/**
	 * <p>Construye un objecto <code>TOCurriculumSubject</code> a partir de la informaci&oacute;n
	 * que se recibe como par&aacute;metro.</p>	
	 * 
	 * @param id <code>int</code> identificador de la Asignatura.
	 * @param name <code>String</code> nombre de la Asignatura.
	 * @param code <code>String</code> c&oacute;digo de la Asignatura.
	 * @param credits <code>double</code> n&uacute;mero de cr&eacute;ditos de la asignatura.
	 * @param type <code>String</code> tipo de Asignaturas.
	 */
	public TOCurriculumSubject(int id, String name, String code,
			double credits, String type)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.id = id;
		this.name = name;
		this.code = code;
		this.credits = credits;
		this.type = type;
	}

	// -------- MTODOS PBLICOS:
	@Override
	public int compareTo(TOCurriculumSubject o)
	{
		// Comprobamso que el objeto "o" no sea nulo.
		if (o == null)
		{
			// Si "o" es nulo entonces vamos a suponer que siempre el objeto "this" ser mayor.
			return 1;
		}
		// Si llegamos a este punto quiere decir que "o" no es null por lo que simplemente 
		// comparamos los nombres (atributo "name").
		return this.name.toLowerCase().compareTo(o.getName().toLowerCase());
	}
	
	// -------- GETTERS & SETTERS
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	public String getCode()
	{
		return code;
	}
	public void setCode(String code)
	{
		this.code = code;
	}
	public double getCredits()
	{
		return credits;
	}
	public void setCredits(double credits)
	{
		this.credits = credits;
	}
	public String getType()
	{
		return type;
	}
	public void setType(String type)
	{
		this.type = type;
	}
	
}
