package edu.uoc.pfc.business.transfers;

import java.io.Serializable;

import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que modela a las Preguntas registradas en el sistema.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOQuestion implements Serializable, Comparable<TOQuestion>
{

	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	/** <p>Tipo de Pregunta: Verdadero o Falso.</p> */
	public static final String TYPE_TRUE_OR_FALSE = "TF";
	/** <p>Tipo de Pregunta: Respuesta M&uacute;ltiple.</p> */
	public static final String TYPE_MULTIPLE_CHOICES = "MC";
	/** <p>Tipo de Pregunta: Respuesta corta.</p> */
	public static final String TYPE_SHORT_ANSWER = "SA";
	/** <p>Tipo de Pregunta: Num&eacute;rica.</p> */
	public static final String TYPE_NUMERIC = "NU";
	/** <p>Tipo de Pregunta: Emparejar.</p> */
	public static final String TYPE_MATCH = "MA";
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la Pregunta dentro del sistema.</p> */
	protected int id;
	/** <p>T&iacute;tulo de la Pregunta.</p> */
	protected String title;
	/** <p>Enunciado de la Pregunta.</p> */
	protected String statement;
	/** <p>Tipo de Pregunta.</p> */
	protected String type;
	/** <p>Identificador de la categor&iacute;a de Pregunta a la que pertenece.</p> */
	protected int categoryId;
	
	/** <p>Indicar&aacute; si el Estudiante ha constestado correctamente a la pregunta.</p> */
	protected boolean correct;
	/** <p>Valor de la pregunta dentro de un Cuestionario.</p> */
	protected double value;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOQuestion</code> e inicializa sus atributos a los
	 * valores por defecto.</p>	
	 */
	public TOQuestion()
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	/**
	 * <p>Construye un objeto <code>TOQuestion</code> a partir de la informaci&oacute;n que
	 * se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la Pregunta.
	 * @param title <code>String</code> t&iacute;tulo de la Pregunta.
	 * @param statement <code>String</code> enunciado de la Pregunta.
	 * @param type <code>String</code> tipo de Pregunta.
	 */
	public TOQuestion(int id, String title, String statement, String type)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.id = id;
		this.title = title;
		this.statement = statement;
		this.type = type;
	}
	
	// -------- MTODOS PBLICOS.	
	@Override
	public int compareTo(TOQuestion o)
	{
		// Comprobamos que el objeto que se recibe como parmetro no sea nulo.
		if (o == null)
		{
			// Al ser nulo suponemos que "this" siempre es mayor.
			return 1;
		}
		// En este caso retornamos el resultado de comparar los dos ttulos
		// de las preguntas.
		return this.title.toLowerCase().compareTo(o.getTitle().toLowerCase());
	}
	
	/**
	 * <p>Retorna el valor del atributo <i>statement</i> (enunciado de la pregunta)
	 * formateado correctamente para ser mostrado en la web.</p>
	 * 
	 * @return <code>String</code> con el valor del enunciado formateado.
	 */
	public String getStatementHTML()
	{
		// Retornamos el enunciado formateado.
		return (WebUtils.stringToHMTLParagraph(this.statement));
	}
	
	// -------- GETTERS & SETTERS.
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public String getTitle()
	{
		return title;
	}
	public void setTitle(String title)
	{
		this.title = title;
	}
	public String getStatement()
	{
		return statement;
	}
	public void setStatement(String statement)
	{
		this.statement = statement;
	}
	public String getType()
	{
		return type;
	}
	public void setType(String type)
	{
		this.type = type;
	}
	public int getCategoryId() 
	{
		return categoryId;
	}
	public void setCategoryId(int categoryId) 
	{
		this.categoryId = categoryId;
	}
	public boolean isCorrect()
	{
		return correct;
	}
	public void setCorrect(boolean correct)
	{
		this.correct = correct;
	}
	public double getValue()
	{
		return value;
	}
	public void setValue(double value)
	{
		this.value = value;
	}
	
}
