package edu.uoc.pfc.business.transfers;

import java.io.Serializable;
import java.util.List;

import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que ser&aacute; utilizada para realizar el intercambio de informaci&oacute;n 
 * entre capas relacionada con las Categor&iacute;as de Preguntas dentro del Banco de 
 * Preguntas de un determinado Usuario.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOQuestionCategory implements Serializable, Comparable<TOQuestionCategory>
{

	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	/** <p>Identificador de la Categor&iacute;a de Preguntas dentro del sistema.</p> */
	private int id;
	/** <p>Nombre de la Categor&iacute;a de Preguntas.</p> */
	private String name;
	/** <p>Descripci&oacute;n de la Categor&iacute;a de Preguntas.</p> */
	private String description;
	/** <p>N&uacute;mero total de Preguntas que posee la categor&iacute;a.</p> */
	private int totalQuestions;
	/** <p>Lista con todas las Preguntas que pertenecen a la Categor&iacute;a.</p> */
	private List<TOQuestion> questions;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOQuestionCategory</code> a partir de la informaci&oacute;n
	 * que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la Categor&iacute;a de Preguntas.
	 * @param name <code>String</code> nombre de la Categor&iacute;a de Preguntas.
	 * @param totalQuestions <code>int</code> n&uacute;mero de preguntas dentro de la Categor&iacute;a.
	 */
	public TOQuestionCategory(int id, String name, int totalQuestions)
	{
		// Constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.id = id;
		this.name = name;
		this.totalQuestions = totalQuestions;
	}
	
	/**
	 * <p>Construye un objeto <code>TOQuestionCategory a partir de la informaci&oacute;n que se 
	 * le pasa como par&aacute;metro.</p>
	 * 
	 *  @param id <code>int</code> identificador de la Categor&iacute;a de Preguntas.
	 * @param  name <code>String</code> nombre de la Categor&iacute;a de Preguntas.
	 * @param description <code>String</code> descripci&oacute;n de la Categor&iacute;a de Preguntas.
	 * @param questions <code>List&lt;TOQuestion&gt; con todas las Preguntas que pertenecen a la
	 * Categor&iacute;a.
	 */
	public TOQuestionCategory(int id, String name, String description, List<TOQuestion> questions)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los atributos.
		this.id = id;
		this.name = name;
		this.description = description;
		this.questions = questions;
		// Comprobamos si el parmetro "questions" es nulo o no.
		if (questions == null) 
		{
			// Al ser nulo el nmero de preguntas de la categora ser 0.
			this.totalQuestions = 0;
		}
		else 
		{
			// Al no ser nulo el nmero de preguntas de la Categora ser igual
			// al nmero de elementos dentro de questions.
			this.totalQuestions = questions.size();
		}
	}
	
	// -------- MTODOS PBLICOS.
	@Override
	public int compareTo(TOQuestionCategory o)
	{
		// Comprobamos que el parmetor no sea nulo.
		if (o != null)
		{
			// Retornamos el resultado de comparar los nombres de las categoras.
			return this.name.toLowerCase().compareTo(o.getName().toLowerCase());
		}
		// Si llegamos aqu quiere decir que el parmetro es nulo por lo que vamos a suponer
		// que el objeto "this" es siempre mayor.
		return 1;
	}
	
	/**
	 * <p>Retorna la descripci&oacute;n de la Categor&iacute;a de Preguntas de tal manera que
	 * pueda ser mostrada en una pgina HTML (formateada correctamente).</p>
	 * 
	 * @return
	 */
	public String getDescriptionHTML()
	{
		// Retornarmos la descripcin formateada.
		return WebUtils.stringToHMTLParagraph(WebUtils.HTMLtoString(this.description));
	}
	
	// -------- GETTERS & SETTERS.
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	public int getTotalQuestions()
	{
		return totalQuestions;
	}
	public void setTotalQuestions(int totalQuestions)
	{
		this.totalQuestions = totalQuestions;
	}
	public List<TOQuestion> getQuestions()
	{
		return questions;
	}
	public void setQuestions(List<TOQuestion> questions)
	{
		this.questions = questions;
	}

}
