package edu.uoc.pfc.business.transfers;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import edu.uoc.pfc.lib.utils.ValidationUtils;
import edu.uoc.pfc.struts.forms.AFQuestionBank;

/**
 * <p>Clase que ser&aacute; utilizada para realizar el intercambio de informaci&oacute;n 
 * relacionada con las preguntas de tipo "Emparejar" entre las diferentes capas.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOQuestionMatch extends TOQuestion implements Serializable 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>N&uacute;mero de opciones.</p> */
	private int options;
	/** <p>Lista de opciones de respuesta.</p> */
	private List<TOQuestionMatchAnswer> answers;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOQuestionMatch</code> a partir de la informaci&oacute;n
	 * contenida en el formulario <code>AFQuestionBank</code> que recibe como par&aacute;metro.</p>
	 * 
	 * @param af <code>AFQuestionBank</code> con la informaci&oacute;n para construir el objeto.
	 */
	public TOQuestionMatch(AFQuestionBank af)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Comprobamos que el atributo no es nulo.
		if (af != null)
		{
			// Establecemos los valores de los atributos.
			this.title = af.getTitle();
			this.statement = af.getStatement();
			this.type = TOQuestion.TYPE_MATCH;
			// Creamos una lista de objetos TOQuestionMatchAnswer
			this.answers = new ArrayList<TOQuestionMatchAnswer>();
			// Recorremos la lista de opciones (parejas). Pareja 1.
			if ((!(ValidationUtils.isEmpty(af.getMaTextLeft1()))) && 
					(!(ValidationUtils.isEmpty(af.getMaTextRight1()))))
			{
				// Creamos un objeto TOQuestionMatchAnswer y lo aadimos a la lista.
				this.answers.add(new TOQuestionMatchAnswer(af.getMaTextLeft1(), 
						af.getMaTextRight1(), this.answers.size(), af.getMaExplain1()));
			}
			// Pareja 2.
			if ((!(ValidationUtils.isEmpty(af.getMaTextLeft2()))) && 
					(!(ValidationUtils.isEmpty(af.getMaTextRight2()))))
			{
				// Creamos un objeto TOQuestionMatchAnswer y lo aadimos a la lista.
				this.answers.add(new TOQuestionMatchAnswer(af.getMaTextLeft2(), 
						af.getMaTextRight2(), this.answers.size(), af.getMaExplain2()));
			}
			// Pareja 3.
			if ((!(ValidationUtils.isEmpty(af.getMaTextLeft3()))) && 
					(!(ValidationUtils.isEmpty(af.getMaTextRight3()))))
			{
				// Creamos un objeto TOQuestionMatchAnswer y lo aadimos a la lista.
				this.answers.add(new TOQuestionMatchAnswer(af.getMaTextLeft3(), 
						af.getMaTextRight3(), this.answers.size(), af.getMaExplain3()));
			}
			// Pareja 4.
			if ((!(ValidationUtils.isEmpty(af.getMaTextLeft4()))) && 
					(!(ValidationUtils.isEmpty(af.getMaTextRight4()))))
			{
				// Creamos un objeto TOQuestionMatchAnswer y lo aadimos a la lista.
				this.answers.add(new TOQuestionMatchAnswer(af.getMaTextLeft4(), 
						af.getMaTextRight4(), this.answers.size(), af.getMaExplain4()));
			}
			// Pareja 5.
			if ((!(ValidationUtils.isEmpty(af.getMaTextLeft5()))) && 
					(!(ValidationUtils.isEmpty(af.getMaTextRight5()))))
			{
				// Creamos un objeto TOQuestionMatchAnswer y lo aadimos a la lista.
				this.answers.add(new TOQuestionMatchAnswer(af.getMaTextLeft5(), 
						af.getMaTextRight5(), this.answers.size(), af.getMaExplain5()));
			}
			// Establecemos el nmero de opciones de respues.
			this.options = this.answers.size();
		}
	}
	
	/**
	 * <p>Construye un objeto <code>TOQuestionMatch</code> a partir de la informaci&oacute;n
	 * que se recibe como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la preguntas.
	 * @param title <code>String</code> t&iacute;tulo.
	 * @param statement <code>String</code> enunciado.
	 * @param answers <code>List&lt;TOQuestionMatchAnswer&gt;</code> lista de respuestas.
	 * @param castegoryId <code>int</code> identificador de la categor&iacute;a de preguntas
	 * a la que pertenece.
	 */
	public TOQuestionMatch(int id, String title, String statement, 
			List<TOQuestionMatchAnswer> answers, int castegoryId)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asigamos los valores de los atributos.
		this.id = id;
		this.title = title;
		this.statement = statement;
		this.type = TOQuestion.TYPE_MATCH;
		this.answers = answers;
		// Establecemos el nmero de respuesta. Vemos si "answers" es nulo.
		if (answers != null)
		{
			// El nmero de respuestas es el total de elementos de la Lista.
			this.options = answers.size();
		}
		// Establacemos el resto de atributos.
		this.categoryId = castegoryId;
	}
	
	
	// -------- GETTERS & SETTERS.
	public int getOptions() 
	{
		return options;
	}
	public void setOptions(int options) 
	{
		this.options = options;
	}
	public List<TOQuestionMatchAnswer> getAnswers() 
	{
		return answers;
	}
	public void setAnswers(List<TOQuestionMatchAnswer> answers) 
	{
		this.answers = answers;
	}
	
}
