package edu.uoc.pfc.business.transfers;

import java.io.Serializable;

import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que ser&aacute; utilizada para realizar el intercambio de informaci&oacute;n entre
 * capas relacionada con el las diferentes opciones de respuesta en una pregunta de tipo
 * "Emparejar".</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOQuestionMatchAnswer implements Serializable, Comparable<TOQuestionMatchAnswer> 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la Respuesta.</p> */
	private int id;
	/** <p>Uno de los dos miembros del par de la respuesta.</p> */
	private String text1;
	/** <p>El otro miembro de la pareja.</p> */
	private String text2;
	/** <p>Posici&oacute;n en la que ser&aacute; mostrada la respuesta.</p> */
	private int position;
	/** <p>Texto de refuerzo.</p> */
	private String explain;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOQuestionMatchAnswer</code> a partir de la 
	 * informaci&oacute;n que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param text1 <code>String</code> texto de la izquierda del del par de opciones.
	 * @param text2 <code>String</code> texto de la derecha del par de opciones.
	 * @param position <code>int</code> posici&oacute;n de la respuesta.
	 * @param explain <code>String</code> texto de refuerzo.
	 */
	public TOQuestionMatchAnswer(String text1, String text2, int position, String explain)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Establecemos los valores de los atributos.
		this.text1 = text1;
		this.text2 = text2;
		this.position = position;
		this.explain = explain;
	}
	
	/**
	 * <p>Construye un objeto <code>TOQuestionMatchAnswer</code> a partir de la 
	 * informaci&oacute;n que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la opci&oacute;n de respuesta.
	 * @param text1 <code>String</code> texto de la izquierda del del par de opciones.
	 * @param text2 <code>String</code> texto de la derecha del par de opciones.
	 * @param position <code>int</code> posici&oacute;n de la respuesta.
	 * @param explain <code>String</code> texto de refuerzo.
	 */
	public TOQuestionMatchAnswer(int id, String text1, String text2, int position,
			String explain)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.id = id;
		this.text1 = text1;
		this.text2 = text2;
		this.position = position;
		this.explain = explain;
	}
	
	// -------- MTODOS PBLICOS.
	@Override
	public int compareTo(TOQuestionMatchAnswer o) 
	{
		// Comprobamos que el parmetro no sea nulo.
		if (o == null)
		{
			// Al ser nulo supondendremo que "this" siempre es mayor.
			return 1;
		}
		// Retornamos el resultado de comparar los atributos "position" (restarlos)
		return (this.position - o.getPosition());
	}
	
	/**
	 * <p>Retorna el valor del atributo "text1" de tal forma que pueda ser mostrado 
	 * forteado de manera correcta dentro de una p&aacute;gina web.</p>
	 * 
	 * @return <code>String</code> con el valor de "text1" correctamente formateado.
	 */
	public String getText1HTML()
	{
		// Retornamos el valor del atributo formateado.
		return (WebUtils.stringToHMTLParagraph(this.text1));
	}
	
	/**
	 * <p>Retorna el valor del atributo "text2" de tal forma que pueda ser mostrado 
	 * forteado de manera correcta dentro de una p&aacute;gina web.</p>
	 * 
	 * @return <code>String</code> con el valor de "text2" correctamente formateado.
	 */
	public String getText2HTML()
	{
		// Retornamos el valor del atributo formateado.
		return (WebUtils.stringToHMTLParagraph(this.text2));
	}

	/**
	 * <p>Retorna el valor del atributo "explain" de tal forma que pueda ser mostrado 
	 * forteado de manera correcta dentro de una p&aacute;gina web.</p>
	 * 
	 * @return <code>String</code> con el valor de "explain" correctamente formateado.
	 */
	public String getExplainHTML()
	{
		// Retornamos el valor del atributo formateado.
		return (WebUtils.stringToHMTLParagraph(this.explain));
	}
	// -------- GETTERS & SETTERS.	
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public String getText1() 
	{
		return text1;
	}
	public void setText1(String text1) 
	{
		this.text1 = text1;
	}
	public String getText2() 
	{
		return text2;
	}
	public void setText2(String text2) 
	{
		this.text2 = text2;
	}
	public int getPosition() 
	{
		return position;
	}
	public void setPosition(int position) 
	{
		this.position = position;
	}
	public String getExplain() 
	{
		return explain;
	}
	public void setExplain(String explain) 
	{
		this.explain = explain;
	}
	
}
