package edu.uoc.pfc.business.transfers;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.pfc.lib.utils.ValidationUtils;
import edu.uoc.pfc.struts.forms.AFQuestionBank;


/**
 * <p>Clase que ser&aacute; utilizada para intercambiar entre las diferentes capas de la 
 * aplicaci&oacute;n la informaci&oacute;n de las preguntas de tipo "Respuesta M&uacute;ltiple".</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOQuestionMultipleChoices extends TOQuestion 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>N&uacute;mero de respuestas correctas.</p> */
	private int correct;
	/** <p>Lista de respuestas asociadas a la pregunta.</p> */
	private List<TOQuestionMultipleChoicesAnswer> answers;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Crea un objeto <code>TOQuestionMultipleChoices</code> a partir de la 
	 * informaci&oacute;n que se encuentra recogida en el objeto <code>AFQuestionBank</code>
	 * que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param af <code>AFQuestionBank</code> con la informaci&oacute;n para crear el objeto.
	 */
	public TOQuestionMultipleChoices(AFQuestionBank af)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Comprobamos que el parmetro no sea nulo.
		if (af != null)
		{
			// Establecemos los valores de los parmetros.
			this.title = af.getTitle();
			this.statement = af.getStatement();
			this.type = TOQuestion.TYPE_MULTIPLE_CHOICES;
			// Creamos una lista para almacenar todas las opciones.
			this.answers = new ArrayList<TOQuestionMultipleChoicesAnswer>();
			// Contador para determinar de qu opcin se trata (orden de las respuestas).
			int order = 0;
			// Comprobamos si tenemos que crear un objeto TOQuestionMultipleChoice a partir
			// de la respuesta recogida en la opcin 1.
			if (!ValidationUtils.isEmpty(af.getMcAnswer1()))
			{
				// No est vaco as que tenemos que aadirlo a la lista. 
				// Definimos una variable local para ver si es una respuesta correcta.
				boolean isCorrect = true;
				// Comprobamos el valor de "mcCorrect1"
				if ("op_false".equals(af.getMcCorrect1())) isCorrect = false;
				// Creamos un objeto TOQuestionMultipleChoicesAnswer y lo aadimos a la lista.
				this.answers.add(new TOQuestionMultipleChoicesAnswer(order, af.getMcAnswer1(), 
						isCorrect, af.getMcText1()));
				// Incrementamos el valor de order.
				order++;
			}
			// Repetimos el mismo proceso que con la opcin 1 con la opcin 2.
			if (!ValidationUtils.isEmpty(af.getMcAnswer2()))
			{
				// No est vaco as que tenemos que aadirlo a la lista. 
				// Definimos una variable local para ver si es una respuesta correcta.
				boolean isCorrect = true;
				// Comprobamos el valor de "mcCorrect2"
				if ("op_false".equals(af.getMcCorrect2())) isCorrect = false;
				// Creamos un objeto TOQuestionMultipleChoicesAnswer y lo aadimos a la lista.
				this.answers.add(new TOQuestionMultipleChoicesAnswer(order, af.getMcAnswer2(), 
						isCorrect, af.getMcText2()));
				// Incrementamos el valor de order.
				order++;
			}
			// Ahora lo repetimos con la opcin 3.
			if (!ValidationUtils.isEmpty(af.getMcAnswer3()))
			{
				// No est vaco as que tenemos que aadirlo a la lista. 
				// Definimos una variable local para ver si es una respuesta correcta.
				boolean isCorrect = true;
				// Comprobamos el valor de "mcCorrect3"
				if ("op_false".equals(af.getMcCorrect3())) isCorrect = false;
				// Creamos un objeto TOQuestionMultipleChoicesAnswer y lo aadimos a la lista.
				this.answers.add(new TOQuestionMultipleChoicesAnswer(order, af.getMcAnswer3(), 
						isCorrect, af.getMcText3()));
				// Incrementamos el valor de order.
				order++;
			}
			// Lo mismo con la opcin 4...
			if (!ValidationUtils.isEmpty(af.getMcAnswer4()))
			{
				// No est vaco as que tenemos que aadirlo a la lista. 
				// Definimos una variable local para ver si es una respuesta correcta.
				boolean isCorrect = true;
				// Comprobamos el valor de "mcCorrect4"
				if ("op_false".equals(af.getMcCorrect4())) isCorrect = false;
				// Creamos un objeto TOQuestionMultipleChoicesAnswer y lo aadimos a la lista.
				this.answers.add(new TOQuestionMultipleChoicesAnswer(order, af.getMcAnswer4(), 
						isCorrect, af.getMcText4()));
				// Incrementamos el valor de order.
				order++;
			}
			// Y, por ltimo, con la opcin 5.
			if (!ValidationUtils.isEmpty(af.getMcAnswer5()))
			{
				// No est vaco as que tenemos que aadirlo a la lista. 
				// Definimos una variable local para ver si es una respuesta correcta.
				boolean isCorrect = true;
				// Comprobamos el valor de "mcCorrect5"
				if ("op_false".equals(af.getMcCorrect5())) isCorrect = false;
				// Creamos un objeto TOQuestionMultipleChoicesAnswer y lo aadimos a la lista.
				this.answers.add(new TOQuestionMultipleChoicesAnswer(order, af.getMcAnswer5(), 
						isCorrect, af.getMcText5()));
			}
		}
	}
	
	/**
	 * <p>Crea un objeto <code>TOQuestionMultipleChoices</code> a partir de la informaci&oacute;n que se
	 * le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la pregunta.
	 * @param title <code>String</code> t&iacute;tulo.
	 * @param statement <code>String</code> enunciado.
	 * @param answers <code>List&lt;TOQuestionMultipleChoicesAnswer&gt;</code> con la informaci&oacute;n
	 * de todas las respuestas asociada a la pregunta.
	 * @param categoryId <code>int</code> identificador de la categor&iacute;a de preguntas.
	 */
	public TOQuestionMultipleChoices(int id, String title, String statement, 
			List<TOQuestionMultipleChoicesAnswer> answers, int categoryId)
	{
		// Llamamos al constructor de la superclase.
		super(id, title, statement, TOQuestion.TYPE_MULTIPLE_CHOICES);
		// Establecemos las opciones de respuesta.
		this.answers = answers;
		this.categoryId = categoryId;
	}
		
	// -------- GETTERS & SETTERS
	public int getCorrect() 
	{
		return correct;
	}
	public void setCorrect(int correct) 
	{
		this.correct = correct;
	}
	public List<TOQuestionMultipleChoicesAnswer> getAnswers() 
	{
		return answers;
	}
	public void setAnswers(List<TOQuestionMultipleChoicesAnswer> answers) 
	{
		this.answers = answers;
	}

}
