package edu.uoc.pfc.business.transfers;

import java.io.Serializable;

import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que ser&aacute; utilizada para realizar el intercambio de informaci&oacute;n 
 * entre capas relacionadas con cada una de las opciones de respuesta en una pregunta de 
 * tipo "Opciones M&uacute;ltiples".</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOQuestionMultipleChoicesAnswer 
		implements Serializable, Comparable<TOQuestionMultipleChoicesAnswer> 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Indentificador de la opci&oacute;n.</p> */
	private int id;
	/** <p>Posici&oacute;n que ocupa dentro de las opciones respuesta.</p> */
	private int order;
	/** <p>Texto de la respuesta.</p> */
	private String answer;
	/** <p>Indica si es correcta o no.</p> */
	private boolean correct;
	/** <p>Texto de refuerzo asociado a la opci&oacute;n.</p> */
	private String text;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOQuestionMultipleChoicesAnswer</code> a partir de la 
	 * informaci&oacute;n que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param order <code>int</code> orden de la respuesta.
	 * @param answer <code>String</code> texto de la respuesta.
	 * @param correct <code>boolean</code> indica si la respuesta es correcta o no.
	 * @param text <code>String</code> texto de refuerzo.
	 */
	public TOQuestionMultipleChoicesAnswer(int order, String answer, boolean correct, String text)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.order = order;
		this.answer = answer;
		this.correct = correct;
		this.text = text;
	}
	
	/**
	 * <p>Construye un objeto <code>TOQuestionMultipleChoicesAnswer</code> a partir de la 
	 * informaci&oacute;n que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la respuesta en el sistema.
	 * @param order <code>int</code> orden de la respuesta.
	 * @param answer <code>String</code> texto de la respuesta.
	 * @param correct <code>boolean</code> indica si la respuesta es correcta o no.
	 * @param text <code>String</code> texto de refuerzo.
	 */
	public TOQuestionMultipleChoicesAnswer(int id, int order, String answer, boolean correct, String text)
	{
		// Llamamos a otro constructor de la clase.
		this(order, answer, correct, text);
		// Establecemo el resto de valores de los atributos.
		this.id = id;
	}
	
	// -------- MTODOS PBLICOS.
	@Override
	public int compareTo(TOQuestionMultipleChoicesAnswer o) 
	{
		// Comprobamos si el parmetro es nulo o no.
		if (o == null)
		{
			// En el caso de que el parmetro sea nulo consideramos a "this" como mayor.
			return 1;
		}
		// Retornamos el resultado de comparar el orden de las pregunta.
		return (this.order - o.getOrder());
	}
	
	/**
	 * <p>Retorna el valor del atributo "answer" en un formato adecuado para ser mostrado
	 * dentro de una p&aacute;gina Web.</p>
	 * 
	 * @return <code>String</code> con el valor del atributo "answer" formateado.
	 */
	public String getAnswerHTML()
	{
		// Retornamos el valor del atributo formateado.
		return (WebUtils.stringToHMTLParagraph(this.answer));
	}
	
	/**
	 * <p>Retorna el valor del atributo "text" en un formato adecuado para ser mostrado
	 * dentro de una p&aacute;gina Web.</p>
	 * 
	 * @return <code>String</code> con el valor del atributo "text" formateado.
	 */
	public String getTextHTML()
	{
		// Retornamos el valor del atributo formateado.
		return (WebUtils.stringToHMTLParagraph(this.text));
	}
	
	// -------- GETTERS & SETTERS
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public int getOrder() 
	{
		return order;
	}
	public void setOrder(int order) 
	{
		this.order = order;
	}
	public String getAnswer() 
	{
		return answer;
	}
	public void setAnswer(String answer) 
	{
		this.answer = answer;
	}
	public boolean isCorrect() 
	{
		return correct;
	}
	public void setCorrect(boolean correct) 
	{
		this.correct = correct;
	}
	public String getText() 
	{
		return text;
	}
	public void setText(String text) 
	{
		this.text = text;
	}
	
}
