package edu.uoc.pfc.business.transfers;

import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que ser&aacute utilizada para realizar el intercambio de informaci&oacute;n
 * entre las capas de la arquitectura de la aplicaci&oacute;n relacionadas con las preguntas
 * de tipo Num&eacute;rico.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOQuestionNumeric extends TOQuestion
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Valor correcto de la respuesta.</p> */
	private double value;
	/** <p>Margen de error permitido en la respuesta para ser considerada correcta.</p> */
	private double error; 
	/** <p>Texto de refuerzo.</p> */
	private String text;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOQuestionNumeric</code> a partir de la informaci&oacute;n que 
	 * se le pasa como par&aacute;metro.</p>
	 * 
	 * @param title <code>String</code> t&iacute;tulo de la pregunta.
	 * @param statement <code>String</code> enunciado.
	 * @param value <code>double</code> respuesta correcta.
	 * @param error <code>double</code> margen de error permitido.
	 * @param text <code>String</code> texto de refuerzo.
	 */
	public TOQuestionNumeric(String title, String statement, double value, double error, String text)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.title = title;
		this.statement = statement;
		this.value = value;
		this.error = error;
		this.text = text;
	}
	
	/**
	 * <p>Construye un objeto <code>TOQuestionNumeric</code> a partir de la informaci&oacute;n que 
	 * se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la pregunta.
	 *  @param title <code>String</code> t&iacute;tulo de la pregunta.
	 * @param statement <code>String</code> enunciado.
	 * @param value <code>double</code> respuesta correcta.
	 * @param error <code>double</code> margen de error permitido.
	 * @param text <code>String</code> texto de refuerzo.
	 * @param categoryId <code>int</code> identificador de la Categor&iacute;a de Preguntas a la que
	 * pertenece la Pregunta.
	 */
	public TOQuestionNumeric(int id, String title, String statement, double value, double error, 
			String text, int categoryId)
	{
		// Llamamos al constructor de la superclase.
		super(id, title, statement, TOQuestion.TYPE_NUMERIC);
		// Asignamos los atributos.
		this.value = value;
		this.error = error;
		this.text = text;
		this.categoryId = categoryId;
	}
	
	/**
	 * <p>Retorna el valor del atributo "text" en un formato adecuado para ser mostrado
	 * dentro de una p&aacute;gina web.</p>
	 * 
	 * @return <code>String</code> valor del atributo "text" formateado para ser mostrado
	 * en una p&aacute;gina web.
	 */
	public String getTextHTML()
	{
		// Retornamos el valor del atributo formateado.
		return (WebUtils.stringToHMTLParagraph(this.text));
	}
	
	// -------- GETTERS & SETTERS.
	public double getValue()
	{
		return value;
	}
	public void setValue(double value)
	{
		this.value = value;
	}
	public double getError()
	{
		return error;
	}
	public void setError(double error)
	{
		this.error = error;
	}
	public String getText()
	{
		return text;
	}
	public void setText(String text)
	{
		this.text = text;
	}	

}
