package edu.uoc.pfc.business.transfers;

import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que modela a las preguntas de tipo "Respuesta Corta" dentro del sistema.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOQuestionShortAnswer extends TOQuestion 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Indica si la respuesta es sensible a may&uacute;sculas y min%uacute;sculas.</p> */
	private boolean caseSensitive;
	/** <p>Valor de la respuesta correcta.</p> */
	private String answer;
	/** <p>Texto de refuerzo.</p> */
	private String text;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOQuestionShortAnswer</code> a partir de la informaci&oacute;n
	 * que recibe como par&aacute;metro.</p>
	 * 
	 * @param title <code>String</code> t&iacute;tulo de la pregunta.
	 * @param statement <code>String</code> enunciado de la pregunta.
	 * @param answer <code>String</code> respuesta.
	 * @param caseSensitive <code>boolean</code> indica si se ha de distinguir entre may&uacute;sculas
	 * y min&uacute;sculas.
	 * @param text <code>String</code> texto de refuerzo.
	 */
	public TOQuestionShortAnswer(String title, String statement, String answer, 
			boolean caseSensitive, String text)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.title = title;
		this.statement = statement;
		this.answer = answer;
		this.caseSensitive = caseSensitive;
		this.text = text;
		this.type = TOQuestion.TYPE_SHORT_ANSWER;
	}
	
	/**
	 * <p>Construye un objeto <code>TOQuestionShortAnswer</code> a partir de la informaci&oacute;n
	 * que se recibe como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la pregunta.
	 * @param title <code>String</code> t&iacute;tulo de la pregunta.
	 * @param statement <code>String</code> enunciado de la pregunta.
	 * @param answer <code>String</code> respuesta.
	 * @param caseSensitive <code>boolean</code> indica si se ha de distinguir entre may&uacute;sculas
	 * y min&uacute;sculas.
	 * @param text <code>String</code> texto de refuerzo.
	 * @param categoryId <code>int</cod> identificador de la Categor&iacute;a de Preguntas a 
	 * la que pertenece la pregunta.
	 */
	public TOQuestionShortAnswer(int id, String title, String statement, String answer,
			boolean caseSensitive, String text, int categoryId)
	{
		// Llamamos al mismo constructor de la clase.
		this(title, statement, answer, caseSensitive, text);
		// Asignamos el resto de parmetros.
		this.id = id;
		this.categoryId = categoryId;
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna el valor del atributo "text" formateado de tal manera que pueda ser 
	 * mostrado dentro de una p&aacute;gina web.</p>
	 * 
	 * @return <code>String</code> con el valor del atributo "text" formateado.
	 */
	public String getTextHTML()
	{
		// Retornamos el valor de "text" formateado.
		return (WebUtils.stringToHMTLParagraph(this.text));
	}
	
	// -------- GETTERS & SETTERs.
	public boolean isCaseSensitive() 
	{
		return caseSensitive;
	}
	public void setCaseSensitive(boolean caseSensitive) 
	{
		this.caseSensitive = caseSensitive;
	}
	public String getAnswer() 
	{
		return answer;
	}
	public void setAnswer(String answer) 
	{
		this.answer = answer;
	}
	public String getText() 
	{
		return text;
	}
	public void setText(String text) 
	{
		this.text = text;
	}

}
