package edu.uoc.pfc.business.transfers;

import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que recoge la informaci&oacute;n que se pasa entre capas relacionada
 * con un Pregunta de tipo "Verdadero o Falso".</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOQuestionTrueOrFalse extends TOQuestion 
{

	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p><code>true</Code> si la respuesta correcta es la opci&oacute;n "verdadero" o 
	 * <code>false</code> si la opci&oacute;n correcta es la respuesta "falso".</p> */
	private boolean value;
	/** <p>Texto de refuerzo de la opci&oacute;n "verdadero".</p> */
	private String txtTrue;
	/** <p>Texto de refuerzo de la opci&oacute;n "falso".</p> */
	private String txtFalse;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOQuestionTrueOrFalse</code> a partir de la informaci&oacute;n
	 * que se le pasa como par&aacute;metro.</p>
	 * 
	 * @param title <code>String</code> t&iacute;tulo de la pregunta.
	 * @param statement <code>String</code> enunciado.
	 * @param value <code>boolean</code> <code>true</Code> si la respuesta correcta es la opci&oacute;n 
	 * "verdadero" o <code>false</code> si la opci&oacute;n correcta es la respuesta "falso".
	 * @param txtTrue <code>String</code> texto de refuerzo de la opci&oacute;n verdadero.
	 * @param txtFalse <code>String</code> texto de refuerzo de la opci&oacute;n falso.
	 */
	public TOQuestionTrueOrFalse(String title, String statement, 
			boolean value, String txtTrue, String txtFalse) 
	{
		// Llamamos al constructor de la superclase.
		super();
		// Establecemos los valores de los atributos.
		this.title = title;
		this.statement = statement;
		this.value = value;
		this.txtTrue = txtTrue;
		this.txtFalse = txtFalse;
	}
	
	/**
	 * <p>Construye un objeto <code>TOQuestionTrueOrFalse</code> a partir de la informaci&oacute;n que
	 * se le pasa como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la pretunta.
	 * @param title <code>String</code> t&iacute;tulo de la pregunta.
	 * @param statement <code>String</code> enunciado.
	 * @param value <code>boolean</code> <code>true</Code> si la respuesta correcta es la opci&oacute;n 
	 * "verdadero" o <code>false</code> si la opci&oacute;n correcta es la respuesta "falso".
	 * @param txtTrue <code>String</code> texto de refuerzo de la opci&oacute;n verdadero.
	 * @param txtFalse <code>String</code> texto de refuerzo de la opci&oacute;n falso.
	 * @param categoryId <code>int</code> identificador de la Categor&iacute;a la que pertenece
	 * la Pregunta.
	 */
	public TOQuestionTrueOrFalse(int id, String title, String statement, boolean value,
			String txtTrue, String txtFalse, int categoryId)
	{
		// Llamamos al constructor de la superclase.
		super(id, title, statement, TOQuestion.TYPE_TRUE_OR_FALSE);
		// Asignamos los valores de los atributos.
		this.value = value;
		this.txtTrue = txtTrue;
		this.txtFalse = txtFalse;
		this.categoryId = categoryId;
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna el valor del atributo "txtTrue" formateado para ser mostrado dentro de una 
	 * pa&aacute;gina web.</p>
	 * 
	 * @return <code>String</code> con el valor del atributo "txtTrue" formateado.
	 */
	public String getTxtTrueHTML()
	{
		// Retornamos el valor del atributo formateado.
		return (WebUtils.stringToHMTLParagraph(this.txtTrue));
	}
	
	/**
	 * <p>Retorna el valor del atributo "txtFalse" formateado para ser mostrado dentro de una 
	 * pa&aacute;gina web.</p>
	 * 
	 * @return <code>String</code> con el valor del atributo "txtFalse" formateado.
	 */
	public String getTxtFalseHTML()
	{
		// Retornamos el valor del atributo formateado.
		return (WebUtils.stringToHMTLParagraph(this.txtFalse));
	}

	// -------- GETTERS & SETTERS.	
	public boolean isValue() 
	{
		return value;
	}
	public void setValue(boolean value) 
	{
		this.value = value;
	}
	public String getTxtTrue() 
	{
		return txtTrue;
	}
	public void setTxtTrue(String txtTrue) 
	{
		this.txtTrue = txtTrue;
	}
	public String getTxtFalse() 
	{
		return txtFalse;
	}
	public void setTxtFalse(String txtFalse) 
	{
		this.txtFalse = txtFalse;
	}
	
}
