package edu.uoc.pfc.business.transfers;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que recoge la informaci&oacute;n de las asignaturas que est&aacute;n
 * registradas dentro de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOSubject implements Serializable, Comparable<TOSubject> {

	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la asignatura.</p> */
	private int id;
	/** <p>Nombre de la asignatura.</p> */
	private String name;
	/** <p>Fecha de creaci&oacute;n de la asignatura.</p> */
	private Date created;
	/** <p>Descripci&oacute;n de la asignatura.</p> */
	private String description;
	/** <p>Lista de Planes de Estudio donde se imparte la Asignatura.</p> */
	private List<TOSubjectCurriculum> curriculum;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objecto <code>TOSubject</code> he inicializa los valores de sus atributos
	 * a los valores que se pasan en los par&aacute;metros.</p> 
	 * 
	 * @param id <code>int</code> identificador dentro del sistema de la asignatura.
	 * @param name <code>String</code> nombre de la asignatura.
	 * @param created <code>Date</code> fecha de creaci&oacute;n.
	 * @param description <code>String</code> descripcin de la asignatura.
	 */
	public TOSubject(int id, String name, Date created, String description)
	{
		// Constructor de la superclase.
		super();
		// Asignamos los valores de los atributos a sus homnimos.
		this.id = id;
		this.name = name;
		this.created = created;
		this.description = description;
	}
	
	/**
	 * <p>Constructor por defecto (deja los valores de todos sus atributos a sus valores por
	 * defecto).</p>
	 */
	public TOSubject() 
	{
		// Llamamos al constructor de la superclase.
		super();
	}
	
	
	// -------- MTODOS PBLICOS.
	@Override
	public int compareTo(TOSubject o) 
	{
		// Comprobamos que el objeto no sea null.
		if (o != null)
		{
			// Realizamos la comparacin de los nombres.
			return this.name.toLowerCase().compareTo(o.getName().toLowerCase());
		}
		// Si llegamos a este punto es que el parmetro "o" es nulo por lo que 
		// retornaremos 1 indicando que este objeto "this" es mayor.
		return 1;
	}
	
	/**
	 * <p>Retorna la descripci&oacute;n de la asignatura de tal manera que 
	 * pueda ser mostrada correctamente dentro de una p&aacute;gina web.</p>
	 * 
	 * @return <code>String</code> con la descripci&oacute;n de la asignatura.
	 */
	public String getDescriptionHTML()
	{
		// Retornamos valor del atributo "description" formateado.
		return (WebUtils.stringToHMTLParagraph(this.description));
	}
	
	/**
	 * <p>Retorna el valor del atributo <i>description</i> formateado de tal manera que
	 * pueda ser mostrado sin problemas dentro de una textarea de un formulario.</p>
	 * 
	 * @return <code>String</code> con el valor del atributo <i>description</i> formateado.
	 */
	public String getDescriptionForm()
	{
		// Simplemente retornamos el texto formateado.
		return WebUtils.HTMLtoString(this.description);
	}
	
	// -------- GETTERS & SETTERS.
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public String getName() 
	{
		return name;
	}
	public void setName(String name) 
	{
		this.name = name;
	}
	public Date getCreated() 
	{
		return created;
	}
	public void setCreated(Date created) 
	{
		this.created = created;
	}
	public String getDescription() 
	{
		return description;
	}
	public void setDescription(String description) 
	{
		this.description = description;
	}
	public List<TOSubjectCurriculum> getCurriculum() 
	{
		return curriculum;
	}
	public void setCurriculum(List<TOSubjectCurriculum> curriculum) 
	{
		this.curriculum = curriculum;
	}	
	
}
