package edu.uoc.pfc.business.transfers;

import java.io.Serializable;

/**
 * <p>Clase que ser&aacute; utilizada para recoger la informaci&oacute;n de los Planes
 * de Estudio en los que se imparte una determinada asignatura.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public class TOSubjectCurriculum implements Serializable, Comparable<TOSubjectCurriculum>
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador del Plan de Estudios.</p> */
	private int id;
	/** <p>Nombre del Plan de Estudios.</p> */
	private String name;
	/** <p>C&oacute;digo de la asignatura dentro del Plan de Estudios.</p> */
	private String code;
	/** <p>N&uacute;mero de cr&eacute;ditos de la asignatura.</p> */
	private double credits;
	/** <p>Nombre del tipo de Plan de Estudios.</p> */
	private String type;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objecto <code>TOSubjectCurriculum</code> a partir de la 
	 * informaci&oacute;n que recibe como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Plan de Estudios.
	 * @param name <code>String</code> nombre del Plan de Estudios.
	 * @param code <code>String</code> c&oacute;digo de la Asignatura.
	 * @param credits <code>double</code> n&uacute;mero de cr&eacute;ditos.
	 * @param type <code>String</code> nombre del tipo de asignatura.
	 */
	public TOSubjectCurriculum(int id, String name, String code, 
			double credits, String type) {
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.id = id;
		this.name = name;
		this.code = code;
		this.credits = credits;
		this.type = type;
	}
	
	// -------- MTODOS PBLICOS.
	@Override
	public int compareTo(TOSubjectCurriculum o) 
	{
		// Comprobamos que el parmetro no es nulo.
		if (o == null)
		{
			// El parmetro es nulo por lo que retornamos que "this" es mayor.
			return 1;
		}
		// Si llegamos a este punto simplemente tenemos que realizar la comparacin
		// de los nombres de los planes de estudio y retornar el valor obtenido.
		return (this.name.toLowerCase().compareTo(o.getName().toLowerCase()));
	}
	
	// -------- GETTERS & SETTERS.	
	public String getCode() 
	{
		return code;
	}
	public void setCode(String code) 
	{
		this.code = code;
	}
	public double getCredits() 
	{
		return credits;
	}
	public void setCredits(double credits) 
	{
		this.credits = credits;
	}
	public String getType() 
	{
		return type;
	}
	public void setType(String type) 
	{
		this.type = type;
	}
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public String getName() 
	{
		return name;
	}
	public void setName(String name) 
	{
		this.name = name;
	}
	
}
