package edu.uoc.pfc.business.transfers;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import edu.uoc.pfc.lib.utils.DateUtils;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p>Clase que ser&aacute; utilizada para realizar el intercambio de informaci&oacute;n
 * entre diferentes capas relacionadas con los Cuestionarios que los Profesores elaboran 
 * dentro del sistema.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public class TOTest implements Serializable, Comparable<TOTest>
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de la clase.</p> */
	private static final long serialVersionUID = 1L;

	// -------- ATRIBUTOS
	/** <p>Identificador del Cuestionario dentro del sistema.</p> */
	private int id;
	/** <p>T&iacute;tulo del Cuestionario.</p> */
	private String title;
	/** <p>Texto de introducci&oacute;n.</p>*/
	private String introduction;
	/** <p>Indica si se han de barajar las preguntas.</p> */
	private boolean shuffleQuestions;
	/** <p>Indica si se han de barajar las respuestas.</p> */
	private boolean shuffleAnswers;
	/** <p>Indica si se mostrar&aacute; el resultado.</p> */
	private boolean showResult;
	/** <p>Indica si se mostrar&aacute; la revisi&oacute;n.</p> */
	private boolean showReview;
	/** <p>Lista de las Preguntas que pertencen al test.</p> */
	private List<TOTestQuestion> questions;
	/** <p>N&uacute;mero de preguntas que forman el test.</p> */
	private int totalQuestions;
	/** <p>Indica si el Cuestionario est&aacute; publicado en un Aula o no.</p> */
	private boolean published;
	/** <p>Fecha de publicaci&oacute;n del Cuestionario en un Aula.</p> */
	private Date openDate;
	/** <p>Fecha de cierre del cuestionario en un Aula.</p> */
	private Date closeDate;
	/** <p>Fecha en la que un Estudiante ha contestado el Cuestionario.</p> */
	private Date answerDate;
	/** <p>Puntuaci&oacute;n obtenida en el Cuestionario.</p> */
	private double score;
	/** <p>M&aacute;xima puntuaci&oacute;n que se pod&iacute;a obtener en el Cuestionario.</p> */
	private double maxScore;
	/** <p>Puntaci&oacute;n expresada en letras.</p> */
	private String finalScore;
	/** <p>Identificador del Test dentro de un Aula.</p> */
	private int idTestClassroom;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOTest</code> a partir de la informaci&oacute;n que se le pasa
	 * como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Cuestionario.
	 * @param title <code>String</code> t&iacute;tulo del Cuestionario.
	 * @param totalQuestions <code>intt</code> n&uacute;mero total de preguntas del Cuestionario.
	 * @param published <code>boolean</code> indica si el Cuestionario est&aacute; publicado en un 
	 * Aula o no.
	 */
	public TOTest(int id, String title, int totalQuestions, boolean published)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Establecemos los valores de los parmetros.
		this.id = id;
		this.title = title;
		this.totalQuestions = totalQuestions;
		this.published = published;
	}
	
	/**
	 * <p>Construye un objeto <code>TOTest</code> a partir de la informaci&oacute;n que se le pasa
	 * como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Cuestionario.
	 * @param title <code>String</code> t&iacute;tulo del Cuestionario.
	 * @param introduction <code>String</code> texto de introducci&oacute;n.
	 * @param shuffleQuestions <code>boolean</code> indica si hay que barajar las Preguntas.
	 * @param shuffleAnswers <code>boolean</cod> indica si hay que barajar las Respuestas.
	 * @param showResult <code>boolean</code> indica si hay que mostrar los resultados.
	 * @param showReview <code>boolean</code> indica si hay que mostrar la revisi&oacute;n.
	 * @param questions <code>List&lt;TOTestQuestion&gr;</code> lista de Preguntas que conforman
	 * el Cuestionario.
	 * @param published <code>boolean</code> indica si el Cuestionario est&aacute; publicado en un 
	 * Aula o no.
	 */
	public TOTest(int id, String title, String introduction, boolean shuffleQuestions,
			boolean shuffleAnswers, boolean showResult, boolean showReview, 
			List<TOTestQuestion> questions, boolean published)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignacin de los atributos.
		this.id = id;
		this.title = title;
		this.introduction = introduction;
		this.shuffleQuestions = shuffleQuestions;
		this.shuffleAnswers = shuffleAnswers;
		this.showResult = showResult;
		this.showReview = showReview;
		this.questions = questions;
		this.published = published;
		// Asignamos el nmero de preguntas que forman el test.
		if (this.questions != null) this.totalQuestions = this.questions.size();
		else this.totalQuestions = 0;
	}
	
	/**
	 * <p>Construye un objeto <code>TOTest</code> a partir de la informaci&oacute;n que se recibe 
	 * como par&aacute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador del Cuestionario en el sistema.
	 * @param title <code>String</code> t&iacute;tulo del Cuestionario.
	 * @param openDate <code>Date</code> fecha de publicaci&oacute;n.
	 * @param closeDate <code>Date</code> fecha de cierre.
	 * @param totalQuestions <code>int</code> n&uacute;mero de preguntas.
	 */
	public TOTest(int id, String title, Date openDate, Date closeDate, int totalQuestions)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores a los atributos.
		this.id = id;
		this.title = title;
		this.openDate = openDate;
		this.closeDate = closeDate;
		this.totalQuestions = totalQuestions;
	}
	
	// -------- MTODOS PBLICOS.
	@Override
	public int compareTo(TOTest o) 
	{
		// Comprobamos que el objeto que recibe como parmetro no sea nulo.
		if (o == null)
		{
			// Al ser "null" supondremos que "this" siempre es mayor.
			return 1;
		}
		// Si llegamos a este punto simplemente tenemos que comparar los ttulos.
		return this.title.toLowerCase().compareTo(o.getTitle().toLowerCase());
	}
	
	/**
	 * <p>Retorna el valor de atributo "introduction" de tal forma que pueda ser
	 * mostrado correctamente dentro de una p&aacute;gina web.</p> 
	 * 
	 * @return <code>String</code> con el valor del atributo "introduction" correctamente
	 * formateado para ser mostrado en un p&aacute;gina web.
	 */
	public String getIntroductionHTML()
	{
		// Llamamos al mtodo de WebUtils para realizar el formateo y retornamos el resultado.
		return WebUtils.stringToHMTLParagraph(this.introduction);
	}
	
	/**
	 * <p>Retorna el valor <code>true</code> en el caso de que la fecha actual del sistema
	 * sea mayor o igual que la fecha de cierre del Cuestionario o <code>false</code> en 
	 * caso contrario.</p>
	 * 
	 * @return <code>boolean</code> (ver la descripci&oacute;n del m&eacute;todo).
	 */
	public boolean isClosed()
	{
		// Comprobamos si tiene fecha de cierre o no.
		if (this.closeDate == null)
		{
			// Al no tener fecha de cierre se considerar que el test est abierto.
			return false;
		}
		else
		{
			// Obtenemos la fecha del sistema.
			Date now = DateUtils.now();
			// Comprobamos el valor de las fechas y retornamos el resultado de la comparacin.
			return (this.closeDate.getTime() < now.getTime());
		}
	}
	
	/**
	 * <p>Retorna <code>true</code> en el caso de que la fecha del sistema sea mayor o igual
	 * que la fecha de apertura del Cuestionario en el Aula o <code>false</code> en caso 
	 * contrario.</p>
	 * 
	 * @return <code>boolean</code> (ver la descripci&oacute;n del m&eacute;todo).
	 */
	public boolean isOpen()
	{
		// Comprobamos que la fecha de apertura no sea nula.
		if (this.openDate == null)
		{
			// No tenemos fecha de apertura por lo que retornamos false.
			return false;
		}
		else
		{
			// Obtenemos la fecha del sistema.
			Date now = DateUtils.now();
			// Comprobamos el valor de las fechas y retornamos el resultado de la comparacin.
			return (this.openDate.getTime() <= now.getTime());
		}
	}
	
	// -------- GETTERS & SETTERS.
	public int getId() 
	{
		return id;
	}
	public void setId(int id) 
	{
		this.id = id;
	}
	public String getTitle() 
	{
		return title;
	}
	public void setTitle(String title) 
	{
		this.title = title;
	}
	public String getIntroduction() 
	{
		return introduction;
	}
	public void setIntroduction(String introduction) 
	{
		this.introduction = introduction;
	}
	public boolean isShuffleQuestions() 
	{
		return shuffleQuestions;
	}
	public void setShuffleQuestions(boolean shuffleQuestions) 
	{
		this.shuffleQuestions = shuffleQuestions;
	}
	public boolean isShuffleAnswers() 
	{
		return shuffleAnswers;
	}
	public void setShuffleAnswers(boolean shuffleAnswers) 
	{
		this.shuffleAnswers = shuffleAnswers;
	}
	public boolean isShowResult() 
	{
		return showResult;
	}
	public void setShowResult(boolean showResult) 
	{
		this.showResult = showResult;
	}
	public boolean isShowReview() 
	{
		return showReview;
	}
	public void setShowReview(boolean showReview) 
	{
		this.showReview = showReview;
	}
	public List<TOTestQuestion> getQuestions() 
	{
		return questions;
	}
	public void setQuestions(List<TOTestQuestion> questions) 
	{
		this.questions = questions;
	}
	public int getTotalQuestions() 
	{
		return totalQuestions;
	}
	public void setTotalQuestions(int totalQuestions) 
	{
		this.totalQuestions = totalQuestions;
	}
	public boolean isPublished() 
	{
		return published;
	}
	public void setPublished(boolean published) 
	{
		this.published = published;
	}
	public Date getOpenDate() 
	{
		return openDate;
	}
	public void setOpenDate(Date openDate) 
	{
		this.openDate = openDate;
	}
	public Date getCloseDate() 
	{
		return closeDate;
	}
	public void setCloseDate(Date closeDate) 
	{
		this.closeDate = closeDate;
	}
	public Date getAnswerDate() 
	{
		return answerDate;
	}
	public void setAnswerDate(Date answerDate) 
	{
		this.answerDate = answerDate;
	}
	public double getScore() 
	{
		return score;
	}
	public void setScore(double score) 
	{
		this.score = score;
	}
	public double getMaxScore() 
	{
		return maxScore;
	}
	public void setMaxScore(double maxScore) 
	{
		this.maxScore = maxScore;
	}
	public String getFinalScore() 
	{
		return finalScore;
	}
	public void setFinalScore(String finalScore) 
	{
		this.finalScore = finalScore;
	}
	public int getIdTestClassroom()
	{
		return idTestClassroom;
	}
	public void setIdTestClassroom(int idTestClassroom)
	{
		this.idTestClassroom = idTestClassroom;
	}
	
}
