package edu.uoc.pfc.business.transfers;

import java.io.Serializable;
import java.util.Date;

/**
 * <p>Clase que ser&aacute; utilizada para transportar entre las capas de la arquitectura de 
 * la aplicaci&oacute;n la informaci&oacute;n relativa a la soluci&oacute;n de un Cuestionario
 * por parte de un Alumno.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class TOTestAnswer implements Serializable
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para todos los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	/** <p>Nota final obtenida: Matr&iacute;cula de Honor.</p> */
	public static final String FINAL_SCORE_MATRICULA_HONOR = "M";
	/** <p>Nota final obtenida: Sobresaliente.</p> */
	public static final String FINAL_SCORE_SOBRESALIENTE = "EX";
	/** <p>Nota final obtenida: Notable.</p> */
	public static final String FINAL_SCORE_NOTABLE = "NO";
	/** <p>Nota final obtenida: Aprobado.</p> */
	public static final String FINAL_SCORE_APROBADO = "A";
	/** <p>Nota final obtenida: Suspenso.</p> */
	public static final String FINAL_SCORE_SUSPENSO = "SU";
	
	// -------- ATRIBUTOS.
	/** <p>Identificador de la Respuesta.</p> */
	private int id;
	/** <p>Fecha en la que se ha contestado al Cuestionario.</p> */
	private Date answerDate;
	/** <p>Puntuaci&oacute;n obtenida.</p> */
	private double score;
	/** <p>Puntuaci&oacute;n m&aacute;xima que se podr&iacute;a obtener.</p> */
	private double maxScore;
	/** <p>Nota obtenida.</p> */
	private String finalScore;
	/** <p>N&uacute;mero total de Preguntas que ten&iacute;a el Cuestionario.</p> */
	private int totalQuestions;
	/** <p>N&uacute;mero de Preguntas acertadas por el Estudiante.</p> */
	private int correctQuestions;
	
	// -------- CONSTRUCTORES.
	/**
	 * <p>Construye un objeto <code>TOTestAnswer</code> a partir de la informaci&oacute;n que se
	 * recibe como par&acute;metro.</p>
	 * 
	 * @param id <code>int</code> identificador de la respuesta en el sistema.
	 * @param answerDate <code>Date</code> fecha de la respuesta.
	 * @param score <code>double</code> puntuaci&oacute;n obtenida.
	 * @param maxScore <code>double</code> puntuaci&oacugte;n m&aacute;xima que se puede obtener.
	 * @param finalScore <code>String</code> nota final en el Cuestionario. 
	 * @param totalQuestions <code>int</code> n&uacute;mero total de preguntas.
	 * @param correctQuestions <code>int</code> n&uacute;mero de preguntas acertadas.
	 */
	public TOTestAnswer(int id, Date answerDate, double score, double maxScore,
			String finalScore, int totalQuestions, int correctQuestions)
	{
		// Llamamos al constructor de la superclase.
		super();
		// Asignamos los valores de los atributos.
		this.id = id;
		this.answerDate = answerDate;
		this.score = score;
		this.maxScore = maxScore;
		this.finalScore = finalScore;
		this.totalQuestions = totalQuestions;
		this.correctQuestions = correctQuestions;
	}
	// -------- GETTERS & SETTERS.
	public int getId()
	{
		return id;
	}
	public void setId(int id)
	{
		this.id = id;
	}
	public Date getAnswerDate()
	{
		return answerDate;
	}
	public void setAnswerDate(Date answerDate)
	{
		this.answerDate = answerDate;
	}
	public double getScore()
	{
		return score;
	}
	public void setScore(double score)
	{
		this.score = score;
	}
	public double getMaxScore()
	{
		return maxScore;
	}
	public void setMaxScore(double maxScore)
	{
		this.maxScore = maxScore;
	}
	public String getFinalScore()
	{
		return finalScore;
	}
	public void setFinalScore(String finalScore)
	{
		this.finalScore = finalScore;
	}
	public int getTotalQuestions()
	{
		return totalQuestions;
	}
	public void setTotalQuestions(int totalQuestions)
	{
		this.totalQuestions = totalQuestions;
	}
	public int getCorrectQuestions()
	{
		return correctQuestions;
	}
	public void setCorrectQuestions(int correctQuestions)
	{
		this.correctQuestions = correctQuestions;
	}

}
