package edu.uoc.pfc.lib.utils;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;

/**
 * <p>Ofrece utilidades para trabajar con beans.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */

public class BeanUtils 
{
	
	// -------- CONSTANTES.
	/** <p>Constante que contiene la clase Java de un String.</p> */
	public static final String STRING_CLASS = String.class.getName();
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Elimina todos los espacios en blanco al principio  y al final de todas las 
	 * propiedades de tipo String, del bean que se le pasa como parmetro.</p>
	 * 
	 * @param bean <code>Object</code> el bean sobre el cual eliminar los espacios 
	 * en blanco en las propiedades de tipo <code>String</code>.
	 */
	public static void trimProperties(Object bean) 
	{
		// Comprobamos que se recibe un objeto no nulo.
		if (bean != null) 
		{
			try 
			{
				// Obtenemos la informacin del bean.
				BeanInfo beanInfo = Introspector.getBeanInfo(bean.getClass());
				// Obtenemos la descripcin de los atributos del bean.
				PropertyDescriptor[] descriptors = beanInfo.getPropertyDescriptors();
				// Recorremos todo los atributos del bean obteniendo su clase.
				for (PropertyDescriptor oPD : descriptors)
				{
					// Vemos si la propiedad es de tipo String.
					if (STRING_CLASS.equals(oPD.getPropertyType().getName()))
					{
						// Invocamos al mtodo Getter.
						Object obj = oPD.getReadMethod().invoke(bean, new Object[0]);
						// Comprobamos si hemos obtenido un valor.
						if (obj != null)
						{
							// Obtenemos el valor como String y hacemos un trim()
							String value = ((String) obj).trim();
							// Invocamos al mtodo Setter.
							oPD.getWriteMethod().invoke(bean, new Object[]{value});
						}
					}
				}
			} 
			catch (Exception e) 
			{
				// Cualquier excepcin que se produzca la capturamos e imprimimos la pila
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * <p>Transforma todas las propiedades de tipo <code>String</code> del <i>bean</i>
	 * que se recibe como par&aacute;metro transformando los caracteres en su equivalente
	 * en HTML.</p>
	 *  
	 * @param bean <code>Object</code> bean a transformar.
	 * @see edu.uoc.pfc.lib.utils.WebUtils
	 */
	public static void htmlProperties(Object bean)
	{
		// Comprobamos que se recibe un objeto no nulo.
		if (bean != null)
		{
			try
			{
				// Obtenemos la informacin del bean.
				BeanInfo beanInfo = Introspector.getBeanInfo(bean.getClass());
				// Obtenemos la descripcin de los atributos del bean.
				PropertyDescriptor[] descriptors = beanInfo.getPropertyDescriptors();
				// Recorremos todo los atributos del bean obteniendo su clase.
				for (PropertyDescriptor oPD : descriptors)
				{
					// Vemos si la propiedad es de tipo String.
					if (STRING_CLASS.equals(oPD.getPropertyType().getName()))
					{
						// Invocamos al mtodo Getter.
						Object obj = oPD.getReadMethod().invoke(bean, new Object[0]);
						// Comprobamos si hemos obtenido un valor.
						if (obj != null)
						{
							// Obtenemos el valor como String y hacemos un trim()
							String value = ((String) obj).trim();
							// Transformamos los caracteres.
							value = WebUtils.stringToHTML(value);
							// Invocamos al mtodo Setter.
							oPD.getWriteMethod().invoke(bean, new Object[]{value});
						}
					}
				}
			}
			catch (Exception e)
			{
				// Cualquier excepcin que se produzca la capturamos e imprimimos la pila
				e.printStackTrace();
			}
		}
	}
	
}
