package edu.uoc.pfc.lib.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * <p>Clase que ofrece una serie de m&eacute;todos p&uacute;blicos y est&aacute;ticos
 * que permiten trabajar con fechas.</p>
 * 
 * @author @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez.
 * @version 1.0
 */
public class DateUtils
{
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Obtiene la fecha y hora actual del sistema y la retorna como un objeto <code>Date</code>.</p>
	 * 
	 * @return <code>Date</code> que representa la fecha y hora actual del sistema (cuando se llama 
	 * a este m&eacute;todo.
	 */
	public static Date now()
	{
		// Simplemente llamamos al constructor de Date sin argumentos.
		return new Date();
	}
	
	/**
	 * <p>Retorna un objeto <code>Date</code> que representa la fecha y hora que se le pasa como par&aacute;metro.
	 * En el caso de que los par&aacute;metros no formen una fecha y hora v&aacute;lida este m&eeacute;todo retornar&aacute;
	 * <code>null</code>.</p>
	 * 
	 * @param <code>String</code> d&iacute;a del mes.
	 * @param month <code>String</code> mes.
	 * @param year <code>String</code> a&ntilde;o
	 * @param hour <code>String</code> hora del d&iacute;a
	 * @param minutes <code>String</code> minutos dentro de la hora.
	 * @return <code>Date</code> que representa a la fecha y hora pasada o <code>null</code> si los datos no se
	 * corresponden con los de una fecha v&aacute;lida.
	 */
	public static Date createDate(String day, String month, String year, String hour, String minutes)
	{
		// Comprobamos que los parmetros no sean nulos.
		if (!ValidationUtils.isValidDate(day, month, year, hour, minutes)) return null;
		// Preparamos la cadena que formar la fecha.
		String date = day + "-" + month + "-" + year + " " + hour + ":" + minutes;
		// Creamos un SimpleDateFormat con el patrn que vamos a seguir.
		SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy HH:mm");
		// Establecemos que interprete las fechas estrictamente.
		sdf.setLenient(false);
		try
		{
			// Intentamos crear un objeto Date a partir de la informacin que tenemos y lo
			// retornamos.
			 return sdf.parse(date);
		}
		catch (ParseException pe)
		{
			// Se ha producido una excepcin debido a que la fecha no es correcta.
			return null;
		}
	}
	
}
