package edu.uoc.pfc.lib.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/** <p>Clase que ofrece una serie de m&eacute;todos est&aacute;ticos que ser&aacute;n
 * utilizados para trabajar con datos provenientes o mostrados en la web.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0 
 */
public class WebUtils
{
	
	// -------- MTODOS PBLICOS Y ESTTICOS.
	/**
	 * <p>Transforma el <code>String</code> que recibe como par&aacute;metro en contenido
	 * HTML que ser&aacute; mostrado en la WEB. Estas transformaciones incluyen:</p>
	 * 
	 * <ul>
	 * <li>Los retornos de carro y saltos de l&iacute;nea dividir&aacute;n al <code>String</code>
	 * en diferentes p&aacute;rrafos.</p>
	 * </ul>
	 * 
	 * @param text <code>String</code> con el texto a formatear.
	 * @return <code>String</code> con el texto formateado.
	 */
	public static String stringToHMTLParagraph(String text)
	{
		// Comprobamos si el parmetro es nulo o no.
		if (text == null)
		{
			// Al ser nulo retornamos la cadena vaca.
			return "";
		}
		// Eliminamos los caracteres al principio y final.
		text = text.trim();
		// Creamos un string que contendr el texto formateado.
		String newText = "";
		// Rompemos el String buscando saltos de lnes.
		StringTokenizer st = new StringTokenizer(text, "\r");
		// Definimos una lista de String donde metermos los prrafos no vacos.
		List<String> list = new ArrayList<String>();
		// Recorremos los tokens.
		while (st.hasMoreTokens())
		{
			// Obtenemos el siguiente elemento de la lista.
			String next_token = (String) st.nextToken().trim();
			// Solamente lo insertamos si no es vaco.
			if (next_token.length() >= 1)
			{
				// Lo aadimos a la lista de String.
				list.add(next_token);
			}
		}
		// Definimos un contador.
		int count = 1;
		// Recorremos la lista de String.
		for (String s : list)
		{
			// Comprobamos si es el ltimo prrafo
			if (count == list.size())
			{
				// Encerramos el texto con <p class="last">
				newText += "<p class='last' style='text-align: justify;'>" + s + "</p>";
			}
			else 
			{
				// Encerramos el texto con <p>
				newText += "<p style='text-align: justify;'>" + s + "</p>";
			}
			// Incrementamos count
			count++;
		}
		// Retornamos el texto formateado.
		return newText;
	}
	
	/**
	 * <p>Transforma el <code>String</code> que recibe como par&aacute;metro en 
	 * c&oacute;digo HTML de tal manera que todos aquellos caracteres que no son 
	 * est&aacute;ndar (como por ejemplo &aacute;) se transforman en la entidad 
	 * HTML equivalente (en el ejemplo &amp;aacute;).</p>
	 * 
	 * @param text <code>String</code> texto a transformar.
	 * @return <code>String</code> texto transformado.
	 */
	public static String stringToHTML(String text)
	{
		// Comprobamos si el parmetro es nulo o no.
		if (text == null)
		{
			// Retornamos la cadena vaca.
			return "";
		}
		// Eliminamos espacios en blanco al principio y final.
		text = text.trim();
		// Comprobamos que la cadena tiene al menos un carcter.
		if (text.length() == 0)
		{
			// Cadena vaca. La retornamos.
			return text;
		}
		// Llegados a este punto "text" tiene al menos un carcter.
		// Realizamos la sustituciones
		text = text.replace("&", "&amp;");
		text = text.replace("", "&aacute;");
		text = text.replace("", "&eacute;");
		text = text.replace("", "&iacute;");
		text = text.replace("", "&oacute;");
		text = text.replace("", "&uacute;");
		text = text.replace("", "&Aacute;");
		text = text.replace("", "&Eacute;");
		text = text.replace("", "&iacute;");
		text = text.replace("", "&Oacute;");
		text = text.replace("", "&Uacute;");
		text = text.replace("", "&ntilde;");
		text = text.replace("", "&Ntilde;");
		text = text.replace("", "&uuml;");
		text = text.replace("", "&Uuml;");
		text = text.replace("\"", "&quot;");
		text = text.replace("<", "&lt;");
		text = text.replace(">", "&gt;");
		// Retornamos el texto.
		return text;
	}
	
	/**
	 * <p>Transforma el <code>String</code> que recibe como par&aacute;metro el cual
	 * condendr&aacute; entidades HTML a sus caracteres equivalentes y retorna dicha
	 * cadena con todas al entidades transformadas.</p>
	 * 
	 * @param text <code>String</code> el texto a transformar.
	 * @return <code>String</code> texto transformado.
	 */
	public static String HTMLtoString(String text)
	{
		// Comprobamos si se ha recibido la cadena nula.
		if (text == null)
		{
			// Retornamso la cadena vaca.
			return "";
		}
		// Eliminamos espacios en blanco del principio y final
		text = text.trim();
		// Comprobamos que la cadena posee al menos un carcter.
		if (text.length() == 0)
		{
			// Retornamos la cadena vaca.
			return "";
		}
		// Llegados a este punto "text" tiene al menos un carcter.
		// Realizamos la sustituciones
		text = text.replace("&amp;", "&");
		text = text.replace("&aacute;", "");
		text = text.replace("&eacute;", "");
		text = text.replace("&iacute;", "");
		text = text.replace("&oacute;", "");
		text = text.replace("&uacute;", "");
		text = text.replace("&Aacute;", "");
		text = text.replace("&Eacute;", "");
		text = text.replace("&iacute;", "");
		text = text.replace("&Oacute;", "");
		text = text.replace("&Uacute;", "");
		text = text.replace("&ntilde;", "");
		text = text.replace("&Ntilde;", "");
		text = text.replace("&uuml;", "");
		text = text.replace("&Uuml;", "");
		text = text.replace("&quot;", "\"");
		text = text.replace("&lt;", "<");
		text = text.replace("&gt;", ">");
		// Retornamos el texto.
		return text;
	}
	
}
