package edu.uoc.pfc.struts.actions;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;

import edu.uoc.pfc.business.exceptions.AccountLockedException;
import edu.uoc.pfc.business.exceptions.BusinessRulesViolationException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.interfaces.IBAccountManager;
import edu.uoc.pfc.business.transfers.TOConfig;
import edu.uoc.pfc.business.transfers.TOUser;
import edu.uoc.pfc.lib.constants.CTPresentation;
import edu.uoc.pfc.lib.constants.CTStruts;
import edu.uoc.pfc.lib.utils.ValidationUtils;
import edu.uoc.pfc.lib.utils.WebUtils;
import edu.uoc.pfc.struts.forms.AFAccount;

/**
 * <code>DispatchAction</code> que ser&aacute; la encargada de manejar
 * las transacciones relacionadas con la gesti&oacute;n de los usuarios
 * de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class DAAccount extends DABaseDispatchAction 
{
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eeacute;to que ser&aacute; llamado por un administrador del sistema 
	 * cuando desee crear una nueva cuenta de usuario.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward createAccount(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para almancenar los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un administrador.
		if (isAdmin(request) && (form instanceof AFAccount))
		{
			// Cast al formulario.
			AFAccount af = (AFAccount) form;
			// Realizamos la validacin simple de los datos. 
			ActionErrors errors = af.validateCreateAccount();
			// Comprobamos si ha habido errores o no.
			if (errors.size() > 0)
			{
				// Ha habido errores en los datos. Los grabamos.
				addErrors(request, errors);
			}
			else 
			{
				// Creamos tres variables locales para ver el tipo de usuario.
				boolean isAdmin = false;
				boolean isTeacher = false;
				boolean isStudent = false;
				// Asignamos los valores a estas variables.
				if (!ValidationUtils.isEmpty(af.getUserTypeAdmin())) isAdmin = true;
				if (!ValidationUtils.isEmpty(af.getUserTypeTeacher())) isTeacher = true;
				if (!ValidationUtils.isEmpty(af.getUserTypeStudent())) isStudent = true;
				// Creamos una variable local para ver si est activo a partir del formulario.
				boolean isActive = Boolean.parseBoolean(af.getActive());
				// Creamos un objeto TOUser en el que volcamos la informacin del formulario.
				TOUser user = new TOUser(af.getLogin(), af.getPassword(), af.getEmail(),
						isAdmin, isTeacher, isStudent, isActive, WebUtils.stringToHTML(af.getName()), 
						WebUtils.stringToHTML(af.getSurname()));
				try 
				{
					// Obtenemos una instancia de IBAccountManager.
					IBAccountManager instance = createIBAccountManager();
					// Llamamos al mtodo que permite crear la cuenta del usuario.
					user = instance.createAccount(user);
					// Guardamos la informacin del usuario en la request.
					request.setAttribute("user", user);
					// Indicamos que la operacin ha sido realizada con xito.
					request.setAttribute("operation_ok", true);
				}
				catch (DuplicatedFieldException dfe)
				{
					// El campo "login" o "email" est duplicado. Guardamos
					// el mensaje de error en la request.
					request.setAttribute("error_form", dfe.getMessage());
				}
				catch (Exception ex)
				{
					// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
					// mtodo que permite mostrar un mensaje con la informacin del error.
					return processException(DAAccount.class, ex, 
							CTStruts.CP_TRANSACTION_ACCOUNT_CREATE,
							mapping, request);
				}
			}
			// Establecemos los parmetros de la query string.
			qs = CTStruts.PARAMS_ADMIN_USER_FORM;
		}
		// Retonarmos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema
	 * cuando desee cambiar el estado (activado o desactivado) de una cuenta.</p> 
	 *  
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward changeState(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para establecer los parmetros para la vista.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un administrador que existe
		// el parmetro "id" y adems que dicho pamtro es numrico.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBAccountManager.
				IBAccountManager instance = createIBAccountManager();
				// Ejecutamos el mtodo que permite cambiar el estado y guardamos el resultado.
				request.setAttribute("user", 
						instance.changeState(Integer.parseInt(request.getParameter("id"))));
				// Obtenemos la lista de todas las cuentas y la guardamos en la request.
				request.setAttribute("user_list", instance.getAccounts());
				// Indicamos que no ha habido errores.
				request.setAttribute("operation_ok", "changeState");
				// Establecemos los parmetros para la query string.
				qs = CTStruts.PARAMS_ADMIN_USER_LIST;
			}
			catch (BusinessRulesViolationException brve)
			{
				// Se trata del nico administrador del sistema. Indicamos el error.
				request.setAttribute("error_form", brve.getMessage());
				// Establecemos los parmetros para la queryString.
				qs = CTStruts.PARAMS_ADMIN_USER_LIST;
				try
				{
					// Obtenemos una instancia de IBAccountManager.
					IBAccountManager instance = createIBAccountManager();
					// Obtenemos la lista de todas las cuentas y la guardamos en la request.
					request.setAttribute("user_list", instance.getAccounts());
				}
				catch (Exception ex)
				{
					// Se ha producido una excepcin. Llamamos al mtodo que permite 
					// mostrar un mensaje con la informacin del error.
					return processException(DAAccount.class, ex, 
							CTStruts.CP_TRANSACTION_ACCOUNT_CHANGE_STATE,
							mapping, request);
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAAccount.class, ex, 
						CTStruts.CP_TRANSACTION_ACCOUNT_CHANGE_STATE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando quiera
	 * eliminar una de las cuentas del sistema, concretamente aquella cuyo <i>identificador</i>
	 * se recibe dentro del par&aacute;metro de la request denominado <i>id</i>.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward deleteAccount(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros para la vista.
		String qs = "";
		// Comprobamos que el mtodo ha sido llamado por un administrador, que existe el parmetro
		// "id" y adems que dicho parmetro es numrico.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBAccountManager
				IBAccountManager instance = createIBAccountManager();
				// Llamamos al mtodo para borrar la cuenta del usuario.
				instance.deleteAccount(Integer.parseInt(request.getParameter("id")));
				// Guardamos el id del usuario que ha sido borrado.
				request.setAttribute("user_id", request.getParameter("id")); 
				// Obtenemos la lista de todas las cuentas y la guardamos en la request.
				request.setAttribute("user_list", instance.getAccounts());
				// Indicamos que no ha habido errores.
				request.setAttribute("operation_ok", "deleteAccount");
				// Establecemos los parmetros para la query string.
				qs = CTStruts.PARAMS_ADMIN_USER_LIST;
			}
			catch (BusinessRulesViolationException brve)
			{
				// Se intenta borrar la cuenta del nico administrador del sistema.
				request.setAttribute("error_form", brve.getMessage());
				// Establecemos los parmetros para la queryString.
				qs = CTStruts.PARAMS_ADMIN_USER_LIST;
				try
				{
					// Obtenemos una instancia de IBAccountManager.
					IBAccountManager instance = createIBAccountManager();
					// Obtenemos la lista de todas las cuentas y la guardamos en la request.
					request.setAttribute("user_list", instance.getAccounts());
				}
				catch (Exception ex)
				{
					// Se ha producido una excepcin. Llamamos al mtodo que permite 
					// mostrar un mensaje con la informacin del error.
					return processException(DAAccount.class, ex, 
							CTStruts.CP_TRANSACTION_ACCOUNT_DELETE,
							mapping, request);
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAAccount.class, ex, 
						CTStruts.CP_TRANSACTION_ACCOUNT_DELETE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por cualquier usuario de la aplicaci&oacute;n
	 * que quiera modificar su perfil de usuario dentro del sistema.</p>
	 * 
	 * @param <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward editAccount(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la vista.
		String qs = "";
		// Comprobamos que el usuario ha iniciado sesin en el sistema viendo si se ha 
		// guardado un objeto dentro de CTPresentation.USER
		Object oUser = request.getSession().getAttribute(CTPresentation.USER);
		// Vemos que no es nulo y es de tipo TOUser y que el formulario es de tipo AFUser.
		if ((oUser != null) && (oUser instanceof TOUser) && (form instanceof AFAccount))
		{
			// Cast al objeto TOUser.
			TOUser user = (TOUser) oUser;
			// Cast del formulario.
			AFAccount af = (AFAccount) form;
			// Comprobamos si existe o no el parmetro "doOperation" y adems
			// si este parmetro tiene el valor "true".
			if (!ValidationUtils.isEmpty(request.getParameter("doOperation")) 
					&& ("yes".equals(request.getParameter("doOperation"))))
			{
				// Realizamos la validacin simple de los datos del formulario.
				ActionErrors errors = af.validateEditMyAccount();
				// Comprobamos si ha habido errores.
				if (errors.size() > 0)
				{
					// Guardamos los errores para mostrrselos al usuario.
					addErrors(request, errors);
				}
				else
				{
					// No hay errores de validacin simples en los datos.
					try
					{
						// Obtenemos una instancia de IBAccountManager.
						IBAccountManager instance = createIBAccountManager();
						// Realizamos la modificacin y guardamos el 
						// resultado en la sesin (ya que es la cuenta del usuario).
						request.getSession().setAttribute(CTPresentation.USER, 
								instance.modifyAccount(user, af.getPassword(), 
								af.getName(), af.getSurname()));
						// Guardamos un valor en la request indicando que la operacin
						// se ha realizado correctamente.
						request.setAttribute("ok_form", true);
					}
					catch (Exception ex)
					{
						// Se ha producido una excepcin. Llamamos al mtodo que permite 
						// mostrar un mensaje con la informacin del error.
						return processException(DAAccount.class, ex, 
								CTStruts.CP_TRANSACTION_ACCOUNT_DELETE,
								mapping, request);
					}
				}
			}
			else 
			{
				// Tenemos que mostrar la informacin del usuario en el formulario.
				// Establecemos "contrasea"
				af.setPassword(user.getPassword());
				// Establecemos "repertir constasea"
				af.setRepeatPassword(user.getPassword());
				// Establecemos el nombre real.
				af.setName(user.getName());
				// Establecemos los apellidos.
				af.setSurname(user.getSurname());
			}
			// Establecemos los parmetros de la query string.
			qs = CTStruts.PARAMS_ALL_USERS_EDIT_ACCOUNT;
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por cualquier usuario del sistema cuando quiera 
	 * recuperar su nombre de usuario.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward forgotLogin(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la vista.
		String qs = CTStruts.PARAMS_PUBLIC_FORGOT_LOGIN;
		// Comprobamos que el formulario es de tipo AFUser.
		if (form instanceof AFAccount)
		{
			// Cast al formulario.
			AFAccount af = (AFAccount) form;
			// Validacin simple de los datos de entrada
			ActionErrors errors = af.validateForgotLogin();
			// Comprobamos si ha habido errores.
			if (errors.size() > 0)
			{
				// Grabamos los errores.
				addErrors(request, errors);
			}
			else
			{
				try
				{
					// Sin errores. Obtenemos una instancia de IBAccountManager.
					IBAccountManager instance = createIBAccountManager();
					// Llamamos al mtodo que permite recuperar el nombre de usuario.
					instance.forgotLogin(af.getEmail());
					// Si llegamos a este punto querr decir que se ha enviado el 
					// correo electrnico correctamente por lo que informaremos de ello
					// al usuario.
					request.setAttribute("operation_ok", true);
					// Guardamos la direccin de correo electrnico para ser mostrada.
					request.setAttribute("email", af.getEmail());
				}
				catch (Exception ex)
				{
					// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
					// mtodo que permite mostrar un mensaje con la informacin del error.
					return processException(DAAccount.class, ex, 
							CTStruts.CP_TRANSACTION_ACCOUNT_FORGOT_LOGIN,
							mapping, request);
				}
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por cualquier usuario del sistema cuando quiera 
	 * recuperar su contrase&ntilde;a de acceso.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward forgotPassword(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la vista.
		String qs = CTStruts.PARAMS_PUBLIC_FORGOT_PASSWORD;
		// Comprobamos que el formulario es de tipo AFUser.
		if (form instanceof AFAccount)
		{
			// Cast al formulario.
			AFAccount af = (AFAccount) form;
			// Validacin simple de los datos de entrada
			ActionErrors errors = af.validateForgotPassword();
			// Comprobamos si ha habido errores.
			if (errors.size() > 0)
			{
				// Grabamos los errores.
				addErrors(request, errors);
			}
			else
			{
				try
				{
					// Sin errores. Obtenemos una instancia de IBAccountManager.
					IBAccountManager instance = createIBAccountManager();
					// Llamamos al mtodo que permite recuperar la contrasea del usuario.
					instance.forgotPassword(af.getEmail());
					// Si llegamos a este punto querr decir que se ha enviado el 
					// correo electrnico correctamente por lo que informaremos de ello
					// al usuario.
					request.setAttribute("operation_ok", true);
					// Guardamos la direccin de correo electrnico para ser mostrada.
					request.setAttribute("email", af.getEmail());
				}
				catch (Exception ex)
				{
					// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
					// mtodo que permite mostrar un mensaje con la informacin del error.
					return processException(DAAccount.class, ex, 
							CTStruts.CP_TRANSACTION_ACCOUNT_FORGOT_PASSWORD,
							mapping, request);
				}
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando quiera
	 * recuperar la informaci&oacute;n de un usuario, en concreto de aquel usuario cuyo identificador
	 * se le pasa dentro del par&aacute;metro <i>id</i>.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward getAccount(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la vista.
		String qs = "";
		// Comprobamos que el mtodo ha sido llamado por un administrador del sistema,
		// que se ha recibido el parmetro "id" con un valor entero y que el formulario
		// es de tipo AFUser.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))) && 
				(form instanceof AFAccount))
		{
			try
			{
				// Realizamos un cast del formulario.
				AFAccount af = (AFAccount) form;
				// Obtenemos una instancia de IBAccountManage.
				IBAccountManager instance = createIBAccountManager();
				// Recuperamos la informacin del usuario.
				TOUser user = instance.getAccount(Integer.parseInt(request.getParameter("id")));
				// Copiamos la informacin del usuario en el formulario.
				af.copyProperties(user);
				// Establecemos los parmetros de la query string.
				qs = CTStruts.PARAMS_ADMIN_USER_INFO;
			}
			catch (Exception ex)
			{
				// Se ha producidouna excepcin as que llamamos al mtodo que 
				// mostrar un mensaje con la informacin del error.
				return processException(DAAccount.class, ex, 
						CTStruts.CP_TRANSACTION_ACCOUNT_DELETE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema siempre que 
	 * quiera obtener la lista de todas las cuentas de usuario que est&aacute;n registradas en
	 * el sistema.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward listAccount(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para los valores de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo ha sido un administrador.
		if (isAdmin(request))
		{
			try
			{
				// Se trata de un administrador. Obtenemos una instancia de IBAccountManager.
				IBAccountManager instance = createIBAccountManager();
				// Llamamos al mtodo que nos da la lista de cuentas y guardamos el 
				// resultado en la request.
				request.setAttribute("user_list", instance.getAccounts());
				// Grabamos la informacin de fecha y hora.
				saveTimestamp(request);
				// Indicamos cual es la siguiente vista a mostrar.
				qs = CTStruts.PARAMS_ADMIN_USER_LIST;
			}
			catch (Exception ex)
			{
				// Se ha producidouna excepcin as que llamamos al mtodo que 
				// mostrar un mensaje con la informacin del error.
				return processException(DAAccount.class, ex, 
						CTStruts.CP_TRANSACTION_ACCOUNT_LIST,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cada vez que quiera
	 * modificar la informaci&oacute;n de la cuenta de usuario.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario
	 */
	public ActionForward modifyAccount(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la vista.
		String qs = "";
		// Comprobamos que quien ha llamado a este mtodo ha sido un administrador y que
		// el formulario obtenido es de tipo AFUser.
		if ((isAdmin(request)) && (form instanceof AFAccount))
		{
			try
			{
				// Cast del formulario.
				AFAccount af = (AFAccount) form;
				// Realizamos las validaciones simples sobre los datos.
				ActionErrors errors = af.validateModifyAccount();
				// Comprobamos si ha habido errores de validacin simples.
				if (errors.size() > 0)
				{
					// Grabamos los errores de validacin simples.
					addErrors(request, errors);
				}
				// Comprobamos que el campo "id" tiene un valor numrico.
				else if (ValidationUtils.isInteger(af.getId()))
				{
					// Creamos tres variables locales para ver el tipo de usuario.
					boolean isAdmin = false;
					boolean isTeacher = false;
					boolean isStudent = false;
					// Asignamos los valores a estas variables.
					if (!ValidationUtils.isEmpty(af.getUserTypeAdmin())) isAdmin = true;
					if (!ValidationUtils.isEmpty(af.getUserTypeTeacher())) isTeacher = true;
					if (!ValidationUtils.isEmpty(af.getUserTypeStudent())) isStudent = true;
					// Creamos una variable local para ver si est activo a partir del formulario.
					boolean isActive = Boolean.parseBoolean(af.getActive());
					// No ha habido errores. Obtenemos una instance IBAccountManager.
					IBAccountManager instance = createIBAccountManager();
					// Llamamos al mtodo que modifica la cuenta y guardamos
					// el resultado en la request.
					request.setAttribute("user_modify", 
							instance.modifyAccount(Integer.parseInt(af.getId()), 
									af.getPassword(), isAdmin, isTeacher, isStudent, 
									isActive, af.getName(), af.getSurname()));
					// Indicamos que la operacin ha sido realizada correctamente.
					request.setAttribute("operation_ok", true);
				}
				else
				{
					// El parmetro "id" no contiene un valor numrico
					// Lanzamos una excepcin de tipo InvalidIdException.
					throw (new InvalidIdException(InvalidIdException.MSG_INVALID_USER_ID));
				}
				// Establecemos los parmetros de la query string.
				qs = CTStruts.PARAMS_ADMIN_USER_INFO;
			}
			catch (BusinessRulesViolationException brve)
			{
				// Si se produce esta excepcin es que se ha violado una regla de
				// negocio y como tal se lo mostramos al administrador.
				request.setAttribute("error_form", brve.getMessage());
				// Establecemos los parmetros de la query string.
				qs = CTStruts.PARAMS_ADMIN_USER_INFO;
			}
			catch (Exception ex)
			{
				// Se ha producidouna excepcin as que llamamos al mtodo que 
				// mostrar un mensaje con la informacin del error.
				return processException(DAAccount.class, ex, 
						CTStruts.CP_TRANSACTION_ACCOUNT_MODIFY,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado cuando un usuario intenta acceder a su cuenta
	 * dentro de la aplicaci&aocute;n.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario
	 */
	public ActionForward signin(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Definimos una variable local que nos ayadar a escribir los parmetros de la query string.
		String qs = "";
		// Comprobamos que el formulario es de tipo AFUser.
		if (form instanceof AFAccount)
		{
			// Cast al formulario.
			AFAccount af = (AFAccount) form;
			// Realizamos la validacin simple de los datos del fomulario.
			ActionErrors errors = af.validateSignin();
			// Comprobamos si ha habido errores.
			if (errors.size() > 0)
			{
				// Grabamos los errores.
				addErrors(request, errors);
				// Escribimos los parmetro de la query string para la siguiente vista.
				qs = CTStruts.PARAMS_PUBLIC_SIGNIN;
			}
			else
			{
				try
				{
					// No ha habido errores de validacin simples.
					// Obtenemos el nmero de intentos de acceso que le quedan al usuario.
					Object oAttempts = request.getSession().getAttribute(CTPresentation.LOGIN_ATTEMPTS);
					// Comprobamos si hemos obtenido un valor y si es entero.
					if ((oAttempts == null) || (!(oAttempts instanceof Integer)))
					{
						// Tenemos que establecer el valor inicial para el nmero de intentos de acceso.
						// Obtenemos el objeto TOConfig.
						TOConfig config = getActiveConfiguration();
						// Establemos el nmero de intentos de acceso.
						request.getSession().setAttribute(CTPresentation.LOGIN_ATTEMPTS, config.getLoginAttempts());
					}
					// Obtenemos el nmero de intentos de acceso que nos quedan.
					int totalAttempts = ((Integer) request.getSession().getAttribute(CTPresentation.LOGIN_ATTEMPTS)).intValue();
					// Vemos si todava nos quedan intentos de acceso.
					if (totalAttempts > 0)
					{
						// Creamos una instancia de IBAccountManager
						IBAccountManager instance = createIBAccountManager();
						// Llamamos al mtodo que recupera la informacin del usuario y guardamos el 
						// resultado de su ejecucin en un parmetro de la sesin.
						request.getSession().setAttribute(CTPresentation.USER, 
								instance.signin(af.getLogin(), af.getPassword()));
						// Establecemos el nmero de intentos de acceso al valor mximo.
						request.getSession().setAttribute(CTPresentation.LOGIN_ATTEMPTS, 
								getActiveConfiguration().getLoginAttempts());
					}
					else
					{
						// Guardamos en la sesin que no le quedan ms intentos de acceso.
						request.getSession().setAttribute(CTPresentation.NO_MORE_LOGIN_ATTEMPTS, true);
					}
				}
				catch (AccountLockedException ale)
				{
					try
					{
						// Reestablecemos el nmero de intentos de acceso.
						request.getSession().setAttribute(CTPresentation.LOGIN_ATTEMPTS, getActiveConfiguration().getLoginAttempts());
						// Indicamos que se ha producido un error
						request.setAttribute("errors_form", true);
						// Creamos un ActionErrors para guardar el error.
						ActionErrors ae = new ActionErrors();
						// Aadimos el mensaje de error.
						ae.add("error_txt", new ActionMessage("common.account.login.account.locked"));
						// Guardamos los errores.
						addErrors(request, ae);
						// Indicamos la siguiente vista que se tiene que mostrar.
						qs = CTStruts.PARAMS_PUBLIC_SIGNIN;
					}
					catch (Exception ex)
					{
						// Se ha producido una excepcin as que la procesamos.
						return processException(DAAccount.class, ex, 
								CTStruts.CP_TRANSACTION_ACCOUNT_SIGNIN,
								mapping, request);
					}
				}
				catch (InvalidIdException iie)
				{
					// En el caso de que se produzca esta excepcin querr decir que los
					// datos del usuario no corresponden por ninguna cuenta del sistema.
					// Recuperamos el nmero de intentos de acceso que le quedan.
					int totalAttempts = ((Integer) request.getSession().getAttribute(CTPresentation.LOGIN_ATTEMPTS)).intValue();
					// Decrementamos en una unidad este nmero y lo volvemos a guardar.
					request.getSession().setAttribute(CTPresentation.LOGIN_ATTEMPTS, --totalAttempts);
					// Indicamos que se ha producido un error
					request.setAttribute("errors_form", true);
					// Creamos un ActionErrors para guardar el error.
					ActionErrors ae = new ActionErrors();
					// Aadimos el mensaje de error.
					ae.add("error_txt", new ActionMessage("common.account.login.invalid.id", totalAttempts));
					// Guardamos los errores.
					addErrors(request, ae);
					// Indicamos la siguiente vista que se tiene que mostrar.
					qs = CTStruts.PARAMS_PUBLIC_SIGNIN;
				}
				catch (Exception ex)
				{
					// Se ha producidouna excepcin as que llamamos al mtodo que 
					// mostrar un mensaje con la informacin del error.
					return processException(DAAccount.class, ex, 
							CTStruts.CP_TRANSACTION_ACCOUNT_SIGNIN,
							mapping, request);
				}
			}
		}
		// Retornamos la siguiente vista que ser mostrada al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado en el momento que un usuario quiera cerrar
	 * su sesi&oacute;n dentro de su cuenta en la aplicaci&oacute;n.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward signout(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Eliminamos de la sesin la informacin del usuario.
		request.getSession().removeAttribute(CTPresentation.USER);
		// Eliminamos cualquier informacin de los Cuestionarios.
		request.getSession().removeAttribute(CTPresentation.TEST);
		// Retornamos la siguiente visgta a mostrar al usuario.
		return getView(CTStruts.PARAMS_PUBLIC_SIGNOUT, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por cualquier usuario de la aplicaci&oacute;n
	 * cuando quiera enviar una solicitud de creaci&oacute;n de una nueva cuenta.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward signup(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Vemos si el formulario que hemos recibido es de tipo AFUser.
		if (form instanceof AFAccount)
		{
			// Cast al formulario.
			AFAccount af = (AFAccount) form;
			// Realizamos la validacin simple de los datos.
			ActionErrors errors = af.validateSignup();
			// Comprobamos si ha habido errores.
			if (errors.size() > 0)
			{
				// Guardamos los errores.
				addErrors(request, errors);
			}
			else
			{
				// No ha habido errores de validacin simples.
				// Creamos un objeto TOUser.
				TOUser user = new TOUser();
				// Establecemos su "login"
				user.setLogin(af.getLogin());
				// Establecemos su "password"
				user.setPassword(af.getPassword());
				// Establecemos su correo electrnico.
				user.setEmail(af.getEmail());
				// Establecemos su "nombre"
				user.setName(WebUtils.stringToHTML(af.getName()));
				// Establecemos sus apellidos.
				user.setSurname(WebUtils.stringToHTML(af.getSurname()));
				try
				{
					// Obtenemos una instancia de IBAccoutManager.
					IBAccountManager instance = createIBAccountManager();
					// Llamamos al mtodo para la creacin de una cuenta.
					instance.createAccount(user);
					// Indicamos que no se ha producido ningn error.
					request.setAttribute("operation_ok", true);
				}
				catch (DuplicatedFieldException dfe)
				{
					// El "login" o el "email" ya estn siendo utilizados en otra
					// cuenta de usuario. Grabamos un mensaje de error.
					request.setAttribute("errors_txt", dfe.getMessage());
					// Indicamos que se ha producido un error.
					request.setAttribute("errors_form", true);
				}
				catch (Exception ex)
				{
					// Se ha producidouna excepcin as que llamamos al mtodo que 
					// mostrar un mensaje con la informacin del error.
					return processException(DAAccount.class, ex, 
							CTStruts.CP_TRANSACTION_ACCOUNT_SINGUP,
							mapping, request);
				}
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(CTStruts.PARAMS_PUBLIC_SIGNUP, mapping);
	}
	
}
