package edu.uoc.pfc.struts.actions;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;

import edu.uoc.pfc.business.exceptions.BusinessRulesViolationException;
import edu.uoc.pfc.business.exceptions.DataStoreException;
import edu.uoc.pfc.business.exceptions.InvalidIdException;
import edu.uoc.pfc.business.interfaces.IBAccountManager;
import edu.uoc.pfc.business.interfaces.IBClassroomManager;
import edu.uoc.pfc.business.interfaces.IBConfigManager;
import edu.uoc.pfc.business.interfaces.IBCurriculumManager;
import edu.uoc.pfc.business.interfaces.IBQuestionBankManager;
import edu.uoc.pfc.business.interfaces.IBSubjectManager;
import edu.uoc.pfc.business.interfaces.IBTestManager;
import edu.uoc.pfc.business.transfers.TOConfig;
import edu.uoc.pfc.business.transfers.TOUser;
import edu.uoc.pfc.lib.constants.CTPresentation;
import edu.uoc.pfc.lib.constants.CTStruts;
import edu.uoc.pfc.lib.utils.DateUtils;

/**
 * <p>Clase que extiende directamente de <code>DispatchAction</code> de Struts y que ofrece
 * una serie de m&eacute;todos que ser&aacute;n heredados por todas las "actions" que 
 * formar&aacute;n la aplicaci&aocute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 * @see edu.uoc.pfc.struts.plugins.PFCPlugIn
 */
public abstract class DABaseDispatchAction extends DispatchAction
{
	
	// -------- MTODOS PROTEGIDOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; encargado de obtener una instancia de un objeto que
	 * implemente la interfaz <code>IBAccountManager</code> que define los m&eacute;todos que
	 * la capa de negocio ofrece a la capa de presentaci&oacute;n asociados a la gesti&oacute;n
	 * de las cuentas de usurio.</p>
	 * 
	 * @return <code>IBAccountManager</code> objeto que implementa esta interfaz.
	 * @throws ClassNotFoundException Si no se encuentra la clase que implementa la interfaz.
	 * @throws IllegalAccessException Si no se tiene acceso al constructor de esa clase.
	 * @throws InstantiationException Si no se puede construir un objeto de la clase.
	 */
	protected IBAccountManager createIBAccountManager() throws InstantiationException, 
			IllegalAccessException, ClassNotFoundException
	{
		// Tenemos que recuperar del mbito de aplicacin el nombre de la clase de la 
		// capa de negocio que implementa la interfaz.
		String className = (String) getServlet().getServletContext().getAttribute(CTPresentation.IB_ACCOUNT_MANAGER_KEY);
		// Retornamos una nueva intancia de esta clase.
		return (IBAccountManager) Class.forName(className).newInstance();
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; el encargado de obtener una instancia de un objeto que
	 * implemente la interfaz <code>IBClassroomManager</code> la cual define los m&eacute;todos 
	 * que la capa de negocio ofrece a la capa de presentaci&oacute; asociados a la gesti&oacute;n
	 * de las aulas.</p>
	 * 
	 * @return <code>IBClassroomManager</code> objeto que implenta esta interfaz.
	 * @throws InstantiationException Si no se encuentra la clase que implementa la interfaz.
	 * @throws IllegalAccessException Si no se tiene acceso al constructor de esa clase.
	 * @throws ClassNotFoundException Si no se puede construir un objeto de la clase.
	 */
	protected IBClassroomManager createIBClassroomManager() throws InstantiationException,
			IllegalAccessException, ClassNotFoundException
	{
		// Tenemos que recuperar del mbito de aplicacin el nombre de la clase
		// de la capa de negocio que implementa la interfaz.
		String className = (String) getServlet().getServletContext().getAttribute(CTPresentation.IB_CLASSROOM_MANAGER_KEY);
		// Retornamos una nueva intancia de esta clase.
		return (IBClassroomManager) Class.forName(className).newInstance();
	}
		
	/**
	 * <p>M&eeacute;todo que ser&aacute; el encargado de obtener una instancia de un objeto que
	 * implemente la interfaz <code>IBConfigManager</code> la cual define los m&eacute;todos 
	 * que la capa de negocio ofrece a la capa de presentaci&oacute; asociados a la gesti&oacute;n
	 * de la configuraci&oacute;n de la aplicaci&oacute;n.</p>
	 * 
	 * @return <code>IBConfigManager</code> objeto que implenta esta interfaz.
	 * @throws InstantiationException Si no se encuentra la clase que implementa la interfaz.
	 * @throws IllegalAccessException Si no se tiene acceso al constructor de esa clase.
	 * @throws ClassNotFoundException Si no se puede construir un objeto de la clase.
	 */
	protected IBConfigManager createIBConfigManager() throws InstantiationException,
			IllegalAccessException, ClassNotFoundException
	{			
		// Tenemos que recuperar del mbito de aplicacin el nombre de la clase de la
		// capa de negocio que implementa la interfaz.
		String className = (String) getServlet().getServletContext().getAttribute(CTPresentation.IB_CONFIG_MANAGER_KEY);
		// Retornamos una nueva instancia de la clase que acabamos de recuperar.
		return (IBConfigManager) Class.forName(className).newInstance();
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; el encargado de obtener una instancia de un objeto que
	 * implemente la interfaz <code>IBCurriculumManager</code> la cual define los m&eacute;todos 
	 * que la capa de negocio ofrece a la capa de presentaci&oacute; asociados a la gesti&oacute;n
	 * de los planes de estudio de la aplicaci&oacute;n.</p>
	 * 
	 * @return <code>IBCurriculumManager</code> objeto que implenta esta interfaz.
	 * @throws InstantiationException Si no se encuentra la clase que implementa la interfaz.
	 * @throws IllegalAccessException Si no se tiene acceso al constructor de esa clase.
	 * @throws ClassNotFoundException Si no se puede construir un objeto de la clase.
	 */
	protected IBCurriculumManager createIBCurriculumManager() throws InstantiationException,
			IllegalAccessException, ClassNotFoundException
	{			
		// Tenemos que recuperar del mbito de aplicacin el nombre de la clase de la
		// capa de negocio que implementa la interfaz.
		String className = (String) getServlet().getServletContext().getAttribute(CTPresentation.IB_CURRICULUM_MANAGER_KEY);
		// Retornamos una nueva instancia de la clase que acabamos de recuperar.
		return (IBCurriculumManager) Class.forName(className).newInstance();
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; el encargado de obtener una instancia de un objeto que
	 * implemente la interfaz <code>IBSubjectManager</code> la cual define los m&eacute;todos 
	 * que la capa de negocio ofrece a la capa de presentaci&oacute; asociados a la gesti&oacute;n
	 * de las asignaturas de la aplicaci&oacute;n.</p>
	 * 
	 * @return <code>IBSubjectManager</code> objeto que implenta esta interfaz.
	 * @throws InstantiationException Si no se encuentra la clase que implementa la interfaz.
	 * @throws IllegalAccessException Si no se tiene acceso al constructor de esa clase.
	 * @throws ClassNotFoundException Si no se puede construir un objeto de la clase.
	 */
	protected IBSubjectManager createIBSubjectManager() throws InstantiationException,
			IllegalAccessException, ClassNotFoundException
	{
		// Recuperamos del mbito de aplicacin el nombre de la clase de la capa
		// de negocio que implementa la interfaz.
		String className = (String) getServlet().getServletContext().getAttribute(CTPresentation.IB_SUBJECT_MANAGER_KEY);
		// Retornamos una nueva instancia de la clase que acabamos de recuperar.
		return (IBSubjectManager) Class.forName(className).newInstance();
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; el encargado de obtener una instancia de un objeto que
	 * implemente la interfaz <code>IBQuestionManager</code> la cual define los m&eacute;todos 
	 * que la capa de negocio ofrece a la capa de presentaci&oacute; asociados a la gesti&oacute;n
	 * de las preguntas de los cuestionarios.</p>
	 * 
	 * @return <code>IBQuestionManager</code> objeto que implenta esta interfaz.
	 * @throws InstantiationException Si no se encuentra la clase que implementa la interfaz.
	 * @throws IllegalAccessException Si no se tiene acceso al constructor de esa clase.
	 * @throws ClassNotFoundException Si no se puede construir un objeto de la clase.
	 */
	protected IBQuestionBankManager createIBQuestionBankManager() throws InstantiationException,
			IllegalAccessException, ClassNotFoundException
	{			
		// Tenemos que recuperar del mbito de aplicacin el nombre de la clase de la
		// capa de negocio que implementa la interfaz.
		String className = (String) getServlet().getServletContext().getAttribute(CTPresentation.IB_QUESTION_BANK_MANAGER_KEY);
		// Retornamos una nueva instancia de la clase que acabamos de recuperar.
		return (IBQuestionBankManager) Class.forName(className).newInstance();
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; el encargado de obtener una instancia de un objeto que
	 * implemente la interfaz <code>IBTestManager</code> la cual define los m&eacute;todos 
	 * que la capa de negocio ofrece a la capa de presentaci&oacute; asociados a la gesti&oacute;n
	 * de los test (cuestionarios).</p>
	 * 
	 * @return <code>IBConfigManager</code> objeto que implenta esta interfaz.
	 * @throws InstantiationException Si no se encuentra la clase que implementa la interfaz.
	 * @throws IllegalAccessException Si no se tiene acceso al constructor de esa clase.
	 * @throws ClassNotFoundException Si no se puede construir un objeto de la clase.
	 */
	protected IBTestManager createIBTestManager() throws InstantiationException,
			IllegalAccessException, ClassNotFoundException
	{			
		// Tenemos que recuperar del mbito de aplicacin el nombre de la clase de la
		// capa de negocio que implementa la interfaz.
		String className = (String) getServlet().getServletContext().getAttribute(CTPresentation.IB_TEST_MANAGER_KEY);
		// Retornamos una nueva instancia de la clase que acabamos de recuperar.
		return (IBTestManager) Class.forName(className).newInstance();
	}
		
	/**
	 * <p>Prepara los par&aacute;metros que deber&aacute;n ser mostrados al usuario en la p&aacute;gina
	 * que se le muestra cuando se produce una excepci&oacute;n no controlada en la aplicac&iacute;n y 
	 * retorna un <code>ActionForward</code> que contendr&aacute; la informaci&oacute;n de la siguiente
	 * vista que se le tendr&aacute; que mostrar.</p>
	 * 
	 * @param class <code>Class</code> la clase que ha lanzado la excepci&oacute;n.
	 * @param ex <code>Exception</code> la excepci&oacute;n que se ha producido.
	 * @param transaction <code>String</code> la transacci&oacute;n que se estaba ejecutando.
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario que ha
	 * producido la excepci&oacute;n.
	 * @return <code>ActionForward</code> con la informaci&oacute;n de la siguiente vista a mostrar.
	 */
	@SuppressWarnings("rawtypes")
	protected ActionForward processException(Class zClass, Exception ex, String transaction,
			ActionMapping mapping, HttpServletRequest request)
	{
		// Almacenamos la clase que ha lanzado la excepcin.
		request.setAttribute("zClass", zClass.getName());
		// Almacenamos en la request la clase de la excepcin.
		request.setAttribute("type", ex.getClass().getName());
		// Almacenamos en la request el mensaje de la excepcin.
		request.setAttribute("message", ex.getMessage());
		// Almacenamos la transaccin.
		request.setAttribute("transaction", transaction);
		// Siguiente vista a mostrar.
		ActionForward next = mapping.findForward("Index");
		// Escribimos la traza de la pila por la salida estndar.
		ex.printStackTrace();
		// Retornamos la siguiente vista a mostrar.
		return (new ActionForward(next.getPath() + "?page=error"));		
	}
	
	/**
	 * <p>Recupera la informaci&oacute;n de la configuraci&oacute;n que est&aacute; activa dentro de la
	 * aplicaci&oacute;n y la retorna como un objeto de la clase <code>TOConfig</code>.</p>
	 * 
	 * @return <code>TOConfig</code> con la informaci&oacute;n de la configuraci&oacute;n activa.
	 * @throwsSi no se encuentra la clase que implementa la interfaz.
	 * @throws IllegalAccessException Si no se tiene acceso al constructor de esa clase.
	 * @throws ClassNotFoundException Si no se puede construir un objeto de la clase.
	 * @throws DataStoreException Si no se ha podido recuperar la informaci&oacute;n.
	 * @throws BusinessRulesViolationException  si no existe una configuraci&oacute;n activa.
	 */
	protected TOConfig getActiveConfiguration() 
			throws InstantiationException, IllegalAccessException, 
			ClassNotFoundException, DataStoreException, BusinessRulesViolationException
	{
		// Referencia al objeto que guarda la configuracin activa.
		TOConfig config = null;
		// Obtenemos del mbito de aplicacin el objeto con los datos de configuracin.
		Object obj = servlet.getServletContext().getAttribute(CTPresentation.ACTIVE_CONFIGURATION_KEY);
		// Comprobamos si el objeto es nulo o no es de la clase TOConfig.
		if ((obj == null) || (!(obj instanceof TOConfig)))
		{
			// No se ha obtenido la configuracin activa. Obtenemos la clase que implementa
			// a la interfaz IBConfigManager.
			String className = (String) servlet.getServletContext().getAttribute(CTPresentation.IB_CONFIG_MANAGER_KEY);
			// Creamos una nueva instancia de esa clase.
			IBConfigManager configManager = (IBConfigManager) Class.forName(className).newInstance();
			// Obtenemos la informacin de la configuracin activa.
			config = configManager.getConfiguration();
		}
		else 
		{
			// Realizamos un cast del objeto guardado en la aplicacin.
			config = (TOConfig) obj;
		}
		// Retornamos el objeto con la informacin de configuracin.
		return config;
	}
	
	/**
	 * <p>Permite comprobar si la petici&oacute;n HTTP que recibe como par&aacute;metro se 
	 * corresponde con la de un usuario que ha iniciado sesi&oacute;n en su cuenta y adem&aacute;s
	 * es un administrador.</p>
	 * 
	 * @param request <code>HttpServletRequest</code> con la petici&oacute;n HTTP.
	 * @return <code>true</code> si la petici&oacute;n ha sido realizada por un administrador
	 * que ha iniciado sesi&oacute;n o <code>false</code> en caso contrario.
	 */
	protected boolean isAdmin(HttpServletRequest request)
	{
		// Recuperamos de la sesin el objeto con los datos del usuario.
		Object oUser = request.getSession().getAttribute(CTPresentation.USER);
		// Comprobamos que no es nulo y adems que es de tipo TOUser.
		if ((oUser != null) && (oUser instanceof TOUser))
		{
			// Realizamos el cast.
			TOUser user = (TOUser) oUser;
			// Comprobamos si se trata de un administrador o no.
			return user.isAdmin();
		}
		// Si se llega a este punto querr decir que no se ha cumplido alguna
		// de las condiciones que se ha verificado.
		return false;
	}
	
	/**
	 * <p>Permite comprobar si la petici&oacute;n HTTP que recibe como par&aacute;metro
	 * ha sido efectuada por un usuario de tipo <i>profesor</i>.</p>
	 * 
	 * @param request <code>HttpServletRequest</code> con la petici&oacute;n HTTP.
	 * @return <code>true</code> si la petici&oacute;n ha sido efectuada por un profesor
	 * o <code>false</code> en caso contrario.
	 */
	protected boolean isTeacher(HttpServletRequest request)
	{
		// Recuperamos de la sesin el objeto con los datos del usuario.
		Object oUser = request.getSession().getAttribute(CTPresentation.USER);
		// Comprobamos que no es nulo y adems que es de tipo TOUser.
		if ((oUser != null) && (oUser instanceof TOUser))
		{
			// Cast del objeto.
			TOUser user = (TOUser) oUser;
			// Retornamos el valor de su atributo "teacher".
			return user.isTeacher();
		}
		// Si se llega a este punto no se cumple la condicin.
		return false;
	}
	
	/**
	 * <p>Permite comprobar si la petici&oacute;n HTTP que recibe como par&aacute;metro
	 * ha sido efectuada por un usuario de tipo <i>alumno</i>.</p>
	 * 
	 * @param request <code>HttpServletRequest</code> con la petici&oacute;n HTTP.
	 * @return <code>true</code> si la petici&oacute;n ha sido efectuada por un profesor
	 * o <code>false</code> en caso contrario.
	 */
	protected boolean isStudent(HttpServletRequest request)
	{
		// Recuperamos de la sesin el objeto con los datos del usuario.
		Object oUser = request.getSession().getAttribute(CTPresentation.USER);
		// Comprobamos que no es nulo y adems que es de tipo TOUser.
		if ((oUser != null) && (oUser instanceof TOUser))
		{
			// Cast del objeto.
			TOUser user = (TOUser) oUser;
			// Retornamos el valor de su atributo "teacher".
			return user.isStudent();
		}
		// Si se llega a este punto no se cumple la condicin.
		return false;
	}
	
	/**
	 * <p>Retorna la siguiente vista que ser&aacute; mostrada al usuario (el &uacute;nico
	 * &lt;global-forward&gt; de Struts que est&aacute; definido en la aplicaci&oacute;n
	 * a&ntilde;adi&eacute;ndole los par&aacute;metros que se le pasan en el <code>String</code>
	 * que recibe el m&eeacute;todo.</p>
	 * 
	 * @param params <code>String</code> par&aacute;metros a a&ntilde;adir a la vista.
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @return <code>ActionForward</code> con la siguiente vista a mostrar al usuario.
	 */
	protected ActionForward getView(String params, ActionMapping mapping)
	{
		// Obtenemos la vista por defecto.
		ActionForward next = mapping.findForward(CTStruts.GLOBAL_FORWARD_INDEX);
		// Creamos la siguiente vista a mostrar al usuario y al retornamos.
		return (new ActionForward(next.getPath() + params));
	}
	
	/**
	 * <p>Graba en el &aacute;mbito de la <i>request</i> la informaci&oacute;n de la fecha
	 * y hora en el momento en que se invaca a este m&eeacute;todo. Esta informaci&oacute;n 
	 * es grabada en el atributo <i>now</i> dentro del objeto HttpServletResquest que 
	 * recibe como par&aacute;metro.</p>
	 * 
	 * @param request <code>HttpServletRequest</code> en el que grabar el atributo.
	 */
	protected void saveTimestamp(HttpServletRequest request)
	{
		// Grabamos la informacin de la hora.
		request.setAttribute("now", DateUtils.now());
	}
	
	/**
	 * <p>Retorna el identificador del Usuario que est&aacute; guardado en la HttpSession.</p>
	 * 
	 * @param request <code>HttpServletRequest</code> asociada a una petici&oacute;n de usuario.
	 * @return <code>int</code> el identificador del usuario.
	 * @throws InvalidIdException en el caso de que la informaci&oacute;n del usurio no haya
	 * sido previamente guardada en la HttpSession.
	 */
	protected int getUserId(HttpServletRequest request) throws InvalidIdException
	{
		// Recuperamos de la sesin el objeto con los datos del usuario.
		Object oUser = request.getSession().getAttribute(CTPresentation.USER);
		// Comprobamos que el objeto no sea nulo y sea de tipo TOUser.
		if ((oUser != null) && (oUser instanceof TOUser))
		{
			// Retornamos el identificador del usuario.
			return ((TOUser) oUser).getId();
		}
		else
		{
			// No hay un usuario guardado en la sesin. Lanzamos una InvalidIdException.
			throw (new InvalidIdException(InvalidIdException.MSG_INVALID_USER_ID));
		}
	}
	
	
}
