package edu.uoc.pfc.struts.actions;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;

import edu.uoc.pfc.business.exceptions.BusinessRulesViolationException;
import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.interfaces.IBAccountManager;
import edu.uoc.pfc.business.interfaces.IBClassroomManager;
import edu.uoc.pfc.business.interfaces.IBCurriculumManager;
import edu.uoc.pfc.business.interfaces.IBTestManager;
import edu.uoc.pfc.business.transfers.TOClassroom;
import edu.uoc.pfc.lib.constants.CTLinks;
import edu.uoc.pfc.lib.constants.CTStruts;
import edu.uoc.pfc.lib.utils.DateUtils;
import edu.uoc.pfc.lib.utils.ValidationUtils;
import edu.uoc.pfc.struts.forms.AFBase;
import edu.uoc.pfc.struts.forms.AFClassroom;

/**
 * <p><code>DispatchAction</code> que ser&aacute; la encargada de atender las 
 * peticiones de relacionadas con las gesti&oacute;n de las Aulas dentro de la 
 * aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class DAClassroom extends DABaseDispatchAction 
{
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera asignar uno de sus
	 * Cuestionarios a una de sus Aulas.</p> 
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward addTest(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor, que se recibe el parmetro
		// "id" y que este tiene un valor numrico, y que el formulario que se recibe es de tipo
		// AFClassroom.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))) && 
				(form instanceof AFClassroom))
		{
			try
			{
				// Comprobamos la existencia del parmetro "action".
				if (ValidationUtils.isEmpty(request.getParameter("action")))
				{
					// Obtenemos una instancia de IBTestManager
					IBTestManager instance = createIBTestManager();
					// Obtenemos la lista de Cuestionarios del Profesor y la guardamos.
					request.setAttribute("test_list", instance.listTest(getUserId(request)));
					// Guardamos en la request el identificador del aula.
					request.setAttribute("classroom_id", request.getParameter("id"));
					// Asignamos los parmetros de la query string.
					qs = CTLinks.CLASSROOM_DATA_ADD_TEST_PARAMS;
				}
				else
				{
					// Tenemos que realizar la asignacin. Cast al formulario.
					AFClassroom af = (AFClassroom) form;
					// Realizamos las validaciones simples sobre los datos del formulario.
					ActionErrors errors = af.validateAddTest();
					// Comprobamos si ha habido errores de validacin simples.
					if (errors.size() > 0)
					{
						// Guardamos los errores de validacin simples.
						addErrors(request, errors);
						// Obtenemos una instancia de IBTestManager
						IBTestManager instance = createIBTestManager();
						// Obtenemos la lista de Cuestionarios del Profesor y la guardamos.
						request.setAttribute("test_list", instance.listTest(getUserId(request)));
						// Guardamos en la request el identificador del aula.
						request.setAttribute("classroom_id", request.getParameter("id"));
						// Asignamos los parmetros de la query string.
						qs = CTLinks.CLASSROOM_DATA_ADD_TEST_PARAMS;
					}
					else
					{
						// Sin errores de validacin simples. Obtenemos una instancia de IBClassroomManager.
						IBClassroomManager instance = createIBClassroomManager();
						// Llamamos al mtodo para asignar el test.
						instance.addTest(Integer.parseInt(request.getParameter("id")), 
								Integer.parseInt(af.getTestId()), getUserId(request), 
								af.getOpenDate(), af.getCloseDate());
						// Indicamos que la operacin ha sido realizada con xito.
						request.setAttribute("operation_ok", true);
						// Guardamos en la request el identificador del aula.
						request.setAttribute("classroom_id", request.getParameter("id"));
						// Establecemos los parmetros de la query string.
						qs = CTLinks.CLASSROOM_DATA_ADD_TEST_PARAMS;
					}
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_ADD_TEST,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando
	 * quiera asignar un Aulumno a un aula.</p> 
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward assignStudent(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Definimos una variable local para los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo sea un Administrador, que se ha
		// recibido el parmetro "id", que dicho parmetro es numrico y, adems, que 
		// "form" es de tipo AFClassroom.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))) && 
				(form instanceof AFClassroom))
		{
			try
			{
				// Creamos una instancia de IBClassroomManager
				IBClassroomManager instance = createIBClassroomManager();
				// Comprobamos la existencia del parmetro "action".
				if (ValidationUtils.isEmpty(request.getParameter("action")))
				{
					// Guardamos el identificador del Aula.
					request.setAttribute("classroom_id", request.getParameter("id"));
					// Guardamos la informacin del Aula en la request
					request.setAttribute("classroom_data", 
							instance.getClassroom(Integer.parseInt(request.getParameter("id"))));
					// Obtenemos la lista de Alumnos que no han sido asignados al Aula.
					request.setAttribute("student_list", 
							instance.getUnassignedStudents(Integer.parseInt(request.getParameter("id"))));
					// Indicamos los parmetros de la query string.
					qs = CTLinks.CLASSROOM_ASSIGN_PARAMS;			
				}
				else if (ValidationUtils.isInteger(request.getParameter("total_students")))
				{
					// Se ha recibido el parmetro "action" y el nmero de alumnos candidatos.
					// Creamos una lista donde almacenaremos los identificadores de los alumnos seleccionados.
					List<Integer> list_id_students = new ArrayList<Integer>();
					// Recorremos el total de alumnos.
					for (int i = 1; i <= Integer.parseInt(request.getParameter("total_students")); i++)
					{
						// Comprobamos si se ha recibido el parmetro "studentId_#" y adems 
						// que dicho parmetro contiene un valor entero.
						if (ValidationUtils.isInteger(request.getParameter("studentId_" + i)))
						{
							// Se ha recibido el parmetro (se ha chequeado el checkbox) por lo
							// que lo aadimos a la lista de identificadores seleccionados.
							list_id_students.add(Integer.parseInt(request.getParameter("studentId_" + i)));
						}
					}
					// Vemos si al menos se ha aadido un alumno.
					if (list_id_students.size() == 0)
					{
						// No se ha aadido ningn alumno. Creamos un objeto ActionErrors.
						ActionErrors errors = new ActionErrors();
						// Aadimos el mensaje debe seleccionar uno.
						errors.add("StudentId", new ActionMessage(AFBase.ERRORS_KEY_CHOOSE_ONE, "Estudiante"));
						// Guardamos el mensaje de error.
						addErrors(request, errors);
						// Guardamos el identificador del Aula.
						request.setAttribute("classroom_id", request.getParameter("id"));
						// Guardamos la informacin del Aula en la request
						request.setAttribute("classroom_data", 
								instance.getClassroom(Integer.parseInt(request.getParameter("id"))));
						// Obtenemos la lista de Alumnos que no han sido asignados al Aula.
						request.setAttribute("student_list", 
								instance.getUnassignedStudents(Integer.parseInt(request.getParameter("id"))));
						// Indicamos los parmetros de la query string.
						qs = CTLinks.CLASSROOM_ASSIGN_PARAMS;	
					}
					else
					{
						try
						{
							// Recorremos la lista con los identificadores de los Alumnos a Asignar.
							for (Integer idStudent : list_id_students)
							{
								// Llamamos al mtodo que permite asignar al Alumno al Aula.
								instance.assignStudent(Integer.parseInt(request.getParameter("id")), idStudent);
							}
							// Indicamos que la operacin ha finalizado con xito.
							request.setAttribute("operation_ok", "assign_student");
							// Guardamos el identificador del Aula.
							request.setAttribute("classroom_id", request.getParameter("id"));
							// Indicamos los parmetros de la siguiente vista a mostrar.
							qs = CTLinks.CLASSROOM_ASSIGN_PARAMS;
						}
						catch (BusinessRulesViolationException brve)
						{
							// Se intenta asignar como un Alumno a un Usuario que ya ha sido 
							// asignado al Aula como un Profesor. Guardamos el mensaje de error.
							request.setAttribute("error_form", brve.getMessage());
							// Guardamos el identificador del Aula.
							request.setAttribute("classroom_id", request.getParameter("id"));
							// Guardamos la informacin del Aula en la request
							request.setAttribute("classroom_data", 
									instance.getClassroom(Integer.parseInt(request.getParameter("id"))));
							// Obtenemos la lista de Alumnos que no han sido asignados al Aula.
							request.setAttribute("student_list", 
									instance.getUnassignedStudents(Integer.parseInt(request.getParameter("id"))));
							// Indicamos los parmetros de la query string.
							qs = CTLinks.CLASSROOM_ASSIGN_PARAMS;	
						}
					}
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_ASSIGN_TEACHER,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando
	 * quiera asignar un Profesor a un aula.</p> 
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward assignTeacher(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comproboamos que quien ha llamado al mtodo es una administrador, que se recibe el 
		// parmetro "id", que adems dicho parmetro contiene un valor numrico y que el formulario
		// es de tipo AFClassroom
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))) && 
				(form instanceof AFClassroom))
		{
			try
			{
				// Creamos una instancia de IBClassroomManager
				IBClassroomManager instance = createIBClassroomManager();		
				// Cast del formulario.
				AFClassroom af = (AFClassroom) form;
				// Comprobamos la presencia del parmetro "action".
				if (ValidationUtils.isEmpty(request.getParameter("action")))
				{
					// No se ha definido el parmetro "action" por lo que tenemos que obtener
					// la lista de profesores y mostrrsela al usuario para que elija a uno.
					// Creamos una instancia de IBAccountManager.
					IBAccountManager instanceAccount = createIBAccountManager();
					// Obtenemos la lista de profesores y los guardamos en la request.
					request.setAttribute("teacher_list", instanceAccount.getTeachers());
					// Obtenemos la informacin del Aula.
					TOClassroom classroom = instance.getClassroom(Integer.parseInt(request.getParameter("id")));
					// Guardamos la informacin del Aula en la request
					request.setAttribute("classroom_data", classroom);
					// Coprobamos si el aula tiene ya un profesor asignado.
					if (classroom.getTeacher() != null)
					{
						// Guardamos el identificador del Profesor dentro del formulario.
						af.setTeacherId(Integer.toString(classroom.getTeacher().getId()));
					}
					// Indicamos los parmetros de la query string.
					qs = CTLinks.CLASSROOM_ASSIGN_PARAMS;					
				}
				else
				{
					// Validamos que se ha seleccionado un Profesor.
					ActionErrors errors = af.validateAssignTeacher();
					// Comprobamos si ha habido errores.
					if (errors.size() > 0)
					{
						// Guardamos los errores.
						addErrors(request, errors);
						// Creamos una instancia de IBAccountManager.
						IBAccountManager instanceAccount = createIBAccountManager();
						// Obtenemos la lista de profesores y los guardamos en la request.
						request.setAttribute("teacher_list", instanceAccount.getTeachers());
						// Obtenemos la informacin del Aula y la guardamos en la request
						request.setAttribute("classroom_data", 
								instance.getClassroom(Integer.parseInt(request.getParameter("id"))));
						// Indicamos los parmetros de la query string.
						qs = CTLinks.CLASSROOM_ASSIGN_PARAMS;	
					}
					else
					{
						try
						{
							// No ha habido errores de validacin simples en los datos.
							// Llamamos al mtodo de la capa de negocio que permite realizar
							// la asignacin del profesor.
							instance.assignTeacher(Integer.parseInt(request.getParameter("id")), 
									Integer.parseInt(af.getTeacherId()));
							// Indicamos que la operacin se ha realizado correctamente.
							request.setAttribute("operation_ok", "assign_teacher");
							// Guardamos el identificador del Aula.
							request.setAttribute("classroom_id", request.getParameter("id"));
							// Indicamos los parmetros de la query string.
							qs = CTLinks.CLASSROOM_ASSIGN_PARAMS;
						}
						catch (BusinessRulesViolationException brve)
						{
							// El Profesor ya haba sido asignado previamente como Alumno al Aula.
							// Guardamos el mensaje de error.
							request.setAttribute("error_form", brve.getMessage());
							// Creamos una instancia de IBAccountManager.
							IBAccountManager instanceAccount = createIBAccountManager();
							// Obtenemos la lista de profesores y los guardamos en la request.
							request.setAttribute("teacher_list", instanceAccount.getTeachers());
							// Obtenemos la informacin del Aula y la guardamos en la request
							request.setAttribute("classroom_data", 
									instance.getClassroom(Integer.parseInt(request.getParameter("id"))));
							// Indicamos los parmetros de la query string.
							qs = CTLinks.CLASSROOM_ASSIGN_PARAMS;
						}
					}
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_ASSIGN_TEACHER,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando
	 * quiera crear una nueva Aula dentro de la aplicaci&oacute;n.</p> 
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward createClassroom(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local para recoger los parmetros de la query string
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Administrador
		// y adems que el formulario es de tipo AFClassroom.
		if ((isAdmin(request)) && (form instanceof AFClassroom))
		{
			try
			{
				// Cast al formulario.
				AFClassroom af = (AFClassroom) form;
				// Comprobamos la existencia del parmetro "step".
				if (ValidationUtils.isEmpty(request.getParameter("step")))
				{
					// Tenemos que obtener la lista de todos los planes de estudio
					// que estn registrados en el sistema.
					IBCurriculumManager instance = createIBCurriculumManager();
					// Obtenemos la lista de Planes de Estudio y los guardamos
					// en la request.
					request.setAttribute("curriculum_list", instance.listPlan());
					// Guardamos en la request un valor indicando en el paso que estamos.
					request.setAttribute("step", "step_1");
					// Establecemos los parmetros de la query string.
					qs = CTLinks.CLASSROOM_CREATE_PARAMS;
				}
				else if ("step_2".equals(request.getParameter("step")))
				{
					// Estamos en el paso para la seleccin de un Plan de Estudios.
					// Realizamos la validacin simple del formulario.
					ActionErrors errors = af.validateCreateClassroomStep1();
					// Comprobamos si ha habido errores.
					if (errors.size() > 0)
					{
						// Guardamos los errores.
						addErrors(request, errors);
						// Tenemos que obtener la lista de todos los planes de estudio
						// que estn registrados en el sistema.
						IBCurriculumManager instance = createIBCurriculumManager();
						// Obtenemos la lista de Planes de Estudio y los guardamos
						// en la request.
						request.setAttribute("curriculum_list", instance.listPlan());
						// Guardamos en la request un valor indicando en el paso que estamos.
						request.setAttribute("step", "step_1");
						// Establecemos los parmetros de la query string.
						qs = CTLinks.CLASSROOM_CREATE_PARAMS;
					}
					else 
					{
						// Creamos una instancia de IBCurriculumManager.
						IBCurriculumManager instance = createIBCurriculumManager();
						// Obtenemos la informacin del Plan de Estudios y la
						// guardamos en la request.
						request.setAttribute("curriculum_data", 
								instance.getPlan(Integer.parseInt(af.getCurriculumId())));
						// Indicamos que el paso en el que estamos es el 2.
						request.setAttribute("step", "step_2");
						// Establecemos los parmetros de la query string.
						qs = CTLinks.CLASSROOM_CREATE_PARAMS;
					}
				}
				else if ("step_3".equals(request.getParameter("step")))
				{
					// Estamos en el paso 3 donde se debe crear el Aula.
					// Realizamos la validacin simple de los datos.
					ActionErrors errors = af.validateCreateClassroomStep2();
					// Comprobamos si existen errores de validacin simples en los datos.
					if (errors.size() > 0)
					{
						// Guardamos los errores.
						addErrors(request, errors);
						// Creamos una instancia de IBCurriculumManager.
						IBCurriculumManager instance = createIBCurriculumManager();
						// Obtenemos la informacin del Plan de Estudios y la
						// guardamos en la request.
						request.setAttribute("curriculum_data", 
								instance.getPlan(Integer.parseInt(af.getCurriculumId())));
						// Indicamos que el paso en el que estamos es el 2.
						request.setAttribute("step", "step_2");
						// Establecemos los parmetros de la query string.
						qs = CTLinks.CLASSROOM_CREATE_PARAMS;
					}
					else if (ValidationUtils.isInteger(af.getCurriculumId()))
					{
						// Sin errores de validacin simples y adems el campo "curriculumId"
						// tiene un valor entero. Creamos una instancia de IBClassroomManager
						IBClassroomManager instance = createIBClassroomManager();
						// Llamamos al mtodo que permite crear el aula y guardamos el resultado de su 
						// ejecucin como un parmetro de la request.
						request.setAttribute("classroom_data", instance.createClassroom(Integer.parseInt(af.getCurriculumId()), 
								Integer.parseInt(af.getSubjectId()), Integer.parseInt(af.getCourse()), af.getLetter()));
						// Guardamos en la request que la operacin se ha ejecutado con xito.
						request.setAttribute("operation_ok", true);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.CLASSROOM_CREATE_PARAMS;
					}
				}
			}
			catch (DuplicatedFieldException dfe)
			{
				// Se ha intentado crear un Aula que ya estaba creada. 
				// Guardamos el mensaje de error en la request.
				request.setAttribute("error_form", dfe.getMessage());
				// Cast del ActionForm
				AFClassroom af = (AFClassroom) form;
				try
				{
					// Creamos una instancia de IBCurriculumManager.
					IBCurriculumManager instance = createIBCurriculumManager();
					// Obtenemos la informacin del Plan de Estudios y la
					// guardamos en la request.
					request.setAttribute("curriculum_data", 
							instance.getPlan(Integer.parseInt(af.getCurriculumId())));
					// Indicamos que el paso en el que estamos es el 2.
					request.setAttribute("step", "step_2");
					// Establecemos los parmetros de la query string.
					qs = CTLinks.CLASSROOM_CREATE_PARAMS;
				}
				catch (Exception ex)
				{
					// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
					// mtodo que permite mostrar un mensaje con la informacin del error.
					return processException(DAClassroom.class, ex, 
							CTStruts.CP_TRANSACTION_CLASSROOM_CREATE,
							mapping, request);
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_CREATE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usaurio.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un Administrador del sistema 
	 * cada vez que quiera eliminar un Aula.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward deleteClassroom(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local donde definimos los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Administrador, que se recibe 
		// el parmetro "id" y que dicho parmetro tiene un valor numrico.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBClassroomManager
				IBClassroomManager instance = createIBClassroomManager();
				// Llamamos al mtodo para eliminar el Aula.
				instance.deleteClassroom(Integer.parseInt(request.getParameter("id")));
				// Indicamos que la operacin se ha realizado con xito.
				request.setAttribute("operation_ok", "deleteClassroom");
				// Llamamos al mtodo para obtener la lista de Aulas y 
				// guardamos el resultado en la request.
				request.setAttribute("classroom_list", instance.listClassroom());
				// Indicamos los parmetros de la query string.
				qs = CTLinks.CLASSROOM_LIST_PARAMS;
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_DELETE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cada 
	 * vez que quiera obtener la informaci&oacute;n de un Aula.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward getClassroom(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local para los parmetros de la query string.
		String qs = "";
		// Comprobamos que se trata de un administrador del sistema, que se recibe el 
		// parmetro "id" y que dicho parmetro tiene un valor numrico.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Creamos una instancia de IBClassroomManager
				IBClassroomManager instance = createIBClassroomManager();
				// Obtenemos la informacin del Aula y la guardamos en la request.
				request.setAttribute("classroom_data", 
						instance.getClassroom(Integer.parseInt(request.getParameter("id"))));
				// Obtenemos una instancia de IBAccountManager.
				IBAccountManager instanceUsers = createIBAccountManager();
				// Obtenemos la lista de profesores y la guardamos su tamao en la request.
				request.setAttribute("total_teachers", instanceUsers.getTeachers().size());
				// Obtenemos la lista de alumnos y guardamos su tamao en la request.
				request.setAttribute("total_students", instanceUsers.getStudents().size());
				// Obtenemos la fecha y hora de la informacin.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.CLASSROOM_DATA_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_MODIFY,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Estudiante cuando quiera obtener la informaci&oacute;n
	 * detallada de una de las Aulas en las que est&aacute; asignado.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward getClassroomStudent(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Estudiante, que se recibe el parmetro "id"
		// y que dicho parmetro contiene un valor numrico.
		if ((isStudent(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBClassroomManager.
				IBClassroomManager instance = createIBClassroomManager();
				// Llamamos al mtodo que permite obtener la informacin del Aula y guardamos el resultado
				// en la request.
				request.setAttribute("classroom_data", instance.getClassroomStudent(getUserId(request), 
						Integer.parseInt(request.getParameter("id"))));
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query String.
				qs = CTLinks.CLASSROOM_DATA_STUDENT_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_DATA_STUDENT,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera obtener los detalles
	 * de una de la Aulas que tiene asignadas como profesor.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward getClassroomTeacher(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor, que se recibe el parmetro
		// "id" y que este tiene un valor numrico.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBClassroomManager.
				IBClassroomManager instance = createIBClassroomManager();
				// Llamamos al mtodo de la interfaz para obtener la informacin y guardamos el 
				// resultado obtenido en la request.
				request.setAttribute("classroom_data", instance.getClassroomTeacher(getUserId(request), 
						Integer.parseInt(request.getParameter("id"))));
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Indicamos los parmetros de la siguiente vista a mostrar.
				qs = CTLinks.CLASSROOM_DATA_TEACHER_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_DATA_TEACHER,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema
	 * cada vez que quiera obtener un listado de todas las aulas que est&aacute;n
	 * registradas en la aplicacaci&oacute;n.</p>
	 *   
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward listClassroom(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local para recoger los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un administrador.
		if (isAdmin(request))
		{
			try
			{
				// Obtenemos una instancia de IBClassroomManager
				IBClassroomManager instance = createIBClassroomManager();
				// Llamamos al mtodo para obtener la lista de Aulas y 
				// guardamos el resultado en la request.
				request.setAttribute("classroom_list", instance.listClassroom());
				// Indicamos los parmetros de la query string.
				qs = CTLinks.CLASSROOM_LIST_PARAMS;
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_LIST,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado cuando un Estudiante desea obtener un listado de las Aulas
	 * en las que est&aacute; asignado.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward listClassroomStudent(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local para los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Estudiante.
		if (isStudent(request))
		{
			try
			{
				// Obtenemos una instancia de IBClassroomManager
				IBClassroomManager instance = createIBClassroomManager();
				// Llamamos al mtodo que nos permite obtener la lista de Aulas y guardamos
				// el resultado que nos retorna en la request.
				request.setAttribute("classroom_list", instance.listClassroomStudent(getUserId(request)));
				// Guardamos la informacin de fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.CLASSROOM_LIST_STUDENT_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_LIST_STUDENT,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor del sistema cada vez que quiera
	 * desasignar uno de sus Cuestionarios de un Aula.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward removeTest(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien llama al mtodo es un Profesor, que se recibe los parmetros
		// "idTest" e "idClassroom" y que ambos poseen un valor numrico.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("idTest")))
				&& (ValidationUtils.isInteger(request.getParameter("idClassroom"))))
		{
			try
			{
				// Obtenemos una instancia de IBClassroomManager.
				IBClassroomManager instance = createIBClassroomManager();
				// Llamamos al mtodo que permite realizar la desasignacin.
				instance.removeTest(Integer.parseInt(request.getParameter("idTest")), 
						Integer.parseInt(request.getParameter("idClassroom")), 
						getUserId(request));
				// Indicamos que la operacin se ha realizado con xito.
				request.setAttribute("operation_ok", "removeTest");
				// Llamamos al mtodo de la interfaz para obtener la informacin y guardamos el 
				// resultado obtenido en la request.
				request.setAttribute("classroom_data", instance.getClassroomTeacher(getUserId(request), 
						Integer.parseInt(request.getParameter("idClassroom"))));
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Indicamos los parmetros de la siguiente vista a mostrar.
				qs = CTLinks.CLASSROOM_DATA_TEACHER_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido una excepcin por lo que llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_REMOVE_TEST,
						mapping, request);
			}
		}
		// Retornamos al siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor con el fin de obtener la informaci&oacute;n
	 * de todas las Aulas en las que est&aacute; asignado como Profesor.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward listClassroomTeacher(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Definimos una variable local para establecer los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un profesor.
		if (isTeacher(request))
		{
			try
			{
				// Obtenemos una instancia de IBClassroomManager
				IBClassroomManager instance = createIBClassroomManager();
				// Llamamos al mtodo que permite obtener la lista de Aulas y guardamos el resultado
				// en la request.
				request.setAttribute("classroom_list", instance.listClassroomTeacher(getUserId(request)));
				// Establecemos los parmetros de la query string.
				qs = CTLinks.CLASSROOM_LIST_TEACHER_PARAMS;
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
			}
			catch (Exception ex)
			{
				// Se ha producido una excepcin. Llamamos al mtodo que permite mostrar 
				// un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_LIST_TEACHER,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Administrador del sistema cuando
	 * quiera desasignar un Alumno de un Aula.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward unassignStudent(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo sea un administrador, que se recibe 
		// el parmetro "id" (identificador del Aula) y que contiene un valor numrico, 
		// y que se recibe el parmetro "student" (identificador del Alumno) con un valor numrico.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id")))
				&& (ValidationUtils.isInteger(request.getParameter("student"))))
		{
			try
			{
				// Obtenemos una instancia de IBClassroomManager.
				IBClassroomManager instance = createIBClassroomManager();
				// Llamamos al mtodo que se encarga de eliminar el Alumno.
				instance.unassignStudent(Integer.parseInt(request.getParameter("id")), 
						Integer.parseInt(request.getParameter("student")));
				// Indicamos que la operacin se ha realizado correctamente.
				request.setAttribute("operation_ok", "unassignStudent");
				// Obtenemos la informacin del Aula y la guardamos en la request.
				request.setAttribute("classroom_data", 
						instance.getClassroom(Integer.parseInt(request.getParameter("id"))));
				// Obtenemos una instancia de IBAccountManager.
				IBAccountManager instanceUsers = createIBAccountManager();
				// Obtenemos la lista de profesores y la guardamos su tamao en la request.
				request.setAttribute("total_teachers", instanceUsers.getTeachers().size());
				// Obtenemos la lista de alumnos y guardamos su tamao en la request.
				request.setAttribute("total_students", instanceUsers.getStudents().size());
				// Obtenemos la fecha y hora de la informacin.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.CLASSROOM_DATA_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido una excepcin. Llamamos al mtodo que permite mostrar 
				// un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_UNASSIGN_STUDENT,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Administrador del sistema cuando quiera
	 * desasignar un Profesor de una Asignatura.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward unassignTeacher(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Administrador, que se recibe el 
		// parmetro "id" y que este contiene un valor numrico.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBClassroomManager
				IBClassroomManager instance = createIBClassroomManager();
				// Llamamos al mtodo para desasignar el Profesor.
				instance.unassignTeacher(Integer.parseInt(request.getParameter("id")));
				// Indicamos que la operacin se ha realizado correctamente.
				request.setAttribute("operation_ok", "unassignTeacher");
				// Obtenemos la informacin del Aula y la guardamos en la request.
				request.setAttribute("classroom_data", 
						instance.getClassroom(Integer.parseInt(request.getParameter("id"))));
				// Obtenemos una instancia de IBAccountManager.
				IBAccountManager instanceUsers = createIBAccountManager();
				// Obtenemos la lista de profesores y la guardamos su tamao en la request.
				request.setAttribute("total_teachers", instanceUsers.getTeachers().size());
				// Obtenemos la lista de alumnos y guardamos su tamao en la request.
				request.setAttribute("total_students", instanceUsers.getStudents().size());
				// Obtenemos la fecha y hora de la informacin.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.CLASSROOM_DATA_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido una excepcin. Llamamos al mtodo que permite mostrar 
				// un mensaje con la informacin del error.
				return processException(DAClassroom.class, ex, 
						CTStruts.CP_TRANSACTION_CLASSROOM_UNASSIGN_TEACHER,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
}
