package edu.uoc.pfc.struts.actions;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import edu.uoc.pfc.business.interfaces.IBConfigManager;
import edu.uoc.pfc.business.transfers.TOConfig;
import edu.uoc.pfc.lib.constants.CTLinks;
import edu.uoc.pfc.lib.constants.CTPresentation;
import edu.uoc.pfc.lib.constants.CTStruts;
import edu.uoc.pfc.struts.forms.AFConfiguration;

/**
 * <p><code>DispatchAction</code> que ser&aacute; utilizada para gestionar las opciones
 * de configuraci&oacute;n de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class DAConfiguration extends DABaseDispatchAction
{
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Administrador de la aplicaci&oacute;n
	 * cuando quiera obtener la informaci&oacute;n de la configuraci&oacute;n activa (vigente)
	 * de la aplicaci&oacute;n.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward getConfiguration(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo sea un Administrador y que el formulario
		// es de tipo AFConfiguration.
		if ((isAdmin(request)) && (form instanceof AFConfiguration))
		{
			// Tenemos que recordar que los datos de la configuracin activa ya 
			// estn almacenados en el mbito de la aplicacin cuando la aplicacin
			// arranca gracias a la ejecucin del plug-in "PFCPlugIn". 
			// Cast del formulario.
			AFConfiguration af = (AFConfiguration) form;
			// Recuperamos la informacin de la configuracin del mbito de la aplicacin.
			Object objConfiguration = getServlet().getServletContext().getAttribute(CTPresentation.ACTIVE_CONFIGURATION_KEY);
			// Vemos si objConfiguration es de tipo TOConfig.
			if (objConfiguration instanceof TOConfig)
			{
				// Cast de objConfiguration.
				TOConfig configuration = (TOConfig) objConfiguration;
				// Establecemos los valores del formulario.
				af.setLoginAttempts(Integer.toString(configuration.getLoginAttempts()));
				af.setActiveCourse(Integer.toString(configuration.getCourse()));
				af.setAdminEmail(configuration.getAdminEmail());
				af.setSmtpServer(configuration.getSmtpServer());
				// Comprobamos si el puerto SMTP es 0
				if (configuration.getSmtpPort() == 0) af.setSmtpPort("");
				else af.setSmtpPort(Integer.toString(configuration.getSmtpPort()));
				af.setSmtpUser(configuration.getSmtpUser());
				af.setSmtpPassword(configuration.getSmtpPassword());
				af.setFpFrom(configuration.getForgotPasswordFrom());
				af.setFpSubject(configuration.getForgotPasswordSubject());
				af.setFpBody(configuration.getForgotPasswordText());
				af.setFlFrom(configuration.getForgotLoginFrom());
				af.setFlSubject(configuration.getForgotLoginSubject());
				af.setFlBody(configuration.getForgotLoginText());
				// Guardamos si se enviarn emails o no.
				request.setAttribute("send_email", configuration.isSendEmail());
				// Guardamos si el servidor SMTP requiere autentificacin o no.
				request.setAttribute("smtp_authentication", configuration.isSmtpAuthentication());
				// Guardamos si el servidor SMTP admite el protocolo TLS.
				request.setAttribute("smtp_tls", configuration.isSmtpTLS());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.CONFIGURATION_DATA_PARAMS;
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return (getView(qs, mapping));
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Administrador del sistema cuando 
	 * quiera modificar las opciones de configuraci&oacute;n de la Aplicaci&oacute;n.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward modifyConfiguration(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Definimos una variable local que recoja los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Administrador del sistema y
		// que el formulario es de tipo AFConfiguration.
		if ((isAdmin(request)) && (form instanceof AFConfiguration))
		{
			try
			{
				// Cast del formulario.
				AFConfiguration af = (AFConfiguration) form;
				// Realizamos la validacin simple sobre los datos del formulario.
				ActionErrors errors = af.validateModify();
				// Vemos si ha habido errores de validacin simples.
				if (errors.size() > 0)
				{
					// Ha habido errores de validacin simples por lo que los guardamos.
					addErrors(request, errors);
					// Definimos tres variables para los atributos booleanos.
					boolean sendEmail = false;
					boolean smtpAuthentication = false;
					boolean smtpTLS = false;
					// Vemos si tenemos que cambiar los valores de estas variables en 
					// funcin de lo que se haya seleccionado en los formularios.
					if ("yes".equals(af.getSendEmail())) sendEmail = true;
					if ("yes".equals(af.getSmtpAuthentication())) smtpAuthentication = true;
					if ("yes".equals(af.getSmtpTLS())) smtpTLS = true;
					// Guardamos si se enviarn emails o no.
					request.setAttribute("send_email", sendEmail);
					// Guardamos si el servidor SMTP requiere autentificacin o no.
					request.setAttribute("smtp_authentication", smtpAuthentication);
					// Guardamos si el servidor SMTP admite el protocolo TLS.
					request.setAttribute("smtp_tls", smtpTLS);
				}
				else
				{
					// No ha habido errores de validacin. Obtenemos una instancia de IBConfigManager
					IBConfigManager instance = createIBConfigManager();
					// Creamos una referencia de tipo TOConfig.
					TOConfig config = null;
					// Vemos si hay que actualizar o no la informacin de envo de emails.
					if ("yes".equals(af.getSendEmail()))
					{
						// S hay informacin de envo de correo electrnico.
						boolean smtpAuthentication = false;
						boolean smtpTLS = false;
						// Vemos si tenemos que cambiar los valores de estas variables en 
						// funcin de lo que se haya seleccionado en los formularios.
						if ("yes".equals(af.getSmtpAuthentication())) smtpAuthentication = true;
						if ("yes".equals(af.getSmtpTLS())) smtpTLS = true;
						// Llamamos al mtodo de la interfaz para realizar la actualizacin.
						config = instance.modifyConfiguration(Integer.parseInt(af.getLoginAttempts()), 
								Integer.parseInt(af.getActiveCourse()), af.getAdminEmail(), af.getSmtpServer(), 
								Integer.parseInt(af.getSmtpPort()), smtpAuthentication, smtpTLS, af.getSmtpUser(),
								af.getSmtpPassword(), af.getFpFrom(), af.getFpSubject(), af.getFpBody(),
								af.getFlFrom(), af.getFlSubject(), af.getFlBody());
					}
					else
					{
						// No tenemos que enviar correos electrnicos.
						// Llamamos al mtodo de la interfaz para realizar la actualizacin.
						config = instance.modifyConfiguration(Integer.parseInt(af.getLoginAttempts()), 
								Integer.parseInt(af.getActiveCourse()), af.getAdminEmail());
					}
					// Una vez realizada la actualizacin debemos actualizar la informacin de la
					// misma en el contexto de la aplicacin.
					request.getServletContext().setAttribute(CTPresentation.ACTIVE_CONFIGURATION_KEY, config);
					// Indicamos que la operacin se ha realizado con xito.
					request.setAttribute("operation_ok", true);
				}				
				// Establecemos los parmetros de la query string.
				qs = CTLinks.CONFIGURATION_DATA_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAConfiguration.class, ex, 
						CTStruts.CP_TRANSACTION_CONFIGURATION_MODIFY,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return (getView(qs, mapping));
	}
	
}
