package edu.uoc.pfc.struts.actions;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.interfaces.IBCurriculumManager;
import edu.uoc.pfc.business.transfers.TOCurriculum;
import edu.uoc.pfc.lib.constants.CTLinks;
import edu.uoc.pfc.lib.constants.CTStruts;
import edu.uoc.pfc.lib.utils.DateUtils;
import edu.uoc.pfc.lib.utils.ValidationUtils;
import edu.uoc.pfc.struts.forms.AFCurriculum;

/**
 * <p><code>DispatchAction</code> de Struts que ser&aacute; la encargada de 
 * atender todas las peticiones de los usuarios de la aplicaci&oacute;n relacionadas
 * con la gesti&oacute;n de los planes de estudios.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class DACurriculum extends DABaseDispatchAction 
{
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cada vez 
	 * que quiera asignar una asigantura a un Plan de Estudios.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward addSubject(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Definimos una variable local con los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un administrador, que se recibe
		// el parmetro "id" y que este tiene un valor entero.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBCurriculumManager.
				IBCurriculumManager instance = createIBCurriculumManager();
				// Comprobamos la presencia del parmetro "action"
				if (ValidationUtils.isEmpty(request.getParameter("action")))
				{
					// No se ha recibido el parmetro "action" por lo que tendremos que
					// obtener la lista de asignaturas que no estn asignadas a un plan 
					// de estudios en concreto. 
					// Obtenemos la informacin del Plan de Estudios y lo guardamos.
					request.setAttribute("curriculum_data", 
							instance.getPlan(Integer.parseInt(request.getParameter("id"))));
					// Obtenemos la lista de Asignaturas no asignadas.
					request.setAttribute("subject_list", 
							instance.getUnassignedSubjects(Integer.parseInt(request.getParameter("id"))));
					// Establecemos los parmetros de la query string
					qs = CTLinks.CURRICULUM_ADD_SUBJECT_PARAMS;
				}
				else if (form instanceof AFCurriculum)
				{
					// Se ha pasado el parmetro "action" y adems el formulario es de tipo AFCurriculum
					// Cast del formulario.
					AFCurriculum af = (AFCurriculum) form;
					// Realizamos la validacin simple de los datos del formulario.
					ActionErrors errors = af.validateAddSubject();
					// Comprobamos si ha habido errores.
					if (errors.size() > 0)
					{
						// Grabamos los errores.
						addErrors(request, errors);
						// Indicamos los parmetros de la query string.
						qs = CTLinks.CURRICULUM_ADD_SUBJECT_PARAMS;
						// de estudios en concreto. 
						// Obtenemos la informacin del Plan de Estudios y lo guardamos.
						request.setAttribute("curriculum_data", 
								instance.getPlan(Integer.parseInt(request.getParameter("id"))));
						// Obtenemos la lista de Asignaturas no asignadas.
						request.setAttribute("subject_list", 
								instance.getUnassignedSubjects(Integer.parseInt(request.getParameter("id"))));
					}
					else
					{
						// Llamamos al mtodo que realiza la asignacin.
						instance.addSubject(Integer.parseInt(request.getParameter("id")), Integer.parseInt(af.getSubjectId()), 
								af.getSubjectType(), af.getCode(), Double.parseDouble(af.getCredits()));
						// Guardamos el identificador del Plan de Estudios.
						request.setAttribute("curriculum_id", request.getParameter("id"));
						// Indicamos que la operacin se ha realizado con xito.
						request.setAttribute("operation_ok", true);
						// Indicamos los parmetros de query string.
						qs = CTLinks.CURRICULUM_ADD_SUBJECT_PARAMS;
					}
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DACurriculum.class, ex, 
						CTStruts.CP_TRANSACTION_CURRICULUM_ADD_SUBJECT,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema siempre que quiera
	 * crear un nuevo Plan de Estudios dentro de la aplicaci&oacute;n.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward createPlan(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que el usuario que ha llamado al mtodo es un administrador y el tipo de formulario.
		if ((isAdmin(request)) && (form instanceof AFCurriculum))
		{
			// Cast al formulario.
			AFCurriculum af = (AFCurriculum) form;
			// Obtenemos los errores de validacin simples (si es que los hay)
			ActionErrors errors = af.validateCreateCurriculum();
			// Comprobamos si ha habido errores.
			if (errors.size() > 0)
			{
				// Guardamos los errores.
				addErrors(request, errors);
				// Indicamos los parmetros de la query string.
				qs = CTLinks.CURRICULUM_CREATE_PARAMS;
			}
			else
			{
				// No ha habido errores de validacin simples.
				try
				{
					// Obtenemos una instancia de IBCurriculumManager
					IBCurriculumManager instance = createIBCurriculumManager();
					// Llamamos al mtodo que permite crear el Plan de Estudios y guardamos 
					// el resultado de la ejecucin en la request.
					request.setAttribute("curriculum_data", instance.createPlan(af.getName(), 
							af.getDescription(), af.getType()));
					// Indicamos que la operacin se ha realizado correctamente.
					request.setAttribute("operation_ok", true);
					// Indicamos los parmetros de la query string.
					qs = CTLinks.CURRICULUM_CREATE_PARAMS;
				}
				catch (DuplicatedFieldException dfe)
				{
					// Se ha intentado registrar un Plan de Estudios que ya estaba registrado
					// Indicamos que ha habido un error.
					request.setAttribute("error_form", dfe.getMessage());
					// Indicamos la siguiente vista que se ha de mostrar.
					qs = CTLinks.CURRICULUM_CREATE_PARAMS;
				}
				catch (Exception ex)
				{
					// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
					// mtodo que permite mostrar un mensaje con la informacin del error.
					return processException(DACurriculum.class, ex, 
							CTStruts.CP_TRANSACTION_CURRICULUM_CREATE,
							mapping, request);
				}
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema cuando quiera 
	 * borrar uno de los Planes de Estudio que est&aacute;n registrados.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward deletePlan(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local donde almacenaremos los parmetros para la query String.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un administrador, que se ha recibido
		// un parmetro denominado "id" y que el valor de ese parmetro es numrico.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBCurricumulManager.
				IBCurriculumManager instance = createIBCurriculumManager();
				// Llamamos al mtodo que elimina el Plan de Estudios del sistema.
				instance.deletePlan(Integer.parseInt(request.getParameter("id")));
				// Indicamos que la operacin ha sido realizada con xito.
				request.setAttribute("operation_ok", "deletePlan");
				// Volvemos a obtener la lista de Planes de Estudio.
				request.setAttribute("curriculum_list", instance.listPlan());
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.CURRICULUM_LIST_PARAMS;
			}
			catch (Exception ex)
			{
				// Llamamos al mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DACurriculum.class, ex, 
						CTStruts.CP_TRANSACTION_CURRICULUM_DELETE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamada para obtener toda la informaci&oacute;n de un 
	 * determinado Plan de Estudios que est&aacute; registrada en el sistema.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward getPlan(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local que nos ayudar a definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un administrador, que se ha recibido
		// el parmetro "id" y que dicho parmetro tiene un valor numrico.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBCurriculumManager.
				IBCurriculumManager instance = createIBCurriculumManager();
				// LLamamos al mtodo que permite obtener la informacin del Plan de Estudios y
				// guardamos el resultado de su ejecucin en la request.
				request.setAttribute("curriculum_data", 
						instance.getPlan(Integer.parseInt(request.getParameter("id"))));
				// Establecemos los parmetros de la Query String.
				qs = CTLinks.CURRICULUM_GET_PARAMS;
				// Guardamos la fecha y hora actual.
				request.setAttribute("now", DateUtils.now());
			}
			catch (Exception ex)
			{
				// Llamamos al mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DACurriculum.class, ex, 
						CTStruts.CP_TRANSACTION_CURRICULUM_GET,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por parte de un administrador cuando desee
	 * obtener un listado de todos los Planes de Estudio registrados en la Aplicaci&oacute;n.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward listPlan(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local donde almacenaremos los parmetros de la query String.
		String qs = "";
		// Comprobamos que quien ha llamado a este mtodo es un administrador.
		if (isAdmin(request))
		{
			try
			{
				// Obtenemos una instancia de IBCurriculumManager.
				IBCurriculumManager instance = createIBCurriculumManager();
				// Llamamos al mtodo para obtener el listado de los planes de estudio
				// y guardamos el resultado dentro de un atributo de la request.
				request.setAttribute("curriculum_list", instance.listPlan());
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.CURRICULUM_LIST_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DACurriculum.class, ex, 
						CTStruts.CP_TRANSACTION_CURRICULUM_LIST,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema cuando 
	 * quiera modificar la informaci&aocute;n de un Plan de Estudios.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward modifyPlan(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un administrador, que se recibe el 
		// parmetro "id", que dicho parmetro contiene un valor numrico y que el formulario
		// que se recibe es de tipo AFCurriculum.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id")))
				&& (form instanceof AFCurriculum))
		{
			try
			{
				// Obtenemos una instancia de IBCurriculumManager.
				IBCurriculumManager instance = createIBCurriculumManager();
				// Realizamos un cast del formulario.
				AFCurriculum af = (AFCurriculum) form;
				// Comprobamos la existencia del parmetro "action"
				if (ValidationUtils.isEmpty(request.getParameter("action")))
				{
					// Obtenemos la informacin del Plan de Estudios.
					TOCurriculum curriculum = instance.getPlan(Integer.parseInt(request.getParameter("id")));
					// Guardamos la informacin en la request.
					request.setAttribute("curriculum_data", curriculum);
					// Establecemos el valor de la descripcin.
					af.setDescription(curriculum.getDescriptionForm());
					// Indicamos los parmetros de la siguiente pgina.
					qs = CTLinks.CURRICULUM_MODIFY_PARAMS;
				}
				else
				{
					// Llamamos al mtodo que nos permite modificar la informacin.
					TOCurriculum curriculum = instance.modifyPlan(
							Integer.parseInt(request.getParameter("id")), 
							af.getDescription());
					// Establecemos el valor de la descripcin en el formulario.
					af.setDescription(curriculum.getDescriptionHTML());
					// Indicamos los parmetros de la siguiente pgina.
					qs = CTLinks.CURRICULUM_MODIFY_PARAMS;
					// Indicamos que la operacin se ha realizado con xito.
					request.setAttribute("operation_ok", true);
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DACurriculum.class, ex, 
						CTStruts.CP_TRANSACTION_CURRICULUM_LIST,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema cuando quiera desasingar
	 * una Asignatura de un Plan de Estudios.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward removeSubject(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo ha sido un administrador, que se han pasado
		// los parmetros "idCurriculum" e "idSubject" y que ambos contienen un valor entero.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("idCurriculum"))) && 
				(ValidationUtils.isInteger(request.getParameter("idSubject"))))
		{
			try
			{
				// Obtenemos una instacia de IBCurriculumManager.
				IBCurriculumManager instance = createIBCurriculumManager();
				// Llamamos al mtodo que permite realizar la desasignacin.
				instance.removeSubject(Integer.parseInt(request.getParameter("idCurriculum")), 
						Integer.parseInt(request.getParameter("idSubject")));				
				// LLamamos al mtodo que permite obtener la informacin del Plan de Estudios y
				// guardamos el resultado de su ejecucin en la request.
				request.setAttribute("curriculum_data", 
						instance.getPlan(Integer.parseInt(request.getParameter("idCurriculum"))));
				// Establecemos los parmetros de la Query String.
				qs = CTLinks.CURRICULUM_GET_PARAMS;
				// Guardamos la fecha y hora actual.
				request.setAttribute("now", DateUtils.now());
				// Indicamos que la operacin ha sido realizada con xito.
				request.setAttribute("operation_ok", "removeSubject");
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DACurriculum.class, ex, 
						CTStruts.CP_TRANSACTION_CURRICULUM_REMOVE_SUBJECT,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
}
