package edu.uoc.pfc.struts.actions;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.interfaces.IBQuestionBankManager;
import edu.uoc.pfc.business.transfers.TOQuestion;
import edu.uoc.pfc.business.transfers.TOQuestionCategory;
import edu.uoc.pfc.business.transfers.TOQuestionMatch;
import edu.uoc.pfc.business.transfers.TOQuestionMultipleChoices;
import edu.uoc.pfc.business.transfers.TOQuestionNumeric;
import edu.uoc.pfc.business.transfers.TOQuestionShortAnswer;
import edu.uoc.pfc.business.transfers.TOQuestionTrueOrFalse;
import edu.uoc.pfc.lib.constants.CTLinks;
import edu.uoc.pfc.lib.constants.CTStruts;
import edu.uoc.pfc.lib.utils.DateUtils;
import edu.uoc.pfc.lib.utils.ValidationUtils;
import edu.uoc.pfc.lib.utils.WebUtils;
import edu.uoc.pfc.struts.forms.AFQuestionBank;

/**
 * <p><code>DispatchAction</code> que ser&aacute; la encargada de gestionar todas las 
 * transacciones que tienen que ver con la gesti&oacute;n de los Bancos de Preguntas de 
 * los Profesores.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class DAQuestionBank extends DABaseDispatchAction
{
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que ser&acute; llamado por un Profesor cuando quiera crear una nueva
	 * Categor&iacute;a de Preguntas dentro de su Banco de Preguntas.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward createCategory(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local que recoger los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor y que el formulario "form" es
		// de tipo AFQuestionBank
		if ((isTeacher(request)) && (form instanceof AFQuestionBank))
		{
			try
			{
				// Cast del formulario.
				AFQuestionBank af = (AFQuestionBank) form;
				// Realizamos las validaciones simples sobre los datos.
				ActionErrors errors = af.validateCreateCategory();
				// Comprobamos si ha habido errores de validacin simples.
				if (errors.size() > 0)
				{
					// Tenemos errores de validacin simples. Los guardamos.
					addErrors(request, errors);
					// Establecemos los parmetros de la query string.
					qs = CTLinks.QUESTION_BANK_CATEGORY_ADD_PARAMS;
				}
				else
				{
					// Al no haber habido errores de validacin simples llamamos al mtodo
					// de la interfaz que nos permitir crear la nueva Categora. Aunque
					// este mtodo retorna un objeto TOQuestionCategory no lo vamos a 
					// guardar ya que no mostraremos informacin de l en la siguiente vista.
					// Creamos una instancia de IBQuestionBankManager.
					IBQuestionBankManager instance = createIBQuestionBankManager();
					// Llamamos al mtodo para registrar la categora.
					instance.createCategory(getUserId(request), af.getCategoryName(), 
							af.getCategoryDescription());
					// Indicamos la siguiente vista a mostrar al usuario.
					qs = CTLinks.QUESTION_BANK_CATEGORY_ADD_PARAMS;
					// Indicamos que la categora ha sido registrada con xito.
					request.setAttribute("operation_ok", true);
				}
			}
			catch (DuplicatedFieldException dfe)
			{
				// Guardamos el mensaje de error en la request.
				request.setAttribute("error_form", dfe.getMessage());
				// Indicamos la siguiente vista a mostrar al usuario.
				qs = CTLinks.QUESTION_BANK_CATEGORY_ADD_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAQuestionBank.class, ex, 
						CTStruts.CP_TRANSACTION_QUESTION_CATEGORY_LIST,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera crear una Pregunta
	 * dentro de una Categor&iacute;a de su Banco de Preguntas.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward createQuestion(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Definimos una variable local que contenga los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor, que el formulario
		// que ser recibe es de tipo "AFQuestionBank", que se recibe el parmetro categoryId
		// y que dicho parmetro tiene un valor entero.
		if ((isTeacher(request)) && (form instanceof AFQuestionBank) && 
				(ValidationUtils.isInteger(request.getParameter("categoryId"))))
		{
			// Cast del formulario.
			AFQuestionBank af = (AFQuestionBank) form;
			try
			{
				// Vemos el tipo de Pregunta que se quiere crear.
				if (TOQuestion.TYPE_TRUE_OR_FALSE.equals(request.getParameter("type")))
				{
					// Estamos creando una pregunta de tipo Verdadero o Falso.
					// Realizamos la validacin simple de los datos del formulario.
					ActionErrors errors = af.validateCreateQuestionTrueOrFalse();
					// Vemos si ha habido errores de validacin simples.
					if (errors.size() > 0)
					{
						// Guardamos los errores de validacin simples.
						addErrors(request, errors);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.QUESTION_CREATE_PARAMS + "&id=" + af.getCategoryId() + 
								"&name=" + af.getCategoryName() + "&type=TF";
					}
					else
					{
						// Declaramos una variable local para ver cual es la opcin correcta.
						boolean correct = true;
						// Comprobamos si se ha seleccionado la opcin "falso"
						if ("op_false".equals(af.getTfCorrectAnswer())) correct = false;
						// Creamos un nuevo objeto TOQuestionTrueOrFalse a partir de af.
						TOQuestionTrueOrFalse question = new TOQuestionTrueOrFalse(af.getTitle(), 
								af.getStatement(), correct, af.getTfTextTrue(), af.getTfTextFalse());
						// Creamos una instancia de IBQuestionBankManager
						IBQuestionBankManager instance = createIBQuestionBankManager();
						// Llamamos al mtodo que nos permite crear la pregunta y no guardamos
						// el objeto que retorna ya que no nos interesa.
						instance.createQuestion(getUserId(request), 
								Integer.parseInt(af.getCategoryId()), question);
						// Guardamos el identificador de la Categora.
						request.setAttribute("category_id", af.getCategoryId());
						// Indicamos que la operacin se ha realizado con xito.
						request.setAttribute("operation_ok", true);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.QUESTION_CREATE_PARAMS;
					}
				}
				else if (TOQuestion.TYPE_NUMERIC.equals(request.getParameter("type")))
				{
					// Estamos ante una pregunta de tipo Numrico.
					// Realizamos las validaciones simples de los datos del formulario.
					ActionErrors errors = af.validateCreateQuestionNumeric();
					// Comprobamos si ha habido errores.
					if (errors.size() > 0)
					{
						// Guardamos los errores de validacin simples.
						addErrors(request, errors);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.QUESTION_CREATE_PARAMS + "&id=" + af.getCategoryId() + 
								"&name=" + af.getCategoryName() + "&type=NU";
					}
					else
					{
						// Vamos bien. Tenemos que dar de alta la pregunta numrica.
						// Creamos una instancia de TOQuestionNumeric.
						TOQuestionNumeric question = new TOQuestionNumeric(af.getTitle(), 
								af.getStatement(), Double.parseDouble(af.getNuValue()), 
								Double.parseDouble(af.getNuError()), af.getNuText());
						// Creamos una instancia de IBQuestionBankManager
						IBQuestionBankManager instance = createIBQuestionBankManager();
						// Llamamos al mtodo que nos permite crear la pregunta y no guardamos
						// el objeto que retorna ya que no nos interesa.
						instance.createQuestion(getUserId(request), 
								Integer.parseInt(af.getCategoryId()), question);
						// Guardamos el identificador de la Categora.
						request.setAttribute("category_id", af.getCategoryId());
						// Indicamos que la operacin se ha realizado con xito.
						request.setAttribute("operation_ok", true);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.QUESTION_CREATE_PARAMS;
					}
				}
				else if (TOQuestion.TYPE_SHORT_ANSWER.equals(request.getParameter("type")))
				{
					// Estamos ante una pregunta de tipo "Respuesta Corta"
					// Realizamos las validaciones simples del formulario.
					ActionErrors errors = af.validateCreateQuestionShortAnswer();
					// Comprobamos si ha habido errores.
					if (errors.size() > 0)
					{
						// Guardamos los errores de validacin simples.
						addErrors(request, errors);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.QUESTION_CREATE_PARAMS + "&id=" + af.getCategoryId() + 
								"&name=" + af.getCategoryName() + "&type=SA";
					}
					else
					{
						// Vamos bien. Tenemos que dar de alta la pregunta de respuesta corta.
						// Definimos una variable local para ver si hay que distinguir las mayusculas.
						boolean caseSensitive = false;
						// Vemos si tenemos que distinguir o no.
						if ("op_true".equals(af.getSaCaseSensitive())) caseSensitive = true;
						// Creamos una instancia de TOQuestionShortAnswer.
						TOQuestionShortAnswer question = new TOQuestionShortAnswer(af.getTitle(), 
								af.getStatement(), af.getSaAnswer(), caseSensitive, af.getSaText());
						// Creamos una instancia de IBQuestionBankManager
						IBQuestionBankManager instance = createIBQuestionBankManager();
						// Llamamos al mtodo que nos permite crear la pregunta y no guardamos
						// el objeto que retorna ya que no nos interesa.
						instance.createQuestion(getUserId(request), 
								Integer.parseInt(af.getCategoryId()), question);
						// Guardamos el identificador de la Categora.
						request.setAttribute("category_id", af.getCategoryId());
						// Indicamos que la operacin se ha realizado con xito.
						request.setAttribute("operation_ok", true);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.QUESTION_CREATE_PARAMS;
					}
				}
				else if (TOQuestion.TYPE_MULTIPLE_CHOICES.equals(request.getParameter("type")))
				{
					// Estamos ante una pregunta de tipo "Mltiples Opciones".
					// Realizamos las validaciones simples del formulario.
					ActionErrors errors = af.validateCreateQuestionMultipleChoices(request);
					// Comprobamos si ha habido errores.
					if (errors.size() > 0)
					{
						// Guardamos los errores de validacin simples.
						addErrors(request, errors);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.QUESTION_CREATE_PARAMS + "&id=" + af.getCategoryId() + 
								"&name=" + af.getCategoryName() + "&type=MC";
					}
					else
					{
						// No hay errores de validacin simples. Creamos un objeto TOQuestionMultipleChoices
						TOQuestionMultipleChoices question = new TOQuestionMultipleChoices(af);
						// Creamos una instancia de IBQuestionBankManager
						IBQuestionBankManager instance = createIBQuestionBankManager();
						// Llamamos al mtodo que nos permite crear la pregunta y no guardamos
						// el objeto que retorna ya que no nos interesa.
						instance.createQuestion(getUserId(request), 
								Integer.parseInt(af.getCategoryId()), question);
						// Guardamos el identificador de la Categora.
						request.setAttribute("category_id", af.getCategoryId());
						// Indicamos que la operacin se ha realizado con xito.
						request.setAttribute("operation_ok", true);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.QUESTION_CREATE_PARAMS;
					}
				}
				else if (TOQuestion.TYPE_MATCH.equals(request.getParameter("type")))
				{
					// Estamos ante una pregunta de tipo "Emparejar".
					// Realizamos las validaciones simples del formulario.
					ActionErrors errors = af.validateCreateQuestionMatch(request);
					// Comprobamos si ha habido errores.
					if (errors.size() > 0)
					{
						// Guardamos los errores de validacin simples.
						addErrors(request, errors);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.QUESTION_CREATE_PARAMS + "&id=" + af.getCategoryId() + 
								"&name=" + af.getCategoryName() + "&type=MA";
					}
					else
					{
						// No hay errores de validacin simples. Creamos un objeto TOQuestionMatch
						TOQuestionMatch question = new TOQuestionMatch(af);
						// Creamos una instancia de IBQuestionBankManager
						IBQuestionBankManager instance = createIBQuestionBankManager();
						// Llamamos al mtodo que nos permite crear la pregunta y no guardamos
						// el objeto que retorna ya que no nos interesa.
						instance.createQuestion(getUserId(request), 
								Integer.parseInt(af.getCategoryId()), question);
						// Guardamos el identificador de la Categora.
						request.setAttribute("category_id", af.getCategoryId());
						// Indicamos que la operacin se ha realizado con xito.
						request.setAttribute("operation_ok", true);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.QUESTION_CREATE_PARAMS;
					}
				}
				// Si llegamosa  este punto querr decir que se ha pasado un tipo
				// de pregunta que no est contemplado, as que no hacemos nada.
			}
			catch (DuplicatedFieldException dfe)
			{
				// Guardamos el error en la request.
				request.setAttribute("error_form", dfe.getMessage());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.QUESTION_CREATE_PARAMS + "&id=" + af.getCategoryId() + 
						"&name=" + af.getCategoryName() + "&type=" + request.getParameter("type");
			}
			catch (Exception ex)
			{
				// Se ha producido una excepcin. Llamamos al mtodo que permite mostrar
				// un mensaje de error al usuario.
				return processException(DAQuestionBank.class, ex, 
						CTStruts.CP_TRANSACTION_QUESTION_CREATE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamdo por un Profesor cuando desee eliminar una de sus
	 * Categor&iacute;as de Preguntas dentro de su Bando de Preguntas.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward deleteCategory(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor, que ser recibe el parmetro
		// "id" y que adems este posee un valor numrico.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtemeos una instancia de IBQuestionBankManager
				IBQuestionBankManager instance = createIBQuestionBankManager();
				// Llamamos al mtodo que permite eliminar la categora.
				instance.deleteCategory(getUserId(request), Integer.parseInt(request.getParameter("id")));
				// Indicamos que la operacin se ha realizado con xito.
				request.setAttribute("operation_ok", "deleteCategory");
				// Obtenemos la lista de categoras de preguntas y la guardamos en la query string.
				request.setAttribute("category_list", instance.listCategories(getUserId(request)));
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.QUESTION_BANK_CATEGORY_LIST_PARAM;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAQuestionBank.class, ex, 
						CTStruts.CP_TRANSACTION_QUESTION_CATEGORY_DELETE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera eliminar una Pregunta
	 * de su Banco de Preguntas.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward deleteQuestion(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para guardar los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor, que se recibe el
		// parmetro "id", que dicho parmetro contiene un valor numrico, que se recibe 
		// el parmetro "cat" y que este tambin posee un valor numrico.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))) 
				&& (ValidationUtils.isInteger(request.getParameter("cat"))))
		{
			try
			{
				// Creamos una instancia de IBQuestionBankManager
				IBQuestionBankManager instance = createIBQuestionBankManager();
				// Llamamos al mtodo que nos permite borrar la pregunta.
				instance.deleteQuestion(getUserId(request), Integer.parseInt(request.getParameter("id")));
				// Guardamos en la request que la operacin se ha realizado con xito.
				request.setAttribute("operation_ok", "deleteQuestion");
				// Llamamos al mtodo para obtener la informacin y la guardamso en al request.
				request.setAttribute("category_data", instance.getCategory(getUserId(request), 
						Integer.parseInt(request.getParameter("cat"))));
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.QUESTION_BANK_CATEGORY_DATA_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido una excepcin. Llamamos al mtodo que permite mostrar
				// un mensaje de error al usuario.
				return processException(DAQuestionBank.class, ex, 
						CTStruts.CP_TRANSACTION_QUESTION_DELETE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}

	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera obtener
	 * la informaci&oacute;n de una Categor&iacute;a de Preguntas definida dentro de
	 * su Bando de Preguntas.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward getCategory(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la Query String.
		String qs = "";
		// Comprobamos si se trata de un Profesor, si se ha recibido el parmetro "id"
		// y adems que ste contenga un valor numrico.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Creamos una instancia de IBQuestionBankManager.
				IBQuestionBankManager instance = createIBQuestionBankManager();
				// Obtenemos la Categora.
				TOQuestionCategory category = instance.getCategory(getUserId(request), 
						Integer.parseInt(request.getParameter("id")));
				// Llamamos al mtodo para obtener la informacin y la guardamos en al request.
				request.setAttribute("category_data", category);
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.QUESTION_BANK_CATEGORY_DATA_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido una excepcin. Llamamos al mtodo que permite mostrar
				// un mensaje de error al usuario.
				return processException(DAQuestionBank.class, ex, 
						CTStruts.CP_TRANSACTION_QUESTION_CATEGORY_DATA,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera obtener la 
	 * informaci&oacute;n de una de las Preguntas que tiene registradas en su Banco de 
	 * Preguntas.</p>
	 *  
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward getQuestion(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la query string
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor, que se recibe
		// el parmetro "id" y que este contiene un valor numrico.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Creamos una instancia de IBQuestionBankManager
				IBQuestionBankManager instance = createIBQuestionBankManager();
				// Obtenemos los datos de la pregunta y los guardamos en la request.
				request.setAttribute("question_data", instance.getQuestion(getUserId(request), 
						Integer.parseInt(request.getParameter("id"))));
				// Guardamos la informacin de la fecha y hora de la operacin.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.QUESTION_DATA_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAQuestionBank.class, ex, 
						CTStruts.CP_TRANSACTION_QUESTION_DATA,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor para obtener la lista de 
	 * Categor&iacute;as de Preguntas dentro de su Banco de Preguntas.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward listCategory(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para almacenar los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al m&eacute;todo es un Profesor.
		if (isTeacher(request))
		{
			try
			{
				// Obtenemos una instancia de IBQuestionBankManager
				IBQuestionBankManager instance = createIBQuestionBankManager();
				// Obtenemos la lista de categoras de preguntas y la guardamos en la query string.
				request.setAttribute("category_list", instance.listCategories(getUserId(request)));
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.QUESTION_BANK_CATEGORY_LIST_PARAM;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAQuestionBank.class, ex, 
						CTStruts.CP_TRANSACTION_QUESTION_CATEGORY_LIST,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un Profesor cuando quiera modificar una Categor&iacute;a 
	 * de Preguntas dentro de su Banco de Preguntas.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward modifyCategory(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor, que se recibe el 
		// parmetro "id", que ste contiene un valor numrico y que "form" es de tipo
		// AFQuestionBank.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Cast al formulario.
				AFQuestionBank af = (AFQuestionBank) form;
				// Creamos una instancia de IBQuestionBankManager
				IBQuestionBankManager instance = createIBQuestionBankManager();
				// Comprobamos si hemos recibido el parmetro "action"
				if (ValidationUtils.isEmpty(request.getParameter("action")))
				{
					// No hemos recibido el parmetor "action". Tenemos que recuperar la
					// informacin de la categora de preguntas. Obtenemos la informacin.
					TOQuestionCategory category = instance.getCategory(getUserId(request), 
							Integer.parseInt(request.getParameter("id")));
					// Guardamos la informacin en la request.
					request.setAttribute("category_data", category);
					// Establecemos los valores del ActionForm.
					af.setCategoryId(Integer.toString(category.getId()));
					af.setCategoryName(category.getName());
					af.setCategoryDescription(WebUtils.HTMLtoString(category.getDescription()));
					// Establecemos los parmetros de la query string.
					qs = CTLinks.QUESTION_BANK_CATEGORY_MODIFY_PARAMS;
				}
				else
				{
					// Hemos recibido el parmetro "action". En este caso no se debe
					// hacer ninguna validacin simple ya que solamente se puede 
					// modificar la descripcin de la Categora. Llamamos
					// al mtodo de la IBQuestionsBankManager para realizar la moficacin.
					// aunque no guardamos el objeto que retorna ya que no lo vamos a 
					// mostrar al usuario.
					instance.modifyCategory(getUserId(request), Integer.parseInt(request.getParameter("id")), 
							af.getCategoryDescription());
					// Indicamos que la operacin ha sido realizada con xito.
					request.setAttribute("operation_ok", true);
					// Establecmos los parmetros de la query string.
					qs = CTLinks.QUESTION_BANK_CATEGORY_MODIFY_PARAMS;
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAQuestionBank.class, ex, 
						CTStruts.CP_TRANSACTION_QUESTION_CATEGORY_MODIFY,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera modificar la 
	 * informaci&aocute;n de una de las preguntas que tiene definidas en el Banco de Preguntas.</p>
	 * 
	 * @param mapping mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward modifyQuestion(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local para definir los parmetros de la query string.
		String qs = "";
		// Vemos si la persona que la llama al mtodo es un profesor, si se recibe
		// el parmetro "id" y adems que este contenga un valor numrico, que el
		// formulario es de tipo AFQuestionBank, que existe el parmetro "cat" y 
		// que dicho parmetro tiene un valor numrico.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id")))
				&& (form instanceof AFQuestionBank) 
				&& (ValidationUtils.isInteger(request.getParameter("cat"))))
		{
			try
			{
				// Obtenemos una instancia de IBQuestionBankManager
				IBQuestionBankManager instance = createIBQuestionBankManager();
				// Cast al formulario.
				AFQuestionBank af = (AFQuestionBank) form;
				// Comprobamos si hemos recibido el parmetro "action"
				if (ValidationUtils.isEmpty(request.getParameter("action")))
				{
					// Obtenemos la informacin de la Pregunta y la guardamos
					// en los atributos del formulario.
					af.setData(instance.getQuestion(getUserId(request), 
							Integer.parseInt(request.getParameter("id"))));
					// Obtenemos la informacin de la categora.
					request.setAttribute("category_data", 
							instance.getCategory(getUserId(request), 
							Integer.parseInt(request.getParameter("cat"))));
					// Establecemos los parmetros para la siguiente vista.
					qs = CTLinks.QUESTION_MODIFY_PARAMS;
				}
				else
				{
					// Realizamos las validaciones simples sobre af.
					ActionErrors errors = af.validateModify(request);
					// Vemos si ha habido errores de validacin simples.
					if (errors.size() > 0)
					{
						// Guardamos los errores de validacin simples.
						addErrors(request, errors);
						// Obtenemos la informacin de la categora.
						request.setAttribute("category_data", 
								instance.getCategory(getUserId(request), 
								Integer.parseInt(request.getParameter("cat"))));
					}
					else
					{
						// Sin errores de validacin. Realizamos la modificacin.
						modifyQuestion(af, request, instance);
						// Indicamos que la operacin se ha realizado correctamente.
						request.setAttribute("operation_ok", true);
						// Guardamos el identificador de la categora.
						request.setAttribute("category_id", request.getParameter("cat"));
					}
					// Establecemos los parmetros para la siguiente vista.
					qs = CTLinks.QUESTION_MODIFY_PARAMS;
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DAQuestionBank.class, ex, 
						CTStruts.CP_TRANSACTION_QUESTION_MODIFY,
						mapping, request);
			}
		}
		// Si llegamos a este punto es que ha habido un error en los parmetros (o no
		// se trata de un profesor, o no se ha recibido "id", etc. por lo que 
		// retornamos la siguiente vista al usuario "por defecto".
		return getView(qs, mapping);
	}	
		
		
	// -------- MTODOS PRIVADOS.	
	/**
	 * <p>Realiza la modificaci&oacute;n de los datos de una pregunta.</p>
	 *  
	 * @param af <code>AFQuestionBank</code> con los datos de la pregunta.
	 * @param request <code>HttpServletRequest</code> asociado a la petici&aocute;n
	 * HTTP del Profesor.
	 * @param instance <code>IBQuestionBankManager</code> para comunicarse con la
	 * capa de negocio.
	 * @throws Exception si se produce una excepcin a la hora de realizar la modificaci&oacute;n.
	 */
	private void modifyQuestion(AFQuestionBank af, HttpServletRequest request, 
			IBQuestionBankManager instance) throws Exception
	{
		// Comprobamos que los tres parmetros no sean nulos.
		if ((af != null) && (request != null) && (instance != null))
		{
			// Vemos el tipo de Pregunta del que se trata.
			if (TOQuestion.TYPE_TRUE_OR_FALSE.equals(af.getType()))
			{
				// Definimos una varibale local para declarar la respuesta correcta.
				boolean isTrue = true;
				// Comprobamos si la respuesta correcta es "falso"
				if ("op_false".equals(af.getTfCorrectAnswer())) isTrue = false;
				// Creamos un objeto TOQuestionTrueOrFalse a partir de af.
				TOQuestionTrueOrFalse to = new TOQuestionTrueOrFalse(af.getTitle(), 
						af.getStatement(), isTrue, af.getTfTextTrue(), af.getTfTextFalse());
				// Llamamos al mtodo para realizar la modificacin de los datos pero 
				// no guardamos el objeto que retorna ya que no lo vamos a mostrar.
				instance.modifyQuestion(getUserId(request), Integer.parseInt(af.getId()), to);
			}
			else if (TOQuestion.TYPE_NUMERIC.equals(af.getType()))
			{
				// Pregunta de tipo Numrico. Creamos un objeto TOQuestionNumeric a partir
				// de los datos contenidos en el formulario.
				TOQuestionNumeric to = new TOQuestionNumeric(af.getTitle(), af.getStatement(), 
						Double.parseDouble(af.getNuValue()), Double.parseDouble(af.getNuError()), af.getNuText());
				// Llamamos al mtodo para realizar la modificacin de los datos pero
				// no guardamos el objeto que retorna ya que no lo vamos a mostrar.
				instance.modifyQuestion(getUserId(request), Integer.parseInt(af.getId()), to);
			}
			else if (TOQuestion.TYPE_SHORT_ANSWER.equals(af.getType()))
			{
				// Pregunta de tipo Respuesta Corta. Variable local para la distinguir mayscula. 
				boolean caseSensitive = false;
				// Comprobamos si hay que distiguir maysculas y minscualas.
				if ("op_true".equals(af.getSaCaseSensitive())) caseSensitive = true;
				// Creamos un objeto TOQuestionShortAnswer a partir de los datos del formulario.
				TOQuestionShortAnswer to = new TOQuestionShortAnswer(af.getTitle(), af.getStatement(), 
						af.getSaAnswer(), caseSensitive, af.getSaText());
				// Llamamos al mtodo para realizar la modificacin de los datos pero
				// no guardamos el objeto que retorna ya que no lo vamos a mostrar.
				instance.modifyQuestion(getUserId(request), Integer.parseInt(af.getId()), to);
			}
			else if (TOQuestion.TYPE_MULTIPLE_CHOICES.equals(af.getType()))
			{
				// Creamos un objeto TOQuestionMultipleChoices a partir de los datos del formulario.
				TOQuestionMultipleChoices to = new TOQuestionMultipleChoices(af);
				// Llamamos al mtodo para realizar la modificacin de los datos pero 
				// no guardamos el objeto que retorna ya que no lo vamos a mostrar.
				instance.modifyQuestion(getUserId(request), Integer.parseInt(af.getId()), to);
			}
			else if (TOQuestion.TYPE_MATCH.equals(af.getType()))
			{
				// Creamos un objeto TOQuestionMatch a partir de los datos del formulario.
				TOQuestionMatch to = new TOQuestionMatch(af);
				// Llamamos al mtodo para realizar al modificacin de los datos pero
				// no guardamos el objeto que retorna ya que no lo vamos a mostrar.
				instance.modifyQuestion(getUserId(request), Integer.parseInt(af.getId()), to);
			}
		}
	}
	
}
