package edu.uoc.pfc.struts.actions;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.interfaces.IBSubjectManager;
import edu.uoc.pfc.business.transfers.TOSubject;
import edu.uoc.pfc.lib.constants.CTLinks;
import edu.uoc.pfc.lib.constants.CTStruts;
import edu.uoc.pfc.lib.utils.DateUtils;
import edu.uoc.pfc.lib.utils.ValidationUtils;
import edu.uoc.pfc.struts.forms.AFSubject;

/**
 * <b><code>DispatchAction</code> que ser&aacurte; la encargada de atender todas las 
 * peticiones de los usuarios que est&aacute;n asociadas a la gesti&oacute;n de las 
 * diferentes asignaturas que est&aacute;n registradas dentro de la aplicaci&oacute;n.</p>
 * 
 * <p>En concreto esta <code>DispacthAction</code> se encargar&aacute; de:</p>
 * <ul>
 * <li>Obtener una lista con todas las asignaturas que est&aacute;n registradas y
 * mostr&aacute;rsela al usuario.</p>
 * </ul>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class DASubject extends DABaseDispatchAction
{
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema cada vez que 
	 * quiera asignar un Plan de Estudios a la Asignatura.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward addCurriculum(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Definimos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un administrador, que se ha pasado
		// el parmetro "id" y que este contiene un valor entero.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Creamos una instancia de IBSubjectManager
				IBSubjectManager instance = createIBSubjectManager();
				// Comprobamos la existencia del parmetro "action" en la request.
				if (ValidationUtils.isEmpty(request.getParameter("action")))
				{
					// No se ha recibido el parmetro por lo que tendremos que obtener la
					// lista de Planes de Estudio a los cuales se le puede asignar la asignatura.
					request.setAttribute("curriculum_list", 
							instance.getUnassignedCurriculum(Integer.parseInt(request.getParameter("id"))));
					// Obtenemos la informacin de la Asignatura y la guardamos.
					request.setAttribute("subject_data", 
							instance.getSubject(Integer.parseInt(request.getParameter("id"))));
					// Asignamos los parmetros de la query string.
					qs = CTLinks.SUBJECT_ADD_CURRICULUM_PARAMS;
				}
				else if (form instanceof AFSubject)
				{
					// Se ha pasado el parmetro "action" y adems el formulario es de tipo AFSubject.
					// Cast del formulario.
					AFSubject af = (AFSubject) form;
					// Realizamos la validacin simple sobre el formulario.
					ActionErrors errors = af.validateAddCurriculum();
					// Comprobamos si ha habido errores.
					if (errors.size() > 0)
					{
						// Grabamos los errores.
						addErrors(request, errors);
						// Indicamos los parmetros de la query string.
						qs = CTLinks.SUBJECT_ADD_CURRICULUM_PARAMS;
						// de estudios en concreto. 
						// Obtenemos la informacin de la Asignatura y la guardamos
						request.setAttribute("subject_data", 
								instance.getSubject(Integer.parseInt(request.getParameter("id"))));
						// Obtenemos la lista de Planes de Estudio no asignadas.
						request.setAttribute("curriculum_list", 
								instance.getUnassignedCurriculum(Integer.parseInt(request.getParameter("id"))));
					}
					else
					{
						// Llamamos al mtodo que realiza la asignacin.
						instance.addCurriculum(Integer.parseInt(request.getParameter("id")), Integer.parseInt(af.getCurriculumId()), 
								af.getSubjectType(), af.getCode(), Double.parseDouble(af.getCredits()));
						// Guardamos el identificador de la Asignatura.
						request.setAttribute("subject_id", request.getParameter("id"));
						// Indicamos que la operacin se ha realizado co xito.
						request.setAttribute("operation_ok", true);
						// Indicamos los parmetros de la query string.
						qs = CTLinks.SUBJECT_ADD_CURRICULUM_PARAMS;
					}
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DACurriculum.class, ex, 
						CTStruts.CP_TRANSACTION_SUBJECT_ADD_CURRICULUM,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar la usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cada vez
	 * que quiera crear una nueva asignatura.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward createSubject(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Varibale local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que lo ha llamado un administrador y que el formulario es de tipo AFSubject.
		if ((isAdmin(request)) && (form instanceof AFSubject))
		{
			// Cast al formulario.
			AFSubject af = (AFSubject) form;
			// Realizamos las validaciones simples sobre los datos.
			ActionErrors errors = af.validateCreateSubject();
			// Comprobamos si ha habido errores.
			if (errors.size() > 0)
			{
				// Guardamos los errores.
				addErrors(request, errors);
			}
			else
			{
				// No ha habido ningn error de validacin simple.
				try
				{
					// Obtenemos una instancia de IBSubjectManager.
					IBSubjectManager instance = createIBSubjectManager();
					// Llamamos al mtodo para la creacin de la asignatura y
					// guardamos el resultado de su ejecucin en la request.
					request.setAttribute("subject", instance.createSubject(af.getName(),
							af.getDescription()));
					// Indicamos que la operacin ha sido realizada correctamente.
					request.setAttribute("operation_ok", true);
				}
				catch (DuplicatedFieldException dfe)
				{
					// Se ha introducido un nombre de asignatura duplicado. 
					// Lo guardamos en la request para que sea mostrado.
					request.setAttribute("error_form", dfe.getMessage());
				}
				catch (Exception ex)
				{
					// Se produce una excepcin durante el proceso de creacin de 
					// la asignatura. Procesamos la excepcin.
					return processException(DASubject.class, ex, 
							CTStruts.CP_TRANSACTION_SUBJECT_CREATE,
							mapping, request);
				}
			}
			// Establecemos los parmetros de la query string.
			qs = CTLinks.SUBJECT_CREATE_PARAMS;
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return (getView(qs, mapping));
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando
	 * quiera eliminar una asignatura, en concreto aquella cuyo identificador es igual
	 * al valor contenido en el par&aacute;metro <i>id</i>.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward deleteSubject(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local que contendr los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un administrador, que existe el
		// parmetro "id" y que dicho parmetro tiene un valor entero.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBSubjectManager.
				IBSubjectManager instance = createIBSubjectManager();
				// Llamamos al mtodo para eliminar la asignatura.
				instance.deleteSubject(Integer.parseInt(request.getParameter("id")));
				// Obtenemos la lista de asignaturas y la guardamos en la request.
				request.setAttribute("subject_list", instance.listSubject());
				// Definimos los parmetros de la query string.
				qs = CTLinks.SUBJECT_LIST_PARAMS;
				// Grabamos la fecha y hora.
				saveTimestamp(request);
				// Indicamos que la operacin se ha realizado correctamente.
				request.setAttribute("operation_ok", "deleteSubject");
			}
			catch (Exception ex)
			{
				// Se produce una excepcin durante el proceso de obtenicin de la
				// informacin de las asignaturas. Procesamos la excepcin.
				return processException(DASubject.class, ex, 
						CTStruts.CP_TRANSACTION_SUBJECT_DELETE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador del sistema para
	 * obtener la informaci&oacute;n de una determinada asignatura.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward getSubject(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local para guardar los parmetros de la query string.
		String qs = "";
		// Comprobamos que se trata de un administrador, que existe el parmetro "id"
		// y que contiene un valor entero.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			// Se dan ambas condiciones.
			try
			{
				// Obtenemos una instancia de IBSubjectManager
				IBSubjectManager instance = createIBSubjectManager();
				// Llamamos al mtodo que permite obtener la informacin de la asignatura
				// y guardamos el resultado dentro de la request.
				request.setAttribute("subject_data", 
						instance.getSubject(Integer.parseInt(request.getParameter("id"))));
				// Establecemos los parmetros de la query string.
				qs = CTLinks.SUBJECT_DATA_PARAMS;
				// Indicamos la fecha y hora en la que se ha obtenido la informacin.
				request.setAttribute("now", DateUtils.now());
			}
			catch (Exception ex)
			{
				// Se produce una excepcin durante el proceso de obtenicin de la
				// informacin de las asignaturas. Procesamos la excepcin.
				return processException(DASubject.class, ex, 
						CTStruts.CP_TRANSACTION_SUBJECT_DELETE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un administrador de la aplicaci&ocute;n
	 * cada vez que quiera obtener la lista de las asignaturas que est&aacute;n registradas
	 * dentro del sistema.</p> 
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward listSubject(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local para especificar los parmetros de la query string.
		String qs = "";
		// Vemos si la persona que ha llamado al mtodo es un administrador.
		if (isAdmin(request))
		{
			try
			{
				// Obtenemos una instancia de IBSubjectManager.
				IBSubjectManager instance = createIBSubjectManager();
				// Obtenemos la lista de asignaturas y la guardamos en la request.
				request.setAttribute("subject_list", instance.listSubject());
				// Definimos los parmetros de la query string.
				qs = CTLinks.SUBJECT_LIST_PARAMS;
				// Grabamos la fecha y hora.
				saveTimestamp(request);
			}
			catch (Exception ex)
			{
				// Se produce una excepcin durante el proceso de obtenicin de la
				// informacin de las asignaturas. Procesamos la excepcin.
				return processException(DASubject.class, ex, 
						CTStruts.CP_TRANSACTION_SUBJECT_LIST,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuarios.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador siempre que quiera modificar
	 * la informaci&oacute;n de Asignatura dentro del sistema.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward modifySubject(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Definimos una variable local para los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un administrador, que se recibe 
		// el parmetro "id", que este parmetros tiene un valor numrico y que "form"
		// es un objeto de tipo AFSubject.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("id"))) && 
				(form instanceof AFSubject))
		{
			try
			{
				// Obtenemos una instancia de IBSubjectManager.
				IBSubjectManager instance = createIBSubjectManager();
				// Cast al formulario.
				AFSubject af = (AFSubject) form;
				// Comprobamos la existencia o no del parmetro "action"
				if (ValidationUtils.isEmpty(request.getParameter("action")))
				{
					// Obtenemos los datos de la asignatura.
					TOSubject subject = instance.getSubject(Integer.parseInt(request.getParameter("id")));
					// Guardamos la asignatura en la request.
					request.setAttribute("subject_data", subject);
					// Establecemos la descripcin dentro del formulario.
					af.setDescription(subject.getDescriptionForm());
					// Establecemos los parmetros de la query string
					qs = CTLinks.SUBJECT_MODIFY_PARAMS;
				}
				else
				{
					// Llamamos al mtodo que modificar la informacin de la Asignatura.
					instance.modifySubject(Integer.parseInt(request.getParameter("id")), 
							af.getDescription());
					// Indicamos en la request que la operacin se ha realizado correctamente.
					request.setAttribute("operation_ok", true);
					// Establecemos los parmetros de la query string
					qs = CTLinks.SUBJECT_MODIFY_PARAMS;
				}
			}
			catch (Exception ex)
			{
				// Se produce una excepcin durante el proceso de obtenicin de la
				// informacin de las asignaturas. Procesamos la excepcin.
				return processException(DASubject.class, ex, 
						CTStruts.CP_TRANSACTION_SUBJECT_MODIFY,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un administrador del sistema cuando quiera desasignar 
	 * una Asignatura de un Plan de Estudios.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward removeCurriculum(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para almacenar los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un administrador, que se reciben los parmetros
		// "idSubject" e "idCurriculum" y que ambos contienen un valor entero.
		if ((isAdmin(request)) && (ValidationUtils.isInteger(request.getParameter("idSubject"))) && 
				(ValidationUtils.isInteger(request.getParameter("idCurriculum"))))
		{
			try
			{
				// Obtenemos una instance de IBSubjectManager.
				IBSubjectManager instance = createIBSubjectManager();
				// Llamamos al mtodo que permite desasignar la Asignatura del Plan de Estudios.
				instance.removeCurriculum(Integer.parseInt(request.getParameter("idSubject")), 
						Integer.parseInt(request.getParameter("idCurriculum")));
				// Llamamos al mtodo que permite obtener la informacin de la asignatura
				// y guardamos el resultado dentro de la request.
				request.setAttribute("subject_data", 
						instance.getSubject(Integer.parseInt(request.getParameter("idSubject"))));
				// Establecemos los parmetros de la query string.
				qs = CTLinks.SUBJECT_DATA_PARAMS;
				// Guardamos la fecha y hora actual.
				request.setAttribute("now", DateUtils.now());
				// Indicamos que la operacin se ha realizado con xito.
				request.setAttribute("operation_ok", "removeCurriculum");
			}
			catch (Exception ex)
			{
				// Procesamos la excepcin.
				return processException(DASubject.class, ex, 
						CTStruts.CP_TRANSACTION_SUBJET_REMOVE_CURRICULUM,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}	
	
}
