package edu.uoc.pfc.struts.actions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import edu.uoc.pfc.business.exceptions.DuplicatedFieldException;
import edu.uoc.pfc.business.interfaces.IBQuestionBankManager;
import edu.uoc.pfc.business.interfaces.IBTestManager;
import edu.uoc.pfc.business.transfers.TOQuestion;
import edu.uoc.pfc.business.transfers.TOQuestionMatch;
import edu.uoc.pfc.business.transfers.TOQuestionMultipleChoices;
import edu.uoc.pfc.business.transfers.TOQuestionMultipleChoicesAnswer;
import edu.uoc.pfc.business.transfers.TOQuestionNumeric;
import edu.uoc.pfc.business.transfers.TOQuestionShortAnswer;
import edu.uoc.pfc.business.transfers.TOQuestionTrueOrFalse;
import edu.uoc.pfc.business.transfers.TOTest;
import edu.uoc.pfc.business.transfers.TOTestQuestion;
import edu.uoc.pfc.lib.constants.CTLinks;
import edu.uoc.pfc.lib.constants.CTPresentation;
import edu.uoc.pfc.lib.constants.CTStruts;
import edu.uoc.pfc.lib.utils.DateUtils;
import edu.uoc.pfc.lib.utils.ValidationUtils;
import edu.uoc.pfc.lib.utils.WebUtils;
import edu.uoc.pfc.struts.forms.AFTest;


/**
 * <p><code>DispatchAction</code> que ser&aacute; utilizada para realizar el intercambio
 * de informaci&oacute;n entre el sistema y un usuario en todas las transacciones que 
 * est&aacute;n relacionadas con la gesti&oacute;n de los Cuestionarios.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class DATest extends DABaseDispatchAction 
{
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera asignar una o m&aacute;s
	 * preguntas a uno de los test que ha creado.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward addQuestion(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local para establecer los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien llama al mtodo es un Profesor, que se recibe el parmetro
		// "id", que este contiene un valor numrico y que el formulario es de tipo AFTest
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))) &&
				(form instanceof AFTest))
		{
			try
			{
				// Comprobamos la existencia del parmetro "action" en la request.
				if (ValidationUtils.isEmpty(request.getParameter("action")))
				{
					// Tenemos que obtener la lista de Categoras de Preguntas del Profesor.
					// Obtenemos una instancia de IBQuestionBankManager.
					IBQuestionBankManager instance = createIBQuestionBankManager();
					// Obtenemos la lista de Categoras de Preguntas del usuario.
					request.setAttribute("category_list", instance.listCategories(getUserId(request)));
					// Guardamos el identificador del Cuestionario.
					request.setAttribute("test_id", request.getParameter("id"));
					// Especificamos los parmetros de la query string.
					qs = CTLinks.TEST_ADD_QUESTION_PARAMS;
					// Indicamos en que parte del proceso de aadir una pregunta estamos.
					request.setAttribute("add_question_step", "choose_category");
				}
				else if (("getQuestions".equals(request.getParameter("action"))) && (
						ValidationUtils.isInteger(request.getParameter("category_id"))))
				{
					// Se ha eligido una Categora as que tenemos que mostrar todas las Preguntas.
					// Obtenemos una instancia de IBQuestionBankManager.
					IBQuestionBankManager instance = createIBQuestionBankManager();
					// Obtenemos la informacin completa de la categora.
					request.setAttribute("category_data", instance.getCategory(getUserId(request), 
							Integer.parseInt(request.getParameter("category_id"))));
					// Guardamos el identificador del Cuestionario.
					request.setAttribute("test_id", request.getParameter("id"));
					// Especificamos los parmetros de la query string.
					qs = CTLinks.TEST_ADD_QUESTION_PARAMS;
					// Indicamos en que parte del proceso de aadir una pregunta estamos.
					request.setAttribute("add_question_step", "choose_question");
				}
				else if (("assign_value".equals(request.getParameter("action"))) &&
						 (ValidationUtils.isInteger(request.getParameter("question_id"))) &&
						 (ValidationUtils.isInteger(request.getParameter("category_id"))))
				{
					// Obtenemos una intancia de IBQuestionBankManager
					IBQuestionBankManager instance = createIBQuestionBankManager();
					// Obtenemos la informacin de la categora y lo guardamos en la request.
					request.setAttribute("category_data", instance.getCategory(getUserId(request), 
							Integer.parseInt(request.getParameter("category_id"))));
					// Obtenemos la informacin de la pregunta y la guardamos en la request.
					request.setAttribute("question_data", instance.getQuestion(getUserId(request), 
							Integer.parseInt(request.getParameter("question_id"))));
					// Guardamos el identificador del cuestionario.
					request.setAttribute("test_id", request.getParameter("id"));
					// Especificamos los parmetros de la query string.
					qs = CTLinks.TEST_ADD_QUESTION_PARAMS;
					// Indicamos en que parate del proceso de aadir una pregunta estamos.
					request.setAttribute("add_question_step", "set_value");
				}
				else if (("do_add".equals(request.getParameter("action"))) &&
						 (ValidationUtils.isInteger(request.getParameter("question_id"))) &&
						 (ValidationUtils.isInteger(request.getParameter("category_id"))))
				{
					// Tenemos que realizar la validacin simple sobre los datos del formulario.
					// Cast del formulario.
					AFTest af = (AFTest) form;
					// Obtenemos los errores de validacin simples.
					ActionErrors errors = af.validateAddQuestion();
					// Comprobamos si ha habido errores o no.
					if (errors.size() > 0)
					{
						// Tenemos errores de validacin simples. Los guardamos.
						addErrors(request, errors);
						// Obtenemos una intancia de IBQuestionBankManager
						IBQuestionBankManager instance = createIBQuestionBankManager();
						// Obtenemos la informacin de la categora y lo guardamos en la request.
						request.setAttribute("category_data", instance.getCategory(getUserId(request), 
								Integer.parseInt(request.getParameter("category_id"))));
						// Obtenemos la informacin de la pregunta y la guardamos en la request.
						request.setAttribute("question_data", instance.getQuestion(getUserId(request), 
								Integer.parseInt(request.getParameter("question_id"))));
						// Guardamos el identificador del cuestionario.
						request.setAttribute("test_id", request.getParameter("id"));
						// Especificamos los parmetros de la query string.
						qs = CTLinks.TEST_ADD_QUESTION_PARAMS;
						// Indicamos en que parate del proceso de aadir una pregunta estamos.
						request.setAttribute("add_question_step", "set_value");
					}
					else 
					{
						// No hay errores de validacin simples. Creamos una instancia de IBTestManager.
						IBTestManager instance = createIBTestManager();
						// Llamamos al mtodo que permite realizar la asignacin de la pregunta al Test.
						instance.addQuestion(getUserId(request), Integer.parseInt(request.getParameter("id")), 
								Integer.parseInt(request.getParameter("question_id")), 
								Double.parseDouble(af.getQuestionValue()));
						// Indicamos en la request que la operacin de asignacin ha sido correcta.
						request.setAttribute("operation_ok", true);
						// Especificamos los parmetros de la query string.
						qs = CTLinks.TEST_ADD_QUESTION_PARAMS;
						// Guardamos el identificador del Test.
						request.setAttribute("test_id", request.getParameter("id"));
					}
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DATest.class, ex, 
						CTStruts.CP_TRANSACTION_ADD_QUESTION,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera crear un 
	 * nuevo Cuestionario dentro del sistema.</p> 
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward createTest(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Creamos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor y adems que el 
		// formulario "form" es de tipo AFTest.
		if ((isTeacher(request)) && (form instanceof AFTest))
		{
			// Cast del cuestionario.
			AFTest af = (AFTest) form;
			// Realizamos la validacin simple de los datos del formulario.
			ActionErrors errors = af.validateCreateTest();
			// Comprobamos si ha habido errores de validacin simples.
			if (errors.size() > 0)
			{
				// Tenemos errores de validacin simples. Los grabamos.
				addErrors(request, errors);
				// Indicamos los parmetros de la query string para la siguiente vista.
				qs = CTLinks.TEST_CREATE_PARAMS;
			}
			else
			{
				try
				{
					// Creamos una instancia de IBTestManager.
					IBTestManager instance = createIBTestManager();
					// Comprobamos si hay que barajar las preguntas.
					boolean shuffleQuestions = false;
					if ("op_true".equals(af.getShuffleQuestions())) shuffleQuestions = true;
					// Comprobamos si hay que barajar las respuestas.
					boolean shuffleAnswers = false;
					if ("op_true".equals(af.getShuffleAnswers())) shuffleAnswers = true;
					// Comprobamos si hay que mostrar la solucin.
					boolean showSolution = false;
					if ("op_true".equals(af.getShowResult())) showSolution = true;
					// Comprobamos si hay que mostrar la revisin.
					boolean showReview = false;
					if ("op_true".equals(af.getShowReview())) showReview = true;
					// Llamamos al mtodo de la interfaz que permite crear el Cuestionario aunque
					// no guardaremos la informacin que retorna ya que no se la vamos a mostrar 
					// al usuario.
					instance.createTest(getUserId(request), af.getTitle(), af.getIntroduction(), 
							shuffleQuestions, shuffleAnswers, showSolution, showReview);
					// Indicamos que la operacin ha sido realizada con xito.
					request.setAttribute("operation_ok", true);
					// Indicamos la siguiente vista a mostrar al usuario.
					qs = CTLinks.TEST_CREATE_PARAMS;
				}
				catch (DuplicatedFieldException dfe)
				{
					// Guardamos en la request el mensaje de error.
					request.setAttribute("error_form", dfe.getMessage());
					// Indicamos la siguiente vista a mostrar al usuario.
					qs = CTLinks.TEST_CREATE_PARAMS;
				}
				catch (Exception ex)
				{
					// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
					// mtodo que permite mostrar un mensaje con la informacin del error.
					return processException(DATest.class, ex, 
							CTStruts.CP_TRANSACTION_TEST_CREATE,
							mapping, request);
				}
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera obtener la informaci&oacute;n 
	 * de las Estad&iacute;sticas relacionadas con las respuestas que los Alumnos han dado a sus Cuestionarios.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward listTestStats(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Definimos una variable local para los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (isTeacher(request))
		{
			try
			{
				// Obtenemos una instancia de IBTestManager
				IBTestManager instance = createIBTestManager();
				// Llamamos al mtodo que permite obtener las Estadsticas de todos
				// los Test del Profesor y guardamos el resultado en la request.
				request.setAttribute("test_list", instance.listTestStats(getUserId(request)));
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.TEST_STATS_LIST_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DATest.class, ex, 
						CTStruts.CP_TRANSACTION_TEST_STATS_LIST,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera obtener la informaci&oacute;n
	 * de uno de los Cuestionarios que tiene creados dentro del sistema.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward getTest(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una varibale local para almacenar los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor, que se recibe el 
		// parmetro "id" y que dicho parmetro contiene un valor numrico.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Creamos una instancia de IBTestManager.
				IBTestManager instance = createIBTestManager();
				// Obtenemos la informacin del Cuestionario y la guardamos en la request.
				request.setAttribute("test_data", instance.getTest(getUserId(request), 
						Integer.parseInt(request.getParameter("id"))));
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Indicamos la siguiente vista que se ha de mostrar al usuario.
				qs = CTLinks.TEST_DATA_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DATest.class, ex, 
						CTStruts.CP_TRANSACTION_TEST_DATA,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera obtener 
	 * las Estad&iacute;sticas detalladas de uno de sus Cuestionarios.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward getTestStats(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor, que se recibe el
		// parmetro "id" y que este posee un valor numrico.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBTestManager.
				IBTestManager instance = createIBTestManager();
				// Llamamos al mtodo que permite obtener las estadsticas guardando el
				// resultado en un atributo de la request.
				request.setAttribute("stats_data", instance.getTestStats(getUserId(request), 
						Integer.parseInt(request.getParameter("id"))));
				// Guardamos la informacin de la fecha y la hora.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.TEST_STATS_DATA_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DATest.class, ex, 
						CTStruts.CP_TRANSACTION_TEST_STATS_DATA,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor dentro del sistema cuando desee
	 * eliminar uno de los Cuestionarios que tiene creados.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward deleteTest(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor, que se recibe el parmetro
		// "id" y que este contiene un valor numrico.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))))
		{
			try
			{
				// Obtenemos una instancia de IBTestManager.
				IBTestManager instance = createIBTestManager();
				// Llamamos al mtodo que nos permite eliminar el Cuestionario.
				instance.deleteTest(getUserId(request), Integer.parseInt(request.getParameter("id")));
				// Indicamos que se ha eliminado con xito el Cuestionario.
				request.setAttribute("operation_ok", "deleteTest");
				// Llamamos al mtodo de interfaz para obtener la lista de cuestionarios
				// y los guardamos en la request.
				request.setAttribute("test_list", instance.listTest(getUserId(request)));
				// Establecemos los parmetros de la query string.
				qs = CTLinks.TEST_LIST_PARAMS;
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DATest.class, ex, 
						CTStruts.CP_TRANSACTION_TEST_LIST,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera obtener 
	 * la informaci&oacute;n de todos los Cuestionarios que ha elaborado dentro del 
	 * sistema.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward listTest(ActionMapping mapping, ActionForm form, 
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local para recoger los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor.
		if (isTeacher(request))
		{
			try
			{
				// Obtenemos una instancia de IBTestManager.
				IBTestManager instance = createIBTestManager();
				// Llamamos al mtodo de interfaz para obtener la lista de cuestionarios
				// y los guardamos en la request.
				request.setAttribute("test_list", instance.listTest(getUserId(request)));
				// Establecemos los parmetros de la query string.
				qs = CTLinks.TEST_LIST_PARAMS;
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DATest.class, ex, 
						CTStruts.CP_TRANSACTION_TEST_LIST,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}

	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando desee modificar la informaci&oacute;n
	 * de uno de sus Cuestionarios de Preguntas.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward modifyTest(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien llama al mtodo es un profesor, que se recibe el parmetro "id",
		// que dicho parmetro contiene un valor numrico y que el parmetro "form" es de 
		// tipo AFTest.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("id"))) && 
				(form instanceof AFTest))
		{
			try
			{
				// Creamos una instancia de IBTestManager.
				IBTestManager instance = createIBTestManager();
				// Cast del formulario.
				AFTest af = (AFTest) form;
				// Comprobamos la existencia el parmetro "action".
				if (ValidationUtils.isEmpty(request.getParameter("action")))
				{
					// No tenemos el parmetro "action". Tenemos que obtener los datos y establecer los
					// los campos del formulario a partir del objeto TOTest.
					af.setData(instance.getTest(getUserId(request), 
							Integer.parseInt(request.getParameter("id"))));
					// Indicamos los parmetros de la query string.
					qs = CTLinks.TEST_MODIFY_PARAMS;
				}
				else
				{
					// Recibimos el parmetro "action" as que tenemos que modificar el Cuestionario.
					// Realizamos la validacin simple de los datos del formulario.
					ActionErrors errors = af.validateModifyTest();
					// Comprobamos si ha habido errores.
					if (errors.size() > 0)
					{
						// Guardamos los errores de validacin simples.
						addErrors(request, errors);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.TEST_MODIFY_PARAMS;
					}
					else
					{
						// Llamamos al mtodo que permite realizar la modificacin de los datos
						// pero no almacenamos el objeto que retorna ya que no se lo mostraremos
						// al usuario. Primero establacemos las variables booleanas.
						// Barajar preguntas.
						boolean shuffleQuestions = true;
						if ("op_false".equals(af.getShuffleQuestions())) shuffleQuestions = false;
						// Barajar respuestas.
						boolean shuffleAnswers = true;
						if ("op_false".equals(af.getShuffleAnswers())) shuffleAnswers = false;
						// Mostrar el resultado.
						boolean showResult = true;
						if ("op_false".equals(af.getShowResult())) showResult = false;
						// Mostrar la revisin.
						boolean showReview = true;
						if ("op_false".equals(af.getShowReview())) showReview = false;
						// Llamamos al mtodo de la interfaz de negocio.
						instance.modifyTests(getUserId(request), Integer.parseInt(af.getId()), af.getIntroduction(), 
								shuffleQuestions, shuffleAnswers, showResult, showReview);
						// Indicamos que la operacin de modificacin ha finalizado con xito.
						request.setAttribute("operation_ok", true);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.TEST_MODIFY_PARAMS;
					}
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DATest.class, ex, 
						CTStruts.CP_TRANSACTION_TEST_MODIFY,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Profesor cuando quiera desasignar una pregunta
	 * de uno de sus Cuestionarios.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward removeQuestion(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Profesor, que se recibe el parmetro
		// "idTest" y que contiene un valor numrico, que se recibe el parmetro "idQuestion" y 
		// que tambin contiene un valor numrico.
		if ((isTeacher(request)) && (ValidationUtils.isInteger(request.getParameter("idTest"))) && 
				(ValidationUtils.isInteger(request.getParameter("idQuestion"))))
		{
			try
			{
				// Obtenemos un instancia de IBTestManager.
				IBTestManager instance = createIBTestManager();
				// Llamamos al mtodo de la interfaz realiza la eliminacin de la pregunta del test.
				instance.removeQuestion(getUserId(request), Integer.parseInt(request.getParameter("idTest")), 
						Integer.parseInt(request.getParameter("idQuestion")));
				// Indicamos que la operacin ha sido realizada con xito.
				request.setAttribute("operation_ok", "removeQuestion");
				// Obtenemos la informacin del Cuestionario y la guardamos en la request.
				request.setAttribute("test_data", instance.getTest(getUserId(request), 
						Integer.parseInt(request.getParameter("idTest"))));
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
				// Indicamos la siguiente vista que se ha de mostrar al usuario.
				qs = CTLinks.TEST_DATA_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DATest.class, ex, 
						CTStruts.CP_TRANSACTION_REMOVE_QUESTION,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado por un Alumno para realizar la revisi&oacute;n de uno 
	 * de los Cuestionarios que haya respondido previamente.</p> 
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward reviewTest(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Declaramos una variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo sea un Estudiante, que se reciben los 
		// parmetros "idTest" e "idClassroom" y que ambos poseen sendos valores numricos.
		if ((isStudent(request)) && (ValidationUtils.isInteger(request.getParameter("idTest")))
				&& (ValidationUtils.isInteger(request.getParameter("idClassroom"))))
		{
			try
			{
				// Obtenemos una instancia de IBTestManager.
				IBTestManager instance = createIBTestManager();
				// Obtenemos una instancia de IBQuestionBankManager
				IBQuestionBankManager instance2 = createIBQuestionBankManager();
				// Obtenemos la informacin del Test.
				TOTest toTest = instance.getTestClassroom(Integer.parseInt(request.getParameter("idTest")), 
						Integer.parseInt(request.getParameter("idClassroom")));
				// Creamos una lista de Objeto TOQuestion donde guardaremos la informacin de cada una 
				// de las preguntas del test.
				List<TOQuestion> list_questions = new ArrayList<TOQuestion>();
				// Recorremos todas las preguntas que estn asignadas al Test.
				for (TOTestQuestion toTestQuestion : toTest.getQuestions())
				{
					// Obtenemos la informacin de completa de la pregunta del test.
					TOQuestion toQuestion = instance2.getQuestion(toTestQuestion.getQuestion().getId());
					// Guardamos la pregunta en la lista de preguntas.
					list_questions.add(toQuestion);
				}
				// Guardamos la informacin del Test.
				request.setAttribute("test_data", toTest);
				// Guardamos la infomracin de las Preguntas del Test.
				request.setAttribute("question_list", list_questions);
				// Guardamos la informacin de la fecha y hora actual.
				request.setAttribute("now", DateUtils.now());
				// Establecemos los parmetros de la query string.
				qs = CTLinks.TEST_REVIEW_PARAMS;
			}
			catch (Exception ex)
			{
				// Se ha producido una excepcin durante la ejecucin del mtodo as que llamamos
				// al mtodo que nos permitir procesarla.
				return processException(DATest.class, ex, 
						CTStruts.CP_TRANSACTION_TEST_REVIEW,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	/**
	 * <p>M&eeacute;todo que ser&aacute; llamado por un Estudiante cuando quiera resolver uno
	 * de los Cuestionarios que tiene asignados en un Aula.</p>
	 * 
	 * @param mapping <code>ActionMapping</code> con los mapeos de Struts.
	 * @param form <code>ActionForm</code> asociado a esta <code>DispatchAction</code>
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param response <code>HttpServletResponse</code> asociada a la petici&oacute;n del usuario.
	 * @return <code>ActionForward</code> con la siguiente vista que se le deber&aacute; mostrar
	 * al usuario.
	 */
	public ActionForward solveTest(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
	{
		// Variable local para definir los parmetros de la query string.
		String qs = "";
		// Comprobamos que quien ha llamado al mtodo es un Estudiante, que se recibe el
		// parmetro "idTest" y que este parmetro tiene un valor numrico, y que el formulario
		// es de tipo AFTest.
		if ((isStudent(request)) && (ValidationUtils.isInteger(request.getParameter("idTest"))) && 
				(form instanceof AFTest))
		{
			try
			{
				// Comprobamos la existencia o no del parmetro "init" en la request y que adems se
				// recibe el parmetro "idClassroom" teniendo este un valor entero.
				if ((!ValidationUtils.isEmpty(request.getParameter("init"))) && 
						(ValidationUtils.isInteger(request.getParameter("idClassroom"))))
				{
					// En primer lugar borramos cualquier otro Test dentro de la HttpSession
					// que se pueda estar resolviendo.
					request.getSession().removeAttribute(CTPresentation.TEST);
					// Creamos una instancia de IBTestManager.
					IBTestManager instance = createIBTestManager();
					// Llamamos al mtodo que permite obtener la informacin del Test
					TOTest toTest = instance.getTestClassroom(Integer.parseInt(request.getParameter("idTest")), 
							Integer.parseInt(request.getParameter("idClassroom")));
					// Comprobamos si tenemos que barajar las preguntas.
					if (toTest.isShuffleQuestions())
					{
						// Tenemos que barajar las preguntas del Test. Obtenemos la lista de ellas.
						List<TOTestQuestion> list_questions = toTest.getQuestions();
						// Barajamos las preguntas.
						Collections.shuffle(list_questions);
						// Asignamos la lista de preguntas.
						toTest.setQuestions(list_questions);
					}
					// Guardamos el test ya configurado en la sesin.
					request.getSession().setAttribute(CTPresentation.TEST, toTest);
					// Guardamos la informacin de la fecha y hora.
					request.setAttribute("now", DateUtils.now());
					// Establecemos los parmetros de la query string.
					qs = CTLinks.TEST_SOLVE_PARAMS;
				}
				else if (!ValidationUtils.isEmpty(request.getParameter("processQuestion"))) 
				{
					// En este caso tenemos que evaluar la respuesta dada por el usuario. Cast del formulario.
					AFTest af = (AFTest) form;
					// Definimos una referencia local que contendr los errores de validacin.
					ActionErrors errors = new ActionErrors();
					// Las validaciones simples sobre los datos depender del tipo de pregunta as que 
					// examinamos el parmetro "type" de la request y en funcin de este llamamos a uno
					// u otro mtodo del AFTest para realizar las validaciones simples.
					if (TOQuestion.TYPE_MATCH.equals(request.getParameter("type")))
						errors = af.validateAnswerMA(request);	// Pregunta de tipo Emparejar.
					else if (TOQuestion.TYPE_MULTIPLE_CHOICES.equals(request.getParameter("type")))
						errors = af.validateAnswerMC(request);	// Pregunta de tipo Mltiples Opciones.
					else if (TOQuestion.TYPE_NUMERIC.equals(request.getParameter("type")))
						errors = af.validateAnswerNU();			// Pregunta de tipo Numrico.
					else if (TOQuestion.TYPE_SHORT_ANSWER.equals(request.getParameter("type")))
						errors = af.validateAnswerSA();			// Pregunta de tipo Respuesta Corta.
					else if (TOQuestion.TYPE_TRUE_OR_FALSE.equals(request.getParameter("type")))
						// En este caso no hay que realizar ninguna validacin ya que por defecto
						// estar seleccionada la opcin "true" as que escribimos una instruccin vaca.
						;
					else
					{
						// No es de ninguno de los tipos de preguntas permitidos por lo que lanzamos una
						// Excepcin indicndolo.
						throw (new Exception("Tipo de Pregunta invalido."));
					}
					// Llegados a este punto tenemos que comprobar si tenemos errores de validacin simples
					if (errors.size() > 0)
					{
						// Guardamos los errores.
						addErrors(request, errors);
						// Obtenemos de nuevo la informacin de la pregunta.
						getQuestion(request, false);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.TEST_SOLVE_PARAMS;
					}
					else
					{
						// No ha habido errores de validacin simples as que tenemos que procesar la respuesta.
						processAnswer(request);
						// Una vez actualizada la respuesta obtenemos la siguiente pregunta del test.
						getQuestion(request, true);
						// Establecemos los parmetros de la query string.
						qs = CTLinks.TEST_SOLVE_PARAMS;
					}
				}
				else
				{
					// En este caso solamente se debera ir en el caso de que tengamos que mostrar la
					// primera pregunta del Test al Alumno.
					getQuestion(request, true);
					// Guardamos la informacin de la fecha y la hora.
					request.setAttribute("now", DateUtils.now());
					// Establecemos los parmetros de la query string.
					qs = CTLinks.TEST_SOLVE_PARAMS;
				}
			}
			catch (Exception ex)
			{
				// Se ha producido cualquier otro tipo de excepcin. Llamamos al 
				// mtodo que permite mostrar un mensaje con la informacin del error.
				return processException(DATest.class, ex, 
						CTStruts.CP_TRANSACTION_TEST_SOLVE,
						mapping, request);
			}
		}
		// Retornamos la siguiente vista a mostrar al usuario.
		return getView(qs, mapping);
	}
	
	// -------- MTODOS PRIVADOS.
	/**
	 * <p>M&eacute;todo que ser&aacute; el encargado de determinar la siguiente Pregunta que ser&aacute; 
	 * mostrada al Estudiante durante la resoluci&oacute;n de un Cuestionario.</p>
	 * 
	 * @param request <code>HttpServletRequest</code> asociada a la petici&oacute;n del usuario.
	 * @param next <code>boolean</code> <code>true</code> si queremos que se avance una pregunta o <code>false</code>
	 * para que se vuelva a procesar la misma pregunta (siempre referenci&aacute;ndonos a la posici&oacute;n que 
	 * apunta el valor almacenado en la <i>request</i> en el atributo "questionNumber").
	 * @throws Exception si se produce alg&uacute;n tipo de error a la hora de obtener la siguiente pregunta
	 * del Cuestionario.
	 */
	private void getQuestion(HttpServletRequest request, boolean next) throws Exception
	{
		if ((request.getSession().getAttribute(CTPresentation.TEST) != null) &&
				(request.getSession().getAttribute(CTPresentation.TEST) instanceof TOTest) &&
				(ValidationUtils.isInteger(request.getParameter("questionNumber"))))
		{
			// Acabamos de comprobar que en la HttpSession tenemos el Test que est resolviendo
			// el Estudiante, que recibimos el parmetros "questionNumber" y que este posee
			// un valor entero. Recuperamos de la HttpSession el Test.
			TOTest toTest = (TOTest) request.getSession().getAttribute(CTPresentation.TEST);
			// Variable local que contendr el valor de "questionNumber".
			int position = Integer.parseInt(request.getParameter("questionNumber"));
			// Comprobamos si tenemos que avanzar o no (recordemos que "questionNumber" siempre nos indica
			// el valor de la siguiente pregunta a procesar por lo que tenemos que restar 1 a "position" si
			// el parmetro "next" es false.
			if (!next) --position;
			// Comprobamos que "toTest" contiene la informacin de las preguntas y que el 
			// valor de "questionNumber" no es mayor que el nmero de ellas.
			if ((toTest.getQuestions() != null) && (toTest.getQuestions().size() > position))
			{	
				// Extraemos la informacin de la siguiente pregunta del Test.
				TOQuestion nextQuestion = toTest.getQuestions().get(position).getQuestion();
				// Obtenemos una instancia de IBQuestionBankManager
				IBQuestionBankManager instance = createIBQuestionBankManager();
				// Obtenemos la informacin de la pregunta.
				TOQuestion requestQuestion = instance.getQuestion(nextQuestion.getId());
				// Vamos a comprobar si se trata de una pregunta de tipo "Multiples Opciones" y adems
				// en el test que estamos resolviendo se tienen que barajar las diferentes respuestas.
				if ((toTest.isShuffleAnswers()) && (requestQuestion instanceof TOQuestionMultipleChoices))
				{
					// Cast de requestQuestion.
					TOQuestionMultipleChoices toMC = (TOQuestionMultipleChoices) requestQuestion;
					// Obtenemos la lista de respuestas.
					List<TOQuestionMultipleChoicesAnswer> list_answers = toMC.getAnswers();
					// Barajamos las opciones de respuesta con Collections.shuffle
					Collections.shuffle(list_answers);
					// Una vez barajadas se las volvemos a asignar a la pregunta.
					toMC.setAnswers(list_answers);
				}
				// Guardamos la informacin de la pregunta en la request la guardamos en la request.
				request.setAttribute("next_question", requestQuestion);
				// Guardamos el valor de la pregunta para ser mostrado.
				request.setAttribute("next_question_value", toTest.getQuestions().get(position).getValue());
				// Guardamos en la request el valor de la siguiente pregunta que se deber consultar.
				request.setAttribute("next_question_number", position + 1);
			}
			else
			{
				// El nmero de pregunta que se ha pasado en la request es mayor que el nmero de preguntas
				// que tenemos en el formulario, as que en la request guardamos un atributo indicando que
				// la resolucin del Test ha finalizado.
				request.setAttribute("end_test", true);
				// Declaramos cuatro variables locales que nos servirn para mostrar los resultados finales
				// de la respuesta al Cuestionario: nmero de preguntas, nmero de preguntas acertadas, nota
				// obtenida y nota mxima que se poda obtener en el Cuestionario.
				int questionTotal = 0;
				int questionOK = 0;
				double score = 0.0;
				double maxScore = 0.0;
				// Recorremos las respuestas efectuadas por al alumno para ir obteniendo la informacin.
				for (TOTestQuestion testQuestion : toTest.getQuestions())
				{
					// Obtenemos una referencia a la pregunta que es donde se ha guardado la informacin.
					TOQuestion question = testQuestion.getQuestion();
					// Vemos si el Estudiante la ha contestado correctamente.
					if (question.isCorrect())
					{
						// Incrementamos en una unidad el nmero de respuestas correctas.
						questionOK++;
						// Incrementamos la puntuacin obtenida por el Estudiante.
						score += testQuestion.getValue();
					}
					// Incrementamos en una unidad el nmero de preguntas del Cuestionario.
					questionTotal++;
					// Incrementamos la puntuacin mxima que se puede obtener en el Test.
					maxScore += testQuestion.getValue();
				}
				// Obtenemos una instancia de IBTestManager.
				IBTestManager instance = createIBTestManager();
				// Llamamos al mtodo que permite registrar la respuesta y guardamos el resultado de 
				// su ejecucin como un atributo de la request.
				request.setAttribute("test_answer_data", instance.saveAnswer(getUserId(request), 
						Integer.parseInt(request.getParameter("idTest")), score, maxScore,
						questionTotal, questionOK));
				// Guardamos la informacin de la fecha y hora.
				request.setAttribute("now", DateUtils.now());
			}
		}
		else
		{
			// No se ha guardado previamente la informacin del Cuestionario en la sesin por lo que lanzamos
			// una Exception.
			throw (new Exception("Imposible recuperar de la HttpSession el Cuestionario."));
		}
	}	
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para procesar la respuesta del Estudiante a una de las preguntas
	 * del Cuestionario que est&aacute; contestando.</p>
	 * 
	 * @param request <code>HttpServletRequest</code> asociado a la petici&oacute;n HTTP donde se env&iacute;a 
	 * la respuesta del Estudiante.
	 * @throws Exception si se produce alg&uacute;n tipo de error a la hora de procesar la respuesta.
	 */
	private void processAnswer(HttpServletRequest request) throws Exception
	{
		// Comprobamos que en la sesin tenemos almacenado el cuestionario y que se ha recibido el parmetro
		// "questionNumber" continiendo este un valor numrico.
		if ((request.getSession().getAttribute(CTPresentation.TEST) != null) &&
				(request.getSession().getAttribute(CTPresentation.TEST) instanceof TOTest) &&
				(ValidationUtils.isInteger(request.getParameter("questionNumber"))))
		{
			// Recuperamos de la sesin el cuestionario.
			TOTest toTest = (TOTest) request.getSession().getAttribute(CTPresentation.TEST);
			// La pregunta que se est contestando es la que se encuentra en la posicin "questionNumber" - 1.
			int position = Integer.parseInt(request.getParameter("questionNumber")) - 1;
			// Recuperamos la pregunta que se encuentra en dicha posicin.
			TOQuestion questionInSession = toTest.getQuestions().get(position).getQuestion();
			// Creamos una instancia de IBQuestionBankManager.
			IBQuestionBankManager instance = createIBQuestionBankManager();
			// Obtenemos toda la informacin de la pregunta.
			TOQuestion question = instance.getQuestion(questionInSession.getId());
			// Variable local que nos servir para determinar si se ha contestado correctamente o no.
			boolean ok = false;
			// Tenemos que determinar el tipo de pregunta del que se trata.
			if (question instanceof TOQuestionMultipleChoices)
				ok = processAnswerMultipleChoices((TOQuestionMultipleChoices) question, request);	// Tipo: Mltiples Respuestas.
			else if (question instanceof TOQuestionShortAnswer)
				ok = processAnswerShortAnswer((TOQuestionShortAnswer) question, request);			// Tipo: Respuesta Corta.
			else if (question instanceof TOQuestionTrueOrFalse)
				ok = processAnswerTrueOrFalse((TOQuestionTrueOrFalse) question, request);			// Tipo: Verdadero o Falso. 
			else if (question instanceof TOQuestionMatch)
				ok = processAnswerMatch((TOQuestionMatch) question, request);						// Tipo: Emparejar. 
			else if (question instanceof TOQuestionNumeric)
				ok = processAnswerNumeric((TOQuestionNumeric) question, request);					// Tipo: Numrico.
			else
				// No se trata de una de las preguntas definidas en la aplicacin as que 
				// lanzamos una Exception.
				throw (new Exception("Tipo de pregunta invalido."));
			// Una vez procesada la respuesta vemos si esta ha sido correcta.
			if (ok)
			{
				// La respuesta ha sido correcta y as lo indicamos en la pregunta de la sesin.
				questionInSession.setCorrect(true);
				// Tambin almacenamos su valor.
				questionInSession.setValue( toTest.getQuestions().get(position).getValue());
			}
		}
		else
		{
			// No se ha guardado previamente la informacin del Cuestionario en la sesin por lo que
			// lanzaremos una Excepcin.
			throw (new Exception("Imposible recuperrar de la HttpSession el Cuestionario."));
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para procesar una respuesta del Estudiante a una pregunta de tipo
	 * "Num&eacute;rico".</p>
	 * 
	 * @param question <code>TOQuestionNumeric</code> la pregunta a contestar.
	 * @param request <code>HttpServletRequest</code> asociado a la petici&oacute;n HTTP donde se env&iacute;a 
	 * la respuesta del Estudiante.
	 * @return <code>true</code> si la respuesta del Estudiante es correcta o <code>false</code> en caso contrario.
	 */
	private boolean processAnswerNumeric(TOQuestionNumeric question, HttpServletRequest request)
	{
		// Comprobamos que se ha recibido el parmetro "nuAnswer" y adems que dicho parmetro contiene
		// un valor numrico.
		if (!ValidationUtils.isDouble(request.getParameter("nuAnswer")))
		{
			// No se ha recibido el parmetro o no tiene un valor numrico por lo que retornamos false.
			return false;
		}
		// Declaramos una variable local con el valor de la respuesta del usuario.
		double answer = Double.parseDouble(request.getParameter("nuAnswer"));
		// Valor mnimo de la respuesta.
		double minValue = question.getValue() - question.getError();
		// Valor mximo de la respuesta.
		double maxValue = question.getValue() + question.getError();
		// Comprobamos si la respuesta cae entre esos valores y lo retornamos.
		return ((answer >= minValue) && (answer <= maxValue));
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para procesar una respuesta del Estudiante a una pregunta de tipo
	 * "Emparejar".</p>
	 * 
	 * @param question <code>TOQuestionMatch</code> la pregunta a contestar.
	 * @param request <code>HttpServletRequest</code> asociado a la petici&oacute;n HTTP donde se env&iacute;a 
	 * la respuesta del Estudiante.
	 * @return <code>true</code> si la respuesta del Estudiante es correcta o <code>false</code> en caso contrario.
	 */
	private boolean processAnswerMatch(TOQuestionMatch question, HttpServletRequest request)
	{
		// Definimos una variable local que nos indicar si la respuesta ha sido correcta o no.
		boolean ok = true;
		// Recorremos todos las parejas que se han formado.
		for (int i = 1; i <= 5; i++)
		{
			// Vemos si existe la pareja.
			if (ValidationUtils.isEmpty(request.getParameter("maSelect" + i + "Left")))
			{
				// No hay ms parejas a comprobar as que finalizamos la ejecucin del bucle.
				break;
			}
			else 
			{
				// Tenemos que ver si coinciden los valores de "maSelectLeft" y "maSelectRight".
				// Declaramos dos variables locales para clarificar el cdigo.
				String leftValue = request.getParameter("maSelect" + i + "Left");
				String rightValue = request.getParameter("maSelect" + i + "Right");
				// Ahora vemos si son iguales o no.
				if (!(leftValue.equals(rightValue)))
				{
					// No son iguales. Establecemos "ok" a false.
					ok = false;
					// Finalizamos la ejecucin del bucle.
					break;
				}
			}
		}
		// Retornamos el valor de "ok"
		return ok;
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para procesar una respuesta del Estudiante a una pregunta de tipo
	 * "Verdadero o Falso".</p>
	 * 
	 * @param question <code>TOQuestionTrueOrFalse</code> la pregunta a contestar.
	 * @param request <code>HttpServletRequest</code> asociado a la petici&oacute;n HTTP donde se env&iacute;a 
	 * la respuesta del Estudiante.
	 * @return <code>true</code> si la respuesta del Estudiante es correcta o <code>false</code> en caso contrario.
	 */
	private boolean processAnswerTrueOrFalse(TOQuestionTrueOrFalse question, HttpServletRequest request)
	{
		// Comprobamos que se ha recibido la respuesta del usuario.
		if (ValidationUtils.isEmpty(request.getParameter("answer")))
		{
			// No se ha recibido la respuesta del usuario as que retornamos false.
			return false;
		}
		// En este punto estamos seguros de que s que se ha recibido la respuesta del usuario. La analizamos.
		// Declaramos una variable local para contendr la respuesta del usuario.
		boolean userAnswer = false;
		// Comprobamos el valor que se ha recibido del usuario.
		if ("true".equals(request.getParameter("answer")))
		{
			// El usuario ha marcado la opcin "verdadero".
			userAnswer = true;
		}
		else if ("false".equals(request.getParameter("answer")))
		{
			// El usuario ha marcado la opcin "falso".
			userAnswer = false;
		}
		else
		{
			// Se ha recibido cualquier otro valor de respuesta que no est permitido por lo que 
			// retornaremos false.
			return false;
		}
		// Tenemos que comparar la opcin del usuario con la que es correcta en la pregunta.
		return (userAnswer == question.isValue());
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para procesar una respuesta del Estudiante a una pregunta de tipo
	 * "Respuesta Corta".</p>
	 * 
	 * @param question <code>TOQuestionShortAnswer</code> la pregunta a contestar.
	 * @param request <code>HttpServletRequest</code> asociado a la petici&oacute;n HTTP donde se env&iacute;a 
	 * la respuesta del Estudiante.
	 * @return <code>true</code> si la respuesta del Estudiante es correcta o <code>false</code> en caso contrario.
	 */
	private boolean processAnswerShortAnswer(TOQuestionShortAnswer question, HttpServletRequest request)
	{
		// Comrprobamos que hemos recibido una respuesta del usuario.
		if (ValidationUtils.isEmpty(request.getParameter("saAnswer")))
		{
			// Al no haber recibido una respuesta retornamos el valor false
			return false;
		}
		// Si llegamos aqu querr decir que s hemos recibido una respuesta del usuario. Elimamos los posibles
		// espacios en blanco que pudesemos tener en ella al principio y final y la trasformamos las entidades HTML.
		String userAnswer = WebUtils.stringToHTML(request.getParameter("saAnswer").trim());
		// Realizamos la comparacin teniendo en cuenta si hay que distinguir o no entre maysculas y minsculas.
		if (question.isCaseSensitive())
		{
			// Tenemos que realizar la comparacin distinguiendo entre maysculas y minsculas.
			return userAnswer.equals(question.getAnswer());
		}
		else
		{
			// Tenemos que realizar la comparacin sin distinguir entre maysculas y minsculas.
			return userAnswer.equalsIgnoreCase(question.getAnswer());
		}
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para procesar una respuesta del Estudiante a una pregunta de tipo
	 * "M&uacute;ltiples Opciones".</p>
	 * 
	 * @param question <code>TOQuestionMultipleChoices</code> la pregunta a contestar.
	 * @param request <code>HttpServletRequest</code> asociado a la petici&oacute;n HTTP donde se env&iacute;a 
	 * la respuesta del Estudiante.
	 * @return <code>true</code> si la respuesta del Estudiante es correcta o <code>false</code> en caso contrario.
	 */
	private boolean processAnswerMultipleChoices(TOQuestionMultipleChoices question, HttpServletRequest request)
	{
		// En primer lugar vamos a determinar cuntas respuestas correctas se definen en la pregunta. Variable local.
		int totalCorrectAnswers = 0;
		// Recorremos las respuestas.
		for (TOQuestionMultipleChoicesAnswer answer : question.getAnswers())
		{
			// Vemos si se trata de una respuesta correcta.
			if (answer.isCorrect())
			{
				// Incrementamos el nmero de respuestas correctas.
				totalCorrectAnswers++;
			}
		}
		// Vamos a ver cuntas opciones de respuesta ha marcado el usuario.
		int totalUserAnswers = 0;
		// Recorremos los parmetros de la respuesta HTTP.
		for (int i = 1; i <= 5; i++)
		{
			// Vemos si se ha recibido una respuesta y adems que dicha respuesta es de tipo entero.
			if (ValidationUtils.isInteger(request.getParameter("mcAnswer" + i)))
			{
				// Incrementamos el nmero de resputas recibidas.
				totalUserAnswers++;
			}
		}
		// Comprobamos si coincide el nmero de respuestas correctas con el nmero de respuestas enviadas
		// por el usuario.
		if (totalCorrectAnswers != totalUserAnswers)
		{
			// La respuesta del usuario no puede ser correcta as que retornamos false.
			return false;
		}
		// Creamos un array de "int" que almacenar los identificadores de las respuestas correctas.
		int[] idCorrectAnswers = new int[totalCorrectAnswers];
		// Contador de posiciones dentro del array.
		int position = 0;
		// Guardamos los identificadores de las respuestas correctas.
		for (TOQuestionMultipleChoicesAnswer answer : question.getAnswers())
		{
			// Vemos si se trata de una pregunta correcta.
			if (answer.isCorrect())
			{
				// Guardamos el identificador de la respuesta.
				idCorrectAnswers[position] = answer.getId();
				// Incrementamos el valor de position.
				position++;
			}
		}
		// Creamos un array de "int" que almacenar los identificadores de las respuestas del usuario.
		int[] idUserAnswers = new int[totalUserAnswers];
		// Inicializamos nuevamente el contador.
		position = 0;
		// Guardamos los identificadores de las respuestas del usuario.
		for (int i = 1; i <= 5; i++)
		{
			// Vemos si se ha recibido una respuesta y adems que dicha respuesta es de tipo entero.
			if (ValidationUtils.isInteger(request.getParameter("mcAnswer" + i)))
			{
				// Guardamos el identificador de la respuesta del usuario.
				idUserAnswers[position] = Integer.parseInt(request.getParameter("mcAnswer" + i));
				// Incrementamos el valor de "position".
				position++;
			}
		}
		// Definimos un array de "boolean" que indicar si se ha encontrado la respuesta correcta
		// entre las que ha indicado el usuario.
		boolean[] findAnswer = new boolean[totalCorrectAnswers];
		// Recorremos el array de respuestas correctas. 
		for (int i = 0; i < idCorrectAnswers.length; i++)
		{
			// Variable local que indicar si se ha encontrado el elemento o no.
			boolean find = false;
			// Recorremos los identificadores de las respuestas enviadas por el usuario.
			for (int idUserAnswer : idUserAnswers)
			{
				// Comparamos los identificadores.
				if (idUserAnswer == idCorrectAnswers[i])
				{
					// Hemos encontrado el identificador en la respuesta y as lo indicamos.
					find = true;
					// Rompemos la ejecucin del bucle.
					break;
				}
			}
			// Comprobamos si lo hemos encontrado o no.
			if (find)
			{
				// Al haberlo encontrado actualizamos su valor en "findAnswer"
				findAnswer[i] = true;
			}
			else
			{
				// No lo hemos encontrado as que la respuesta no ser correcta. Rompemos la
				// ejecucin del bucle.
				break;
			}
		}
		// Ahora simplemente tenemos que comprobar si todos los elementos de findAnswer son true.
		boolean allTrue = true;
		// Recorremos todos los elementos de findAnswer.
		for (boolean value : findAnswer)
		{
			// Comprobamos si value es "false"
			if (!value)
			{
				// No se ha marcado una de las respuestas correctas.
				allTrue = false;
				// Rompemos la ejecucin del bucle.
				break;
			}
		}
		// Retornamos el valor de "allTrue"
		return allTrue;
	}
	
}