package edu.uoc.pfc.struts.forms;

import org.apache.struts.action.ActionErrors;

import edu.uoc.pfc.business.transfers.TOUser;
import edu.uoc.pfc.lib.utils.BeanUtils;
import edu.uoc.pfc.lib.utils.ValidationUtils;

/**
 * <p><code>ActionForm</code> que ser&aacute; utilizado en las transacciones asociadas
 * a la gesti&oacute;n de cuentas de usuario.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class AFAccount extends AFBase 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	/** <p>N&uacute;mero m&iacute;nimo de caracteres para el campo <i>login</i>.</p> */
	public static final int LOGIN_MIN_LENGTH = 3;
	/** <p>N&uacute;mero m&aacute;ximo de caracteres para el campo <i>login</i>.</p> */
	public static final int LOGIN_MAX_LENGTH = 20;
	/** <p>N&uacute;mero m&iacute;nimo de caracteres para el campo <i>password</i>.</p> */
	public static final int PASSWORD_MIN_LENGTH = 6;
	/** <p>N&uacute;mero m&aacute;ximo de caracteres para el campo <i>password</i>.</p> */
	public static final int PASSWORD_MAX_LENGTH = 20;
	
	// -------- ATRIBUTOS PRIVADOS
	/** <p>Identificador del usuario en el sistema.</p> */
	private String id;
	/** <p>Nombre de usuario (login).</p> */
	private String login;
	/** <p>Contrase&ntilde;a de acceso.</p> */
	private String password;
	/** <p>Repetir contrase&ntilde;a.</p> */
	private String repeatPassword;
	/** <p>Correo electr&oacute;nico.</p> */
	private String email;
	/** <p>Tipo de usuario: administrador.</p> */
	private String userTypeAdmin;
	/** <p>Tipo de usuario: profesor.</p> */
	private String userTypeTeacher;
	/** <p>Tipo de usuario: alumno.</p> */
	private String userTypeStudent;
	/** <p>Cuenta activa.</p> */
	private String active;
	/** <p>Nombre real del usuario.</p> */
	private String name;
	/** <p>Apellidos del usuario.</p> */
	private String surname;
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Valida que los datos introducidos en el formulario para la creaci&oacute;n
	 * de una nueva cuenta son correctos desde el punto de vista de las validaciones
	 * simples que se han de realizar sobre ellos.</p>
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples sobre los datos del formulario. En el caso de no haber errores retorna
	 * un objeto <code>ActionErrors</code> que est&aacute; vac&iacute;o.
	 */
	public ActionErrors validateCreateAccount()
	{
		// Borramos los errores de validacin simples que pudiese haber.
		clearErrors();
		// Validamos el campo "login".
		validateLogin();
		// Validamos el campo "password".
		validatePassword();
		// Validamos el campo "repeatPassword"
		validateRepeatPassword();
		// Validamos el campo "email"
		validateEmail();
		// Validamos el tipo de usuario.
		validateType();
		// Validamos el campo "name"
		validateName();
		// Validamos el campo "surname"
		validateSurname();
		// Vemos si todo ha ido bien.
		if (this.errors.size() == 0)
		{
			// No ha habido errores. Eliminamos espacios en blanco 
			// de todos los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	/**
	 * <p>Valida que los datos introducidos en el formulario para la modificaci&oacugte;n de 
	 * una cuenta de usuario son correctos desde el punto de las validaciones simples que se
	 * deber&aacute;n llevar a cabo sobre ellos.</p>
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples sobre los datos del formulario. En el caso de no haber errores retorna
	 * un objeto <code>ActionErrors</code> que est&aacute; vac&iacute;o.
	 */
	public ActionErrors validateModifyAccount()
	{
		// Borramos los errores de validacin simples que pudiese haber.
		clearErrors();
		// Validamos el campo contrasea.
		validatePassword();
		// Validamos el campo "repeatPassword"
		validateRepeatPassword();
		// Validamos el tipo de usuario.
		validateType();
		// Validamos el campo "name"
		validateName();
		// Validamos el campo "surname"
		validateSurname();
		// Vemos si todo ha ido bien.
		if (this.errors.size() == 0)
		{
			// No ha habido errores. Eliminamos los espacios en blanco de todos
			// los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado siempre que se quiera validar la 
	 * informaci&oacute;n del formulario asociada a la modificaci&oacute;n de un
	 * perfil de usuario.</p>
	 * 
	 * return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples sobre los datos del formulario. En el caso de no haber errores retorna
	 * un objeto <code>ActionErrors</code> que est&aacute; vac&iacute;o.
	 */
	public ActionErrors validateEditMyAccount()
	{
		// Borramos los errores de validacin simples que pudiese haber.
		clearErrors();
		// Validamos "contrasea".
		validatePassword();
		// Validamos "repetir contrasea".
		validateRepeatPassword();
		// Vemos si todo ha ido bien.
		if (this.errors.size() == 0)
		{
			// No ha habido errores por lo que eliminanos los espacios en blanco.
			BeanUtils.trimProperties(this);
		}
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	/**
	 * <p>M&eeacute;todo que realiza las validaciones simples sobre los campos del formulario
	 * cuando la transacci&oacute;n que se est&aacute; ejecutando es <i>recuperar el nombre
	 * de usuario</i>.</p>
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples sobre los datos del formulario. En el caso de no haber errores retorna
	 * un objeto <code>ActionErrors</code> que est&aacute; vac&iacute;o.
	 */
	public ActionErrors validateForgotLogin()
	{
		// Borramos los errores de validacin simples que pudiese haber.
		clearErrors();
		// Validamos el campo "email"
		validateEmail();
		// Vemos si todo ha ido bien.
		if (this.errors.size() == 0)
		{
			// No ha habido errores. Eliminamos los espacios en blanco de todos
			// los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
		
	/**
	 * <p>M&eeacute;todo que realiza las validaciones simples sobre los campos del formulario
	 * cuando la transacci&oacute;n que se est&aacute; ejecutando es <i>recuperar la contrase&ntilde;a
	 * de un usuario</i>.</p>
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples sobre los datos del formulario. En el caso de no haber errores retorna
	 * un objeto <code>ActionErrors</code> que est&aacute; vac&iacute;o.
	 */
	public ActionErrors validateForgotPassword()
	{
		// Borramos los errores de validacin simples que pudiese haber.
		clearErrors();
		// Validamos el campo "email".
		validateEmail();
		// Vemos si todo ha ido bien.
		if (this.errors.size() == 0)
		{
			// No ha habido errores. Eliminamos los espacios en blanco de todos
			// los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	/**
	 * <p>M&eacute;todo que ser&aacute; llamado para realizar las validaciones simples sobre los 
	 * datos del formulario en una transacci&oacute;n de acceso al sistema por parte de un usuario.</p> 
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples sobre los datos del formulario. En el caso de no haber errores retorna
	 * un objeto <code>ActionErrors</code> que est&aacute; vac&iacute;o.
	 */
	public ActionErrors validateSignin()
	{
		// Borramos los posibles errores de validacin simples que pudiese haber.
		clearErrors();
		// Validamos el campo "login".
		validateLogin();
		// Validamos el campo "password".
		validatePassword();
		// Vemos si todo ha ido bien.
		if (this.errors.size() == 0)
		{
			// No ha habido errores as que eliminamos los espacios en blanco de las propiedades.
			BeanUtils.trimProperties(this);
		}
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	/**
	 * <p>M&eacute;todoq ue ser&aacute; llamado para realizar las validaciones simples sobre 
	 * los datos del formulario en una transacci&oacute;n de petici&oacute;n de creaci&oacute;n
	 * de una cuenta de usuario.</p>
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples sobre los datos del formulario. En el caso de no haber errores retorna
	 * un objeto <code>ActionErrors</code> que est&aacute; vac&iacute;o.
	 */
	public ActionErrors validateSignup()
	{
		// Borramos los posibles errores de validacin simples que pudiese haber.
		clearErrors();
		// Validamos el campo "login"
		validateLogin();
		// Validamos el campo "password"
		validatePassword();
		// Validamos el campo "repeatPassword"
		validateRepeatPassword();
		// Validamos el campo "email"
		validateEmail();
		// Validamos el campo "name"
		validateName();
		// Validamos el campo "surname"
		validateSurname();
		// Vemos si todo ha ido bien.
		if (this.errors.size() == 0)
		{
			// Eliminamos los espacios en blanco de todos los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	/**
	 * <p>Copia los valores de las propiedades del objeto <code>TOUser</code> que recibe como
	 * par&aacute;metro en los atributos correspondientes a los campos del formulario.</p>
	 * 
	 * @param user <code>TOUser</code> con la informaci&oacute;n del usuario.
	 */
	public void copyProperties(TOUser user)
	{
		// Comprobamos que el parmetro no sea nulo.
		if (user != null)
		{
			// Identificador del usuario.
			this.id = Integer.toString(user.getId());
			// Nombre de usuario
			this.login = user.getLogin();
			// Contrasea
			this.password = user.getPassword();
			// Repetir contrasea.
			this.repeatPassword = user.getPassword();
			// Correo electrnico.
			this.email = user.getEmail();
			// Tipo de usuario: Administrador.
			if (user.isAdmin())
			{
				// Indicamos que s es un administrador.
				this.userTypeAdmin = "yes";
			}
			// Tipo de usuario: Profesor.
			if (user.isTeacher())
			{
				// Indicamos que s es un profesor.
				this.userTypeTeacher = "yes";
			}
			// Tipo de usuario: Estudiante.
			if (user.isStudent())
			{
				// Indicamos que s es un estudiante.
				this.userTypeStudent = "yes";
			}
			
			// Cuenta activa.
			this.active = Boolean.toString(user.isActive());
			// Nombre real.
			this.name = user.getName();
			// Apellidos.
			this.surname = user.getSurname();
		}
	}
	
	// -------- MTODOS PRIVADOS.	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>login</i>. En el caso
	 * de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateLogin()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.login))
		{
			// El campo "login" est vaco. Grabamos un mensaje de error.
			saveEmptyError("Login");
		}
		else if (!ValidationUtils.isInRange(this.login, LOGIN_MIN_LENGTH, LOGIN_MAX_LENGTH))
		{
			// El campo "login" no tiene un nmero de caracteres vlido.
			saveOutOfRange("Login", LOGIN_MIN_LENGTH, LOGIN_MAX_LENGTH);
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>password</i>. En el caso
	 * de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validatePassword()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.password))
		{
			// El campo "password" est vaco. Grabamos un mensaje de error.
			saveEmptyError("Password");
		}
		else if (!ValidationUtils.isInRange(this.password, PASSWORD_MIN_LENGTH, PASSWORD_MAX_LENGTH))
		{
			// El campo "password" no tiene el nmero de caracteres vlido.
			saveOutOfRange("Password", PASSWORD_MIN_LENGTH, PASSWORD_MAX_LENGTH);
		}
	}
	
	/** <p>Realiza las validaciones simples sobre el campo <i>repeatPassword</i>. En el 
	 * caso de que exista alg&uacute;n error ser&aacute; depositado dentro del objeto 
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateRepeatPassword()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.repeatPassword))
		{
			// El campo "repeatPassword" est vaco. Grabamos un mensaje de error.
			saveEmptyError("RepeatPassword", "Repetir Contrase&ntilde;a");
		}
		else if (!ValidationUtils.isInRange(this.password, PASSWORD_MIN_LENGTH, PASSWORD_MAX_LENGTH))
		{
			// El campo "repeatPassword" no tiene el nmero de caracteres vlido.
			saveOutOfRange("RepeatPassword", "Repetir Contrase&ntilde;a", PASSWORD_MIN_LENGTH, PASSWORD_MAX_LENGTH);
		}
		else if (!ValidationUtils.isEmpty(this.password))
		{
			// Comprobamos si "password" y "repeatPassword" tienen el mismo valor
			if (!this.password.equals(this.repeatPassword.trim()))
			{
				// No tienen el mismo valor. Grabamos un error.
				saveNotEqual("RepeatPassword", "Contrase&ntilde;a", "Repetir Contrase&ntilde;a");
			}
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>Email</i>. En el caso de que exista
	 * alg&uacute;n error ser&aacute; depositado dentro del objeto <code>ActionErrors</code> 
	 * asociado al atributo <i>errors</i>.</p>
	 */
	private void validateEmail()
	{
		// Validamos que el campo no est vaco.
		if (ValidationUtils.isEmpty(this.email))
		{
			// El campo "email" est vaco. Grabamos un mensaje de error.
			saveEmptyError("Email", "Correo Electr&oacute;nico");
		}
		else if (!ValidationUtils.isEmail(this.email))
		{
			// No se ha escrito una direccin de correo electrnico 
			saveNotEmail("Email", "Correo Electr&oacute;nico");
		}
	}
	
	/**
	 * <p>Realiza las validaciones sobre el campo <i>Tipo</i>. En concreto validar&aacute; que 
	 * al menos se ha indicado un tipo de usuario. En el caso de que exista alg&uacute;n error
	 * ser&aacute; depositado dentro del objeto <code>ActionErrors</code> asociado al atributo
	 * <i>errors</i>.</p>
	 */
	private void validateType()
	{
		// Comprobamos que se ha elegido al menos un tipo de usuario.
		if ((ValidationUtils.isEmpty(this.userTypeAdmin)) && (ValidationUtils.isEmpty(this.userTypeTeacher)) 
				&& (ValidationUtils.isEmpty(this.userTypeStudent)))
		{
			// Grabamos el error.
			saveChooseOne("UserType", "Tipo de Usuario");
		}
	}
	
	/**
	 * <p>Realiza las validaciones sobre el campo <i>name</i>. En el caso de que exista 
	 * alg&uacute;n error ser&aacute; depositado dentro del objeto <code>ActionErrors</code> 
	 * asociado al atributo <i>errors</i>.</p>
	 */
	private void validateName()
	{
		// Comprobamos si el campo est vaco.
		if (ValidationUtils.isEmpty(this.name))
		{
			// Grabamos el error "campo vaco".
			saveEmptyError("Name", "Nombre");
		}
	}
	
	/**
	 * <p>Realiza las validaciones sobre el campo <i>surname</i>. En el caso de que exista 
	 * alg&uacute;n error ser&aacute; depositado dentro del objeto <code>ActionErrors</code> 
	 * asociado al atributo <i>errors</i>.</p>
	 */
	private void validateSurname()
	{
		// Comprobamos si el campo est vaco.
		if (ValidationUtils.isEmpty(this.surname))
		{
			// Grabamos el error "campo vaco".
			saveEmptyError("Surname", "Apellidos");
		}
	}
	
	// -------- GETTERS & SETTERS
	public String getLogin() 
	{
		return login;
	}
	public void setLogin(String login) 
	{
		this.login = login;
	}
	public String getPassword() 
	{
		return password;
	}
	public void setPassword(String password) 
	{
		this.password = password;
	}
	public String getRepeatPassword() 
	{
		return repeatPassword;
	}
	public void setRepeatPassword(String repeatPassword) 
	{
		this.repeatPassword = repeatPassword;
	}
	public String getEmail() 
	{
		return email;
	}
	public void setEmail(String email) 
	{
		this.email = email;
	}
	public String getUserTypeAdmin() 
	{
		return userTypeAdmin;
	}
	public void setUserTypeAdmin(String userTypeAdmin) 
	{
		this.userTypeAdmin = userTypeAdmin;
	}
	public String getUserTypeTeacher() 
	{
		return userTypeTeacher;
	}
	public void setUserTypeTeacher(String userTypeTeacher) 
	{
		this.userTypeTeacher = userTypeTeacher;
	}
	public String getUserTypeStudent() 
	{
		return userTypeStudent;
	}
	public void setUserTypeStudent(String userTypeStudent) 
	{
		this.userTypeStudent = userTypeStudent;
	}
	public String getActive() 
	{
		return active;
	}
	public void setActive(String active) 
	{
		this.active = active;
	}
	public String getName() 
	{
		return name;
	}
	public void setName(String name) 
	{
		this.name = name;
	}
	public String getSurname() 
	{
		return surname;
	}
	public void setSurname(String surname) 
	{
		this.surname = surname;
	}

	public String getId()
	{
		return id;
	}

	public void setId(String id)
	{
		this.id = id;
	}
	
}
