package edu.uoc.pfc.struts.forms;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMessage;

import edu.uoc.pfc.lib.utils.ValidationUtils;

/**
 * <code>ActionForm</code> abstracto del cual heredar&aacute;n todos los
 * <code>ActionForm</code> que son utilizados por las transacciones de 
 * la aplicaci&oacutE;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public abstract class AFBase extends ActionForm 
{

	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	/** <p>Clave dentro del fichero "Common.properties" para el error "elegir al menos uno".</p> */
	public static final String ERRORS_KEY_CHOOSE_ONE = "errors.choose.one";
	/** <p>Clave dentro del fichero "Common.properties" para el error "campo vac&iacute;o".</p> */
	public static final String ERRORS_KEY_EMPTY = "errors.empty";
	/** <p>Clave dentro del fichero "Common.properties" para el error "email inv&aacute;lido".</p> */
	public static final String ERRORS_KEY_NOT_EMAIL = "errors.not.email";
	/** <p>Clave dentro del fichero "Common.properties" para el error "no iguales".</p> */
	public static final String ERRORS_KEY_NOT_EQUAL = "errors.not.equal";
	/** <p>Clave dentro del fichero "Common.properties" para el error "valor fuera de rango".</p> */
	public static final String ERRORS_KEY_OUT_OF_RANGE = "errors.out.of.range";
	/** <p>Clave dentro del fichero "Common.properties" para el error "seleccionar uno".</p> */
	public static final String ERRORS_KEY_SELECT_ONE = "errors.select.one";
	/** <p>Clave dentro del fichero "Common.properties" para el error "no num&eacute;rico".</p> */
	public static final String ERRORS_KEY_NO_NUMBER = "errors.no.number";
	/** <p>Clave dentro del fichero "Common.properties" para el error "fecha inv&aacute;lida".</p> */
	public static final String ERRORS_KEY_INVALID_DATE = "errors.no.date";
	
	
	// -------- ATRIBUTOS PROTEGIDOS.
	/** <p>M&eeacute;todo que deber&aacute; ejecutarse de la <code>DispatchAction</code> asociada con el <code>ActionForm</code>. */
	protected String cmd;
	/** <p><code>ActionErrors</code> que recoger&aacute; los errores de validaci&oacute;n simples.</p> */
	protected ActionErrors errors;
	
	// --------- MTODOS PROTEGIDOS.
	/**
	 * <p>Borra todos los errores que pudiera haber dentro del objeto <code>ActionErrors</code>.
	 */
	protected void clearErrors()
	{
		// Comprobamos si es vaco.
		if (this.errors == null)
		{
			// Al ser nulo simplemente lo volvemos a crear.
			this.errors = new ActionErrors();
		}
		else
		{
			// Ya se haba creado previamente as que lo vaciamos.
			this.errors.clear();
		}
	}
	
	/**
	 * <p>Inicializa el objeto <code>ActionErrors</code> siempre que no haya sido inicializado
	 * previamente (en ese caso el m&eacute;todo no har&aacute; nada).</p>
	 */
	protected void initErrors()
	{
		// Comprobamos que "errors" no ha sido inicializado previamente.
		if (this.errors == null)
		{
			// Inicializamos "errors"
			this.errors = new ActionErrors();
		}
	}
	
	/**
	 * <p>Almacena el error "fecha inv&aacute;lida" dentro del objeto <code>ActionErrors</code>
	 * definido en el atributo <i>errors</i> refiri&eacute;ndose al campo que se recibe como
	 * par&aacutemetro.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 */
	protected void saveInvalidDate(String field)
	{
		// Validamos que el campo recibido no sea vaco.
		if (!ValidationUtils.isEmpty(field))
		{
			// Inicializamos "errors"
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_INVALID_DATE, field));
		}
	}
	
	/**
	 * <p>Almacena el error "fecha inv&aacute;lida" dentro del objeto <code>ActionErrors</code>
	 * definido en el atributo <i>errors</i> refiri&eacute;ndose al campo que se recibe como
	 * par&aacute;metro. El nombre del campo que ser&aacute; mostrado en la p&aacute;gina
	 * web ser&aacute; el que se recoge en el par&aacute;metro <i>fieldWeb</i>.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 * @param fieldWeb <code>String</code> nombre del campo que ser&aacute; mostrado en la web.
	 */
	protected void saveInvalidDate(String field, String fieldWeb)
	{
		// Validamos que ambos campos recibidos no sean nulos.
		if ((!ValidationUtils.isEmpty(field)) && (!ValidationUtils.isEmpty(fieldWeb)))
		{
			// Inicializamos "errors"
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_INVALID_DATE, fieldWeb));
		}
		else
		{
			// Llamamos al mtodo sobreescrito
			saveInvalidDate(field);
		}
	}
	
	/**
	 * <p>Almacena el error "campo vac&iacute;o" dentro del objeto <code>ActionErrors</code>
	 * definido en el atributo <i>errors</i> refiri&eacute;ndose al campo que se recibe como
	 * par&aacutemetro.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 */
	protected void saveEmptyError(String field)
	{
		// Validamos que el campo recibido no sea vaco.
		if (!ValidationUtils.isEmpty(field))
		{
			// Inicializamos "errors"
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_EMPTY, field));
		}
	}
	
	/**
	 * <p>Almacena el error "campo vac&iacute;o" dentro del objeto <code>ActionErrors</code>
	 * definido en el atributo <i>errors</i> refiri&eacute;ndose al campo que se recibe como
	 * par&aacute;metro. El nombre del campo que ser&aacute; mostrado en la p&aacute;gina
	 * web ser&aacute; el que se recoge en el par&aacute;metro <i>fieldWeb</i>.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 * @param fieldWeb <code>String</code> nombre del campo que ser&aacute; mostrado en la web.
	 */
	protected void saveEmptyError(String field, String fieldWeb)
	{
		// Validamos que ambos campos recibidos no sean nulos.
		if ((!ValidationUtils.isEmpty(field)) && (!ValidationUtils.isEmpty(fieldWeb)))
		{
			// Inicializamos "errors"
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_EMPTY, fieldWeb));
		}
		else
		{
			// Llamamos al mtodo sobreescrito
			saveEmptyError(field);
		}
	}
	
	/**
	 * <p>Almacena el error "fuera de rango" dentro del objeto <code>ActionErrors</code>
	 * definido en el atributo <i>errors</i> refiri&eacute;ndose al campo que se recibe como
	 * par&aacutemetro.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 * @param min <code>int</code> valor m&iacute;nimo.
	 * @param max <code>int</code> valor m&aacute;ximo.
	 */
	protected void saveOutOfRange(String field, int min, int max)
	{
		// Validamos que el campo no sea vaco.
		if (!ValidationUtils.isEmpty(field))
		{
			// Inicializamos "errors"
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_OUT_OF_RANGE, field, min, max));  
		}
	}
	
	/**
	 * <p>Almacena el error "fuera de rango" dentro del objeto <code>ActionErrors</code> 
	 * definido en el atributo <i>errors</i> refiri&eacute;ndose al campo <i>field</i> que
	 * se recibe como par&aacute;metro. El campo <i>fieldWeb</i> contendr&aacute; el nombre
	 * del campo tal y como se vea en la web.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 * @param fieldWeb <code>String</code> nombre del campo a mostrar en la web.
	 * @param min <code>int</code> valor m&iacute;nimo.
	 * @param max <code>int</code> valor m&aacute;ximo.
	 */
	protected void saveOutOfRange(String field, String fieldWeb, int min, int max)
	{
		// Validamos que el campo no sea vaco ni el campo fieldWeb
		if ((!ValidationUtils.isEmpty(field)) && (!ValidationUtils.isEmpty(fieldWeb)))
		{
			// Inicializamos "errors"
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_OUT_OF_RANGE, fieldWeb, min, max));
		}
		else 
		{
			// Llamamos al mtodo sobreescrito.
			saveOutOfRange(field, min, max);
		}
	}
	
	/**
	 * <p>Almacena el error "elegir uno" dentro del objeto <code>ActionErrors</code>
	 * definido en el atributo <i>errors</i> refiri&eacute;ndose al campo <i>field</i> que 
	 * se recibe como pa&aacute;metro.</p>
	 * @param field <code>String</code> campo al que corresponde el error.
	 */
	protected void saveChooseOne(String field)
	{
		// Comprobamos que no sea vaco.
		if (!ValidationUtils.isEmpty(field))
		{
			// Inicializamos los errores.
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_CHOOSE_ONE, field));
		}
	}
	
	/**
	 * <p>Almacena el error "elegir uno" dentro del objeto <code>ActionErrors</code> definido 
	 * en el atributo <i>error</i> refiri&eacute;ndose al campo <i>field</i> que se recibe como
	 * par&aacute;metro. El campo <i>fieldWeb</i> contedr&aacute; el nombre del campo tal y
	 * como ser&aacute; mostrado en la web.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 * @param fieldWeb <code>String</code> nombre del campo a mostrar en la web.
	 */
	protected void saveChooseOne(String field, String fieldWeb)
	{
		// Comprobamos que no sean vacos ninguno de los dos campos.
		if ((!ValidationUtils.isEmpty(field)) && (!ValidationUtils.isEmpty(fieldWeb)))
		{
			// Inicializamos los errores.
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_CHOOSE_ONE, fieldWeb));
		}
		else
		{
			// Llamamos al mtodo que sobreescribimos.
			saveChooseOne(field);
		}
	}
	
	/**
	 * <p>Almacena el error "seleccionar uno" dentro del objeto <code>ActionErrors</code>
	 * definido en el atributo <i>errors</i> refiri&eacute;ndose al campo <i>field</i> que 
	 * se recibe como pa&aacute;metro.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 */
	protected void saveSelectOne(String field)
	{
		// Comprobamos que no sea vaco.
		if (!ValidationUtils.isEmpty(field))
		{
			// Inicializamos los errores.
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_SELECT_ONE, field));
		}
	}
	
	/**
	 * <p>Almacena el error "seleccionar uno" dentro del objeto <code>ActionErrors</code> definido 
	 * en el atributo <i>error</i> refiri&eacute;ndose al campo <i>field</i> que se recibe como
	 * par&aacute;metro. El campo <i>fieldWeb</i> contedr&aacute; el nombre del campo tal y
	 * como ser&aacute; mostrado en la web.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 * @param fieldWeb <code>String</code> nombre del campo a mostrar en la web.
	 */
	protected void saveSelectOne(String field, String fieldWeb)
	{
		// Comprobamos que no sean vacos ninguno de los dos campos.
		if ((!ValidationUtils.isEmpty(field)) && (!ValidationUtils.isEmpty(fieldWeb)))
		{
			// Inicializamos los errores.
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_SELECT_ONE, fieldWeb));
		}
		else
		{
			// Llamamos al mtodo que sobreescribimos.
			saveChooseOne(field);
		}
	}
	
	/**
	 * <p>Almacena el error "no iguales" dentro del objeto <code>ActionErrors</code> definido
	 * en el atributo <i>error</i> refiri&eacute;ndose al campo <i>field</i> que se recibe 
	 * como par&aacute;metro. El campo <i>fieldWeb1</i> se refiere al primero de los campos
	 * que no son iguales y el campo <i>fieldWeb2</i> al segundo.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 * @param fieldWeb1 <code>String</code> nombre del primer campo a mostrar en la web.
	 * @param fieldWeb2 <code>String</code> nombre del segundo campo a mostrar en la web.
	 */
	protected void saveNotEqual(String field, String fieldWeb1, String fieldWeb2)
	{
		// Comprobamos que no sean vacos ninguno de los tres parmetros.
		if ((!ValidationUtils.isEmpty(field)) && (!ValidationUtils.isEmpty(fieldWeb1)) 
				&& (!ValidationUtils.isEmpty(fieldWeb2)))
		{
			// Inicializamos los errores.
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_NOT_EQUAL, fieldWeb1, fieldWeb2));
		}
	}
	
	/**
	 * <p>Almacena el error "not email" dentro del objeto <code>ActionErrors</code> definido
	 * en el atributo <i>errors</i> refiri&eacute;ndose al campo <i>field</i> que se recibe 
	 * como par&aacute;metro.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 */
	protected void saveNotEmail(String field)
	{
		// Comprobamos que no sea vaco el parmetro.
		if (!ValidationUtils.isEmpty(field))
		{
			// Inicializamos los errores.
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_NOT_EMAIL, field));
		}
	}
	
	/**
	 * <p>Almacena el error "not email" dentro del objeto <code>ActionErrors</code> definido
	 * en el atributo <i>errors</i> refiri&eacute;ndose al campo <i>field</i> que se recibe 
	 * como par&aacute;metro. El campo <i>fieldWeb</i> contedr&aacute; el nombre del campo tal y
	 * como ser&aacute; mostrado en la web.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 * @param fieldWeb <code>String</code> nombre del campo a mostrar en la web.
	 */
	protected void saveNotEmail(String field, String fieldWeb)
	{
		// Comprobamos que ambos parmetros no son nulos.
		if ((!ValidationUtils.isEmpty(field)) && (!ValidationUtils.isEmpty(fieldWeb)))
		{
			// Inicializamos los errores.
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_NOT_EMAIL, fieldWeb));
		}
		else 
		{
			// Llamamos al mtodo sobreescrito.
			saveNotEmail(field);
		}
	}
	
	/**
	 * <p>Almacena el error "no num&eeacute;rico" dentro del objeto <code>ActionErrors</code>
	 * definido en el atributo <i>errors</i> refiri&eacute;ndose al campo <i>field</i> que 
	 * se recibe como pa&aacute;metro.</p>
	 * @param field <code>String</code> campo al que corresponde el error.
	 */
	protected void saveNoNumber(String field)
	{
		// Comprobamos que no sea vaco.
		if (!ValidationUtils.isEmpty(field))
		{
			// Inicializamos los errores.
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_NO_NUMBER, field));
		}
	}
	
	/**
	 * <p>Almacena el error "no num&eeacute;rico" dentro del objeto <code>ActionErrors</code> definido 
	 * en el atributo <i>error</i> refiri&eacute;ndose al campo <i>field</i> que se recibe como
	 * par&aacute;metro. El campo <i>fieldWeb</i> contedr&aacute; el nombre del campo tal y
	 * como ser&aacute; mostrado en la web.</p>
	 * 
	 * @param field <code>String</code> campo al que corresponde el error.
	 * @param fieldWeb <code>String</code> nombre del campo a mostrar en la web.
	 */
	protected void saveNoNumber(String field, String fieldWeb)
	{
		// Comprobamos que no sean vacos ninguno de los dos campos.
		if ((!ValidationUtils.isEmpty(field)) && (!ValidationUtils.isEmpty(fieldWeb)))
		{
			// Inicializamos los errores.
			initErrors();
			// Grabamos el error.
			this.errors.add(field, new ActionMessage(ERRORS_KEY_NO_NUMBER, fieldWeb));
		}
		else
		{
			// Llamamos al mtodo que sobreescribimos.
			saveNoNumber(field);
		}
	}
	
	
	// -------- GETTERS & SETTERS.
	public String getCmd() 
	{
		return cmd;
	}

	public void setCmd(String cmd) 
	{
		this.cmd = cmd;
	}

}
