package edu.uoc.pfc.struts.forms;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMessage;

import edu.uoc.pfc.lib.utils.ValidationUtils;

/**
 * <p><code>ActionForm</code> utilizado en las transacciones de gesti&oacute;n de las
 * Aulas dentro de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class AFClassroom extends AFBase 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	/** <p>Identificador del Plan de Estudios.</p> */
	private String curriculumId;
	/** <p>Identificador de la Asignatura.</p> */
	private String subjectId;
	/** <p>Curso al que pertenece el Aula.</p> */
	private String course;
	/** <p>Letra del al Aula.</p> */
	private String letter;
	/** <p>Identificador de Profesor asignado a la Asignatura.</p> */
	private String teacherId;
	
	/** <p>Identificar del Cuestionario a asignar a un Aula.</p> */
	private String testId;
	/** <p>D&iacute;a de apertura del Cuestionario en el Aula.</p> */
	private String openDateDay;
	/** <p>Mes de apertura del Cuestionario en el Aula.</p> */
	private String openDateMonth;
	/** <p>A&ntilde;o de apertura del Cuestionario en el Aula.</p> */
	private String openDateYear;
	/** <p>D&iacute;a de cierre del Cuestionario en el Aula.</p> */
	private String closeDateDay;
	/** <p>Mes de cierre del Cuestionario en el Aula.</p> */
	private String closeDateMonth;
	/** <p>A&ntilde;o de cierre del Cuestionario en el Aula.</p> */
	private String closeDateYear;
	
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Retorna un objeto <code>ActionErrors</code> con los errores de validaci&oacute;n simples
	 * que hay en los datos del formulario durante el proceso de selecci&oacute;n del Plan de Estudios
	 * en el cual se va a crear un Aula. En el caso de que no haya errores este m&eeacute;todo
	 * retornar&aacute; un objeto <code>ActionErrors</code> vac&iacute;o.</p> 
	 * 
	 * @return <code>ActionErrors</code> con los errores de validaci&oacute;n simples en los datos
	 * del formulario.
	 */
	public ActionErrors validateCreateClassroomStep1()
	{
		// Borramos los posibles errores que tuvisemos.
		clearErrors();
		// Validamos el campo curriculumId
		validateCurriculumId();
		// retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	/**
	 * <p>Retorna un objeto <code>ActionErrors</code> con los errores de validaci&oacute;n simples
	 * que hay en los datos del formulario durante el proceso de creaci&oacute; de un Aula. 
	 * En el caso de que no haya errores este m&eeacute;todo retornar&aacute; un objeto <code>ActionErrors</code> 
	 * vac&iacute;o.</p> 
	 * 
	 * @return <code>ActionErrors</code> con los errores de validaci&oacute;n simples en los datos
	 * del formulario.
	 */
	public ActionErrors validateCreateClassroomStep2()
	{
		// Borramos los posibles errores que tuvisemos.
		clearErrors();
		// En este paso se supone que "curriculumId" tiene un valor correcto.
		// Validamos el campo "subjectId"
		validateSubjectId();
		// Validamos el campo "course"
		validateCourse();
		// Validamos el campo "letter"
		validateLetter();
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	/**
	 * <p>Retorna un objeto <code>ActionErrors</code> con los errores de validaci&oacute;n simples
	 * que hay en los datos del formulario durante el proceso de asignaci&oacute;n de un Profesor a 
	 * un Aula. En el caso de que no haya errores este m&eeacute;todo retornar&aacute; un objeto 
	 * <code>ActionErrors</code> vac&iacute;o.</p> 
	 * 
	 * @return <code>ActionErrors</code> con los errores de validaci&oacute;n simples en los datos
	 * del formulario.
	 */
	public ActionErrors validateAssignTeacher() 
	{
		// Borramos los posibles errores que tuvisemos.
		clearErrors();
		// Validamos el campo "teacherId"
		validateTeacherId();
		// Retornamos el objeto ActionErrors.
		return this.errors;
		
	}
	
	/**
	 * <p>Retorna un objeto <code>ActionErrors</code> con los errores de validaci&oacute;n simples
	 * que hay en los datos del formulario durante el proceso de asignaci&oacute;n de un Cuestionario
	 *  a un Aula. En el caso de que no haya errores este m&eeacute;todo retornar&aacute; un objeto 
	 *  <code>ActionErrors</code> vac&iacute;o.</p> 
	 * 
	 * @return <code>ActionErrors</code> con los errores de validaci&oacute;n simples en los datos
	 * del formulario.
	 */
	public ActionErrors validateAddTest()
	{
		// Borramos los posibles errores de validacin simples.
		clearErrors();
		// Validamos el campo "testId"
		validateTestId();
		// Validamos la fecha de publicacin.
		validateOpenDate();
		// Validamos la fecha de cierre.
		validateCloseDate();
		// Validamos que "openDate" sea menor o igual que "closeDate".
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());
		// Establecemos que value las fechas en sentido estricto (bisiestos).
		sdf.setLenient(false);
		try
		{
			// Interpretamos la fecha "openDate"
			Date openDate = sdf.parse(this.openDateDay + "/" + this.openDateMonth + "/" + this.openDateYear);
			// Interpretamos la fecha "closeDate"
			Date closeDate = sdf.parse(this.closeDateDay + "/" + this.closeDateMonth + "/" + this.closeDateYear);
			// Vamos si openDate es menor o igual que closeDate.
			if (openDate.getTime() > closeDate.getTime())
			{
				// openDate es mayor. Guardamos un error.
				this.errors.add("CloseDate", new ActionMessage("errors.no.date.invalid", 
						"Fecha de Publicacion", "Fecha de Cierre"));
			}
		} 
		catch (Exception ex)
		{
			// En la excepcin no hacemos nada ya que nunca se dar porque previamente hemos verificado
			// que tanto "openDate" como "closeDate" forman dos fecha vlidas.
		}
		// Retornamos el objeto ActionErrors.
		return this.errors;
	}
	
	/**
	 * <p>Retorna un objeto <code>Date</code> formado a partir de la informaci&oacute;n de los
	 * campos que definen la fecha de apertura de un Cuestionario en un Aula.</p>
	 * 
	 * @return <code>Date</code> representando la fecha de apertura del Cuestionario.
	 */
	public Date getOpenDate()
	{
		// Comprobamos que los atributos forman una fecha vlida.
		if (ValidationUtils.isValidDate(this.openDateDay, this.openDateMonth, this.openDateYear))
		{
			try
			{
				// Creamos un objeto SimpleDateFormat
				SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());
				// Establecemos que value las fechas en sentido estricto (bisiestos).
				sdf.setLenient(false);
				// Retornamos la fecha "openDate"
				return (sdf.parse(this.openDateDay + "/" + this.openDateMonth + "/" + this.openDateYear));
			}
			catch (Exception e)
			{
				// Esta excepcin no se va a producir ya que previamente hemos comprobado
				// que los atributos forman una fecha vlida. No hacemos nada en ella pero
				// si que tenemos que retornar un valor porque si no no compila.
				return null;
			}
		}
		else
		{
			// No tenemos una fecha vlida. Retornamos "null".
			return null;
		}
	}
	
	/**
	 * <p>Retorna un objeto <code>Date</code> formado a partir de la informaci&oacute;n de los
	 * campos que definen la fecha de cierre de un Cuestionario en un Aula.</p>
	 * 
	 * @return <code>Date</code> representando la fecha de cierre del Cuestionario.
	 */
	public Date getCloseDate()
	{
		// Comprobamos que los atributos forman una fecha vlida.
		if (ValidationUtils.isValidDate(this.closeDateDay, this.closeDateMonth, this.closeDateYear))
		{
			try
			{
				// Creamos un objeto SimpleDateFormat
				SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());
				// Establecemos que value las fechas en sentido estricto (bisiestos).
				sdf.setLenient(false);
				// Retornamos la fecha "closeDate"
				return (sdf.parse(this.closeDateDay + "/" + this.closeDateMonth + "/" + this.closeDateYear));
			}
			catch (Exception e)
			{
				// Esta excepcin no se va a producir ya que previamente hemos comprobado
				// que los atributos forman una fecha vlida. No hacemos nada en ella pero
				// si que tenemos que retornar un valor porque si no no compila.
				return null;
			}
		}
		else
		{
			// No tenemos una fecha vlida. Retornamos "null".
			return null;
		}
	}
	
	// -------- MTODOS PRIVADOS
	/**
	 * <p>Realiza las validaciones simples sobre la fecha de publicaci&oacute;n de un Cuesationario. 
	 * En el caso de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateOpenDate()
	{
		// Validamos que "openDateDay", "openDateMonth" y "openDateYear" forman una fecha.
		if (!ValidationUtils.isValidDate(this.openDateDay, this.openDateMonth, this.openDateYear))
		{
			// Fecha invlida. Grabamos un error.
			saveInvalidDate("OpenDate", "Fecha de Publicacion");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre la fecha de cierre de un Cuesationario. 
	 * En el caso de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateCloseDate()
	{
		// Validamos que "openDateDay", "openDateMonth" y "openDateYear" forman una fecha.
		if (!ValidationUtils.isValidDate(this.closeDateDay, this.closeDateMonth, this.closeDateYear))
		{
			// Fecha invlida. Grabamos un error.
			saveInvalidDate("CloseDate", "Fecha de Cierre");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>testId</i>. En el caso
	 * de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateTestId()
	{
		// Validamos que el campo no sea vaco.
		if (ValidationUtils.isEmail(this.testId))
		{
			// El campo est vaco. Grabamos un error.
			saveEmptyError("TestId", "Cuestionario");
		}
		else if ("_nil_".equals(this.testId))
		{
			// No se ha seleccionado un cuestionario.
			saveSelectOne("TestId", "Cuestionario");
		}
		else if (!ValidationUtils.isInteger(this.testId))
		{
			// No se ha enviado un valor entero.
			saveNoNumber("TestId", "Cuestionario");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>curriculumId</i>. En el caso
	 * de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateCurriculumId()
	{
		// Validamos que el campo no est vaco
		if (ValidationUtils.isEmpty(this.curriculumId))
		{
			// El campo "curriculum" est vo. Grabamos un error.
			saveEmptyError("CurriculumId", "Plan de Estudios");
		}
		else if ("_nil_".equals(this.curriculumId))
		{
			// No se ha seleccionado ninguna opcin del <select>
			saveSelectOne("CurriculumId", "Plan de Estudios");
		}
		else if (!ValidationUtils.isInteger(this.curriculumId))
		{
			// No se ha enviado un valor entero en "curriculumId"
			saveNoNumber("CurriculumId", "Plan de Estudio");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>subjectId</i>. En el caso
	 * de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateSubjectId()
	{
		// Validamos que el campo no est vaco
		if (ValidationUtils.isEmpty(this.subjectId))
		{
			// El campo "curriculum" est vo. Grabamos un error.
			saveEmptyError("SubjectId", "Asignatura");
		}
		else if ("_nil_".equals(this.subjectId))
		{
			// No se ha seleccionado ninguna opcin del <select>
			saveSelectOne("SubjectId", "Asignatura");
		}
		else if (!ValidationUtils.isInteger(this.subjectId))
		{
			// No se ha enviado un valor entero en "curriculumId"
			saveNoNumber("SubjectId", "Asignatura");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>course</i>. En el caso
	 * de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateCourse()
	{
		// Validamos que el campo no est vaco
		if (ValidationUtils.isEmpty(this.course))
		{
			// El campo "course" est vaci. Grabamos un error.
			saveEmptyError("Course", "Curso");
		}
		else if (!ValidationUtils.isInteger(this.course))
		{
			// El campo "course" no contiene un valor entero.
			saveNoNumber("Course", "Curso");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>letter</i>. En el caso
	 * de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateLetter()
	{
		// Validamos que el campo no est vaco
		if (ValidationUtils.isEmpty(this.letter))
		{
			// El campo "letter" est vaco. Grabamos un errors.
			saveEmptyError("Letter", "Letra");
		}
		else if ("_nil_".equals(this.letter))
		{
			// No se ha seleccionado una letra. Grabamos un error.
			saveSelectOne("Letter", "Letra");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>teacherId</i>. En el caso
	 * de que exista al&uacute;a error ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateTeacherId()
	{
		// Comprobamos si es vaco.
		if (ValidationUtils.isEmpty(this.teacherId))
		{
			// Grabamos el error "campo vaco".
			saveEmptyError("TeacherId", "Profesor");
		}
		else if ("_nil_".equals(this.teacherId))
		{
			// No se ha seleccionado un progesor. Guardamos un error.
			saveSelectOne("TeacherId", "Profesor");
		}
		else if (!ValidationUtils.isInteger(this.teacherId))
		{
			// El identificador del profesor no es numrico. Grabamos un
			// error indicando que elija un elemento de la lista desplegable.
			saveSelectOne("TeacherId", "Profesor");
		}
	}
	
	
	
	// -------- GETTERS & SETTERS
	public String getCurriculumId() 
	{
		return curriculumId;
	}
	public void setCurriculumId(String curriculumId) 
	{
		this.curriculumId = curriculumId;
	}
	public String getSubjectId()
	{
		return subjectId;
	}
	public void setSubjectId(String subjectId)
	{
		this.subjectId = subjectId;
	}
	public String getCourse()
	{
		return course;
	}
	public void setCourse(String course)
	{
		this.course = course;
	}
	public String getLetter()
	{
		return letter;
	}
	public void setLetter(String letter)
	{
		this.letter = letter;
	}
	public String getTeacherId() 
	{
		return teacherId;
	}
	public void setTeacherId(String teacherId) 
	{
		this.teacherId = teacherId;
	}
	public String getTestId() 
	{
		return testId;
	}
	public void setTestId(String testId) 
	{
		this.testId = testId;
	}
	public String getOpenDateDay() 
	{
		return openDateDay;
	}
	public void setOpenDateDay(String openDateDay) 
	{
		this.openDateDay = openDateDay;
	}
	public String getOpenDateMonth() 
	{
		return openDateMonth;
	}
	public void setOpenDateMonth(String openDateMonth) 
	{
		this.openDateMonth = openDateMonth;
	}
	public String getOpenDateYear() 
	{
		return openDateYear;
	}
	public void setOpenDateYear(String openDateYear) 
	{
		this.openDateYear = openDateYear;
	}
	public String getCloseDateDay() 
	{
		return closeDateDay;
	}
	public void setCloseDateDay(String closeDateDay) 
	{
		this.closeDateDay = closeDateDay;
	}
	public String getCloseDateMonth() 
	{
		return closeDateMonth;
	}
	public void setCloseDateMonth(String closeDateMonth) 
	{
		this.closeDateMonth = closeDateMonth;
	}
	public String getCloseDateYear() 
	{
		return closeDateYear;
	}
	public void setCloseDateYear(String closeDateYear) 
	{
		this.closeDateYear = closeDateYear;
	}
	
}
