package edu.uoc.pfc.struts.forms;

import org.apache.struts.action.ActionErrors;

import edu.uoc.pfc.lib.utils.BeanUtils;
import edu.uoc.pfc.lib.utils.ValidationUtils;

/**
 * <p><code>ActionForm</code> que ser&aacute; utilizada en las transacciones de gesti&oacute;n
 * de la Configuraci&oacute;n de la aplicaci&oacute;n.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class AFConfiguration extends AFBase
{

	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>N&uacute;mero de intentos de acceso a la aplicaci&oacute;n.</p> */
	private String loginAttempts;
	/** <p>Curso actual (activo).</p> */
	private String activeCourse;
	/** <p>Enviar correos electr&oacute;nicos?</p> */
	private String sendEmail;
	/** <p>Servidor SMTP.</p> */
	private String smtpServer;
	/** <p>Puerto SMTP.</p> */
	private String smtpPort;
	/** <p>El servidor SMTP requiere autentificaci&oacute;n?</p> */
	private String smtpAuthentication;
	/** <p>El servidor SMTP admite el protocolo TLS?</p> */
	private String smtpTLS;
	/** <p>Usuario SMTP.</p> */
	private String smtpUser;
	/** <p>Contrase&ntilde;a SMTP.</p> */
	private String smtpPassword;
	/** <p>Quien env&iacute;a el email de recordar contrase&ntilde;a.</p> */
	private String fpFrom;
	/** <p>Asunto del email para recordar contrase&ntilde;a.</p> */
	private String fpSubject;
	/** <p>Cuerpo del email para recordar contrase&ntilde;a.</p> */
	private String fpBody;
	/** <p>Quien env&iacute;a el email para recordar el login.</p> */
	private String flFrom;
	/** <p>Asunto del email para recoradar el nombre de usuario.</p> */
	private String flSubject;
	/** <p>Cuerpo del email para recordar el nombre de usuario.</p> */
	private String flBody;
	/** <p>Email del administrador.</p> */
	private String adminEmail;
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Realiza las validaciones simples sobre los datos del formulario cuando se est&aacute;
	 * realizando una operaci&oacute;n de modificaci&oacute;n de la Configuraci&oacueta;n.</p>
	 * 
	 * @return <code>ActionErrors</code> con los errores de validaci&oacute;n simples en los datos
	 * del formulario.
	 */
	public ActionErrors validateModify()
	{
		// Borramos los posibles errores que pudiramos tener.
		clearErrors();
		// Validamos "loginAttempts"
		validateLoginAttempts();
		// Validamos "activeCourse"
		validateActiveCourse();
		// Validamos el campo "adminEmail"
		validateAdminEmail();
		// Comprobamos si se deben validar las opciones de envo de correo electrnico.
		if ("yes".equals(this.sendEmail))
		{
			// Validamos el campo "smtpServer".
			validateSmtpServer();
			// Validamos el campo "smtpPort".
			validateSmtpPort();
			// Vemos si tenemos que validar las opciones de autentificacin.
			if ("yes".equals(this.smtpAuthentication))
			{
				// Validamos el campo "smtpUser".
				validateSmtpUser();
				// Validamos el campo "smtpPassword".
				validateSmtpPassword();
				// Validamos el campo "fpFrom"
				validateForgotPasswordFrom();
				// Validamos el campo "fpSubject"
				validateForgotPasswordSubject();
				// Validamos el campo "fpBody"
				validateForgotPasswordBody();
				// Validamos el campo "flFrom"
				validateForgotLoginFrom();
				// Validamos el campo "flSubject"
				validateForgotLoginSubject();
				// Validamos el campo "flBody"
				validateForgotLoginBody();
			}
		}
		// Comprobamos si ha habido errores de validacin simples.
		if (this.errors.size() > 0)
		{
			// Eliminamos espacios en blanco al principio y final de los campos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos los errores.
		return this.errors;
	}
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>loginAttempts</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateLoginAttempts()
	{
		// Vemos si el campo est vaco.
		if (ValidationUtils.isEmpty(this.loginAttempts))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("LoginAttempts", "Intentos de Acceso");
		}
		else if (!(ValidationUtils.isIntegerPositive(this.loginAttempts)))
		{
			// El campo no contiene un valor entero positivo.
			saveNoNumber("LoginAttempts", "Intentos de Acceso");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>course</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateActiveCourse()
	{
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.activeCourse))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("ActiveCourse", "Curso Actual");
		}
		else if (!ValidationUtils.isIntegerPositive(this.activeCourse))
		{
			// El campo no contiene un valor entero positivo.
			saveNoNumber("ActiveCourse", "Intentos de Acceso");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>adminEmail</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateAdminEmail()
	{
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.adminEmail))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("AdminEmail", "Email Administrador");
		}
		else if (!ValidationUtils.isEmail(this.adminEmail))
		{
			// Guardamos el error "email invlido"
			saveNotEmail("AdminEmail", "Email Administrador");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>smtpServer</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateSmtpServer()
	{
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.smtpServer))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("SmtpServer", "Servidor SMTP");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>smtpPort</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateSmtpPort()
	{
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.smtpPort))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("SmtpPort", "Puerto");
		}
		else if (!ValidationUtils.isIntegerPositive(this.smtpPort))
		{
			// El campo no contiene un valor entero positivo.
			saveNoNumber("SmtpPort", "Puerto");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>smtpUser</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateSmtpUser()
	{
		// Vemos si est vaco.
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.smtpUser))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("SmtpUser", "Usuario");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>smtpPassword</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateSmtpPassword()
	{
		// Vemos si est vaco.
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.smtpPassword))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("SmtpPassword", "Contresea");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>fpFrom</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateForgotPasswordFrom()
	{
		// Vemos si est vaco.
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.fpFrom))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("FpFrom", "De");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>fpSubject</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateForgotPasswordSubject()
	{
		// Vemos si est vaco.
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.fpSubject))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("FpSubject", "Asunto");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>fpBody</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateForgotPasswordBody()
	{
		// Vemos si est vaco.
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.fpBody))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("FpBody", "Texto");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>flFrom</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateForgotLoginFrom()
	{
		// Vemos si est vaco.
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.flFrom))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("FlFrom", "De");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>flSubject</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateForgotLoginSubject()
	{
		// Vemos si est vaco.
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.flSubject))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("FlSubject", "Asunto");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el campo <i>fpBody</i>. En el caso
	 * de que exista alg&uacute;n error este ser&aacute; depositado dentro del objeto
	 * <code>ActionErrors</code> asociado al atributo <i>errors</i>.</p>
	 */
	private void validateForgotLoginBody()
	{
		// Vemos si est vaco.
		// Vemos si est vaco.
		if (ValidationUtils.isEmpty(this.flBody))
		{
			// Guardamos el error "campo vaco".
			saveEmptyError("FlBody", "Texto");
		}
	}
	
	// -------- GETTERS & SETTERS.
	public String getLoginAttempts()
	{
		return loginAttempts;
	}
	public void setLoginAttempts(String loginAttempts)
	{
		this.loginAttempts = loginAttempts;
	}
	public String getActiveCourse()
	{
		return activeCourse;
	}
	public void setActiveCourse(String activeCourse)
	{
		this.activeCourse = activeCourse;
	}
	public String getSendEmail()
	{
		return sendEmail;
	}
	public void setSendEmail(String sendEmail)
	{
		this.sendEmail = sendEmail;
	}
	public String getSmtpServer()
	{
		return smtpServer;
	}
	public void setSmtpServer(String smtpServer)
	{
		this.smtpServer = smtpServer;
	}
	public String getSmtpPort()
	{
		return smtpPort;
	}
	public void setSmtpPort(String smtpPort)
	{
		this.smtpPort = smtpPort;
	}
	public String getSmtpAuthentication()
	{
		return smtpAuthentication;
	}
	public void setSmtpAuthentication(String smtpAuthentication)
	{
		this.smtpAuthentication = smtpAuthentication;
	}
	public String getSmtpTLS()
	{
		return smtpTLS;
	}
	public void setSmtpTLS(String smtpTLS)
	{
		this.smtpTLS = smtpTLS;
	}
	public String getSmtpUser()
	{
		return smtpUser;
	}
	public void setSmtpUser(String smtpUser)
	{
		this.smtpUser = smtpUser;
	}
	public String getSmtpPassword()
	{
		return smtpPassword;
	}
	public void setSmtpPassword(String smtpPassword)
	{
		this.smtpPassword = smtpPassword;
	}
	public String getFpFrom()
	{
		return fpFrom;
	}
	public void setFpFrom(String fpFrom)
	{
		this.fpFrom = fpFrom;
	}
	public String getFpBody()
	{
		return fpBody;
	}
	public void setFpBody(String fpBody)
	{
		this.fpBody = fpBody;
	}
	public String getFlFrom()
	{
		return flFrom;
	}
	public void setFlFrom(String flFrom)
	{
		this.flFrom = flFrom;
	}
	public String getFlSubject()
	{
		return flSubject;
	}
	public void setFlSubject(String flSubject)
	{
		this.flSubject = flSubject;
	}
	public String getFlBody()
	{
		return flBody;
	}
	public void setFlBody(String flBody)
	{
		this.flBody = flBody;
	}
	public String getFpSubject()
	{
		return fpSubject;
	}
	public void setFpSubject(String fpSubject)
	{
		this.fpSubject = fpSubject;
	}
	public String getAdminEmail()
	{
		return adminEmail;
	}
	public void setAdminEmail(String adminEmail)
	{
		this.adminEmail = adminEmail;
	}
	
}
