package edu.uoc.pfc.struts.forms;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMessage;

import edu.uoc.pfc.business.transfers.TOQuestion;
import edu.uoc.pfc.business.transfers.TOQuestionMatch;
import edu.uoc.pfc.business.transfers.TOQuestionMatchAnswer;
import edu.uoc.pfc.business.transfers.TOQuestionMultipleChoices;
import edu.uoc.pfc.business.transfers.TOQuestionMultipleChoicesAnswer;
import edu.uoc.pfc.business.transfers.TOQuestionNumeric;
import edu.uoc.pfc.business.transfers.TOQuestionShortAnswer;
import edu.uoc.pfc.business.transfers.TOQuestionTrueOrFalse;
import edu.uoc.pfc.lib.utils.BeanUtils;
import edu.uoc.pfc.lib.utils.ValidationUtils;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <p><code>ActionForm</code> que ser&aacute; utilizada en todas las trasacciones relacionadas
 * con la gesti&oacute;n del Banco de Preguntas de un Profesor.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class AFQuestionBank extends AFBase
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS
	/** <p>Identificador de la Categor&iacute;a de Preguntas.</p> */
	private String categoryId;
	/** <p>Nombre de una Categor&iacute;a de Preguntas.</p> */
	private String categoryName;
	/** <p>Descripci&oacugte;n de una Categor&iacute;a de Preguntas.</p> */
	private String categoryDescription;
	
	/** <p>T&iacute;tulo de la Pregunta.</p> */
	private String title;
	/** <p>Enunciado de la Pregunta.</p> */
	private String statement;
	/** <p>Identificador de la Pregunta.</p> */
	private String id;
	/** <p>Tipo de pregunta.</p> */
	private String type;
	
	/** <p>Pregunta Verdadero/Falso: Respuesta correcta.</p> */
	private String tfCorrectAnswer;
	/** <p>Pregunta Verdadero/Falso: Refuerzo opci&oacute;n verdadero.</p> */
	private String tfTextTrue;
	/** <p>Pregunta Verdadero/Falso: Refuerzo opci&oacute;n falso.</p> */
	private String tfTextFalse;
	
	/** <p>Pregunta Num&eacuet;rica: Valor correcto.</p> */
	private String nuValue;
	/** <p>Pregunta Num&eacuet;rica: Margen de error.</p> */
	private String nuError;
	/** <p>Pregunta Num&eacuet;rica: Texto de refuerzo.</p> */
	private String nuText;
	
	/** <p>Pregunta Respuesta Corta: Respuesta.</p> */
	private String saAnswer;
	/** <p>Pregunta Respuesta Corta: Distiguir May&uacute;sculas.</p> */
	private String saCaseSensitive;
	/** <p>Pregunta Respuesta Corta: Texto de refuerzo.</p> */
	private String saText;
	
	/** <p>Opciones de Respuesta M&uacute;ltiple.</p> */
	private String mcAnswer1;
	private String mcAnswer2;
	private String mcAnswer3;
	private String mcAnswer4;
	private String mcAnswer5;
	/** <p>Indica si la opci&oacute;n es correcta en una pregunta de Respuesta M&uacute;ltiple.</p> */
	private String mcCorrect1;
	private String mcCorrect2;
	private String mcCorrect3;
	private String mcCorrect4;
	private String mcCorrect5;
	/** <p>Texto de Refuerzo en las opciones de respuesta de un Pregunta con M&uacute;ltiples Opciones.</p> */
	private String mcText1;
	private String mcText2;
	private String mcText3;
	private String mcText4;
	private String mcText5;
	
	/** <p>Elementos de la izquierda en una Respuesta de Emparejar.</p> */
	private String maTextLeft1;
	private String maTextLeft2;
	private String maTextLeft3;
	private String maTextLeft4;
	private String maTextLeft5;
	/** <p>Elementos de la derecha en una Respuesta de Emparejar.</p> */
	private String maTextRight1;
	private String maTextRight2;
	private String maTextRight3;
	private String maTextRight4;
	private String maTextRight5;
	/** <p>Textos de refuerzo de las opciones a emparejar.</p> */
	private String maExplain1;
	private String maExplain2;
	private String maExplain3;
	private String maExplain4;
	private String maExplain5;
	
	// -------- MTODOS PBLICOS
	/**
	 * <p>Retorna un objeto <code>ActionErrors</code> con todos los errores de validaci&oacute;n
	 * simples que pueda haya en el formulario de creaci&oacute;n de una nueva Categor&iacute;a 
	 * de Preguntas en el sistema.</p>
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n simples. Si no
	 * los hay este objeto estar&aacute; vac&iacute;o.
	 */
	public ActionErrors validateCreateCategory()
	{
		// Borramos los posibles errores de validacin simples.
		clearErrors();
		// Validamos el nombre de la categora.
		validateCategoryName();
		// Comprobamos si hay errors.
		if (this.errors.size() == 0)
		{
			// No hay errores de validacin simples por lo que eliminamos los posibles 
			// espacios en blanco de los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos la lista con los errores de validacin.
		return this.errors;
	}
	
	/**
	 * <p>M&eacute;todo que realiza la validaci&oacute;n simple sobre los datos
	 * del formulario cuando se pretende crear una nueva pregunta dentro del sistema
	 * de tipo "Verdadero o Falso".</p>
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples. Si no los hay este objeto estar&aacute; vac&iacute;o.
	 */
	public ActionErrors validateCreateQuestionTrueOrFalse()
	{
		// Borramos los errores de validacin simples.
		clearErrors();
		// Validamos el campo "title"
		validateTitle();
		// Validamos el campo "statement"
		validateStatement();
		// Comprobamos si ha habido errores de validacin simples.
		if (this.errors.size() == 0)
		{
			// No hay errores de validacin simples por lo que eliminamos los posibles 
			// espacios en blanco de los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos la lista con los errores de validacin.
		return this.errors;
	}
	
	/**
	 * <p>M&eacute;todo que realiza la validaci&oacute;n simple sobre los datos
	 * del formulario cuando se pretende crear una nueva pregunta dentro del sistema
	 * de tipo "Num&eacute;rico".</p>
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples. Si no los hay este objeto estar&aacute; vac&iacute;o.
	 */
	public ActionErrors validateCreateQuestionNumeric()
	{
		// Borramos los errores de validacin simples.
		clearErrors();
		// Validamos el campo "title"
		validateTitle();
		// Validamos el campo "statement"
		validateStatement();
		// Validamos el campo "nuValue"
		validateNuValue();
		// Validamos el campo "nuError"
		validateNuError();
		// Comprobamos si ha habido errores de validacin simples.
		if (this.errors.size() == 0)
		{
			// No hay errores de validacin simples por lo que eliminamos los posibles 
			// espacios en blanco de los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos la lista con los errores de validacin.
		return this.errors;
	}
	
	/**
	 * <p>M&eacute;todo que realiza la validaci&oacute;n simple sobre los datos
	 * del formulario cuando se pretende crear una nueva pregunta dentro del sistema
	 * de tipo "Respuesta Corta".</p>
	 * 
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples. Si no los hay este objeto estar&aacute; vac&iacute;o.
	 */
	public ActionErrors validateCreateQuestionShortAnswer()
	{
		// Borramos los errores de validacin simples.
		clearErrors();
		// Validamos el campo "title"
		validateTitle();
		// Validamos el campo "statement"
		validateStatement();
		// Validamos el campo "saAnswer"
		validateSaAnswer();
		// Comprobamos si ha habido errores de validacin simples.
		if (this.errors.size() == 0)
		{
			// No hay errores de validacin simples por lo que eliminamos los posibles 
			// espacios en blanco de los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos la lista con los errores de validacin.
		return this.errors; 
	}
	
	/**
	 * <p>M&eacute;todo que realiza la validaci&oacute;n simple sobre los datos
	 * del formulario cuando se pretende crear una nueva pregunta dentro del sistema
	 * de tipo "Emparejar".</p>
	 * 
	 * @param request <code>HttpServletRequest</code> asociado a la petici&oacute; de
	 * creaci&oacute;n de la pregunta.
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples. Si no los hay este objeto estar&aacute; vac&iacute;o.
	 */
	public ActionErrors validateCreateQuestionMatch(HttpServletRequest request)
	{
		// Borramos los posibles errores de validacin que tuvisemos.
		clearErrors();
		// Validamos "title"
		validateTitle();
		// Validamos "statement"
		validateStatement();
		// Declaramos una variable local para contar el nmero de parejas formadas.
		int total = 0;
		// Vemos las opciones de pareja una a una. Pareja: 1.
		if (((ValidationUtils.isEmpty(this.maTextLeft1)) && (!ValidationUtils.isEmpty(this.maTextRight1)))
				|| ((!ValidationUtils.isEmpty(this.maTextLeft1)) && (ValidationUtils.isEmpty(this.maTextRight1))))
		{
			// Solamente se ha especificado una de las dos opciones de la pareja 1.
			this.errors.add("Match_1", new ActionMessage("question.ma.form.invalid.match"));
		}
		else if ((!(ValidationUtils.isEmpty(this.maTextLeft1))) && (!(ValidationUtils.isEmpty(this.maTextRight1))))
		{
			// Incrementamos el nmero de parejas.
			total++;
		}
		// Pareja 2.
		if (((ValidationUtils.isEmpty(this.maTextLeft2)) && (!ValidationUtils.isEmpty(this.maTextRight2)))
				|| ((!ValidationUtils.isEmpty(this.maTextLeft2)) && (ValidationUtils.isEmpty(this.maTextRight2))))
		{
			// Solamente se ha especificado una de las dos opciones de la pareja 2.
			this.errors.add("Match_2", new ActionMessage("question.ma.form.invalid.match"));
		}
		else if ((!(ValidationUtils.isEmpty(this.maTextLeft2))) && (!(ValidationUtils.isEmpty(this.maTextRight2))))
		{
			// Incrementamos el nmero de parejas.
			total++;
		}
		// Pareja 3.
		if (((ValidationUtils.isEmpty(this.maTextLeft3)) && (!ValidationUtils.isEmpty(this.maTextRight3)))
				|| ((!ValidationUtils.isEmpty(this.maTextLeft3)) && (ValidationUtils.isEmpty(this.maTextRight3))))
		{
			// Solamente se ha especificado una de las dos opciones de la pareja 3.
			this.errors.add("Match_3", new ActionMessage("question.ma.form.invalid.match"));
		}
		else if ((!(ValidationUtils.isEmpty(this.maTextLeft3))) && (!(ValidationUtils.isEmpty(this.maTextRight3))))
		{
			// Incrementamos el nmero de parejas.
			total++;
		}
		// Pareja 4.
		if (((ValidationUtils.isEmpty(this.maTextLeft4)) && (!ValidationUtils.isEmpty(this.maTextRight4)))
				|| ((!ValidationUtils.isEmpty(this.maTextLeft4)) && (ValidationUtils.isEmpty(this.maTextRight4))))
		{
			// Solamente se ha especificado una de las dos opciones de la pareja 4.
			this.errors.add("Match_4", new ActionMessage("question.ma.form.invalid.match"));
		}
		else if ((!(ValidationUtils.isEmpty(this.maTextLeft4))) && (!(ValidationUtils.isEmpty(this.maTextRight4))))
		{
			// Incrementamos el nmero de parejas.
			total++;
		}
		// Pareja 5.
		if (((ValidationUtils.isEmpty(this.maTextLeft5)) && (!ValidationUtils.isEmpty(this.maTextRight5)))
				|| ((!ValidationUtils.isEmpty(this.maTextLeft5)) && (ValidationUtils.isEmpty(this.maTextRight5))))
		{
			// Solamente se ha especificado una de las dos opciones de la pareja 5.
			this.errors.add("Match_5", new ActionMessage("question.ma.form.invalid.match"));
		}
		else if ((!(ValidationUtils.isEmpty(this.maTextLeft5))) && (!(ValidationUtils.isEmpty(this.maTextRight5))))
		{
			// Incrementamos el nmero de parejas.
			total++;
		}
		// Comprobamos el nmero de parejas formadas.
		if (total < 2)
		{
			// Indicamos en la request que ha habido un error.
			request.setAttribute("error_vs", "no_two_options");
			// Aadimos un elemento al ActionErrors.
			this.errors.add("", new ActionMessage("common.no.error"));
		}
		// Comprobamos si ha habido errores de validacin simples.
		if (this.errors.size() == 0)
		{
			// No hay errores de validacin simples por lo que eliminamos los posibles 
			// espacios en blanco de los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos la lista con los errores de validacin.
		return this.errors; 
	}
	
	/**
	 * <p>M&eacute;todo que realiza la validaci&oacute;n simple sobre los datos
	 * del formulario cuando se pretende crear una nueva pregunta dentro del sistema
	 * de tipo "M&uacute;tiples Opciones".</p>
	 * 
	 * @param request <code>HttpServletRequest</code> asociado a la petici&oacute; de
	 * creaci&oacute;n de la pregunta.
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples. Si no los hay este objeto estar&aacute; vac&iacute;o.
	 */
	public ActionErrors validateCreateQuestionMultipleChoices(HttpServletRequest request)
	{
		// Borramos los errores de validacin simples.
		clearErrors();
		// Validamos el campo "title"
		validateTitle();
		// Validamos el campo "statement"
		validateStatement();
		// Validamos que se hayan escrito al menos dos opciones de respuesta.
		// Declaramos una variable local.
		int totalAnswers = 0;
		// Contamos el nmero de opciones de respuesta.
		if (!ValidationUtils.isEmpty(this.mcAnswer1)) totalAnswers++;
		if (!ValidationUtils.isEmpty(this.mcAnswer2)) totalAnswers++;
		if (!ValidationUtils.isEmpty(this.mcAnswer3)) totalAnswers++;
		if (!ValidationUtils.isEmpty(this.mcAnswer4)) totalAnswers++;
		if (!ValidationUtils.isEmpty(this.mcAnswer5)) totalAnswers++;
		// Vemos si tenemso 2 o ms opciones escritas.
		if (totalAnswers < 2)
		{
			// Indicamos que ha habido el error "indicar dos o ms respuestas".
			request.setAttribute("error_vs", "no_two_answers");
			// Aadimos un elemento al ActionErrors.
			this.errors.add("", new ActionMessage("common.no.error"));
		}
		else 
		{
			// Vamos a contar el nmero de respuestas correctas. Inicializamos el contador
			totalAnswers = 0;
			// Contamos el nmero de opciones correctas.
			if ("op_true".equals(this.mcCorrect1)) totalAnswers++;
			if ("op_true".equals(this.mcCorrect2)) totalAnswers++;
			if ("op_true".equals(this.mcCorrect3)) totalAnswers++;
			if ("op_true".equals(this.mcCorrect4)) totalAnswers++;
			if ("op_true".equals(this.mcCorrect5)) totalAnswers++;
			// Vemos el nmero total de opciones correctas.
			if (totalAnswers < 1)
			{
				// Indicamos que se produce el error "indicar una respuesta correcta".
				request.setAttribute("error_vs", "no_one_correct");
				// Aadimos un elemento al ActionErrors.
				this.errors.add("key", new ActionMessage(ERRORS_KEY_CHOOSE_ONE, "key"));
			}
			else 
			{
				// Tenemos que todas las opciones marcadas como correctas 
				// tienen un texto asociado. Definimos una variable local.
				boolean hasErrors = false;
				// Comprobamos una a una las opciones de respuesta.
				if (("op_true".equals(this.mcCorrect1)) && (ValidationUtils.isEmpty(this.mcAnswer1))) hasErrors = true;
				if (("op_true".equals(this.mcCorrect2)) && (ValidationUtils.isEmpty(this.mcAnswer2))) hasErrors = true;
				if (("op_true".equals(this.mcCorrect3)) && (ValidationUtils.isEmpty(this.mcAnswer3))) hasErrors = true;
				if (("op_true".equals(this.mcCorrect4)) && (ValidationUtils.isEmpty(this.mcAnswer4))) hasErrors = true;
				if (("op_true".equals(this.mcCorrect5)) && (ValidationUtils.isEmpty(this.mcAnswer5))) hasErrors = true;
				// Comprobamos si hay errores.
				if (hasErrors)
				{
					// Indicamos que se produce el error "error de formato".
					request.setAttribute("error_vs", "invalid_format");
					// Aadimos un elemento al ActionErrors.
					this.errors.add("key", new ActionMessage(ERRORS_KEY_CHOOSE_ONE, "key"));
				}
			}
		}
		// Comprobamos si ha habido errores de validacin simples.
		if (this.errors.size() == 0)
		{
			// No hay errores de validacin simples por lo que eliminamos los posibles 
			// espacios en blanco de los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos la lista con los errores de validacin.
		return this.errors; 
	}
	
	/**
	 * <p>Establece los valores de los atributos del formulario en funci&oacute;n 
	 * del tipo de pregunta del que se trata.</p>
	 * 
	 * @param question <code>TOQuestion</code> que contiene los valores a asignar
	 * a los atributos del formulario.
	 */
	public void setData(TOQuestion question)
	{
		// Comprobamos que el parmetro no es nulo.
		if (question != null)
		{
			// Determinamos el tipo de Pregunta del que se trata.
			if (question instanceof TOQuestionTrueOrFalse)
			{
				// Llamamos al mtodo para las preguntas: Verdadero o False.
				setData((TOQuestionTrueOrFalse) question);
			}
			else if (question instanceof TOQuestionNumeric)
			{
				// Llamamos al mtodo para las pregunta: Numricas.
				setData((TOQuestionNumeric) question);
			}
			else if (question instanceof TOQuestionShortAnswer)
			{
				// Llamamos al mtodo para las preguntas: Respuesta Corta.
				setData((TOQuestionShortAnswer) question);
			}
			else if (question instanceof TOQuestionMultipleChoices)
			{
				// Llamamos al mtodo para las preguntas: Mltiples Opciones.
				setData((TOQuestionMultipleChoices) question);
			}
			else if (question instanceof TOQuestionMatch)
			{
				// Llamamos al mtodo para las preguntas: Emparejar.
				setData((TOQuestionMatch) question);
			}
		}
	}
	
	/**
	 * <p>M&eacute;todo que realiza la validaci&oacute;n simple sobre los datos
	 * del formulario durante una operaci&oacute;n de modificaci&oacute;n de una
	 * pregunta en funci&oacute;n del tipo de pregunta del que se trate.</p>
	 * 
	 * @parama request <code>HttpServletRequest</code> asociado a la petici&oacute;n
	 * de validaci&oacute;n.
	 * @return <code>ActionErrors</code> con todos los errores de validaci&oacute;n 
	 * simples. Si no los hay este objeto estar&aacute; vac&iacute;o.
	 */
	public ActionErrors validateModify(HttpServletRequest request)
	{
		// Borramos los posibles errores de validacin.
		clearErrors();
		// Vemos el tipo de pregunta ante el que estamos.
		if (TOQuestion.TYPE_TRUE_OR_FALSE.equals(this.type))
		{
			// Pregunta tipo: Verdadero / Falso.
			validateModifyQuestionTrueOrFalse();
		}
		else if (TOQuestion.TYPE_NUMERIC.equals(this.type))
		{
			// Pregunta tipo: Numrico.
			validateModifyQuestionNumeric();
		}
		else if (TOQuestion.TYPE_SHORT_ANSWER.equals(this.type))
		{
			// Pregunta tipo: Respuesta Corta.
			validateModifyQuestionShortAnswer();
		}
		else if (TOQuestion.TYPE_MULTIPLE_CHOICES.equals(this.type))
		{
			// Pregunta tipo: Mltiples opciones.
			validateModifyQuestionMultipleChoices(request);
		}
		else if (TOQuestion.TYPE_MATCH.equals(this.type))
		{
			// Pregunta de tipo: Emparejar.
			validateModifyQuestionMatch(request);
		}
		// Vemos si ha habido errores.
		if (this.errors.size() == 0)
		{
			// No hay errores de validacin simples por lo que eliminamos los posibles 
			// espacios en blanco de los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos la lista con los errores de validacin.
		return this.errors;

	}
	
	// --------MTODOS PRIVADOS
	/**
	 * <p>Realiza las validaciones simples sobre el atributo "saAnswer" depositando los
	 * errores que se encuentre en el campo "errors".
	 */
	private void validateSaAnswer()
	{
		// Comprobamos que no est vaco.
		if (ValidationUtils.isEmpty(this.saAnswer))
		{
			// El campo est vaco por lo que guardamos un mensaje de error.
			saveEmptyError("SaAnswer", "Respuesta");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el atributo "nuValue" depositando los
	 * errores que se encuentre en el campo "errors".
	 */
	private void validateNuValue()
	{
		// Comprobamos que no sea vaco.
		if (ValidationUtils.isEmpty(this.nuValue))
		{
			// El campo est vaco as que guardamos el error.
			saveEmptyError("NuValue", "Respuesta Correcta");
		}
		else if (!ValidationUtils.isDouble(this.nuValue))
		{
			// El campo no contiene un valor double. Guardamos un error.
			saveNoNumber("NuValue", "Respuesta Correcta");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el atributo "nuError" depositando los
	 * errores que se encuentre en el campo "errors".
	 */
	private void validateNuError()
	{
		// Comprobamos que no sea vaco.
		if (ValidationUtils.isEmpty(this.nuError))
		{
			// El campo est vaco as que guardamos el error.
			saveEmptyError("NuError", "Margen de Error");
		}
		else if (!ValidationUtils.isDouble(this.nuError))
		{
			// El campo no contiene un valor double. Guardamos un error.
			saveNoNumber("NuError", "Margen de Error");
		}
	}
	
	
	/**
	 * <p>Realiza las validaciones simples sobre el atributo "categoryName" depositando los
	 * errores que se encuentre en el campo "errors".
	 */
	private void validateCategoryName()
	{
		// Comprobamos que no sea vaco.
		if (ValidationUtils.isEmpty(this.categoryName))
		{
			// El campo est vaco as que guardamos el error.
			saveEmptyError("CategoryName", "Nombre");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el atributo "title" depositando los
	 * errores que se encuentre en el campo "errors".
	 */
	private void validateTitle()
	{
		// Comprobamos que no est vaco.
		if (ValidationUtils.isEmpty(this.title))
		{
			// El campo est vaco. Guardamos un error.
			saveEmptyError("Title", "Titulo");
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre el atributo "statement" depositando los
	 * errores que se encuentre en el campo "errors".
	 */
	private void validateStatement()
	{
		// Comprobamos que no est vaco.
		if (ValidationUtils.isEmpty(this.statement))
		{
			// El campo est vaco. Guardamos el error.
			saveEmptyError("Statement", "Enunciado");
		}
	}
	
	/**
	 * <p>Establece los valores de los atributos a los valores que se reciben en el 
	 * objeto <code>TOQuestionTrueOrFalse</code> que se recibe como par&aacute;metro.</p>
	 *  
	 * @param to <code>TOQuestionTrueOrFalse</code> con la informaci&oacute;n para 
	 * los atributos. 
	 */
	private void setData(TOQuestionTrueOrFalse to)
	{
		// Comprobamos que el parmetro no sea nulo.
		if (to != null)
		{
			// Establecemos los valores de los atributos.
			this.id = Integer.toString(to.getId());
			this.title = WebUtils.HTMLtoString(to.getTitle());
			this.statement = WebUtils.HTMLtoString(to.getStatement());
			// Valor de "tfCorrectAnswer"
			if (to.isValue()) this.tfCorrectAnswer = "op_true";
			else this.tfCorrectAnswer = "op_false";
			// Resto de atributos.
			this.tfTextTrue = WebUtils.HTMLtoString(to.getTxtTrue());
			this.tfTextFalse = WebUtils.HTMLtoString(to.getTxtFalse());
			this.type = to.getType();
		}
	}
	
	/**
	 * <p>Establece los valores de los atributos a los valores que se reciben en el 
	 * objeto <code>TOQuestionNumeric</code> que se recibe como par&aacute;metro.</p>
	 *  
	 * @param to <code>TOQuestionNumeric</code> con la informaci&oacute;n para 
	 * los atributos. 
	 */
	private void setData(TOQuestionNumeric to)
	{
		// Comprobamos que el parmetro no sea nulo.
		if (to != null)
		{
			// Establecemos los valores de los atributos.
			this.id = Integer.toString(to.getId());
			this.title = WebUtils.HTMLtoString(to.getTitle());
			this.statement = WebUtils.HTMLtoString(to.getStatement());
			this.nuValue = Double.toString(to.getValue());
			this.nuError = Double.toString(to.getError());
			this.nuText = WebUtils.HTMLtoString(to.getText());
			this.type = to.getType();
		}
	}
	
	/**
	 * <p>Establece los valores de los atributos a los valores que se reciben en el 
	 * objeto <code>TOQuestionMatch</code> que se recibe como par&aacute;metro.</p>
	 *  
	 * @param to <code>TOQuestionMatch</code> con la informaci&oacute;n para 
	 * los atributos. 
	 */
	private void setData(TOQuestionMatch to)
	{
		// Comprobamos que el parmetro no es nulo.
		if (to != null)
		{
			// Establecemos los valores de los atributos.
			this.id = Integer.toString(to.getId());
			this.title = WebUtils.HTMLtoString(to.getTitle());
			this.statement = WebUtils.HTMLtoString(to.getStatement());
			this.type = to.getType();
			// Comprobamos que tenemos opciones de respuesta.
			if (to.getAnswers() != null)
			{
				// Definimos un contador para establecer los valores de las respuestas.
				int count = 1;
				// Recorremos todas las respuestas.
				for (TOQuestionMatchAnswer answer : to.getAnswers())
				{
					// En funcin del valor de "count" establecemos los valores del formulario.
					switch (count)
					{
						case 1 :
							// Opcin 1
							this.maTextLeft1 = WebUtils.HTMLtoString(answer.getText1());
							this.maTextRight1 = WebUtils.HTMLtoString(answer.getText2());
							this.maExplain1 = WebUtils.HTMLtoString(answer.getExplain());
							break;
						case 2 :
							// Opcin 2
							this.maTextLeft2 = WebUtils.HTMLtoString(answer.getText1());
							this.maTextRight2 = WebUtils.HTMLtoString(answer.getText2());
							this.maExplain2 = WebUtils.HTMLtoString(answer.getExplain());
							break;
						case 3 :
							// Opcin 3
							this.maTextLeft3 = WebUtils.HTMLtoString(answer.getText1());
							this.maTextRight3 = WebUtils.HTMLtoString(answer.getText2());
							this.maExplain3 = WebUtils.HTMLtoString(answer.getExplain());
							break;
						case 4 :
							// Opcin 4
							this.maTextLeft4 = WebUtils.HTMLtoString(answer.getText1());
							this.maTextRight4 = WebUtils.HTMLtoString(answer.getText2());
							this.maExplain4 = WebUtils.HTMLtoString(answer.getExplain());
							break;
						case 5 :
							// Opcin 5
							this.maTextLeft5 = WebUtils.HTMLtoString(answer.getText1());
							this.maTextRight5 = WebUtils.HTMLtoString(answer.getText2());
							this.maExplain5 = WebUtils.HTMLtoString(answer.getExplain());
					}
					// Incrementamos el valor de count.
					count++;
				}
			}
		}
	}
	
	/**
	 * <p>Establece los valores de los atributos a los valores que se reciben en el 
	 * objeto <code>TOQuestionMultipleChoices</code> que se recibe como par&aacute;metro.</p>
	 *  
	 * @param to <code>TOQuestionMultipleChoices</code> con la informaci&oacute;n para 
	 * los atributos. 
	 */
	private void setData(TOQuestionMultipleChoices to)
	{
		// Comprobamos que el atributo no sea nulo.
		if (to != null)
		{
			// Establecemos los valores de los atributos.
			this.id = Integer.toString(to.getId());
			this.title = WebUtils.HTMLtoString(to.getTitle());
			this.statement = WebUtils.HTMLtoString(to.getStatement());
			this.type = to.getType();
			// Comprobamos que tenemos respuesta.
			if (to.getAnswers() != null)
			{
				// Declaramos un contador para establecer los valores a partir de las opciones.
				int count = 1;
				// Recorremos todas las respuestas.
				for (TOQuestionMultipleChoicesAnswer answer : to.getAnswers())
				{
					// Establecemos los valores de los atributos en funcin de count.
					switch (count)
					{
						case 1 :
							// Establecemos los valores de los atributos para la primera respuesta.
							this.mcAnswer1 = WebUtils.HTMLtoString(answer.getAnswer());
							this.mcText1 = WebUtils.HTMLtoString(answer.getText());
							// Vemos si es correcta o no para establecer "mcCorrect1"
							if (answer.isCorrect()) this.mcCorrect1 = "op_true";
							else this.mcCorrect1 = "op_false";
							break;
						case 2 :
							// Igual que antes pero para la opcin 2.
							this.mcAnswer2 = WebUtils.HTMLtoString(answer.getAnswer());
							this.mcText2 = WebUtils.HTMLtoString(answer.getText());
							// Vemos si es correcta o no para establecer "mcCorrect2"
							if (answer.isCorrect()) this.mcCorrect2 = "op_true";
							else this.mcCorrect2 = "op_false";
							break;
						case 3 :
							// Ahora la opcin 3
							this.mcAnswer3 = WebUtils.HTMLtoString(answer.getAnswer());
							this.mcText3 = WebUtils.HTMLtoString(answer.getText());
							// Vemos si es correcta o no para establecer "mcCorrect3"
							if (answer.isCorrect()) this.mcCorrect3 = "op_true";
							else this.mcCorrect3 = "op_false";
							break;
						case 4 :
							// Opcin 4
							this.mcAnswer4 = WebUtils.HTMLtoString(answer.getAnswer());
							this.mcText4 = WebUtils.HTMLtoString(answer.getText());
							// Vemos si es correcta o no para establecer "mcCorrect4"
							if (answer.isCorrect()) this.mcCorrect4 = "op_true";
							else this.mcCorrect4 = "op_false";
							break;
						case 5 :
							// Opcin 5
							this.mcAnswer5 = WebUtils.HTMLtoString(answer.getAnswer());
							this.mcText5 = WebUtils.HTMLtoString(answer.getText());
							// Vemos si es correcta o no para establecer "mcCorrect5"
							if (answer.isCorrect()) this.mcCorrect5 = "op_true";
							else this.mcCorrect5 = "op_false";
					}
					// Incrementamos el valor del contador.
					count++;
				}
				// Tenemos que completar la informacin que nos queda con el nmero de opciones
				// de respuesta hasta 5 (el mximo permitido). Sabemos que el nmero mnimo de
				// respuestas permitido es de dos por lo que el primer case del switch ser 3 y
				// nos aprovechamos de la ejecucin en cascada de los "case" de un "switch".
				switch (count)
				{
					case 3 :
						this.mcCorrect3 = "op_false";
					case 4 :
						this.mcCorrect4 = "op_false";
					case 5 :
						this.mcCorrect5 = "op_false";
				}
			}
		}
	}
	
	/**
	 * <p>Establece los valores de los atributos a los valores que se reciben en el 
	 * objeto <code>TOQuestionShortAnswer</code> que se recibe como par&aacute;metro.</p>
	 *  
	 * @param to <code>TOQuestionShortAnswer</code> con la informaci&oacute;n para 
	 * los atributos. 
	 */
	private void setData(TOQuestionShortAnswer to)
	{
		// Comprobamos que el parmetro no se nulo.
		if (to != null)
		{
			// Establecemos los valores de los atributos.
			this.id = Integer.toString(to.getId());
			this.title = WebUtils.HTMLtoString(to.getTitle());
			this.statement = WebUtils.HTMLtoString(to.getStatement());
			this.type = to.getType();
			this.saAnswer = WebUtils.HTMLtoString(to.getAnswer());
			this.saText = WebUtils.HTMLtoString(to.getText());
			// Establecemos el valor de "saCaseSensitive"
			if (to.isCaseSensitive()) this.saCaseSensitive = "op_true";
			else this.saCaseSensitive = "op_false";
		}
	}
	
	/**
	 * <p>M&eacute;todo que realiza la validaci&oacute;n simple sobre los datos
	 * del formulario cuando se pretende modificar una pregunta dentro del sistema
	 * de tipo "Verdadero o Falso".</p>
	 */
	private void validateModifyQuestionTrueOrFalse()
	{
		// Validamos el campo "statement"
		validateStatement();
	}
	
	/**
	 * <p>M&eacute;todo que realiza la validaci&oacute;n simple sobre los datos
	 * del formulario cuando se pretende modificar una pregunta dentro del sistema
	 * de tipo "Numrico".</p>
	 */
	private void validateModifyQuestionNumeric()
	{
		// Validamos el campo "statement"
		validateStatement();
		// Validamos el campo "nuValue"
		validateNuValue();
		// Validamos el campo "nuError"
		validateNuError();
	}
	
	/**
	 * <p>M&eacute;todo que realiza la validaci&oacute;n simple sobre los datos
	 * del formulario cuando se pretende modificar una pregunta dentro del sistema
	 * de tipo "Respuesta Corta".</p>
	 */
	private void validateModifyQuestionShortAnswer()
	{
		// Validamos el campo "statement"
		validateStatement();
		// Validamos el campo "saAnswer"
		validateSaAnswer();
	}
	
	/**
	 * <p>M&eacute;todo que realiza la validaci&oacute;n simple sobre los datos
	 * del formulario cuando se pretende modificar una pregunta dentro del sistema
	 * de tipo "Emparejar".</p>
	 * 
	 * @parama request <code>HttpServletRequest</code> asociado a la petici&oacute;n
	 * de validaci&oacute;n.
	 */
	private void validateModifyQuestionMatch(HttpServletRequest request)
	{
		// Validamos el campo "statement".
		validateStatement();
		// Declaramos una variable local para contar el nmero de parejas formadas.
		int total = 0;
		// Vemos las opciones de pareja una a una. Pareja: 1.
		if (((ValidationUtils.isEmpty(this.maTextLeft1)) && (!ValidationUtils.isEmpty(this.maTextRight1)))
				|| ((!ValidationUtils.isEmpty(this.maTextLeft1)) && (ValidationUtils.isEmpty(this.maTextRight1))))
		{
			// Solamente se ha especificado una de las dos opciones de la pareja 1.
			this.errors.add("Match_1", new ActionMessage("question.ma.form.invalid.match"));
		}
		else if ((!(ValidationUtils.isEmpty(this.maTextLeft1))) && (!(ValidationUtils.isEmpty(this.maTextRight1))))
		{
			// Incrementamos el nmero de parejas.
			total++;
		}
		// Pareja 2.
		if (((ValidationUtils.isEmpty(this.maTextLeft2)) && (!ValidationUtils.isEmpty(this.maTextRight2)))
				|| ((!ValidationUtils.isEmpty(this.maTextLeft2)) && (ValidationUtils.isEmpty(this.maTextRight2))))
		{
			// Solamente se ha especificado una de las dos opciones de la pareja 2.
			this.errors.add("Match_2", new ActionMessage("question.ma.form.invalid.match"));
		}
		else if ((!(ValidationUtils.isEmpty(this.maTextLeft2))) && (!(ValidationUtils.isEmpty(this.maTextRight2))))
		{
			// Incrementamos el nmero de parejas.
			total++;
		}
		// Pareja 3.
		if (((ValidationUtils.isEmpty(this.maTextLeft3)) && (!ValidationUtils.isEmpty(this.maTextRight3)))
				|| ((!ValidationUtils.isEmpty(this.maTextLeft3)) && (ValidationUtils.isEmpty(this.maTextRight3))))
		{
			// Solamente se ha especificado una de las dos opciones de la pareja 3.
			this.errors.add("Match_3", new ActionMessage("question.ma.form.invalid.match"));
		}
		else if ((!(ValidationUtils.isEmpty(this.maTextLeft3))) && (!(ValidationUtils.isEmpty(this.maTextRight3))))
		{
			// Incrementamos el nmero de parejas.
			total++;
		}
		// Pareja 4.
		if (((ValidationUtils.isEmpty(this.maTextLeft4)) && (!ValidationUtils.isEmpty(this.maTextRight4)))
				|| ((!ValidationUtils.isEmpty(this.maTextLeft4)) && (ValidationUtils.isEmpty(this.maTextRight4))))
		{
			// Solamente se ha especificado una de las dos opciones de la pareja 4.
			this.errors.add("Match_4", new ActionMessage("question.ma.form.invalid.match"));
		}
		else if ((!(ValidationUtils.isEmpty(this.maTextLeft4))) && (!(ValidationUtils.isEmpty(this.maTextRight4))))
		{
			// Incrementamos el nmero de parejas.
			total++;
		}
		// Pareja 5.
		if (((ValidationUtils.isEmpty(this.maTextLeft5)) && (!ValidationUtils.isEmpty(this.maTextRight5)))
				|| ((!ValidationUtils.isEmpty(this.maTextLeft5)) && (ValidationUtils.isEmpty(this.maTextRight5))))
		{
			// Solamente se ha especificado una de las dos opciones de la pareja 5.
			this.errors.add("Match_5", new ActionMessage("question.ma.form.invalid.match"));
		}
		else if ((!(ValidationUtils.isEmpty(this.maTextLeft5))) && (!(ValidationUtils.isEmpty(this.maTextRight5))))
		{
			// Incrementamos el nmero de parejas.
			total++;
		}
		// Comprobamos el nmero de parejas formadas.
		if (total < 2)
		{
			// Indicamos en la request que ha habido un error.
			request.setAttribute("error_vs", "no_two_options");
			// Aadimos un elemento al ActionErrors.
			this.errors.add("", new ActionMessage("common.no.error"));
		}
	}
	
	/**
	 * <p>M&eacute;todo que realiza la validaci&oacute;n simple sobre los datos
	 * del formulario cuando se pretende modificar una pregunta dentro del sistema
	 * de tipo "M&uacute;ltiples Opciones".</p>
	 * 
	 * @parama request <code>HttpServletRequest</code> asociado a la petici&oacute;n
	 * de validaci&oacute;n.
	 */
	private void validateModifyQuestionMultipleChoices(HttpServletRequest request)
	{
		// Validamos el campo "statement"
		validateStatement();
		// Validamos que se hayan escrito al menos dos opciones de respuesta.
		// Declaramos una variable local.
		int totalAnswers = 0;
		// Contamos el nmero de opciones de respuesta.
		if (!ValidationUtils.isEmpty(this.mcAnswer1)) totalAnswers++;
		if (!ValidationUtils.isEmpty(this.mcAnswer2)) totalAnswers++;
		if (!ValidationUtils.isEmpty(this.mcAnswer3)) totalAnswers++;
		if (!ValidationUtils.isEmpty(this.mcAnswer4)) totalAnswers++;
		if (!ValidationUtils.isEmpty(this.mcAnswer5)) totalAnswers++;
		// Vemos si tenemso 2 o ms opciones escritas.
		if (totalAnswers < 2)
		{
			// Indicamos que ha habido el error "indicar dos o ms respuestas".
			request.setAttribute("error_vs", "no_two_answers");
			// Aadimos un elemento al ActionErrors.
			this.errors.add("", new ActionMessage("common.no.error"));
		}
		else 
		{
			// Vamos a contar el nmero de respuestas correctas. Inicializamos el contador
			totalAnswers = 0;
			// Contamos el nmero de opciones correctas.
			if ("op_true".equals(this.mcCorrect1)) totalAnswers++;
			if ("op_true".equals(this.mcCorrect2)) totalAnswers++;
			if ("op_true".equals(this.mcCorrect3)) totalAnswers++;
			if ("op_true".equals(this.mcCorrect4)) totalAnswers++;
			if ("op_true".equals(this.mcCorrect5)) totalAnswers++;
			// Vemos el nmero total de opciones correctas.
			if (totalAnswers < 1)
			{
				// Indicamos que se produce el error "indicar una respuesta correcta".
				request.setAttribute("error_vs", "no_one_correct");
				// Aadimos un elemento al ActionErrors.
				this.errors.add("key", new ActionMessage(ERRORS_KEY_CHOOSE_ONE, "key"));
			}
			else 
			{
				// Tenemos que todas las opciones marcadas como correctas 
				// tienen un texto asociado. Definimos una variable local.
				boolean hasErrors = false;
				// Comprobamos una a una las opciones de respuesta.
				if (("op_true".equals(this.mcCorrect1)) && (ValidationUtils.isEmpty(this.mcAnswer1))) hasErrors = true;
				if (("op_true".equals(this.mcCorrect2)) && (ValidationUtils.isEmpty(this.mcAnswer2))) hasErrors = true;
				if (("op_true".equals(this.mcCorrect3)) && (ValidationUtils.isEmpty(this.mcAnswer3))) hasErrors = true;
				if (("op_true".equals(this.mcCorrect4)) && (ValidationUtils.isEmpty(this.mcAnswer4))) hasErrors = true;
				if (("op_true".equals(this.mcCorrect5)) && (ValidationUtils.isEmpty(this.mcAnswer5))) hasErrors = true;
				// Comprobamos si hay errores.
				if (hasErrors)
				{
					// Indicamos que se produce el error "error de formato".
					request.setAttribute("error_vs", "invalid_format");
					// Aadimos un elemento al ActionErrors.
					this.errors.add("key", new ActionMessage(ERRORS_KEY_CHOOSE_ONE, "key"));
				}
			}
		}
	}
	
	// -------- GETTERS & SETTERS.
	public String getCategoryName()
	{
		return categoryName;
	}
	public void setCategoryName(String categoryName)
	{
		this.categoryName = categoryName;
	}
	public String getCategoryDescription()
	{
		return categoryDescription;
	}
	public void setCategoryDescription(String categoryDescription)
	{
		this.categoryDescription = categoryDescription;
	}
	public String getCategoryId()
	{
		return categoryId;
	}
	public void setCategoryId(String categoryId)
	{
		this.categoryId = categoryId;
	}
	public String getTitle() 
	{
		return title;
	}
	public void setTitle(String title) 
	{
		this.title = title;
	}
	public String getStatement() 
	{
		return statement;
	}
	public void setStatement(String statement) 
	{
		this.statement = statement;
	}
	public String getTfCorrectAnswer() 
	{
		return tfCorrectAnswer;
	}
	public void setTfCorrectAnswer(String tfCorrectAnswer) 
	{
		this.tfCorrectAnswer = tfCorrectAnswer;
	}
	public String getTfTextTrue() 
	{
		return tfTextTrue;
	}
	public void setTfTextTrue(String tfTextTrue) 
	{
		this.tfTextTrue = tfTextTrue;
	}
	public String getTfTextFalse() 
	{
		return tfTextFalse;
	}
	public void setTfTextFalse(String tfTextFalse) 
	{
		this.tfTextFalse = tfTextFalse;
	}
	public String getId() 
	{
		return id;
	}
	public void setId(String id) 
	{
		this.id = id;
	}
	public String getType() 
	{
		return type;
	}
	public void setType(String type) 
	{
		this.type = type;
	}
	public String getNuValue()
	{
		return nuValue;
	}
	public void setNuValue(String nuValue)
	{
		this.nuValue = nuValue;
	}
	public String getNuError()
	{
		return nuError;
	}
	public void setNuError(String nuError)
	{
		this.nuError = nuError;
	}
	public String getNuText()
	{
		return nuText;
	}
	public void setNuText(String nuText)
	{
		this.nuText = nuText;
	}
	public String getSaAnswer() 
	{
		return saAnswer;
	}
	public void setSaAnswer(String saAnswer) 
	{
		this.saAnswer = saAnswer;
	}
	public String getSaCaseSensitive() 
	{
		return saCaseSensitive;
	}
	public void setSaCaseSensitive(String saCaseSensitive) 
	{
		this.saCaseSensitive = saCaseSensitive;
	}
	public String getSaText() 
	{
		return saText;
	}
	public void setSaText(String saText) 
	{
		this.saText = saText;
	}
	public String getMcAnswer1() 
	{
		return mcAnswer1;
	}
	public void setMcAnswer1(String mcAnswer1) 
	{
		this.mcAnswer1 = mcAnswer1;
	}
	public String getMcAnswer2() 
	{
		return mcAnswer2;
	}
	public void setMcAnswer2(String mcAnswer2) 
	{
		this.mcAnswer2 = mcAnswer2;
	}
	public String getMcAnswer3() 
	{
		return mcAnswer3;
	}
	public void setMcAnswer3(String mcAnswer3) 
	{
		this.mcAnswer3 = mcAnswer3;
	}
	public String getMcAnswer4() 
	{
		return mcAnswer4;
	}
	public void setMcAnswer4(String mcAnswer4) 
	{
		this.mcAnswer4 = mcAnswer4;
	}
	public String getMcAnswer5() 
	{
		return mcAnswer5;
	}
	public void setMcAnswer5(String mcAnswer5) 
	{
		this.mcAnswer5 = mcAnswer5;
	}
	public String getMcCorrect1() 
	{
		return mcCorrect1;
	}
	public void setMcCorrect1(String mcCorrect1) 
	{
		this.mcCorrect1 = mcCorrect1;
	}
	public String getMcCorrect2() 
	{
		return mcCorrect2;
	}
	public void setMcCorrect2(String mcCorrect2) 
	{
		this.mcCorrect2 = mcCorrect2;
	}
	public String getMcCorrect3() 
	{
		return mcCorrect3;
	}
	public void setMcCorrect3(String mcCorrect3) 
	{
		this.mcCorrect3 = mcCorrect3;
	}
	public String getMcCorrect4() 
	{
		return mcCorrect4;
	}
	public void setMcCorrect4(String mcCorrect4) 
	{
		this.mcCorrect4 = mcCorrect4;
	}
	public String getMcCorrect5() 
	{
		return mcCorrect5;
	}
	public void setMcCorrect5(String mcCorrect5) 
	{
		this.mcCorrect5 = mcCorrect5;
	}
	public String getMcText1() 
	{
		return mcText1;
	}
	public void setMcText1(String mcText1) 
	{
		this.mcText1 = mcText1;
	}
	public String getMcText2() 
	{
		return mcText2;
	}
	public void setMcText2(String mcText2) 
	{
		this.mcText2 = mcText2;
	}
	public String getMcText3() 
	{
		return mcText3;
	}
	public void setMcText3(String mcText3) 
	{
		this.mcText3 = mcText3;
	}
	public String getMcText4() 
	{
		return mcText4;
	}
	public void setMcText4(String mcText4) 
	{
		this.mcText4 = mcText4;
	}
	public String getMcText5() 
	{
		return mcText5;
	}
	public void setMcText5(String mcText5) 
	{
		this.mcText5 = mcText5;
	}
	public String getMaTextLeft1() 
	{
		return maTextLeft1;
	}
	public void setMaTextLeft1(String maTextLeft1) 
	{
		this.maTextLeft1 = maTextLeft1;
	}
	public String getMaTextLeft2() 
	{
		return maTextLeft2;
	}
	public void setMaTextLeft2(String maTextLeft2) 
	{
		this.maTextLeft2 = maTextLeft2;
	}
	public String getMaTextLeft3() 
	{
		return maTextLeft3;
	}
	public void setMaTextLeft3(String maTextLeft3) 
	{
		this.maTextLeft3 = maTextLeft3;
	}
	public String getMaTextLeft4() 
	{
		return maTextLeft4;
	}
	public void setMaTextLeft4(String maTextLeft4) 
	{
		this.maTextLeft4 = maTextLeft4;
	}
	public String getMaTextLeft5() 
	{
		return maTextLeft5;
	}
	public void setMaTextLeft5(String maTextLeft5) 
	{
		this.maTextLeft5 = maTextLeft5;
	}
	public String getMaTextRight1() 
	{
		return maTextRight1;
	}
	public void setMaTextRight1(String maTextRight1) 
	{
		this.maTextRight1 = maTextRight1;
	}
	public String getMaTextRight2() 
	{
		return maTextRight2;
	}
	public void setMaTextRight2(String maTextRight2) 
	{
		this.maTextRight2 = maTextRight2;
	}
	public String getMaTextRight3() 
	{
		return maTextRight3;
	}
	public void setMaTextRight3(String maTextRight3) 
	{
		this.maTextRight3 = maTextRight3;
	}
	public String getMaTextRight4() 
	{
		return maTextRight4;
	}
	public void setMaTextRight4(String maTextRight4) 
	{
		this.maTextRight4 = maTextRight4;
	}
	public String getMaTextRight5() 
	{
		return maTextRight5;
	}
	public void setMaTextRight5(String maTextRight5) 
	{
		this.maTextRight5 = maTextRight5;
	}
	public String getMaExplain1() 
	{
		return maExplain1;
	}
	public void setMaExplain1(String maExplain1) 
	{
		this.maExplain1 = maExplain1;
	}
	public String getMaExplain2() 
	{
		return maExplain2;
	}
	public void setMaExplain2(String maExplain2) 
	{
		this.maExplain2 = maExplain2;
	}
	public String getMaExplain3() 
	{
		return maExplain3;
	}
	public void setMaExplain3(String maExplain3) 
	{
		this.maExplain3 = maExplain3;
	}
	public String getMaExplain4() 
	{
		return maExplain4;
	}
	public void setMaExplain4(String maExplain4) 
	{
		this.maExplain4 = maExplain4;
	}
	public String getMaExplain5() 
	{
		return maExplain5;
	}
	public void setMaExplain5(String maExplain5) 
	{
		this.maExplain5 = maExplain5;
	}
	
}
