package edu.uoc.pfc.struts.forms;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMessage;

import edu.uoc.pfc.business.transfers.TOTest;
import edu.uoc.pfc.lib.utils.BeanUtils;
import edu.uoc.pfc.lib.utils.ValidationUtils;
import edu.uoc.pfc.lib.utils.WebUtils;

/**
 * <code>DispatchAction</code> que ser&aacute; la encargada de gestionar el intercambio 
 * de informaci&oacute;n entre los usuarios y el sistemas relacionada con la gesti&oacute;n
 * de los cuestionarios.</p>
 * 
 * @author Jos&eacute; Manuel &Aacute;lvarez &Aacute;lvarez
 * @version 1.0
 */
public class AFTest extends AFBase 
{
	
	// -------- CONSTANTES.
	/** <p>N&uacute;mero de serie por defecto para los objetos de esta clase.</p> */
	private static final long serialVersionUID = 1L;
	
	// -------- ATRIBUTOS.
	/** <p>Identificador del Cuestionario.</p> */
	private String id;
	/** <p>T&iacute;tulo del Cuestionario.</p> */
	private String title;
	/** <p>Texto de introducci&oacute;n.</p> */
	private String introduction;
	/** <p>Indica si se barajar&aacute;n las preguntas.</p> */
	private String shuffleQuestions;
	/** <p>Indica si se han de barajar las respuestas.</p> */
	private String shuffleAnswers;
	/** <p>Indica si se ha de mostrar el resultado.</p> */
	private String showResult;
	/** <p>Indica si se ha de mostrar los texto de revisi&oacute;n del Cuestionario.</p> */
	private String showReview;
	
	/** <p>Valor de una pregunta dentro de un Cuestionario.</p> */
	private String questionValue;
	/** <p>Respuesta de una pregunta de tipo Num&eacute;rico.</p> */
	private String nuAnswer;
	/** <p>Respuesta de una pregunta de tipo Respuesta Corta.</p> */
	private String saAnswer;
	
	// -------- MTODOS PBLICOS.
	/**
	 * <p>Realiza las validaciones simples sobre los datos del formulario cuando la transacci&oacute;n
	 * que se est&aacute; ejecutando servir&aacute; para crear un nuevo Cuestionario en el sistema.</p>
	 * 
	 * @return <code>ActionErros</code> con todos los errores de validaci&oacute;n simples en los datos
	 * del formulario. En el caso de que no hubiese ninguno este m&eacute;todo retornar&aacute; un objeto
	 * <code>ActionErrors</code> vac&iacute;o.
	 */
	public ActionErrors validateCreateTest()
	{
		// Eliminamos los posibles errores de validacin simples.
		clearErrors();
		// Validamos el campo "title"
		validateTitle();
		// Validamos el campo "introducction"
		validateIntroduction();
		// Comprobamos si ha habido errores de validacin simples.
		if (this.errors.size() == 0)
		{
			// No ha habido errores de validacin simples por lo que eliminamos los posibles espacios
			// en blanco al principio y final del valor de cada uno de los atributos.
			BeanUtils.trimProperties(this);
		}
		// Retornamos los errores de validacin simples.
		return this.errors;
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre los datos del formulario cuando la transacci&oacute;n
	 * que se est&aacute; ejecutando servir&aacute; para a&ntilde;adir una Pregunta a un Cuestionario.</p>
	 * 
	 * @return <code>ActionErros</code> con todos los errores de validaci&oacute;n simples en los datos
	 * del formulario. En el caso de que no hubiese ninguno este m&eacute;todo retornar&aacute; un objeto
	 * <code>ActionErrors</code> vac&iacute;o.
	 */
	public ActionErrors validateAddQuestion()
	{
		// Eliminamos los posibles errores de validacin simples que tengamos.
		clearErrors();
		// En este caso simplemente tenemos que validar la informacin del campo "questionValue".
		validateQuestionValue();
		// Retornamos la lista de errores de validacin simples.
		return this.errors;
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre los datos del formulario cuando la transacci&oacute;n
	 * que se est&aacute; ejecutando servir&aacute; para modificar un Cuestionario en el sistema.</p>
	 * 
	 * @return <code>ActionErros</code> con todos los errores de validaci&oacute;n simples en los datos
	 * del formulario. En el caso de que no hubiese ninguno este m&eacute;todo retornar&aacute; un objeto
	 * <code>ActionErrors</code> vac&iacute;o.
	 */
	public ActionErrors validateModifyTest()
	{
		// Borramos los errores de valicin simples.
		clearErrors();
		// Validamos el campo "introduction".
		validateIntroduction();
		// Comprobamos si ha habido errores.
		if (this.errors.size() == 0)
		{
			// No hay errores de validacin simples as que borramos los posibles espacios en blanco
			// al principio y final de los atributos del formulario.
			BeanUtils.trimProperties(this);
		}
		// Retornamos los errores de validacin simples.
		return this.errors;
	}
	
	/**
	 * <p>Establece los datos del formulario a partir de los valores de los atributos del objeto 
	 * <code>TOTest</code> que recibe como par&aacute;metro.</p>
	 * 
	 * @param test <code>TOTest</code> a partir del cual establecer los valores de los atributos.
	 */
	public void setData(TOTest test)
	{
		// Comprobamos que el parmetro no es nulo.
		if (test != null)
		{
			// Establecemos los valores de los atributos.
			this.id = Integer.toString(test.getId());
			this.title = WebUtils.HTMLtoString(test.getTitle());
			this.introduction = WebUtils.HTMLtoString(test.getIntroduction());
			// Valor de "Barajar preguntas"
			if (test.isShuffleQuestions()) this.shuffleQuestions = "op_true";
			else this.shuffleQuestions = "op_false";
			// Valor de "Barajar respuestas"
			if (test.isShuffleAnswers()) this.shuffleAnswers = "op_true";
			else this.shuffleAnswers = "op_false";
			// Valor de "Mostrar Solucin"
			if (test.isShowResult()) this.showResult = "op_true";
			else this.showResult = "op_false";
			// Valor de "Mostrar Revisin"
			if (test.isShowReview()) this.showReview = "op_true";
			else this.showReview = "op_false";
		}
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre los datos del formulario cuando se est&aacute; contestando
	 * a una pregunta de tipo "Respuesta Num&eacute;rica".</p>
	 * 
	 * @return <code>ActionErros</code> con todos los errores de validaci&oacute;n simples en los datos
	 * del formulario. En el caso de que no hubiese ninguno este m&eacute;todo retornar&aacute; un objeto
	 * <code>ActionErrors</code> vac&iacute;o.
	 */
	public ActionErrors validateAnswerNU()
	{
		// Borramos los posibles errores de validacin simples.
		clearErrors();
		// Validamos el campo "nuAnswer"
		validateNuAnswer();
		// Retornamos los errores de validacin simples.
		return this.errors;
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre los datos del formulario cuando se est&aacute; contestando
	 * a una pregunta de tipo "Respuesta Corta".</p>
	 * 
	 * @return <code>ActionErros</code> con todos los errores de validaci&oacute;n simples en los datos
	 * del formulario. En el caso de que no hubiese ninguno este m&eacute;todo retornar&aacute; un objeto
	 * <code>ActionErrors</code> vac&iacute;o.
	 */
	public ActionErrors validateAnswerSA()
	{
		// Borramos los posibles errores de validacin simples.
		clearErrors();
		// Validamos el campo "saAnswer"
		validateSaAnswer();
		// Comprobamos si ha habido errores de validacin simples.
		if (this.errors.size() == 0)
		{
			// Eliminamos los espacios en blanco al principio y final de las propiedades.
			BeanUtils.trimProperties(this);
		}
		// Retornamos los errores de validacin simples.
		return this.errors; 
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre los datos del formulario cuando se est&aacute; contestando
	 * a una pregunta de tipo "M&uacugte;ltiples Opciones".</p>
	 * 
	 * @param request <code>HttpServletRequest</code> con la informaci&oacute;n de la petici&oacute;n HTTP
	 * con la respuesta del usuario.
	 * @return <code>ActionErros</code> con todos los errores de validaci&oacute;n simples en los datos
	 * del formulario. En el caso de que no hubiese ninguno este m&eacute;todo retornar&aacute; un objeto
	 * <code>ActionErrors</code> vac&iacute;o.
	 */
	public ActionErrors validateAnswerMC(HttpServletRequest request)
	{
		// Borramos todos los posibles errores de validacin simples.
		clearErrors();
		// Declaramos un variable booleana que nos servir para ver si se ha marcado al menos una opcin.
		boolean hasAnswer = false;
		// Recordemos que como mucho tenemos 5 posibles opciones. Las recorremos.
		for (int i = 1; i <= 5; i++)
		{
			// Vemos si hemos recibido el parmetro mcAnswer"i"
			if (!ValidationUtils.isEmpty(request.getParameter("mcAnswer" + i)))
			{
				// S que se ha marcado la opcin por lo que al menos hay una respuesta. Actualizamos la
				// varibale booleana.
				hasAnswer = true;
				// Rompemos la ejecucin del bucle.
				break;
			}
		}
		// Comprobamos a ver si ha habido al menos una respuesta.
		if (!hasAnswer)
		{
			// No tenemos una respuesta as que guardamos el error simple.
			saveChooseOne("McAnswer", "Respuesta");
		}
		
		// Retornamos los errores de validacin simples.
		return this.errors;
	}
	
	/**
	 * <p>Realiza las validaciones simples sobre los datos del formulario cuando se est&aacute; contestando
	 * a una pregunta de tipo "Emparejar".</p>
	 * 
	 * @param request <code>HttpServletRequest</code> con la informaci&oacute;n de la petici&oacute;n HTTP
	 * con la respuesta del usuario.
	 * @return <code>ActionErros</code> con todos los errores de validaci&oacute;n simples en los datos
	 * del formulario. En el caso de que no hubiese ninguno este m&eacute;todo retornar&aacute; un objeto
	 * <code>ActionErrors</code> vac&iacute;o.
	 */
	public ActionErrors validateAnswerMA(HttpServletRequest request)
	{
		// Eliminamos los posibles errores de validacin simples que tuvisemos.
		clearErrors();
		// Vamos a comprobamos cuntos <select> tenemos.
		int totalSelect = 0;
		// Recorremos los <select> recibidos.
		for (int i = 1; i <= 5; i++)
		{
			// Vemos si hemos recibido un valor del <select> de la izquierda.
			if (ValidationUtils.isEmpty(request.getParameter("maSelect" + i + "Left")))
			{
				// No hemos recibido un valor as que finalizamos la ejecucin del bucle.
				break;
			}
			else
			{
				// S que hemos recibido un valor as que incrementamos el nmero de <select>
				totalSelect++;
			}
		}
		// Declaramos dos arrays de String para almacenar los valores recibidos.
		String[] leftValues = new String[totalSelect];
		String[] rightValues = new String[totalSelect];
		// Rellenamos ambos arrays a partir de la informacin recibida.
		for (int i = 0; i < totalSelect; i++)
		{
			// Valor de la izquierda.
			leftValues[i] = request.getParameter("maSelect" + (i + 1) + "Left");
			// Valor de la derecha.
			rightValues[i] = request.getParameter("maSelect" + (i + 1) + "Right");
		}
		// Definimos una variable booleana que nos indicar si haya alguna opcin duplicada.
		boolean duplicated = false;
		// Comprobamos si existe algn elemento duplicado en los arrays. (Definimos una etiqueta)
		outer :
		for (int i = 0; i < totalSelect; i++)
		{
			// Valores a comprobar de ambos arrays.
			String valueLeft = leftValues[i];
			String valueRight = rightValues[i];
			// Recorremos el resto de valores del array.
			for (int j = (i + 1); j < totalSelect; j++)
			{
				// Comprobamos los elementos de la izquierda y derecha.
				if ((valueLeft.equals(leftValues[j])) || (valueRight.equals(rightValues[j])))
				{
					// El elemento est duplicado.
					duplicated = true;
					// Finalizamos la ejecucin de ambos bucles.
					break outer;
				}
			}
		}
		// Comprobamos si ha habido elementos duplicados o no.
		if (duplicated)
		{
			// S que tenemos elementos duplicados. Inicializamos los errores.
			initErrors();
			// Grabamos el error.
			this.errors.add("MaAnswer", new ActionMessage("test.solve.question.MA.error.duplicated"));
		}
		// Retornamos los errores de validacin simples.
		return this.errors;
	}
	
	// -------- MTODOS PRIVADOS.
	/**
	 * <p>Realiza la validaci&oacute;n simple sobre el atributo "saAnswer". En caso de que hubiese alg&uacute;n
	 * error este ser&aacute; a&ntilde;adido al atributo "errors".
	 */
	private void validateSaAnswer()
	{
		// Coprobamos que el campo no sea vaco.
		if (ValidationUtils.isEmpty(this.saAnswer))
		{
			// Grabamos el error "campo vaco".
			saveEmptyError("SaAnswer", "Respuesta");
		}
	}
	
	/**
	 * <p>Realiza la validaci&oacute;n simple sobre el atributo "nuAnswer". En caso de que hubiese alg&uacute;n
	 * error este ser&aacute; a&ntilde;adido al atributo "errors".
	 */
	private void validateNuAnswer()
	{
		// Coprobamos que el campo no sea vaco.
		if (ValidationUtils.isEmpty(this.nuAnswer))
		{
			// Grabamos el error "campo vaco".
			saveEmptyError("NuAnswer", "Respuesta");
		}
		else if (!ValidationUtils.isDouble(this.nuAnswer))
		{
			// El valor del campo no es double. Grabamos un error.
			saveNoNumber("NuAnswer", "Respuesta");
		}
	}
	
	/**
	 * <p>Realiza la validaci&oacute;n simple sobre el atributo "questionValue". En caso de que hubiese alg&uacute;n
	 * error este ser&aacute; a&ntilde;adido al atributo "errors".
	 */
	private void validateQuestionValue()
	{
		// Comprobamos que el campo no sea vaco.
		if (ValidationUtils.isEmpty(this.questionValue))
		{
			// Aadimos el error "campo obligatorio".
			saveEmptyError("QuestionValue", "Valor");
		}
		else if (!ValidationUtils.isDouble(this.questionValue))
		{
			// El campo no contiene un valor vlido. Aadimos un error
			saveNoNumber("QuestionValue", "Valor");
		}
	}
	/**
	 * <p>Realiza la validaci&oacute;n simple sobre el atributo "title". En caso de que hubiese alg&uacute;n
	 * error este ser&aacute; a&ntilde;adido al atributo "errors".
	 */
	private void validateTitle()
	{
		// Comprobamos que el campo no sea vaco.
		if (ValidationUtils.isEmpty(this.title))
		{
			// Aadimos el error "campo obligatorio".
			saveEmptyError("Title", "Titulo");
		}
	}
	
	/**
	 * <p>Realiza la validaci&oacute;n simple sobre el atributo "introduction". En caso de que hubiese 
	 * alg&uacute;n error este ser&aacute; a&ntilde;adido al atributo "errors".
	 */
	private void validateIntroduction()
	{
		// Comprobamos que el campo no sea vaco.
		if (ValidationUtils.isEmpty(this.introduction))
		{
			// Aadimos el error "campo obligatorio".
			saveEmptyError("Introduction", "Introduccion");
		}
	}
	
	// -------- GETTERS & SETTERS
	public String getTitle() 
	{
		return title;
	}
	public void setTitle(String title) 
	{
		this.title = title;
	}
	public String getIntroduction() 
	{
		return introduction;
	}
	public void setIntroduction(String introduction) 
	{
		this.introduction = introduction;
	}
	public String getShuffleQuestions() 
	{
		return shuffleQuestions;
	}
	public void setShuffleQuestions(String shuffleQuestions) 
	{
		this.shuffleQuestions = shuffleQuestions;
	}
	public String getShuffleAnswers() 
	{
		return shuffleAnswers;
	}
	public void setShuffleAnswers(String shuffleAnswers) 
	{
		this.shuffleAnswers = shuffleAnswers;
	}
	public String getShowResult() 
	{
		return showResult;
	}
	public void setShowResult(String showResult) 
	{
		this.showResult = showResult;
	}
	public String getShowReview() 
	{
		return showReview;
	}
	public void setShowReview(String showReview) 
	{
		this.showReview = showReview;
	}
	public String getId() 
	{
		return id;
	}
	public void setId(String id) 
	{
		this.id = id;
	}
	public String getQuestionValue() 
	{
		return questionValue;
	}
	public void setQuestionValue(String questionValue) 
	{
		this.questionValue = questionValue;
	}
	public String getNuAnswer()
	{
		return nuAnswer;
	}
	public void setNuAnswer(String nuAnswer)
	{
		this.nuAnswer = nuAnswer;
	}
	public String getSaAnswer()
	{
		return saAnswer;
	}
	public void setSaAnswer(String saAnswer)
	{
		this.saAnswer = saAnswer;
	}
	
}
